@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2003 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -52,6 +52,7 @@ static server_t* share_server = NULL;
 static browse_t* popup_browse = NULL;
 static browse_node_t* popup_node = NULL;
 
+
 gint await_conn_ack2(gpointer data, gint source,
 		     GdkInputCondition condition);
 gint send_browse_get(gpointer data, gint source,
@@ -80,41 +81,51 @@ void share_fill_buffer(share_t* share) {
     if (buffer->datamax - buffer->datasize < 1024)
       buffer_resize(buffer, 1024);
 
+    added = sprintf(buffer->data + buffer->datasize,
+		    "\"%s\" %s %lu %d %d %d\n",
+		    file->winname,
+		    file->md5, file->size,
+		    file->bitrate, file->frequency,
+		    file->duration);
 #ifdef HAVE_LIBLCONV_H
-    if (!global.options.use_iconv) {
-#endif
-      added = sprintf(buffer->data + buffer->datasize,
-		      "\"%s\" %s %lu %d %d %d\n",
-		      file->winname,
-		      file->md5, file->size,
-		      file->bitrate, file->frequency,
-		      file->duration);
-#ifdef HAVE_LIBLCONV_H
-    } else {
-      lconv_buf = lconv_strdup_conv(LCONV_SPEC, global.options.dest_codeset, local_codeset, file->winname);
-      added = sprintf(buffer->data + buffer->datasize,
-		      "\"%s\" %s %lu %d %d %d\n",
-		      lconv_buf,
-		      file->md5, file->size,
-		      file->bitrate, file->frequency,
-		      file->duration);
+    if (global.options.use_iconv) {
+      char* lconv_buf =
+	lconv_strdup_conv(LCONV_SPEC, global.options.dest_codeset, local_codeset, 
+			  buffer->data + buffer->datasize);
+      added = strlen(lconv_buf);
+      // replace the original buffer
+      strcpy(buffer->data + buffer->datasize, lconv_buf);
+      free(lconv_buf);
     }
 #endif
+
     buffer->datasize += added;
     node = file_node_next(node);
   }
+
   added = sprintf(buffer->data + buffer->datasize, "\n");
+#ifdef HAVE_LIBLCONV_H
+  if (global.options.use_iconv) {
+    char* lconv_buf = 
+      lconv_strdup_conv(LCONV_SPEC, global.options.dest_codeset, local_codeset, 
+			buffer->data + buffer->datasize);
+    added = strlen(lconv_buf);
+    // replace the original buffer
+    strcpy(buffer->data + buffer->datasize, lconv_buf);
+    free(lconv_buf);
+  }
+#endif
   buffer->datasize += added;
 }
 
 share_t *share_new(net_t* net, char* user) {
   share_t *share;
 
-  share = l_malloc(sizeof(share_t));
-  share->data = transfer_data_new(S_QUEUED);
-  share->data->user_info = user_info_detect(net, user);
+  share = g_malloc(sizeof(share_t));
+  transfer_init(TRANS(share), S_QUEUED);
+  TRANS(share)->user_info = user_info_detect(net, user);
   share->nu.net = net;
-  share->nu.user = l_strdup(user);
+  share->nu.user = g_strdup(user);
   share->buffer = NULL;
 
   return share;
@@ -122,7 +133,8 @@ share_t *share_new(net_t* net, char* user) {
 
 void share_end(socket_t * socket, int mode) {
   share_t *share = socket->data;
-  
+  transfer_t* trans = TRANS(share);
+
   if (!share) return;
 
   share_status_set(socket, mode);
@@ -130,13 +142,11 @@ void share_end(socket_t * socket, int mode) {
   if (share->buffer) {
     buffer_destroy(share->buffer);
     share->buffer = NULL;
-    share->data->timeleft = 
-      share->data->stop_time - share->data->start_time;
-    if (share->data->timeleft > 0)
-      share->data->rate =
-	share->data->transferred / share->data->timeleft;
+    trans->timeleft = trans->stop_time - trans->start_time;
+    if (trans->timeleft > 0)
+      trans->rate = trans->transferred / trans->timeleft;
     else
-      share->data->rate = 0;
+      trans->rate = 0;
   }
 
   share_update(socket);
@@ -161,9 +171,9 @@ void share_hide(socket_t * socket) {
 void share_destroy(share_t *share) {
   if (!share) return;
 
-  transfer_data_destroy(share->data);
   if (share->buffer) buffer_destroy(share->buffer);
-  l_free(share);
+
+  transfer_destroy(TRANS(share));
 }
 
 gint share_push_output(gpointer data, gint source,
@@ -179,7 +189,7 @@ gint share_push_output(gpointer data, gint source,
     return 1;
   }
 
-  maxpush = upload_calc_max(share->data->user_info);
+  maxpush = upload_calc_max(TRANS(share)->user_info);
   if (!maxpush) {
     gdk_input_remove(socket->output);
     socket->output = -1;
@@ -200,8 +210,8 @@ gint share_push_output(gpointer data, gint source,
     }
 
     global.up_width.bytes[0] += sent;
-    share->data->user_info->bytes[1] += sent;
-    share->data->transferred += sent;
+    TRANS(share)->user_info->bytes[1] += sent;
+    TRANS(share)->transferred += sent;
   }
   if (buffer->consumed == buffer->datasize) {
     socket_end(socket, S_FINISHED);
@@ -223,15 +233,15 @@ gint share_send_nick(gpointer data, gint source,
   gdk_input_remove(socket->output);
   socket->output = -1;
 
-  temp_str = l_strdup_printf("%s\n", share->nu.net->user.username);
+  temp_str = g_strdup_printf("%s\n", share->nu.net->user.username);
 
   if (send_safe(source, temp_str, 
 		strlen(temp_str), strlen(temp_str)) == -1) {
-    l_free(temp_str);
+    g_free(temp_str);
     socket_destroy(socket, 0);
     return 1;
   }
-  l_free(temp_str);
+  g_free(temp_str);
 
   share_fill_buffer(share);
   share_status_set(socket, S_UPLOADING);
@@ -253,7 +263,7 @@ socket_t *share_search_mapable() {
     share = socket->data;
     if (!share) continue;
 
-    if (share->data->status == S_WAITING)
+    if (TRANS(share)->status == S_WAITING)
       return socket;
   }
   
@@ -277,7 +287,7 @@ gint share_fw_get_info(gpointer data, gint source ATTR_UNUSED,
   socket2 = share_search_mapable();
   if (socket2) {
     share = socket2->data;
-    if (string_list_search(LIST_ENEMY, share->data->user_info->nick)) {
+    if (string_list_search(LIST_ENEMY, TRANS(share)->user_info->nick)) {
       socket_destroy(socket, S_DELETE);
       return 1;
     }
@@ -344,36 +354,37 @@ void share_status_set(socket_t * socket, int status) {
   chat_page_t *page;
   GtkCList *clist;
   int row;
+  transfer_t* trans;
 
   if (!socket) return;
   share = socket->data;
   if (!share) return;
-  add = transfer_status_dist(share->data->status, status);
+  trans = TRANS(share);
+  add = transfer_status_dist(trans->status, status);
 
   if (add < 0) check_uploads_again();
-
-  if (share->data->user_info)
-    share->data->user_info->cur[1] += add;
+  
+  if (trans->user_info) trans->user_info->cur[1] += add;
   global.limit.cur_uploads += add;
 
   if (status == S_UPLOADING) {
     global.limit.cur_real_uploads++;
-    share->data->user_info->real_cur[1]++;
-    share->data->start_time = global.current_time;
-    share->data->transferred = 0;
+    trans->user_info->real_cur[1]++;
+    trans->start_time = global.current_time;
+    trans->transferred = 0;
   } else if (add > 0)
-    share->data->start_time = global.current_time;
+    trans->start_time = global.current_time;
 
-  if (share->data->status == S_UPLOADING) {
+  if (trans->status == S_UPLOADING) {
     global.limit.cur_real_uploads--;
-    share->data->user_info->real_cur[1]--;
-    share->data->stop_time = global.current_time;
+    trans->user_info->real_cur[1]--;
+    trans->stop_time = global.current_time;
   }
 
   if (status == S_UPLOADING) {
     l_log(NULL, "uploads", LOG_OTHER,
 	  "Uploading [Shared List] to [%s]\n",
-	  share->data->user_info->nick);
+	  trans->user_info->nick);
     
     if ((global.options.no_piping & NOPIPE_UPLOAD) == 0) {
       if (global.options.piping & PIPE_UPLOAD) {
@@ -390,17 +401,17 @@ void share_status_set(socket_t * socket, int status) {
 			    "Shared List", "Shared List",
 			    NULL, NULL);
     }
-  } else if (share->data->status == S_UPLOADING) {
-    if (share->data->stop_time - share->data->start_time > 0)
-      val = share->data->transferred /
-	(share->data->stop_time - share->data->start_time);
+  } else if (trans->status == S_UPLOADING) {
+    if (trans->stop_time - trans->start_time > 0)
+      val = trans->transferred /
+	(trans->stop_time - trans->start_time);
     else
       val = 0;
 
     print_speed(str, val, 1);
     l_log(NULL, "uploads", LOG_OTHER,
 	  "Ending [Shared List] to [%s] :%s: [%ld] (%s)\n",
-	  share->data->user_info->nick,
+	  trans->user_info->nick,
 	  status_names(status),
 	  share->buffer->consumed, str);
     
@@ -420,7 +431,7 @@ void share_status_set(socket_t * socket, int status) {
     }
   }
 
-  share->data->status = status;
+  trans->status = status;
   share_update(socket);
 
   clist = GTK_CLIST(lookup_widget(global.win, "transfer_up"));
@@ -446,17 +457,17 @@ void share_update(socket_t * socket) {
   GdkPixmap *pixmap = NULL;
   GdkBitmap *bitmap = NULL;
   share_t *share;
-  transfer_data_t* data;
+  transfer_t* trans;
 
   if (!socket) return;
   share = socket->data;
-  data = share->data;
+  trans = TRANS(share);
 
   clist = GTK_CLIST(lookup_widget(global.win, "transfer_up"));
   row = gtk_clist_find_row_from_data(clist, socket);
   if (row < 0) return;
 
-  strcpy(str, LineSpeed(data->user_info->linespeed));
+  strcpy(str, LineSpeed(trans->user_info->linespeed));
   gtk_clist_set_text(clist, row, 4, str);
 
   // filesize
@@ -470,14 +481,14 @@ void share_update(socket_t * socket) {
   gtk_clist_set_text(clist, row, 1, str);
 
   // status
-  if (data->status == S_WAITING)
+  if (trans->status == S_WAITING)
     sprintf(str, "%d Waiting", (socket->max_cnt - socket->cnt));
   else
-    strcpy(str, status_names(data->status));
+    strcpy(str, status_names(trans->status));
   gtk_clist_set_text(clist, row, 3, str);
 
   // progress
-  if ((data->status == S_UPLOADING) && (share->buffer)) {
+  if ((trans->status == S_UPLOADING) && (share->buffer)) {
     gtk_clist_get_pixmap(clist, row, 5, &pixmap, &bitmap);
     pixmap =
       transfer_draw_progress(clist, pixmap, 
@@ -490,17 +501,17 @@ void share_update(socket_t * socket) {
     gtk_clist_set_text(clist, row, 5, "");
   }
 
-  if (data->rate > 0 || data->status == S_UPLOADING) {
-    print_speed(str, (int) data->rate, 1);
-    up_speed_pixs(data->rate, &pixmap, &bitmap);
+  if (trans->rate > 0 || trans->status == S_UPLOADING) {
+    print_speed(str, (int) trans->rate, 1);
+    up_speed_pixs(trans->rate, &pixmap, &bitmap);
     gtk_clist_set_pixtext(clist, row, 6, str, 5, pixmap, bitmap);
   } else {
     gtk_clist_set_text(clist, row, 6, "");
   }
 
-  if (data->rate > 0) {
-    print_time_unit(str, data->timeleft);
-  } else if (data->status == S_UPLOADING) {
+  if (trans->rate > 0) {
+    print_time_unit(str, trans->timeleft);
+  } else if (trans->status == S_UPLOADING) {
     sprintf(str, "stalled");
   } else {
     *str = 0;
@@ -514,20 +525,18 @@ void share_show(socket_t* socket) {
   GtkCList *temp;
   int row;
   share_t *share = socket->data;
-  transfer_data_t* data;
+  transfer_t* trans = TRANS(share);
 
   temp = GTK_CLIST(lookup_widget(global.win, "transfer_up"));
   row = gtk_clist_find_row_from_data(temp, socket);
   if (row >= 0) return;
 
-  data = share->data;
-  
   strcpy(tstr[0], "Library upload");
   print_size(tstr[1], 0);
   sprintf(tstr[2], "[%s] %s", 
 	  share->nu.net?NetType[share->nu.net->type]:"?",
-	  data->user_info->nick);
-  strcpy(tstr[4], LineSpeed(share->data->user_info->linespeed));
+	  trans->user_info->nick);
+  strcpy(tstr[4], LineSpeed(trans->user_info->linespeed));
 
   tstr[3][0] = tstr[5][0] = tstr[6][0] = 0;
   row = gtk_clist_append(temp, list);
@@ -680,15 +689,9 @@ void browse_insert_file(browse_t* browse, file_t* file) {
 HANDLER(nap_browse_end);
 
 gint get_browse_files(gpointer data, gint source,
-		      GdkInputCondition condition)
-{
+		      GdkInputCondition condition) {
   int res;
-#ifndef HAVE_LIBLCONV_H
   char buf[2049];
-#else
-  char buf[4097];
-  char lconv_buf[4097];
-#endif
   char *filename;
   char *md5;
   char *size;
@@ -701,30 +704,36 @@ gint get_browse_files(gpointer data, gint source,
   char *pos2;
   browse_t *browse = socket->data;
   buffer_t* buffer = browse->buffer;
-
+  
   if ((condition & GDK_INPUT_READ) == 0) {
     socket_destroy(socket, 0);
     return 1;
   }
+
   if ((res = recv_safe(source, buffer->data + buffer->datasize, 
 		       0, buffer->datamax - buffer->datasize - 1)) == -1) {
     socket_destroy(socket, 0);
     return 1;
   }
-  
-  global.down_width.bytes[0] += res;
-  buffer->datasize += res;
-  buffer->data[buffer->datasize] = 0;
+  buffer->data[buffer->datasize + res] = 0;
 
 #ifdef HAVE_LIBLCONV_H
   if (global.options.use_iconv) {
-    lconv_conv(LCONV_SPEC, local_codeset, global.options.dest_codeset, lconv_buf, 
-	       buffer->data);
-    strcpy(buffer->data, lconv_buf);
-    // FIXME: buffer changed!
+    char* lconv_buf =
+      lconv_strdup_conv(LCONV_SPEC, local_codeset, global.options.dest_codeset,
+			buffer->data + buffer->datasize);
+    res = strlen(lconv_buf);
+    if (res >= buffer->datamax - buffer->datasize -1)
+      buffer_resize(buffer, res);
+    strcpy(buffer->data+buffer->datasize, lconv_buf);
+    free(lconv_buf);
   }
 #endif
 
+  buffer->datasize += res;
+  global.down_width.bytes[0] += res;
+  buffer->data[buffer->datasize] = 0;
+
   pos1 = buffer->data;
   while ((pos2 = strchr(pos1, '\n'))) {
     *pos2 = 0;
@@ -738,8 +747,8 @@ gint get_browse_files(gpointer data, gint source,
 
     file = file_new();
     file_set_winname(file, filename);
-    file->user = l_strdup(FILE_TREE(browse)->name);
-    file->md5 = l_strdup(md5);
+    file->user = g_strdup(FILE_TREE(browse)->name);
+    file->md5 = g_strdup(md5);
     file->size = strtoul(size, NULL, 10);
     file->bitrate = atoi(bitrate);
     file->frequency = atoi(frequency);
@@ -923,7 +932,7 @@ socket_t *browse_search_socket(char* user) {
     socket = dlist->data;
     if (socket->type != S_BROWSE) continue;
     browse2 = socket->data;
-    if (browse2 && !l_strcasecmp(browse2->name, user)) return socket;
+    if (browse2 && !g_strcasecmp(browse2->name, user)) return socket;
   }
   return NULL;
 }
@@ -942,12 +951,15 @@ void browse_direct(browse_t* browse) {
 	       FILE_TREE(browse)->name);
 }
 
-static void
-on_remove_browse_activate(GtkWidget *source ATTR_UNUSED,
-			  gpointer user_data) {
+static gboolean
+on_remove_browse_activate(GtkWidget       *widget ATTR_UNUSED,
+			  GdkEventButton  *event ATTR_UNUSED,
+			  gpointer         user_data) {
   browse_t *browse = user_data;
 
   if (browse) browse_remove(browse, 0);
+
+  return TRUE;
 }
 
 static void
@@ -984,9 +996,9 @@ GtkWidget *create_browse_popup(browse_t* browse,
 
   if (file) {
     if (item_num > 1)
-      text = l_strdup_printf("Download Selected (%d)", item_num);
+      text = g_strdup_printf("Download Selected (%d)", item_num);
     else
-      text = l_strdup("Download File");
+      text = g_strdup("Download File");
     item = gtk_menu_item_new_with_label(text);
 
     gtk_widget_show(item);
@@ -995,9 +1007,9 @@ GtkWidget *create_browse_popup(browse_t* browse,
 		       GTK_SIGNAL_FUNC(on_download2_activate),
 		       GINT_TO_POINTER(0));
     if (item_num > 1)
-      text = l_strdup_printf("Download with Foldername (%d)", item_num);
+      text = g_strdup_printf("Download with Foldername (%d)", item_num);
     else
-      text = l_strdup("Download with Foldername");
+      text = g_strdup("Download with Foldername");
     item = gtk_menu_item_new_with_label(text);
     gtk_widget_show(item);
     gtk_container_add(GTK_CONTAINER(popup), item);
@@ -1082,7 +1094,7 @@ static file_node_t* browse_node_new(char* name) {
 
   if (!name) return NULL;
 
-  node = l_malloc(sizeof(browse_node_t));
+  node = g_malloc(sizeof(browse_node_t));
   if (!node) return NULL;
 
   file_node_init(FILE_NODE(node), name);
@@ -1102,7 +1114,7 @@ browse_search_nick(char *nick) {
 
   for (dlist = global.browses; dlist; dlist = dlist->next) {
     browse = dlist->data;
-    if (!l_strcasecmp(nick, browse->name))
+    if (!g_strcasecmp(nick, browse->name))
       return BROWSE_TREE(browse);
   }
   return NULL;
@@ -1118,7 +1130,7 @@ void browse_user_files(net_t* net, char *nick) {
     if (!FILE_TREE(browse)->files) {
       browse_load_saved(browse);
       browse_create_page(browse);
-      file_tree_show(FILE_TREE(browse), MIME_SIZE, 3);
+      file_tree_show(FILE_TREE(browse), MEDIA_SIZE, 3);
     }
     browse->net = net;
   } else {
@@ -1143,7 +1155,7 @@ browse_t *browse_search_user_net(char* user, net_t* net) {
     browse = dlist->data;
     if (!user || !net || net != browse->net ||
 	browse->status < 0) continue;
-    if (!l_strcasecmp(user, FILE_TREE(browse)->name))
+    if (!g_strcasecmp(user, FILE_TREE(browse)->name))
       return browse;
   }
   return NULL;
@@ -1152,7 +1164,7 @@ browse_t *browse_search_user_net(char* user, net_t* net) {
 static browse_t* browse_new(char* nick, char* des) {
   browse_t *browse;
 
-  browse = l_malloc(sizeof(browse_t));
+  browse = g_malloc(sizeof(browse_t));
 
   file_tree_init(FILE_TREE(browse), nick, des);
   
@@ -1286,11 +1298,11 @@ create_browse2_popup(browse_t* browse, browse_node_t* node) {
   popup_node = node;
 
   item_str =
-    l_strdup_printf("%d files = %s", 
+    g_strdup_printf("%d files = %s", 
 		    fnode->fci[tree->show_mime][tree->show_time],
 		    print_size(str, fnode->fsi[tree->show_mime][tree->show_time]));
   item = gtk_menu_item_new_with_label(item_str);
-  l_free(item_str);
+  g_free(item_str);
   gtk_widget_show(item);
   gtk_container_add(GTK_CONTAINER(popup), item);
 
@@ -1410,7 +1422,7 @@ static void browse_add_saved(browse_t* browse) {
   clist = GTK_CLIST(lookup_widget(global.win, "clist43"));
   row = gtk_clist_find_row_from_data(clist, tree);
   if (row >= 0) {
-    sprintf(tstr[1], "%d", tree->fci[MIME_SIZE][3]);
+    sprintf(tstr[1], "%d", tree->fci[MEDIA_SIZE][3]);
     gtk_clist_set_text(clist, row, 1, tstr[1]);
     strcpy(tstr[2], ctime(&tree->last_refreshed));
     tstr[2][strlen(tstr[2])-1] = 0;
@@ -1419,7 +1431,7 @@ static void browse_add_saved(browse_t* browse) {
     gtk_clist_set_text(clist, row, 3, tstr[3]);
   } else {
     strcpy(tstr[0], tree->name);
-    sprintf(tstr[1], "%d", tree->fci[MIME_SIZE][3]);
+    sprintf(tstr[1], "%d", tree->fci[MEDIA_SIZE][3]);
     strcpy(tstr[2], ctime(&tree->last_refreshed));
     tstr[2][strlen(tstr[2])-1] = 0;
     strcpy(tstr[3], tree->description?tree->description:"");
@@ -1444,7 +1456,7 @@ static void browse_save(browse_t* browse) {
   }
 
   qfprintf(file, "SAVED_BROWSE 1 %S %d %ld %S\n", 
-	   tree->name, tree->fci[MIME_SIZE][3],
+	   tree->name, tree->fci[MEDIA_SIZE][3],
 	   tree->last_refreshed, 
 	   tree->description?tree->description:"");
   browse_save_node(file, tree->files, 0);
@@ -1453,6 +1465,77 @@ static void browse_save(browse_t* browse) {
   browse_add_saved(browse);
 }
 
+static int
+on_file_tree_search_change_frombutton(GtkWidget * widget,
+				      void* user_data) {
+  GtkWidget* item = widget;
+  file_tree_t* tree = user_data;
+  char* text;
+
+  if (!tree || !tree->search_entry || !tree->ctree || !tree->clist) return 0;
+
+  gtk_label_get(GTK_LABEL(GTK_BIN(item)->child), &text);
+
+  gtk_entry_set_text(GTK_ENTRY(tree->search_entry), text);
+
+  return 0;
+}
+
+void popup_position(GtkMenu * menu, gint * x, gint * y, gpointer user_data);
+
+static void
+on_browse_quicker_clicked(GtkButton * button ATTR_UNUSED,
+			  gpointer user_data) {
+  GtkWidget *popup;
+  GtkWidget* item;
+  file_tree_t* tree = user_data;
+  char* text;
+  GList* dlist;
+
+  deactivate_auto_afk();
+
+  popup = gtk_menu_new();
+
+  if (global.string_list[LIST_QSEARCH]) {
+    for (dlist = global.string_list[LIST_QSEARCH]; dlist; dlist = dlist->next) {
+      text = dlist->data;
+      
+      item = gtk_menu_item_new_with_label(text);
+      gtk_widget_show(item);
+      gtk_container_add(GTK_CONTAINER(popup), item);
+      gtk_signal_connect(GTK_OBJECT(item), "activate",
+			 GTK_SIGNAL_FUNC(on_file_tree_search_change_frombutton),
+			 tree);
+    }
+  } else {
+    item = gtk_menu_item_new_with_label("No saved search items");
+    gtk_widget_show(item);
+    gtk_container_add(GTK_CONTAINER(popup), item);
+    gtk_widget_set_sensitive(item, FALSE);
+  }
+  gtk_menu_popup(GTK_MENU(popup), NULL, NULL,
+		 (GtkMenuPositionFunc) popup_position, button, 1, 0);
+}
+
+static void
+on_browse_quicker_add(GtkButton * button ATTR_UNUSED,
+		      gpointer user_data) {
+  GtkWidget *popup;
+  file_tree_t* tree = user_data;
+  char* text;
+  
+  deactivate_auto_afk();
+
+  if (!tree || !tree->search_entry) return;
+  popup = gtk_menu_new();
+  
+  text = gtk_entry_get_text(GTK_ENTRY(tree->search_entry));
+
+  if (!text || *text == 0) return;
+
+  qsearch_add(text);
+}
+
 GtkWidget *browse_create_page(browse_t* browse) {
   GtkWidget *vbox;
   GtkWidget *scrolledwindow;
@@ -1471,12 +1554,15 @@ GtkWidget *browse_create_page(browse_t* browse) {
   GtkNotebook *notebook;
   GtkWidget* pixmap;
   GtkWidget* hpaned;
+  GtkWidget* eventbox;
   GtkTooltips *tooltips;
   file_tree_t* tree = FILE_TREE(browse);
 
   if (tree->ctree) return NULL;
 
-  tooltips = gtk_tooltips_new ();
+  if (global.options.switch_browse) switch_to_page(4);
+
+  tooltips = gtk_tooltips_new();
 
   hpaned = gtk_hpaned_new ();
   tree->main_link = hpaned;
@@ -1502,6 +1588,8 @@ GtkWidget *browse_create_page(browse_t* browse) {
   gtk_clist_column_titles_show (GTK_CLIST (ctree));
   gtk_clist_set_sort_column(GTK_CLIST(ctree), 0);
   gtk_clist_set_auto_sort(GTK_CLIST(ctree), TRUE);
+  gtk_ctree_set_line_style(GTK_CTREE(ctree),
+			   GTK_CTREE_LINES_DOTTED);
   
   tree->signal[0] = 
     gtk_signal_connect (GTK_OBJECT (ctree), "tree_select_row",
@@ -1516,13 +1604,13 @@ GtkWidget *browse_create_page(browse_t* browse) {
   gtk_clist_set_column_widget (GTK_CLIST (ctree), 0, label);
   
 
-  hbox2 = gtk_hbox_new(FALSE, 5);
+  hbox2 = gtk_hbox_new(FALSE, 0);
   gtk_widget_show(hbox2);
   gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, FALSE, 0);
 
-  label = gtk_label_new(" Quick Search");
+  label = gtk_label_new("Search");
   gtk_widget_show(label);
-  gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 5);
 
   entry = gtk_entry_new();
   //  gtk_entry_set_text(GTK_ENTRY(entry), "not functional yet");
@@ -1535,6 +1623,52 @@ GtkWidget *browse_create_page(browse_t* browse) {
 			tree);
   tree->search_entry = entry;
 
+  // quicker_search_button
+  button = gtk_button_new ();
+  gtk_widget_show (button);
+  gtk_box_pack_start (GTK_BOX (hbox2), button, FALSE, FALSE, 0);
+  GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
+  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_HALF);
+
+  hbox3 = gtk_hbox_new (FALSE, 0);
+  gtk_widget_show (hbox3);
+  gtk_container_add (GTK_CONTAINER (button), hbox3);
+
+  pixmap = create_pixmap (global.win, "arrowr.xpm");
+  gtk_widget_show (pixmap);
+  gtk_box_pack_start (GTK_BOX (hbox3), pixmap, FALSE, FALSE, 3);
+  
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+		      GTK_SIGNAL_FUNC (on_browse_quicker_add),
+		      tree);
+
+  button = gtk_button_new ();
+  gtk_widget_show (button);
+  gtk_box_pack_start (GTK_BOX (hbox2), button, FALSE, FALSE, 0);
+  GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
+  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_HALF);
+
+  hbox3 = gtk_hbox_new (FALSE, 0);
+  gtk_widget_show (hbox3);
+  gtk_container_add (GTK_CONTAINER (button), hbox3);
+
+  pixmap = create_pixmap (global.win, "arrowu.xpm");
+  gtk_widget_show (pixmap);
+  gtk_box_pack_start (GTK_BOX (hbox3), pixmap, FALSE, FALSE, 3);
+  /*ev. not necessary: */
+  sep = gtk_vseparator_new ();
+  gtk_widget_show (sep);
+  gtk_box_pack_start (GTK_BOX (hbox3), sep, FALSE, FALSE, 0);
+
+  label = gtk_label_new ("quicker");
+  gtk_widget_show (label);
+  gtk_box_pack_start (GTK_BOX (hbox3), label, FALSE, FALSE, 3);
+
+  /**/
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+		      GTK_SIGNAL_FUNC (on_browse_quicker_clicked),
+		      tree);
+
   // right side.
   vbox = gtk_vbox_new(FALSE, 0);
   gtk_widget_show(vbox);
@@ -1549,6 +1683,9 @@ GtkWidget *browse_create_page(browse_t* browse) {
 
   clist = gtk_clist_new(5);
   tree->clist = GTK_CLIST(clist);
+  gtk_clist_set_sort_column(GTK_CLIST(clist), 0);
+  gtk_clist_set_sort_type(GTK_CLIST(clist), GTK_SORT_ASCENDING); 
+  gtk_clist_set_auto_sort(GTK_CLIST(clist), 1);
   gtk_widget_show(clist);
   gtk_widget_set_events(clist,
 			GDK_POINTER_MOTION_MASK | GDK_LEAVE_NOTIFY_MASK);
@@ -1710,28 +1847,40 @@ GtkWidget *browse_create_page(browse_t* browse) {
   gtk_widget_show(label);
   gtk_box_pack_start (GTK_BOX (tab_label), label, TRUE, TRUE, 0);
 
+  /*
   button = gtk_button_new();
   gtk_widget_show(button);
   gtk_box_pack_start (GTK_BOX (tab_label), button, FALSE, FALSE, 0);
   GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+  */
   //  gtk_widget_set_sensitive(button, FALSE);
 
+  eventbox = gtk_event_box_new ();
+  gtk_widget_show (eventbox);
+  gtk_container_add (GTK_CONTAINER (tab_label), eventbox);
+  gtk_widget_set_events (eventbox, GDK_BUTTON_RELEASE_MASK);
+
   pixmap = create_pixmap (global.win, "close.xpm");
   gtk_widget_show (pixmap);
-  gtk_container_add (GTK_CONTAINER (button), pixmap);
+  gtk_container_add (GTK_CONTAINER (eventbox), pixmap);
 
+  gtk_signal_connect (GTK_OBJECT (eventbox), "button_release_event",
+		      GTK_SIGNAL_FUNC(on_remove_browse_activate),
+                      browse);
+  /*
   gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		     GTK_SIGNAL_FUNC(on_remove_browse_activate),
 		     browse);
-
+  */
   notebook = GTK_NOTEBOOK(lookup_widget(global.win, "notebook6"));
   gtk_notebook_append_page(notebook, hpaned, tab_label);
-  /*
-  gtk_notebook_set_page(notebook,
-			gtk_notebook_page_num(GTK_NOTEBOOK(notebook),
-					      hpaned));
-  */
+  if (global.options.switch_browse) {
+    gtk_notebook_set_page
+      (notebook,
+       gtk_notebook_page_num(GTK_NOTEBOOK(notebook), hpaned));
+  }
+
   gtk_widget_realize(hpaned);
   return ctree;
 }
@@ -1782,11 +1931,11 @@ void browse_remove(browse_t *browse, int del) {
   } else if (del) {
     gtk_clist_remove(clist, row);
 
-    fname = l_strdup_printf("%s%cbrowse%c%s",
+    fname = g_strdup_printf("%s%cbrowse%c%s",
 			    global.options.config_dir,
 			    DIR_SEP, DIR_SEP, tree->name);
     unlink(fname);
-    l_free(fname);
+    g_free(fname);
     
     global.browses = g_list_remove(global.browses, browse);
     buffer_destroy(browse->buffer);
@@ -1834,7 +1983,7 @@ void browse_finish(browse_t* browse, unsigned long ip) {
 
   file_tree_calc_matrix(tree);
   browse_update_user(tree);
-  file_tree_show(tree, MIME_SIZE, 1);
+  file_tree_show(tree, MEDIA_SIZE, 1);
   browse_save(browse);
 }
 
@@ -1873,7 +2022,7 @@ static browse_t* browse_preload_saved(char *name) {
   }
   browse = browse_new(user, des);
   FILE_TREE(browse)->last_refreshed = strtoul(lbrowsed, NULL, 10);
-  FILE_TREE(browse)->fci[MIME_SIZE][3] = atoi(files);
+  FILE_TREE(browse)->fci[MEDIA_SIZE][3] = atoi(files);
   browse_add_saved(browse);
 
   fclose(file);
@@ -1940,15 +2089,15 @@ static void browse_load_folder_0(browse_t* browse, FILE* fd,
 	// create the file
 	file = file_new();
 	file_set_winname(file, wname);
-	file->user = l_strdup(tree->name);
-	file->md5 = l_strdup(md5);
+	file->user = g_strdup(tree->name);
+	file->md5 = g_strdup(md5);
 	file->size = strtol(size, NULL, 0);
 	file->bitrate = atoi(brate);
 	file->frequency = atoi(freq);
 	file->duration = atoi(duration);
 
 	child = BROWSE_NODE
-	  (file_node_insert_file(tree, FILE_NODE(parent), file));
+	  (file_node_insert_file(tree, FILE_NODE(parent), file, FALSE));
 	child->first_seen = strtoul(tstamp, NULL, 10);
 	child->last_seen = strtoul(tstamp2, NULL, 10);
       }
@@ -1996,6 +2145,7 @@ static void browse_load_folder_1(browse_t* browse, FILE* fd,
       tstamp = arg(NULL, 0);
       tstamp2 = arg(NULL, 0);
       if (tstamp2) {
+
 	// create the file
 	file = file_new();
 	pos = strlen(filename);
@@ -2004,19 +2154,19 @@ static void browse_load_folder_1(browse_t* browse, FILE* fd,
 	strcpy(file->longname+pos, filename);
 	file->shortname = get_short_name(file->longname);
 	file->filename = get_file_name(file->shortname);
-	file->winname = l_strdup(file->longname);
+	file->winname = g_strdup(file->longname);
 	local_to_napster(file->winname);
-	file->media_type = get_media_type(file->filename);
+	file->media_type = mtype_get(file->filename);
 
-	file->user = l_strdup(tree->name);
-	file->md5 = l_strdup(md5);
+	file->user = g_strdup(tree->name);
+	file->md5 = g_strdup(md5);
 	file->size = strtol(size, NULL, 0);
 	file->bitrate = atoi(brate);
 	file->frequency = atoi(freq);
 	file->duration = atoi(duration);
 
 	child = BROWSE_NODE
-	  (file_node_insert_file(tree, FILE_NODE(parent), file));
+	  (file_node_insert_file(tree, FILE_NODE(parent), file, FALSE));
 	child->first_seen = strtoul(tstamp, NULL, 10);
 	child->last_seen = strtoul(tstamp2, NULL, 10);
       }
@@ -2059,8 +2209,6 @@ static void browse_load_saved(browse_t* browse) {
   else
     browse_version = 0;
 
-  printf("browse version [%s] %d\n", tree->name, browse_version);
-  
   if (browse_version == 0)
     browse_load_folder_0(browse, file, NULL);
   else if (browse_version == 1)
@@ -2074,14 +2222,14 @@ static void browse_load_saved(browse_t* browse) {
 }
 
 static void 
-on_show_browse_files_activate(GtkMenuItem * menuitem ATTR_UNUSED, 
+on_show_browse_files_activate(GtkMenuItem* menuitem ATTR_UNUSED, 
 			      gpointer user_data) {
   browse_t* browse = user_data;
 
   if (!FILE_TREE(browse)->files) {
     browse_load_saved(browse);
     browse_create_page(browse);
-    file_tree_show(FILE_TREE(browse), MIME_SIZE, 3);
+    file_tree_show(FILE_TREE(browse), MEDIA_SIZE, 3);
   }
 }
 
@@ -2094,8 +2242,8 @@ static int browse_description_func(clist_rename_t* cr) {
   if (row < 0) return 0;
   
   tree = cr->data;
-  if (tree->description) l_free(tree->description);
-  tree->description = l_strdup(cr->new_text);
+  if (tree->description) g_free(tree->description);
+  tree->description = g_strdup(cr->new_text);
   gtk_clist_set_text(cr->clist, row, cr->col, tree->description);  
   
   if (!tree->files) {
