@@ -1,5 +1,4 @@
-
-/* Copyright (C) 2000/2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -287,11 +286,6 @@ void on_button98_clicked(GtkButton * button, gpointer user_data);
 
 void on_button99_clicked(GtkButton * button, gpointer user_data);
 
-void on_play_file2_activate(GtkMenuItem * menuitem, gpointer user_data);
-void on_play_file3_activate(GtkMenuItem * menuitem, gpointer user_data);
-
-void on_play_file_activate(GtkMenuItem * menuitem, gpointer user_data);
-
 void
 on_instant_message_activate(GtkMenuItem * menuitem, gpointer user_data);
 
@@ -507,28 +501,12 @@ on_browse_list_button_event(GtkWidget * widget,
 
 void on_browse_files_activate(GtkMenuItem * menuitem, gpointer user_data);
 
-void on_button182_clicked(GtkButton * button, gpointer user_data);
-
-void on_button180_clicked(GtkButton * button, gpointer user_data);
-
-void
-on_clist17_select_row(GtkCList * clist,
-		      gint row,
-		      gint column, GdkEvent * event, gpointer user_data);
-
-void on_button183_clicked(GtkButton * button, gpointer user_data);
-
 void on_combo_entry17_changed(GtkEditable * editable, gpointer user_data);
 
 void on_button184_clicked(GtkButton * button, gpointer user_data);
 
 void on_changes_activate(GtkMenuItem * menuitem, gpointer user_data);
 
-void
-on_clist18_select_row(GtkCList * clist,
-		      gint row,
-		      gint column, GdkEvent * event, gpointer user_data);
-
 void on_mode_error_activate(GtkMenuItem * menuitem, gpointer user_data);
 
 void on_mode_ban_activate(GtkMenuItem * menuitem, gpointer user_data);
@@ -776,10 +754,6 @@ on_checkbutton53_toggled(GtkToggleButton * togglebutton,
 
 void on_combo_entry13_changed(GtkEditable * editable, gpointer user_data);
 
-void on_entry80_changed(GtkEditable * editable, gpointer user_data);
-
-void on_combo_entry20_changed(GtkEditable * editable, gpointer user_data);
-
 gboolean
 on_drawingarea10_expose_event(GtkWidget * widget,
 			      GdkEventExpose * event, gpointer user_data);
@@ -836,10 +810,6 @@ void
 on_download_man_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_upload_man_activate                 (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 void
 on_button318_clicked                   (GtkButton       *button,
                                         gpointer         user_data);
@@ -1240,15 +1210,6 @@ on_ctree4_tree_select_row              (GtkCTree        *ctree,
                                         gint             column,
                                         gpointer         user_data);
 
-void
-on_scan_progress_win_destroy           (GtkObject       *object,
-                                        gpointer         user_data);
-
-gboolean
-on_scan_progress_win_delete_event      (GtkWidget       *widget,
-                                        GdkEvent        *event,
-                                        gpointer         user_data);
-
 void
 on_button355_clicked                   (GtkButton       *button,
                                         gpointer         user_data);
@@ -1390,3 +1351,48 @@ void
 on_ctree8_drag_end                     (GtkWidget       *widget,
                                         GdkDragContext  *drag_context,
                                         gpointer         user_data);
+
+void
+on_checkbutton90_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+gboolean
+on_drawingarea12_configure_event       (GtkWidget       *widget,
+                                        GdkEventConfigure *event,
+                                        gpointer         user_data);
+
+gboolean
+on_drawingarea1_configure_event        (GtkWidget       *widget,
+                                        GdkEventConfigure *event,
+                                        gpointer         user_data);
+
+gboolean
+on_drawingarea11_configure_event       (GtkWidget       *widget,
+                                        GdkEventConfigure *event,
+                                        gpointer         user_data);
+
+gboolean
+on_drawingarea2_configure_event        (GtkWidget       *widget,
+                                        GdkEventConfigure *event,
+                                        gpointer         user_data);
+void
+on_button364_clicked (GtkButton       *button,
+		      gpointer         user_data);
+
+void
+on_checkbutton92_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_checkbutton93_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_checkbutton94_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_button365_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void on_special_users_activate();
