@@ -1,4 +1,4 @@
-/* Copyright (C) 2000/2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -47,13 +47,13 @@ static int all_servers;
 
 #define COMMAND_ERROR(val, text) \
   if (val) {\
-    client_message("Error", text);\
+    client_message("Error", "%s", text);\
     return;\
   }
 
 #define CHECK_TYPE(typ, text) \
   if (!active_page || (active_page->type != typ)) {\
-    client_message("Error", text);\
+    client_message("Error", "%s", text);\
     return;\
   }
 
@@ -94,11 +94,11 @@ int command_new(int id, char *iname, char *syntax, char *ides,
     g_warning("chat command already exists [%s]", iname);
     return 0;
   }
-  new_comm = (command_t *) l_malloc(sizeof(command_t));
+  new_comm = (command_t *) g_malloc(sizeof(command_t));
   new_comm->id = id;
-  new_comm->name = l_strdup(iname);
-  new_comm->syntax = l_strdup(syntax);
-  new_comm->description = l_strdup(ides);
+  new_comm->name = g_strdup(iname);
+  new_comm->syntax = g_strdup(syntax);
+  new_comm->description = g_strdup(ides);
   new_comm->is_alias = 0;
   new_comm->level = ilevel;
   new_comm->folder = ifolder;
@@ -120,8 +120,8 @@ int command_rename(int iid, char *iname) {
     return 0;
   }
 
-  l_free(comm->name);
-  comm->name = l_strdup(iname);
+  g_free(comm->name);
+  comm->name = g_strdup(iname);
 
   return 1;
 }
@@ -155,7 +155,7 @@ command_t *command_is_command(char *iname) {
 
   for (dlist = global.commands; dlist; dlist = dlist->next) {
     comm = dlist->data;
-    if (!l_strcasecmp(name, comm->name))
+    if (!g_strcasecmp(name, comm->name))
       return comm;
   }
 
@@ -205,14 +205,14 @@ int command_make_alias(char *iname, char *alias)
     return 0;
   }
 
-  new_alias = (command_t *) l_malloc(sizeof(command_t));
+  new_alias = (command_t *) g_malloc(sizeof(command_t));
   new_alias->id = comm->id;
-  new_alias->name = l_strdup(alias);
-  new_alias->syntax = l_strdup_printf("see %s", iname);
+  new_alias->name = g_strdup(alias);
+  new_alias->syntax = g_strdup_printf("see %s", iname);
   strcpy(temp, "This is an alias for <");
   strcat(temp, iname);
   strcat(temp, ">");
-  new_alias->description = l_strdup(temp);
+  new_alias->description = g_strdup(temp);
   new_alias->is_alias = 1;
   new_alias->level = comm->level;
   new_alias->folder = comm->folder;
@@ -223,7 +223,7 @@ int command_make_alias(char *iname, char *alias)
   strcat(temp, alias);
   strcat(temp, " := ");
   strcat(temp, iname);
-  client_message("Message", temp);
+  client_message("Message", "%s", temp);
 
   return 1;
 }
@@ -252,10 +252,10 @@ void command_delete_all()
 
   for (dlist = global.commands; dlist; dlist = dlist->next) {
     comm = (command_t *) (dlist->data);
-    l_free(comm->name);
-    l_free(comm->syntax);
-    l_free(comm->description);
-    l_free(comm);
+    g_free(comm->name);
+    g_free(comm->syntax);
+    g_free(comm->description);
+    g_free(comm);
   }
   g_list_free(global.commands);
   global.commands = NULL;
@@ -369,7 +369,7 @@ COMMAND(lopster_emote) {
   if (!active_page) return;
 
   if (active_page->type == P_PUBLIC)
-    send_public_emote(active_net, active_page->name, message);
+    send_public_emote(active_page, message);
   if (active_page->type == P_PRIVATE)
     send_private_emote(active_net, active_page->name, message);
 }
@@ -451,12 +451,12 @@ COMMAND(lopster_autojoin) {
       COMMAND_ERROR(!text, "Not in autojoin list");
       active_net->auto_join =
 	g_list_remove(active_net->auto_join, text);
-      l_free(text);
+      g_free(text);
     } else {
       char* text = is_string_in_list(active_net->auto_join, channel);
       COMMAND_ERROR(text, "Already in autojoin list");
       active_net->auto_join =
-	g_list_append(active_net->auto_join, l_strdup(channel));
+	g_list_append(active_net->auto_join, g_strdup(channel));
     }
     server_save();
   }
@@ -486,9 +486,9 @@ help_print_folder(chat_page_t* page, folder_t folder, int all) {
   
   chat_print_time_stamp(page, M_PUBLIC);
   chat_print_prefix(page, 1);
-  //  temp_str = l_strdup(FolderNames(folder));
+  //  temp_str = g_strdup(FolderNames(folder));
   chat_print_text(page, M_PUBLIC, "error", FolderNames(folder));
-  //  l_free(temp_str);
+  //  g_free(temp_str);
   cnt = 0;
   for (dlist = global.commands; dlist; dlist = dlist->next) {
     command = dlist->data;
@@ -561,8 +561,8 @@ COMMAND(lopster_help) {
       chat_print_text(page, M_PUBLIC, "message", " /");
       chat_print_text(page, M_PUBLIC, "message", command->name);
       chat_print_text(page, M_PUBLIC, "text", "\n");
-      client_message("Syntax", command->syntax);
-      client_message("Description", command->description);
+      client_message("Syntax", "%s", command->syntax);
+      client_message("Description", "%s", command->description);
       if ((int) command->level > L_ADMIN)
 	if (level > L_ADMIN)
 	  client_message("Access", "Elite (and you are!)");
@@ -704,9 +704,9 @@ COMMAND(lopster_clear) {
 
   where = arg(data, 0);
   if (where) {
-    if (!l_strcasecmp(where, "TOP")) {
+    if (!g_strcasecmp(where, "TOP")) {
       chat_page_clear(active_page, 0);
-    } else if (!l_strcasecmp(where, "BOTTOM")) {
+    } else if (!g_strcasecmp(where, "BOTTOM")) {
       chat_page_clear(active_page, 1);
     } else {
       client_message("Error", "Clear what?");
@@ -811,9 +811,8 @@ COMMAND(lopster_level) {
 
   level = arg(NULL, 0);
   if (!level) {
-#warning FIXME when other networks types are implemented
     COMMAND_ERROR(level2int(nick) >= 0,
-		  "If you are sure that you want change your own user level, then use: /raw 606 <yournick> <level>");
+		  "If you are sure that you like to change your own user level, then use: /raw 606 <yournick> <level>");
     COMMAND_ERROR(TRUE, "Please sepcify a user level!");
   } else {
     command_send(active_net, CMD_SET_USERLEVEL, nick, level);
@@ -876,10 +875,10 @@ COMMAND(lopster_password) {
     if (strcmp(active_net->user.password, nick)) {
       command_send(active_net, CMD_CHANGE_PASS, nick);
       command =
-	l_strdup_printf("Your password was changed to [%s]", nick);
+	g_strdup_printf("Your password was changed to [%s]", nick);
       strcpy(active_net->user.password, nick);
-      client_message("Message", command);
-      l_free(command);
+      client_message("Message", "%s", command);
+      g_free(command);
       server_save();
     } else {
       client_message("Message", "Your password was not changed");
@@ -1012,9 +1011,9 @@ COMMAND(lopster_scheme) {
   scheme = arg(data, 1);
   if (!scheme) {
     command =
-      l_strdup_printf("Current scheme is [%s]", global.scheme->name);
-    client_message("Message", command);
-    l_free(command);
+      g_strdup_printf("Current scheme is [%s]", global.scheme->name);
+    client_message("Message", "%s", command);
+    g_free(command);
   } else {
     scheme1 = scheme_load(scheme);
     if (scheme1) {
@@ -1251,17 +1250,17 @@ COMMAND(lopster_cversion) {
   
   if (uname(&uts) < 0) {
     if (INTERN_MICRO_VERSION)
-      t2 = l_strdup_printf(VERSION_STRING".%d [Uptime %s]",
+      t2 = g_strdup_printf(VERSION_STRING".%d [Uptime %s]",
 			   INTERN_MICRO_VERSION, str);
     else
-      t2 = l_strdup_printf(VERSION_STRING" [Uptime %s]", str);
+      t2 = g_strdup_printf(VERSION_STRING" [Uptime %s]", str);
   } else {
     if (INTERN_MICRO_VERSION)
-      t2 = l_strdup_printf(VERSION_STRING".%d (%s %s) [Uptime: %s]",
+      t2 = g_strdup_printf(VERSION_STRING".%d (%s %s) [Uptime: %s]",
 			   INTERN_MICRO_VERSION,
 			   uts.sysname, uts.release, str);
     else
-      t2 = l_strdup_printf(VERSION_STRING" (%s %s) [Uptime: %s]",
+      t2 = g_strdup_printf(VERSION_STRING" (%s %s) [Uptime: %s]",
 			   uts.sysname, uts.release, str);
   }
   
@@ -1281,9 +1280,9 @@ COMMAND(lopster_cversion) {
       COMMAND_ERROR(1, "You have to switch to a valid page");
     }
   } else {
-    client_message("Message", t2);
+    client_message("Message", "%s", t2);
   }
-  l_free(t2);
+  g_free(t2);
 }
 
 COMMAND(lopster_echo) {
@@ -1416,91 +1415,6 @@ COMMAND(lopster_sconfig) {
   }
 }
 
-COMMAND(lopster_clones) {
-  GList *dlist;
-  GList *dlist2;
-  char *pos;
-  int cnt, cnt2, cnt3;
-  clone_t *clone;
-  char *which;
-  int number;
-  char t[1024];
-
-  which = arg(data, 0);
-  COMMAND_ERROR(!which, "Print what clones?");
-  pos = arg(NULL, 0);
-  if (pos) number = atoi(pos);
-  else number = 2;
-
-  if (!strcmp(which, "all")) {
-    cnt = cnt2 = 0;
-    client_message("Clones", 
-		   "Printing IPs with at least %d nicks", number);
-    for (dlist = global.clones; dlist; dlist = dlist->next) {
-      clone = dlist->data;
-      if (g_list_length(clone->nicks) < (unsigned)number)
-	continue;
-      sprintf(t, "IP with %d clones", g_list_length(clone->nicks));
-      client_message(t, clone->ip);
-      for (dlist2 = clone->nicks; dlist2; dlist2 = dlist2->next) {
-	pos = dlist2->data;
-	client_message("--->", "<%s>", pos);
-	cnt++;
-      }
-      cnt2++;
-    }
-    sprintf(t, "[%d clones found] (%d IPs)", cnt, cnt2);
-    client_message("--Sum-->", t);
-  } else if (!strcmp(which, ".")) {
-    CHECK_TYPE(P_PUBLIC, "Have to switch to a channel");
-    sprintf(t, "Printing IPs with at least %d nicks", number);
-    client_message("Clones", t);
-    cnt = cnt2 = cnt3 = 0;
-    for (dlist = global.clones; dlist; dlist = dlist->next) {
-      clone = dlist->data;
-      if (g_list_length(clone->nicks) < (unsigned)number)
-	continue;
-      for (dlist2 = clone->nicks; dlist2; dlist2 = dlist2->next) {
-	pos = dlist2->data;
-	if (user_search_full(active_page, pos)) break;
-      }
-      if (dlist2) {
-	sprintf(t, "IP with %d clones", g_list_length(clone->nicks));
-	client_message(t, clone->ip);
-	cnt++;
-      } else
-	continue;
-      for (dlist2 = clone->nicks; dlist2; dlist2 = dlist2->next) {
-	pos = dlist2->data;
-	if (user_search_full(active_page, pos)) {
-	  client_message("--->", "(C) <%s>", pos);
-	  cnt3++;
-	} else {
-	  client_message("--->", "( ) <%s>", pos);
-	}
-	cnt2++;
-      }
-    }
-    sprintf(t, "[%d clones found in channel %s] (%d IPs / %d clones)",
-	    cnt3, active_page->name, cnt, cnt2);
-    client_message("--Sum-->", t);
-  } else {
-    sprintf(t, "Printing nicks for IP %s", which);
-    client_message("Clones", t);
-    for (dlist = global.clones; dlist; dlist = dlist->next) {
-      clone = dlist->data;
-      if (strcmp(clone->ip, which)) continue;
-      sprintf(t, "IP with %d clones", g_list_length(clone->nicks));
-      client_message(t, clone->ip);
-      for (dlist2 = clone->nicks; dlist2; dlist2 = dlist2->next) {
-	pos = dlist2->data;
-	client_message("--->", "<%s>", pos);
-      }
-      break;
-    }
-  }
-}
-
 COMMAND(lopster_op) {
   char *nick;
   char *channel;
@@ -1633,10 +1547,10 @@ COMMAND(lopster_links) {
     } else {
       network_links_print(active_net);
     }
-  } else if (!l_strcasecmp(action, "GET")) {
+  } else if (!g_strcasecmp(action, "GET")) {
     client_message("Message", "Evaluating linked server list");
     network_links_get(active_net);
-  } else if (!l_strcasecmp(action, "PRINT")) {
+  } else if (!g_strcasecmp(action, "PRINT")) {
     network_links_print(active_net);
   } else {
     client_message("Error", "Unknown action for /links");
@@ -1837,14 +1751,14 @@ COMMAND(lopster_killghost) {
   COMMAND_ERROR(!ghost, "You have to specify the ghost!");
   passwd = arg(NULL, 0);
   if (!passwd) {
-    command = l_strdup_printf("ghost %s %s", ghost,
+    command = g_strdup_printf("ghost %s %s", ghost,
 			      active_net->user.password);
   } else {
-    command = l_strdup_printf("ghost %s %s", ghost, passwd);
+    command = g_strdup_printf("ghost %s %s", ghost, passwd);
   }
   
   send_notice(active_net, "NickServ", command, 1);
-  l_free(command);
+  g_free(command);
 }
 
 COMMAND(lopster_locate) {
@@ -1888,9 +1802,9 @@ COMMAND(lopster_rcolors) {
   codes = arg(data, 0);
 
   if (!codes) {
-  } else if (!l_strcasecmp(codes, "-reset")) {
-    l_free(global.rainbow_colors);
-    global.rainbow_colors = l_strdup("RrpPBbcCgGYy");
+  } else if (!g_strcasecmp(codes, "-reset")) {
+    g_free(global.rainbow_colors);
+    global.rainbow_colors = g_strdup("RrpPBbcCgGYy");
   } else {
     for (i1 = 0; data[i1] && (i1 < 1023); i1++) {
       if (strchr("krgybmpcwKRGYBMPCWn", data[i1]))
@@ -1898,11 +1812,11 @@ COMMAND(lopster_rcolors) {
     }
     str[pos] = 0;
     if (pos == 0)
-      global.rainbow_colors = l_strdup("RrpPBbcCgGYy");
+      global.rainbow_colors = g_strdup("RrpPBbcCgGYy");
     else
-    global.rainbow_colors = l_strdup(str);
+    global.rainbow_colors = g_strdup(str);
   }
-  client_message("Rainbow colors", global.rainbow_colors);
+  client_message("Rainbow colors", "%s", global.rainbow_colors);
 }
 
 COMMAND(lopster_rainbow) {
@@ -2016,9 +1930,9 @@ COMMAND(lopster_slink) {
   COMMAND_ERROR(!server, "Link a server or what?");
 
   if (remote) {
-    command = l_strdup_printf("%s %s", server, remote);
+    command = g_strdup_printf("%s %s", server, remote);
     command_send(active_net, CMD_SERVER_CONNECT, command);
-    l_free(command);
+    g_free(command);
   } else {
     command_send(active_net, CMD_SERVER_CONNECT, data);
   }
@@ -2178,14 +2092,14 @@ COMMAND(lopster_limit) {
   tag = arg(data, 0);
   ip = arg(NULL, 1);
 
-  if (!tag || !l_strcasecmp(tag, "list")) {
+  if (!tag || !g_strcasecmp(tag, "list")) {
     client_message("Message", "Requesting limits on <%s>",
 		   active_net->name);
     command_send(active_net, CMD_CLASS_LIST);
-  } else if (!l_strcasecmp(tag, "add")) {
+  } else if (!g_strcasecmp(tag, "add")) {
     COMMAND_ERROR(!ip, "No ip specified");
     command_send(active_net, CMD_CLASS_ADD, ip);
-  } else if (!l_strcasecmp(tag, "remove")) {
+  } else if (!g_strcasecmp(tag, "remove")) {
     COMMAND_ERROR(!ip, "No ip specified");
     command_send(active_net, CMD_CLASS_REMOVE, ip);
   } else {
@@ -2201,14 +2115,14 @@ COMMAND(lopster_iline) {
 
   tag = arg(data, 0);
   ip = arg(NULL, 1);
-  if (!tag || !l_strcasecmp(tag, "list")) {
+  if (!tag || !g_strcasecmp(tag, "list")) {
     client_message("Message", "Requesting ilines on <%s>",
 		   active_net->name);
     command_send(active_net, CMD_ILINE_LIST);
-  } else if (!l_strcasecmp(tag, "add")) {
+  } else if (!g_strcasecmp(tag, "add")) {
     COMMAND_ERROR(!ip, "No ip specified");
     command_send(active_net, CMD_ILINE_ADD, ip);
-  } else if (!l_strcasecmp(tag, "remove")) {
+  } else if (!g_strcasecmp(tag, "remove")) {
     COMMAND_ERROR(!ip, "No ip specified");
     command_send(active_net, CMD_ILINE_REMOVE, ip);
   } else {
@@ -2224,14 +2138,14 @@ COMMAND(lopster_dline) {
 
   tag = arg(data, 0);
   ip = arg(NULL, 1);
-  if (!tag || !l_strcasecmp(tag, "list")) {
+  if (!tag || !g_strcasecmp(tag, "list")) {
     client_message("Message", "Requesting ilines on <%s>",
 		   active_net->name);
     command_send(active_net, CMD_DLINE_LIST);
-  } else if (!l_strcasecmp(tag, "add")) {
+  } else if (!g_strcasecmp(tag, "add")) {
     COMMAND_ERROR(!ip, "No ip specified");
     command_send(active_net, CMD_DLINE_ADD, ip);
-  } else if (!l_strcasecmp(tag, "remove")) {
+  } else if (!g_strcasecmp(tag, "remove")) {
     COMMAND_ERROR(!ip, "No ip specified");
     command_send(active_net, CMD_DLINE_REMOVE, ip);
   } else {
@@ -2247,14 +2161,14 @@ COMMAND(lopster_eline) {
 
   tag = arg(data, 0);
   ip = arg(NULL, 1);
-  if (!tag || !l_strcasecmp(tag, "list")) {
+  if (!tag || !g_strcasecmp(tag, "list")) {
     client_message("Message", "Requesting ilines on <%s>",
 		   active_net->name);
     command_send(active_net, CMD_ELINE_LIST);
-  } else if (!l_strcasecmp(tag, "add")) {
+  } else if (!g_strcasecmp(tag, "add")) {
     COMMAND_ERROR(!ip, "No ip specified");
     command_send(active_net, CMD_ELINE_ADD, ip);
-  } else if (!l_strcasecmp(tag, "remove")) {
+  } else if (!g_strcasecmp(tag, "remove")) {
     COMMAND_ERROR(!ip, "No ip specified");
     command_send(active_net, CMD_ELINE_REMOVE, ip);
   } else {
