@@ -1,4 +1,4 @@
-/* Copyright (C) 2000/2002 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -41,102 +41,260 @@
 #  include <pthread.h>
 #endif
 
-typedef struct {
-  call_func_t func;
-  void* data;
-  char* address;
-} thread_data_t;
+//#define RESOLVE_DEBUG
 
-#ifdef THREADS_ENABLED
-static pthread_mutex_t resolver = PTHREAD_MUTEX_INITIALIZER;
-#endif
 static int destroying = 0;
 
 int server_login(gpointer data, gint source, GdkInputCondition condition);
 int connection_timeout(gpointer data);
 void socket_remove_clist(socket_t * socket);
 
+typedef struct {
+  call_func_t func;
+  void* data;
+  char* address;
+  unsigned int ip_long;
+} resolve_data_t;
+
+typedef struct {
+  time_t last_resolved;
+  unsigned int ip;
+} resolve_hist_t;
+
+static GList* ResolveDone = NULL;
+static GHashTable* ResolveHT = NULL;
+
 #ifdef THREADS_ENABLED
-static void * gethostbyname_thread (void *args) {
-  thread_data_t* tdata = args;
-  struct hostent* h;
-  unsigned long addr;
-
-  pthread_mutex_lock(&resolver);
-  h = gethostbyname(tdata->address);
-  pthread_mutex_unlock(&resolver);
-
-  if (h) addr = ((struct in_addr *) (h->h_addr_list[0]))->s_addr;
-  else addr = INADDR_NONE;
-
-#ifdef CONNECTION_DEBUG
-  printf("[RESOLVE] thread done [%s]->[%s]\n", 
-	 tdata->address, ntoa(addr));
+static GList* PendingTasks = NULL;
+static pthread_mutex_t resolver = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t resolver2 = PTHREAD_MUTEX_INITIALIZER;
 #endif
-  tdata->func(tdata->data, addr);
 
-  l_free(tdata->address);
-  l_free(tdata);
+void resolve_check() {
+  resolve_data_t* data;
+  GList* dlist;
 
+  if (!ResolveDone) return;
+
+#ifdef THREADS_ENABLED
+  pthread_mutex_lock(&resolver2);
+#endif
+
+  for (dlist = ResolveDone; dlist; dlist = dlist->next) {
+    data = dlist->data;
+    data->func(data->data, data->ip_long);
+    g_free(data->address);
+    g_free(data);
+  }
+  g_list_free(ResolveDone);
+  ResolveDone = NULL;
+
+#ifdef THREADS_ENABLED
+  pthread_mutex_unlock(&resolver2);
+#endif
+}
+
+
+static void resolve_success(resolve_data_t* data, int real) {
+  resolve_hist_t* hist;
+
+#ifdef THREADS_ENABLED
+  pthread_mutex_lock(&resolver2);
+#endif
+
+#ifdef RESOLVE_DEBUG
+  printf("[RESOLVE] done [%s]->[%s]\n", 
+	 data->address, 
+	 (data->ip_long==INADDR_NONE)?"ERROR":ntoa(data->ip_long));
+#endif
+  // save data in list for processing outside the threads
+  ResolveDone = g_list_append(ResolveDone, data);
+  // store result in hash table only if we did a real resolve
+  if (real) {
+    if (ResolveHT) {
+      hist = g_hash_table_lookup (ResolveHT, data->address);
+    } else {
+      ResolveHT = g_hash_table_new(g_str_hash, g_str_equal);
+      hist = NULL;
+    }
+    if (!hist) {
+      hist = g_malloc(sizeof(resolve_hist_t));
+      g_hash_table_insert (ResolveHT, g_strdup(data->address), hist);
+    }
+    hist->last_resolved = global.current_time;
+    hist->ip = data->ip_long;
+  }
+
+#ifdef THREADS_ENABLED
+  pthread_mutex_unlock(&resolver2);
+#endif
+
+}
+
+#define RESOLVE_INT  600   // 10 minutes
+
+static gboolean resolve_lookup(resolve_data_t* data) {
+  resolve_hist_t* hist;
+  gboolean result;
+
+#ifdef THREADS_ENABLED
+  pthread_mutex_lock(&resolver2);
+#endif
+  // store result in hash table only if we did a real resolve
+  if (ResolveHT) {
+    hist = g_hash_table_lookup (ResolveHT, data->address);
+  } else {
+    hist = NULL;
+  }
+  if (!hist) {
+    // not entry found
+    result = FALSE;
+  } else if (hist->last_resolved + RESOLVE_INT < global.current_time) {
+    // out of date entry found
+#ifdef RESOLVE_DEBUG
+    printf("[RESOLVE] host '%s' cached but outdated\n", 
+	   data->address);
+#endif
+    result =  FALSE;
+  } else {
+    data->ip_long = hist->ip;
+#ifdef RESOLVE_DEBUG
+    printf("[RESOLVE] host '%s' recently cached\n", 
+	   data->address);
+#endif
+    result = TRUE;
+  }
+
+#ifdef THREADS_ENABLED
+  pthread_mutex_unlock(&resolver2);
+#endif
+  return result;
+}
+
+#ifdef THREADS_ENABLED
+static void* gethostbyname_thread(void *args) {
+  gboolean done = FALSE;
+
+  // we resolve as long as we have pending tasks
+  (void)args;
+  do {
+    resolve_data_t* tdata = NULL;
+
+    // get the next task
+    pthread_mutex_lock(&resolver);
+    if (!PendingTasks) {
+      tdata = NULL;
+    } else {
+      tdata = PendingTasks->data;
+      PendingTasks = g_list_remove(PendingTasks, tdata);
+    }
+    if (!PendingTasks) done = TRUE;
+    pthread_mutex_unlock(&resolver);
+
+    // now resolve the address
+    if (tdata) {
+      struct hostent* h;
+      unsigned long addr;
+      h = gethostbyname(tdata->address);
+      if (h) addr = ((struct in_addr *) (h->h_addr_list[0]))->s_addr;
+      else addr = INADDR_NONE;
+      tdata->ip_long = addr;
+      resolve_success(tdata, 1);
+    }
+  } while (!done);
+
+#ifdef RESOLVE_DEBUG
+  printf("[RESOLVE] about to shut thread down\n");
+#endif
   return NULL;
 }
  
-void my_gethostbyname(char *address, call_func_t func, void* data) {
-  pthread_t the_thread;
-  pthread_attr_t the_thread_attr;
-  thread_data_t* tdata;
+void my_gethostbyname(resolve_data_t* tdata) {
 
-  tdata = l_malloc(sizeof(*tdata));
+  pthread_mutex_lock(&resolver);
+  
+  // if list is empty, create a new thread.
+  if (!PendingTasks) {
+    pthread_t the_thread;
+    pthread_attr_t the_thread_attr;
+
+    pthread_attr_init(&the_thread_attr);
+    // make thread to free all data on exit
+    pthread_attr_setdetachstate(&the_thread_attr,
+				PTHREAD_CREATE_DETACHED);
+    
+    if (pthread_create(&the_thread, &the_thread_attr, 
+		       gethostbyname_thread, NULL)) {
+      printf("[RESOLVE] could not create thread\n");
+      tdata->ip_long = INADDR_NONE;
+      resolve_success(tdata, 0);
+    } else {
+#ifdef RESOLVE_DEBUG
+      printf("[RESOLVE] created new thread\n");
+#endif
+      PendingTasks = g_list_append(PendingTasks, tdata);
+    }
+  } else {
+    PendingTasks = g_list_append(PendingTasks, tdata);
+  }
+
+  pthread_mutex_unlock(&resolver);
+}
+#endif
+
+void resolve_address(char *address, call_func_t func, void* data) {
+  unsigned long addr;
+  resolve_data_t* tdata;
+
+  if (!address || *address == 0) return;
+
+#ifdef RESOLVE_DEBUG
+  printf("[RESOLVE] address '%s'\n", address);
+#endif
+
+  tdata = g_malloc(sizeof(*tdata));
   tdata->func = func;
   tdata->data = data;
-  tdata->address = l_strdup(address);
-
-  pthread_attr_init (&the_thread_attr);
-  // make thread to free all data on exit
-  pthread_attr_setdetachstate(&the_thread_attr,
-			      PTHREAD_CREATE_DETACHED);
-
-  pthread_create (&the_thread, &the_thread_attr, 
-		  gethostbyname_thread, tdata);
-}
-#endif
-
-unsigned long resolv_address(char *address, call_func_t func,
-			     void* data) {
-  unsigned long addr;
-#ifndef THREADS_ENABLED
-  struct hostent* h;
-#endif
+  tdata->address = g_strdup(address);
+  tdata->ip_long = INADDR_NONE;
+  
+  // return if we have found a recent IP in the hash table
+  if (resolve_lookup(tdata)) {
+    resolve_success(tdata, 0);
+    return;
+  }
 
   addr = inet_addr(address);
-  if (addr != INADDR_NONE) return addr;
+  if (addr != INADDR_NONE) {
+    tdata->ip_long = addr;
+    resolve_success(tdata, 0);
+    return;
+  }
+  
+#ifdef THREADS_ENABLED /****************************/
 
-#ifdef THREADS_ENABLED
-#  ifdef CONNECTION_DEBUG
-  printf("[RESOLVE] thread [%s]\n", address);
-#  endif
-  my_gethostbyname(address, func, data);
-#else
-#  ifdef CONNECTION_DEBUG
-  printf("[RESOLVE] normal [%s]\n", address);
-#  endif
-  h = gethostbyname(address);
-  if (h) addr = ((struct in_addr *) (h->h_addr_list[0]))->s_addr;
-  else addr = INADDR_NONE;
-#  ifdef CONNECTION_DEBUG
-  printf("[RESOLVE] normal done [%s]->[%s]\n", 
-	 address, ntoa(addr));
-#  endif
-  if (addr != INADDR_NONE) return addr;
-  else func(data, addr);
-#endif
-  return INADDR_NONE;
+  my_gethostbyname(tdata);
+
+#else /*********************************************/
+  {
+    struct hostent* h = gethostbyname(address);
+    if (h) addr = ((struct in_addr *) (h->h_addr_list[0]))->s_addr;
+    else addr = INADDR_NONE;
+    tdata->ip_long = addr;
+    resolve_success(tdata, 1);
+  }
+#endif /********************************************/
+  
+  return;
 }
 
+
+
+
 socket_t *socket_new(int type) {
   socket_t *sock;
 
-  sock = l_malloc(sizeof(socket_t));
+  sock = g_malloc(sizeof(socket_t));
 
   sock->ip_long = 0;
   sock->port = 0;
@@ -184,8 +342,7 @@ socket_t *socket_new(int type) {
 }
 
 static void socket_end_real(socket_t * socket, int data) {
-  upload_t *upload;
-  share_t *share;
+  transfer_t *trans;
   download_t *download;
   browse_t *browse;
   int tim;
@@ -224,18 +381,18 @@ static void socket_end_real(socket_t * socket, int data) {
     download_end(socket, data);
     break;
   case S_SHARE:
-    share = socket->data;
-    if (!share) break;
+    trans = TRANS(socket->data);
+    if (!trans) break;
     
-    if (share->data->status == data) break;
+    if (trans->status == data) break;
     
     share_end(socket, data);
     break;
   case S_UPLOAD:
-    upload = socket->data;
-    if (!upload) break;
+    trans = TRANS(socket->data);
+    if (!trans) break;
     
-    if (upload->data->status == data) break;
+    if (trans->status == data) break;
     
     upload_end(socket, data);
     break;
@@ -344,7 +501,7 @@ void socket_destroy(socket_t * socket, int data) {
   }
 
   destroying = 0;
-  l_free(socket);
+  g_free(socket);
 }
 
 char *socket_get_name(socket_t * socket) {
@@ -413,21 +570,7 @@ int connect_socket(socket_t * s, char *proto, int type) {
 
 gint server_send(gpointer data, gint source,
 		 GdkInputCondition condition);
-
-/*
-static void command_pack(net_t* net, gint16 type, va_list ap) {
-  GList* dlist;
-
-  if (net) {
-    net->command_pack(net, type, ap);
-  } else {
-    for (dlist = global.net_active; dlist; dlist = dlist->next) {
-      net = dlist->data;
-      net->command_pack(net, type, ap);
-    }
-  }
-}
-*/
+gboolean net_has_outgoing(net_t* net);
 
 int command_send(net_t* net, gint16 type, ...) {
   va_list ap;
@@ -443,16 +586,19 @@ int command_send(net_t* net, gint16 type, ...) {
   }
 
   while (net) {
-    // we already might want to send commands when connecting 
-    // (ping from IRC _before_ welcome)
-    // so check NET_CONNECTED() instead of NET_ONLINE()
     if (!net->out_buffer) net->out_buffer = buffer_new(4096);
+    // we already might want to send commands when connecting (ping
+    // from IRC _before_ welcome) so check NET_ONLINE() instead of
+    // NET_CONNECTED() NOTE: i really dont know why i've declared
+    // NET_ONLINE to check the connection and NET_CONNECTED to check
+    // the online status. stupid
     if (NET_ONLINE(net)) {
+      
       va_start(ap, type);
       net->command_pack(net, type, ap);
       va_end(ap);
 
-      if (net->socket->output == -1 && net->out_buffer->datasize) {
+      if (net->socket->output == -1 && net_has_outgoing(net)) {
 	net->socket->output =
 	  gdk_input_add(net->socket->fd, GDK_INPUT_WRITE,
 			GTK_SIGNAL_FUNC(server_send), net->socket);
@@ -471,6 +617,67 @@ int command_send(net_t* net, gint16 type, ...) {
   return 0;
 }
 
+int command_send_unimportant(net_t* net, gint16 type, ...) {
+  va_list ap;
+  GList* dlist = NULL;
+
+  if (type < 0) {
+    g_warning("command out of bounds %d", type);
+    return 1;
+  }
+  if (!net && global.net_active) {
+    dlist = global.net_active;
+    net = dlist->data;
+  }
+
+  while (net) {
+    if (!net->temp_buffer) net->temp_buffer = buffer_new(4096);
+    if (NET_ONLINE(net)) {
+      int pos;
+      int size;
+      buffer_t* buffer = net->out_buffer;
+
+      // switch buffers;
+      net->out_buffer = net->temp_buffer;
+      // leave space for length
+      buffer_need_space(net->out_buffer, sizeof(size));
+      pos = net->out_buffer->datasize;
+      net->out_buffer->datasize += sizeof(size);
+
+      va_start(ap, type);
+      net->command_pack(net, type, ap);
+      va_end(ap);
+      
+      size = net->out_buffer->datasize - pos - 4;
+      if (size > 0) {
+	memcpy(net->out_buffer->data+pos, &size, sizeof(size));
+      } else {
+	printf("--WARNING size <= 0\n");
+	net->out_buffer->datasize -= sizeof(size);
+      }
+      
+      net->out_buffer = buffer;
+      
+      if (net->socket->output == -1 && net_has_outgoing(net)) {
+	net->socket->output =
+	  gdk_input_add(net->socket->fd, GDK_INPUT_WRITE,
+			GTK_SIGNAL_FUNC(server_send), net->socket);
+      }
+    } else {
+      g_warning("net is not online");
+    }
+    if (dlist) {
+      dlist = dlist->next;
+      if (dlist) net = dlist->data;
+      else net = NULL;
+    } else {
+      net = NULL;
+    }
+  }
+  return 0;
+}
+
+
 gint get_con_type(gpointer data, gint source, GdkInputCondition condition) {
 #ifndef HAVE_LIBLCONV_H
   char buffer[1025];
@@ -512,6 +719,10 @@ gint get_con_type(gpointer data, gint source, GdkInputCondition condition) {
 #endif
 #ifdef TRANSFER_DEBUG
   printf("in [%s] %d\n", buffer, cnt);
+  while (cnt >= 0) {
+    printf(" - [%1c] [%3d]\n", buffer[cnt], buffer[cnt]);
+    cnt--;
+  }
 #endif
   if (!strncmp(buffer, "GETLIST", 7)) {
     if (recv_safe(source, buffer, 7, 7) != 7) {
@@ -559,6 +770,20 @@ gint get_con_type(gpointer data, gint source, GdkInputCondition condition) {
     socket->input =
       gdk_input_add(socket->fd, GDK_INPUT_READ,
 		    GTK_SIGNAL_FUNC(download_fw_get_info), socket);
+#ifdef ALLOW_REMOTE_CONTROL
+  } else if (!strncmp(buffer, "LOGIN\r\n", 7)) {
+    if (recv_safe(source, buffer, 7, 7) != 7) {
+      socket_destroy(socket, 0);
+      return 1;
+    }
+#ifdef TRANSFER_DEBUG
+    printf("got LOGIN\n");
+#endif
+    socket->type = S_REMOTE;
+    socket->output =
+      gdk_input_add(socket->fd, GDK_INPUT_WRITE,
+		    GTK_SIGNAL_FUNC(remote_send_login), socket);
+#endif
   } else {
     l_log(NULL, "invalid_requests", LOG_OTHER, "[%s][%s]\n",
 	  ntoa(socket->ip_long), buffer);
@@ -670,8 +895,9 @@ void create_upload_port(gint16 port, int send) {
   client_message("Message", "Searching for port....");
   send = 1;
 
-  port = get_next_port(global.allowed_ports, port);
-  save = port;
+  save = get_next_port(global.allowed_ports, port);
+  if (save == port) goto failure; // only one port in list
+  port = save;
   while (1) {
     global.upload_socket->port = htons(port);
     server.sin_port = global.upload_socket->port;
@@ -849,7 +1075,8 @@ void socket_show_clist(socket_t * socket) {
 	  net->subtype == N_OPENNAP_NG) {
 	sprintf(tstr[1], "%d.%d", net->major, net->minor);
 	gtk_clist_append(clist, list);
-      } else if (net->subtype == N_SLAVANAP) {
+      } else if (net->subtype == N_SLAVANAP ||
+		 net->subtype == N_CLEANNAP) {
 	sprintf(tstr[1], "%d.%d.%d",
 		net->major, net->minor, net->micro);
 	gtk_clist_append(clist, list);
@@ -863,7 +1090,7 @@ void socket_show_clist(socket_t * socket) {
 	gtk_clist_append(clist, list);
 	break;
       }
-      userinfo = download->data->user_info;
+      userinfo = TRANS(download)->user_info;
       copy_string(tstr[0], "Type");
       copy_string(tstr[1], "Download");
       gtk_clist_append(clist, list);
@@ -874,7 +1101,7 @@ void socket_show_clist(socket_t * socket) {
       copy_string(tstr[1], download->file->longname);
       gtk_clist_append(clist, list);
       copy_string(tstr[0], "User");
-      copy_string(tstr[1], download->data->user_info->nick);
+      copy_string(tstr[1], userinfo->nick);
       gtk_clist_append(clist, list);
       copy_string(tstr[0], "Linespeed");
       copy_string(tstr[1], LineSpeed(userinfo->linespeed));
@@ -890,10 +1117,10 @@ void socket_show_clist(socket_t * socket) {
       gtk_clist_append(clist, list);
       }
       copy_string(tstr[0], "Status");
-      copy_string(tstr[1], status_names(download->data->status));
+      copy_string(tstr[1], status_names(TRANS(download)->status));
       gtk_clist_append(clist, list);
       copy_string(tstr[0], "Dcc");
-      copy_string(tstr[1], (download->data->is_dcc) ? "Yes" : "No");
+      copy_string(tstr[1], (TRANS(download)->is_dcc) ? "Yes" : "No");
       gtk_clist_append(clist, list);
 
       if (userinfo->max[0] == 0) strcpy(str1, "No limit");
@@ -919,7 +1146,7 @@ void socket_show_clist(socket_t * socket) {
 	gtk_clist_append(clist, list);
 	break;
       }
-      userinfo = upload->data->user_info;
+      userinfo = TRANS(upload)->user_info;
       copy_string(tstr[0], "Type");
       copy_string(tstr[1], "Upload");
       gtk_clist_append(clist, list);
@@ -945,10 +1172,10 @@ void socket_show_clist(socket_t * socket) {
 	gtk_clist_append(clist, list);
       }
       copy_string(tstr[0], "Status");
-      copy_string(tstr[1], status_names(upload->data->status));
+      copy_string(tstr[1], status_names(TRANS(upload)->status));
       gtk_clist_append(clist, list);
       copy_string(tstr[0], "Dcc");
-      copy_string(tstr[1], (upload->data->is_dcc) ? "Yes" : "No");
+      copy_string(tstr[1], (TRANS(upload)->is_dcc) ? "Yes" : "No");
       gtk_clist_append(clist, list);
       if (userinfo->max[1] == 0) strcpy(str1, "No limit");
       else if (userinfo->max[1] == -1) {
