@@ -1,4 +1,4 @@
-/* Copyright (C) 2000/2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -13,8 +13,8 @@
 
 typedef void (*call_func_t)( void* , unsigned long);
 
-unsigned long resolv_address(char *address, call_func_t func, 
-			     void* data);
+void resolve_address(char *address, call_func_t func, 
+		    void* data);
 
 void connect_fail(socket_t * socket, char *text);
 void connect_progress(char *text);
@@ -26,6 +26,7 @@ void socket_end(socket_t * socket, int data);
 char *socket_get_name(socket_t * socket);
 
 int command_send(net_t* net, gint16 type, ...);
+int command_send_unimportant(net_t* net, gint16 type, ...);
 
 void create_upload_port(gint16 port, int send);
 int connect_socket(socket_t * s, char *proto, int type);
