@@ -1,4 +1,4 @@
-/* Copyright (C) 2000/2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2004 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -35,30 +35,20 @@
 #include "preferences.h"
 #include "utils.h"
 
-int get_color(char chr)
-{
+int get_color(char chr) {
   switch (chr) {
-  case 'r':
-    return 4;
-  case 'b':
-    return 2;
-  case 'g':
-    return 3;
-  case 'c':
-    return 10;
-  case 'y':
-    return 8;
-  case 'Y':
-    return 5;
-  case 'k':
-    return 1;
-  default:
-    return -1;
+  case 'r': return 4;
+  case 'b': return 2;
+  case 'g': return 3;
+  case 'c': return 10;
+  case 'y': return 8;
+  case 'Y': return 5;
+  case 'k': return 1;
+  default:  return -1;
   }
 }
 
-void text_print_text(GtkText * widget, char *fmt, ...)
-{
+void text_print_text(GtkText * widget, char *fmt, ...) {
   char *pos;
   char text2[2048];
   char *text;
@@ -104,7 +94,7 @@ void port_dialog(char *message) {
   text_print_text(text,
 		  "$kAt the moment the data port was set to $b0$k, so you cannot download from users who set their port also to 0.\n");
   text_print_text(text,
-		  "$kIf you do not want this, choose another port at $bPreferences->General->Port for Uploading Files $k and deactivate the $bFirewall $kbutton.\n");
+		  "$kIf you do not want this, choose another port at $bPreferences->General->Dataport for Uploading Files $k and deactivate the $bFirewall $kbutton.\n");
 }
 
 void changes_dialog() {
@@ -112,54 +102,22 @@ void changes_dialog() {
   GtkText *text;
 
   win = create_information_win();
-  gtk_window_set_title(GTK_WINDOW(win), "Changes since Lopster 1.0.1");
+  gtk_window_set_title(GTK_WINDOW(win), "Changes since Lopster 1.2.1");
   text = GTK_TEXT(lookup_widget(win, "text15"));
   gtk_text_set_word_wrap(text, 1);
 
-  text_print_text(text, "$g-----------------------------\nMajor changes since version 1.0.1\n-----------------------------\n");
-
-  text_print_text(text, "\n$k* $rMost important\n");
-  text_print_text(text, "$k - $bMulti server support $k- connecting to more than one server\n");
-  text_print_text(text, "$k - $bMulti source support $k- downloading a file from more than one source\n");
-
-  text_print_text(text, "\n$k* $rServers\n");
-  text_print_text(text, "$k - $bCompletely new server/network handling $k- create your own network groups\n");
-  text_print_text(text, "$k - $bBasic IRC support $k- just create a network with type IRC and add a server there\n");
-  text_print_text(text, "$k - $bCompression support on opennap-ng servers $k- ./configure --with-zlib\n");
-  text_print_text(text, "$k - $bThread support $k- ./configure --with-pthread to get rid of gui freezes\n");
-
-  text_print_text(text, "\n$k* $rLibrary and browse\n");
-  text_print_text(text, "$k - $bQuick search $k- quickly find files in your library or browses\n");
-  text_print_text(text, "$k - $bNew layout $k- two window view for direcotries and files\n");
-  text_print_text(text, "$k - $bDirect browses handled like uploads $k- They are bandwidth restricted, you can abort them\n");
-  text_print_text(text, "$k - $bBrowses are saved to disc $k- You can view the files offline and even queue files of offline users for downloading\n");
-
-  text_print_text(text, "\n$k* $rSearch\n");
-  text_print_text(text, "$k - $bTree view $k- Same files, same node\n");
-  text_print_text(text, "$k - $bMore options $k- more power\n");
-
-  text_print_text(text, "\n$k* $rChat\n");
-  text_print_text(text, "$k - $bLots of right click tokens $k- Networks, channels, nicknames, files, urls. Try it!\n");
-  text_print_text(text, "$k - $bLimit for chat buffers\n");
-
-  text_print_text(text, "\n$k* $rTransfers\n");
-  text_print_text(text, "$k - $bDownload layout changes $k- it is configurable - check prefs\n");
-  text_print_text(text, "$k - $bDownload clone detection\n");
-  text_print_text(text, "$k - $bAdvanced connection between Lopster users $k- You can download several segments with one single TCP connection\n");
-
-  text_print_text(text, "\n$k* $rStatistic\n");
-  text_print_text(text, "$k - $bNew layout $k- more options for the bandwidth graph (click)\n");
-  
-  text_print_text(text, "\n$k* $rAs always:\n");
-  text_print_text(text, "$k - Lots of minor changes, lots of bug fixes\n");
-
-  text_print_text(text, "\n$rHave Fun!!!  Sgop\n");
+  text_print_text(text,
+		  "\n"
+		  "$r**********************************************\n"
+		  "$k This is a bugfix release\n"
+		  "$r**********************************************\n"
+		  "\n"
+		  "\n$rHave Fun!!!  Sgop\n");
   gtk_my_widget_show(win);
 }
 
-GtkWidget *create_info_win(char *text, char **buttons,
-			   GtkSignalFunc * func)
-{
+GtkWidget* create_info_win(char *text, char **buttons,
+			   GtkSignalFunc* func) {
   GtkWidget *delete_win;
   GtkWidget *frame172;
   GtkWidget *vbox;
@@ -252,8 +210,7 @@ void on_open_pref_clicked(GtkButton * button ATTR_UNUSED, gpointer user_data)
   preferences_goto(3);
 }
 
-void openfile_dialog(char *text)
-{
+void openfile_dialog(char *text) {
   GtkSignalFunc func[3];
   char *buttons[3];
   GtkWidget *temp;
@@ -261,56 +218,15 @@ void openfile_dialog(char *text)
   func[0] = (GtkSignalFunc) on_close_clicked;
   func[1] = (GtkSignalFunc) on_open_pref_clicked;
   func[2] = NULL;
-  buttons[0] = l_strdup("Close");
-  buttons[1] = l_strdup("Setup Applications");
+  buttons[0] = g_strdup("Close");
+  buttons[1] = g_strdup("Setup Applications");
   buttons[2] = NULL;
 
   temp = create_info_win(text, buttons, func);
   gtk_widget_show(temp);
 
-  l_free(buttons[0]);
-  l_free(buttons[1]);
-}
-
-void on_download1_pref_clicked(GtkButton * button ATTR_UNUSED, gpointer user_data)
-{
-  gtk_widget_destroy(GTK_WIDGET(user_data));
-  preferences_goto(4);
-}
-
-void on_download2_pref_clicked(GtkButton * button ATTR_UNUSED, gpointer user_data)
-{
-  gtk_widget_destroy(GTK_WIDGET(user_data));
-  preferences_goto(5);
-}
-
-void download_dialog(char *filename, int mime) {
-  GtkSignalFunc func[4];
-  char *buttons[4];
-  GtkWidget *temp;
-  char *text;
-
-  if (global.path_warned) return;
-
-  text =
-    l_strdup_printf("Download of file %s failed.\nEither the incomplete path was not set or\nno valid path was specified for mime type %s.\n",
-		    filename, MimeNames(mime, 2));
-  func[0] = (GtkSignalFunc) on_close_clicked;
-  func[1] = (GtkSignalFunc) on_download1_pref_clicked;
-  func[2] = (GtkSignalFunc) on_download2_pref_clicked;
-  func[3] = NULL;
-  buttons[0] = l_strdup("Close");
-  buttons[1] = l_strdup("Set incomplete path");
-  buttons[2] = l_strdup("Set download path");
-  buttons[3] = NULL;
-
-  temp = create_info_win(text, buttons, func);
-  gtk_widget_show(temp);
-
-  l_free(buttons[0]);
-  l_free(buttons[1]);
-  l_free(buttons[2]);
-  global.path_warned = 1;
+  g_free(buttons[0]);
+  g_free(buttons[1]);
 }
 
 void
@@ -336,8 +252,7 @@ on_delete_selected1_activate(GtkMenuItem * menuitem ATTR_UNUSED, gpointer user_d
     gtk_widget_destroy(GTK_WIDGET(user_data));
 }
 
-void delete_dialog(char *text)
-{
+void delete_dialog(char *text) {
   GtkSignalFunc func[4];
   char *buttons[4];
   GtkWidget *temp;
@@ -346,15 +261,15 @@ void delete_dialog(char *text)
     func[0] = (GtkSignalFunc) on_delete_selected1_activate;
     func[1] = (GtkSignalFunc) on_close_clicked;
     func[2] = NULL;
-    buttons[0] = l_strdup("Do it!");
-    buttons[1] = l_strdup("Cancel");
+    buttons[0] = g_strdup("Do it!");
+    buttons[1] = g_strdup("Cancel");
     buttons[2] = NULL;
 
     temp = create_info_win(text, buttons, func);
     gtk_widget_show(temp);
 
-    l_free(buttons[0]);
-    l_free(buttons[1]);
+    g_free(buttons[0]);
+    g_free(buttons[1]);
   } else {
     on_delete_selected1_activate(NULL, NULL);
   }
@@ -398,15 +313,15 @@ void resume_delete_dialog(char *text)
     func[0] = (GtkSignalFunc) on_delete_selected2_activate;
     func[1] = (GtkSignalFunc) on_close_clicked;
     func[2] = NULL;
-    buttons[0] = l_strdup("Do it!");
-    buttons[1] = l_strdup("Cancel");
+    buttons[0] = g_strdup("Do it!");
+    buttons[1] = g_strdup("Cancel");
     buttons[2] = NULL;
 
     temp = create_info_win(text, buttons, func);
     gtk_widget_show(temp);
 
-    l_free(buttons[0]);
-    l_free(buttons[1]);
+    g_free(buttons[0]);
+    g_free(buttons[1]);
   } else {
     on_delete_selected2_activate(NULL, NULL);
   }
@@ -424,29 +339,28 @@ void on_new_version_noticed(GtkButton * button ATTR_UNUSED, gpointer user_data)
   global.options.check_version = 2;
 }
 
-void new_version_dialog(char *text)
-{
+void new_version_dialog(char *text) {
   GtkSignalFunc func[3];
   char *buttons[3];
   GtkWidget *temp;
   char *str;
 
-  func[0] = (GtkSignalFunc) on_new_version_remember;
-  func[1] = (GtkSignalFunc) on_new_version_noticed;
+  func[0] = (GtkSignalFunc)on_new_version_remember;
+  func[1] = (GtkSignalFunc)on_new_version_noticed;
   func[2] = NULL;
-  buttons[0] = l_strdup("Remember next time");
-  buttons[1] = l_strdup("Noticed!");
+  buttons[0] = g_strdup("Remember next time");
+  buttons[1] = g_strdup("Noticed!");
   buttons[2] = NULL;
 
   str =
-      l_strdup_printf("Lopster %s is available NOW!\nCheck out lopster.sourceforge.net!",
+      g_strdup_printf("Lopster %s is available NOW!\nCheck out lopster.sourceforge.net!",
 		      text);
   temp = create_info_win(str, buttons, func);
   gtk_widget_show(temp);
-  l_free(str);
+  g_free(str);
 
-  l_free(buttons[0]);
-  l_free(buttons[1]);
+  g_free(buttons[0]);
+  g_free(buttons[1]);
 }
 
 void on_check_version_close(GtkButton * button ATTR_UNUSED, gpointer user_data)
@@ -454,36 +368,40 @@ void on_check_version_close(GtkButton * button ATTR_UNUSED, gpointer user_data)
   gtk_widget_destroy(GTK_WIDGET(user_data));
 }
 
-void check_version_dialog(char *text)
-{
+void check_version_dialog(char *text) {
   GtkSignalFunc func[2];
   char *buttons[2];
   GtkWidget *temp;
   char *str;
+  int res;
 
   func[0] = (GtkSignalFunc) on_check_version_close;
   func[1] = NULL;
-  buttons[0] = l_strdup("Close");
+  buttons[0] = g_strdup("Close");
   buttons[1] = NULL;
 
-  if (!version_is_up_to_date(text)) {
+  res = version_is_up_to_date(text); 
+  if (res == 0) {
     str =
-	l_strdup_printf("Latest release is %s\nYour version seems to be outdated!\n",
-			text);
+      g_strdup_printf("Latest version is %s\nYour version seems to be outdated!\n",
+		      text);
+  } else if (res == -1) {
+    str =
+      g_strdup_printf("Latest developement version is %s\nYou are using a release version, but it might be outdated!\n",
+		      text);
   } else {
     str =
-      l_strdup_printf("Latest Lopster is %s\nYour version is up to date!\n",
+      g_strdup_printf("Latest Lopster is %s\nYour version seems to be up to date!\n",
 		      text);
   }
   temp = create_info_win(str, buttons, func);
   gtk_widget_show(temp);
-  l_free(str);
+  g_free(str);
 
-  l_free(buttons[0]);
+  g_free(buttons[0]);
 }
 
-void on_info_close(GtkButton * button ATTR_UNUSED, gpointer user_data)
-{
+void on_info_close(GtkButton * button ATTR_UNUSED, gpointer user_data) {
   gtk_widget_destroy(GTK_WIDGET(user_data));
 }
 
@@ -492,13 +410,14 @@ void info_dialog(char *text) {
   char *buttons[2];
   GtkWidget *temp;
 
-  func[0] = (GtkSignalFunc) on_info_close;
+  func[0] = (GtkSignalFunc)on_info_close;
   func[1] = NULL;
-  buttons[0] = l_strdup("Close");
+  buttons[0] = g_strdup("Close");
   buttons[1] = NULL;
 
   temp = create_info_win(text, buttons, func);
   gtk_widget_show(temp);
 
-  l_free(buttons[0]);
+  g_free(buttons[0]);
 }
+
