@@ -1,4 +1,4 @@
-/* Copyright (C) 2000/2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -59,6 +59,7 @@
 #define O_OPERATOR        4
 #define O_GLOBAL          5
 #define O_WALLOP          6
+#define O_EMOTE           7
 
 static void exec_destroy(exec_t * exec);
 static void get_exec_input(gpointer data, gint source,
@@ -93,11 +94,11 @@ static exec_t *exec_new(char *command,
 {
   exec_t *exec;
 
-  exec = l_malloc(sizeof(exec_t));
-  exec->command = l_strdup(command);
+  exec = g_malloc(sizeof(exec_t));
+  exec->command = g_strdup(command);
   exec->fd_watch = -1;
   exec->output = output;
-  exec->who = who ? l_strdup(who) : 0;
+  exec->who = who ? g_strdup(who) : 0;
   exec->id = exec_find_free_id();
   exec->pid = 0;
   exec->net = net;
@@ -131,13 +132,13 @@ static void exec_try_dequeue()
 }
 
 static void exec_destroy(exec_t * exec) {
-  if (exec->who) l_free(exec->who);
-  l_free(exec->command);
+  if (exec->who) g_free(exec->who);
+  g_free(exec->command);
   process_list = g_list_remove(process_list, exec);
   if (exec->fd_watch >= 0) gdk_input_remove(exec->fd_watch);
   if (exec->fd >= 0) close(exec->fd);
   buffer_destroy(exec->buffer);
-  l_free(exec);
+  g_free(exec);
 }
 
 static void print_exec(exec_t * exec)
@@ -369,7 +370,7 @@ static void exec_failed(exec_t* exec)
 {
   if (exec->verbose)
     client_message("Warning",
-		"Unable to start process %d: %s. Retry in 20 secs.",
+		   "Unable to start process %d: %s. Retry in 20 secs.",
 		   exec->id, strerror(errno));
   if (exec_timerid != -1) {
     //    printf("[EXEC] removing timer %d in exec_failed\n", exec_timerid);
@@ -442,8 +443,7 @@ static void exec_queue(exec_t* exec)
   basic implementation by r00t
   added non blocking output read and more.
 */
-void exec_command(net_t* net, char *cmd)
-{
+void exec_command(net_t* net, char *cmd) {
   char *line;
   char *command;
   int output = O_WINDOW;
@@ -483,11 +483,11 @@ void exec_command(net_t* net, char *cmd)
     command = arg(line, 0);
     who = arg(NULL, 0);
     command = arg(NULL, 1);
-  } else if (!strncasecmp(line, "-wallop", 5)) {
+  } else if (!strncasecmp(line, "-wallop", 7)) {
     output = O_WALLOP;
     command = arg(line, 0);
     command = arg(NULL, 1);
-  } else if (!strncasecmp(line, "-chop", 3)) {
+  } else if (!strncasecmp(line, "-chop", 5)) {
     command = arg(line, 0);
     command = arg(NULL, 1);
     if (global.current_page->type == P_PUBLIC) {
@@ -498,6 +498,18 @@ void exec_command(net_t* net, char *cmd)
       client_message("Error", "Switch to a channel");
       return;
     }
+  } else if (!strncasecmp(line, "-me", 3) ||
+	     !strncasecmp(line, "-emote", 6)) {
+    command = arg(line, 0);
+    command = arg(NULL, 1);
+    if (global.current_page->type == P_PUBLIC) {
+      output = O_EMOTE;
+      who = global.current_page->name;
+      page = global.current_page;
+    } else {
+      client_message("Error", "Switch to a channel");
+      return;
+    }
   } else if (!strncasecmp(line, "-quiet", 6)) {
     output = O_NONE;
     command = arg(line, 0);
@@ -513,7 +525,7 @@ void exec_command(net_t* net, char *cmd)
       client_message("Error", "Kill which process?");
       return;
     }
-    if (!l_strcasecmp(command, "all"))
+    if (!g_strcasecmp(command, "all"))
       exec_kill_all();
     else
       exec_kill(atoi(command));
@@ -556,7 +568,7 @@ void exec_command_safe(void (*exit_cb)(void *, int), void *exit_cb_data,
     }
     nlen++;
     va_end(args);
-    cline = l_malloc(nlen);
+    cline = g_malloc(nlen);
     memcpy(cline, cmd, clen);
     va_start(args, cmd);
     while ((targ = va_arg (args, char *))) {
@@ -642,8 +654,8 @@ static void exec_print_line(exec_t * exec, char *line) {
   case O_NONE:
     break;
   case O_CHANNEL:
-    if (!exec->page) break;
-    send_public(exec->page, line);
+    if (exec->page) 
+      send_public(exec->page, line);
     break;
   case O_PRIVATE:
     send_notice(exec->net, exec->who, line, 1);
@@ -652,8 +664,12 @@ static void exec_print_line(exec_t * exec, char *line) {
     send_wallop(exec->net, line);
     break;
   case O_OPERATOR:
-    if (!exec->page) break;
-    send_chwallop(exec->page, line);
+    if (exec->page) 
+      send_chwallop(exec->page, line);
+    break;
+  case O_EMOTE:
+    if (exec->page) 
+      send_public_emote(exec->page, line);
     break;
   case O_GLOBAL:
     send_global(exec->net, line);
