@@ -1,4 +1,4 @@
-/* Copyright (C) 2000/2002 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -24,7 +24,7 @@ file_node_init(file_node_t* node, char* name) {
   int i1, i2;
 
   if (!node) return;
-  if (name) node->name = l_strdup(name);
+  if (name) node->name = g_strdup(name);
   else node->name = NULL;
   node->parent = NULL;
   node->child = NULL;
@@ -32,7 +32,7 @@ file_node_init(file_node_t* node, char* name) {
   node->prev = NULL;
   node->file = NULL;
 
-  for (i1 = 0; i1 <= MIME_SIZE; i1++) {
+  for (i1 = 0; i1 <= MEDIA_SIZE; i1++) {
     for (i2 = 0; i2 <= 3; i2++) {
       node->fci[i1][i2] = 0;
       node->fsi[i1][i2] = 0.;
@@ -51,7 +51,7 @@ file_node_destroy(file_tree_t* tree, file_node_t* node) {
   
   if (tree->node_free) tree->node_free(node);
   
-  if (node->name) l_free(node->name);
+  if (node->name) g_free(node->name);
   if (node->file) file_destroy(node->file);
 
   child = node->child;
@@ -62,7 +62,7 @@ file_node_destroy(file_tree_t* tree, file_node_t* node) {
   }
   node->child = NULL;
   
-  l_free(node);
+  g_free(node);
 }
 
 /** gets pixmap for node */
@@ -173,13 +173,13 @@ file_tree_init(file_tree_t* tree, char* name, char* des) {
 
   if (!tree) return;
 
-  if (name) tree->name = l_strdup(name);
+  if (name) tree->name = g_strdup(name);
   else tree->name = NULL;
-  if (des) tree->description = l_strdup(des);
+  if (des) tree->description = g_strdup(des);
   else tree->description = NULL;
 
   tree->last_refreshed = 0;
-  for (i1 = 0; i1 <= MIME_SIZE; i1++) {
+  for (i1 = 0; i1 <= MEDIA_SIZE; i1++) {
     for (i2 = 0; i2 <= 3; i2++) {
       tree->fci[i1][i2] = 0;
       tree->fsi[i1][i2] = 0.;
@@ -197,7 +197,7 @@ file_tree_init(file_tree_t* tree, char* name, char* des) {
   tree->resize_cont = NULL;
   tree->files = NULL;
   tree->current_node = NULL;
-  tree->show_mime = MIME_SIZE;
+  tree->show_mime = MEDIA_SIZE;
   tree->show_time = 3;
   tree->mark_timeout = -1;
   tree->search_timeout = -1;
@@ -217,14 +217,17 @@ file_tree_init(file_tree_t* tree, char* name, char* des) {
   tree->signal[2] = -1;
   tree->signal[3] = -1;
   tree->signal[4] = -1;
+  tree->signal[5] = -1;
+  tree->signal[6] = -1;
+  tree->signal[7] = -1;
 }
 
 void file_tree_destroy(file_tree_t* tree) {
   if (!tree) return;
-  if (tree->name) l_free(tree->name);
-  if (tree->description) l_free(tree->description);
+  if (tree->name) g_free(tree->name);
+  if (tree->description) g_free(tree->description);
   if (tree->mark_timeout >= 0) gtk_timeout_remove(tree->mark_timeout);
-  l_free(tree);
+  g_free(tree);
 }
 
 void
@@ -342,12 +345,12 @@ file_tree_split_folder(file_tree_t* tree, file_node_t* node,
   
   parent = node->parent;
 
-  name = l_strdup(node->name);
+  name = g_strdup(node->name);
 
   // remove the original node
   file_tree_node_remove(tree, node);
-  l_free(node->name);
-  node->name = l_strdup(name+pos);
+  g_free(node->name);
+  node->name = g_strdup(name+pos);
 
   // insert the new node
   name[pos] = 0;
@@ -358,22 +361,23 @@ file_tree_split_folder(file_tree_t* tree, file_node_t* node,
   // insert the original node again.
   file_tree_node_insert(tree, child, node, 1);
 
-  l_free(name);
+  g_free(name);
   return child;
 }
 
 file_node_t*
 file_node_insert_file(file_tree_t* tree, file_node_t* node,
-		      file_t* file) {
+		      file_t* file, gboolean check_dup) {
   file_node_t* child;
   
-  child = file_node_search_file(node?node->child:tree->files, file);
-  
-  if (child) {
-    // update the file.
-    child->file->net = file->net;
-    file_destroy(file);
-    return child;
+  if (check_dup) {
+    child = file_node_search_file(node?node->child:tree->files, file);
+    if (child) {
+      // update the file.
+      child->file->net = file->net;
+      file_destroy(file);
+      return child;
+    }
   }
 
   child = tree->node_new(file->filename);
@@ -384,8 +388,7 @@ file_node_insert_file(file_tree_t* tree, file_node_t* node,
   return child;
 }
 
-file_node_t*
-file_tree_insert_file(file_tree_t* tree, file_t * file) {
+file_node_t* file_tree_insert_file(file_tree_t* tree, file_t * file) {
   char* pos;
   file_node_t* node;
   char save;
@@ -400,7 +403,7 @@ file_tree_insert_file(file_tree_t* tree, file_t * file) {
   
 #if 0  // code out of date
   if (pos && pos - file->longname >= 2	&& pos[-1] == '/') {
-    //	Undo stupid XNap behaviour of putting each file	in its own numbered
+    //	Undo XNap behaviour of putting each file in its own numbered
     //	directory e.g. "42//foo.mp3", "43//bar.mp3" which makes	it very	hard
     //	to browse.
     pos -= 2;
@@ -420,7 +423,7 @@ file_tree_insert_file(file_tree_t* tree, file_t * file) {
   node = file_tree_insert_folder(tree, NULL, file->longname, 1);
   // append the filename again.
   *(file->filename) = save;
-  node = file_node_insert_file(tree, node, file);
+  node = file_node_insert_file(tree, node, file, TRUE);
 
   if (tree->update_user) tree->update_user(tree);
 
@@ -594,19 +597,21 @@ static char* TNames(int no) {
 
 static char *MNames(int no) {
   switch (no) {
-  case MIME_MP3:
+  case MEDIA_MP3:
     return "Mp3 files";
-  case MIME_AUDIO:
+  case MEDIA_AUDIO:
     return "Audio files";
-  case MIME_VIDEO:
+  case MEDIA_VIDEO:
     return "Videos";
-  case MIME_APPLICATION:
+  case MEDIA_APPLICATION:
     return "Applications";
-  case MIME_IMAGE:
+  case MEDIA_IMAGE:
     return "Images";
-  case MIME_TEXT:
+  case MEDIA_TEXT:
     return "Text files";
-  case MIME_NONE:
+  case MEDIA_FOLDER:
+    return "Folder";
+  case MEDIA_NONE:
     return "Other";
   default:
     return "Files";
@@ -659,7 +664,7 @@ file_tree_calc_matrix_rec(file_tree_t* tree, file_node_t* node) {
   int i1, i2;
   file_node_t* child;
 
-  for (i1 = 0; i1 <= MIME_SIZE; i1++) {
+  for (i1 = 0; i1 <= MEDIA_SIZE; i1++) {
     for (i2 = 0; i2 <= 3; i2++) {
       node->fci[i1][i2] = 0;
       node->fsi[i1][i2] = 0.;
@@ -672,33 +677,33 @@ file_tree_calc_matrix_rec(file_tree_t* tree, file_node_t* node) {
       // its a new shared file
       node->fci[node->file->media_type][2]++;
       node->fsi[node->file->media_type][2] += node->file->size;
-      node->fci[MIME_SIZE][2]++;
-      node->fsi[MIME_SIZE][2] += node->file->size;
+      node->fci[MEDIA_SIZE][2]++;
+      node->fsi[MEDIA_SIZE][2] += node->file->size;
     }
     if (i1 & (1<<1)) {
       // its an shared file
       node->fci[node->file->media_type][1]++;
       node->fsi[node->file->media_type][1] += node->file->size;
-      node->fci[MIME_SIZE][1]++;
-      node->fsi[MIME_SIZE][1] += node->file->size;
+      node->fci[MEDIA_SIZE][1]++;
+      node->fsi[MEDIA_SIZE][1] += node->file->size;
     }
     if (i1 & (1<<0)) {
       // its unshared
       node->fci[node->file->media_type][0]++;
       node->fsi[node->file->media_type][0] += node->file->size;
-      node->fci[MIME_SIZE][0]++;
-      node->fsi[MIME_SIZE][0] += node->file->size;
+      node->fci[MEDIA_SIZE][0]++;
+      node->fsi[MEDIA_SIZE][0] += node->file->size;
     }
     node->fci[node->file->media_type][3]++;
     node->fsi[node->file->media_type][3] += node->file->size;
-    node->fci[MIME_SIZE][3]++;
-    node->fsi[MIME_SIZE][3] += node->file->size;
+    node->fci[MEDIA_SIZE][3]++;
+    node->fsi[MEDIA_SIZE][3] += node->file->size;
   } else {
     child = node->child;
     while (child) {
       // tree->calc_matrix_rec() should also do the trick
       file_tree_calc_matrix_rec(tree, child);
-      for (i1 = 0; i1 <= MIME_SIZE; i1++) {
+      for (i1 = 0; i1 <= MEDIA_SIZE; i1++) {
 	for (i2 = 0; i2 <= 3; i2++) {
 	  node->fci[i1][i2] += child->fci[i1][i2];
 	  node->fsi[i1][i2] += child->fsi[i1][i2];
@@ -714,7 +719,7 @@ file_tree_calc_matrix(file_tree_t* tree) {
   int i1, i2;
   file_node_t* node;
 
-  for (i1 = 0; i1 <= MIME_SIZE; i1++) {
+  for (i1 = 0; i1 <= MEDIA_SIZE; i1++) {
     for (i2 = 0; i2 <= 3; i2++) {
       tree->fci[i1][i2] = 0;
       tree->fsi[i1][i2] = 0.;
@@ -724,7 +729,7 @@ file_tree_calc_matrix(file_tree_t* tree) {
   node = tree->files;
   while (node) {
     file_tree_calc_matrix_rec(tree, node);
-    for (i1 = 0; i1 <= MIME_SIZE; i1++) {
+    for (i1 = 0; i1 <= MEDIA_SIZE; i1++) {
       for (i2 = 0; i2 <= 3; i2++) {
 	tree->fci[i1][i2] += node->fci[i1][i2];
 	tree->fsi[i1][i2] += node->fsi[i1][i2];
@@ -788,9 +793,9 @@ void file_tree_clear_files(file_tree_t *tree) {
   if (!tree) return;
 
   // updating user stats
-  for (i1 = 0; i1 <= MIME_SIZE; i1++) {
+  for (i1 = 0; i1 <= MEDIA_SIZE; i1++) {
     for (i2 = 0; i2 <= 3; i2++) {
-      if (i1 != MIME_SIZE && i2 != 3) {
+      if (i1 != MEDIA_SIZE && i2 != 3) {
 	tree->fci[i1][i2] = 0;
 	tree->fsi[i1][i2] = 0.;
       }
@@ -882,8 +887,8 @@ int on_file_tree_search_changed(GtkWidget * widget,
   if (!tree || !tree->ctree || !tree->clist) return 0;
 
   text = gtk_entry_get_text(GTK_ENTRY(widget));
-  if (tree->search_token) l_free(tree->search_token);
-  tree->search_token = l_strdup(text);
+  if (tree->search_token) g_free(tree->search_token);
+  tree->search_token = g_strdup(text);
 
   if (tree->search_timeout >= 0)
     gtk_timeout_remove(tree->search_timeout);
@@ -1029,7 +1034,7 @@ char* file_node_get_folder(file_node_t* node, int* size) {
   if (!node->parent) {
     // no more parent, we allocate string here
     // one byte for the \0
-    result = l_malloc(sizeof(char)*(*size+1));
+    result = g_malloc(sizeof(char)*(*size+1));
     result[*size] = 0;
     // thats the return point, now size is index
     *size = 0;
@@ -1090,6 +1095,47 @@ file_tree_search_winname(file_tree_t* tree, file_node_t* node,
 			  (void*)fname);
 }
 
+/*// BEGIN chelaz@chelaz.de, modified by sgop
+void file_tree_save_node_plain(FILE* fd, file_node_t* node,
+			       int indent) {
+  file_t* file;
+  char str[1024];
+
+  while (node) {
+    file = node->file;
+    if (file) {
+      fprintf(fd, "%*s", indent, "");
+      fprintf(fd, "%s [%5.2f MB]%s\n",
+	      file->filename, print_bytes(str, file->size),
+	      (node->flags & LIB_VIRTUAL)!=0 ? " [offline]":"");
+    } else if (node->child) {
+      fprintf(fd, "%*s", indent, "");
+      fprintf(fd, "%s%s\n", node->name, 
+	      (node->flags & LIB_VIRTUAL)!=0 ? " [offline]":"");
+      lib_save_node_plain_text(fd, node->child, indent+2);
+      fprintf(fd, "%*s\n", indent, "");
+    }
+    node = node->next;
+  }
+}
+
+void file_tree_save_plain(file_tree_t* tree, char* filename) {
+  FILE *fd;
+  file_tree_t* tree = FILE_TREE(lib);
+
+  if (!tree) return;
+  
+  if ((fd = fopen(filename_plain_text, "w")) == NULL) {
+    g_warning("Could not save plain file tree %s", filename);
+    return;
+  }
+  fprintf(fd, "BEGIN of list\n"
+	  "-----------------------------------------\n");
+  lib_save_node_plain_text(fd_plain_text, tree->files, 0);
+  fclose(fd_plain_text);
+}
+// END chelaz@chelaz.de */
+
 static GList* 
 file_tree_search_filesize_rec(file_node_t* node, file_t* file, 
 			      GList* ilist) {
@@ -1098,6 +1144,7 @@ file_tree_search_filesize_rec(file_node_t* node, file_t* file,
   if (!node) return ilist;
 
   if (node->file) {
+/*** [fvg] adapt to allow some marge (interval, max delta =...) */
     if (file->size == node->file->size)
       ilist = g_list_prepend(ilist, node->file);
   }
@@ -1162,12 +1209,7 @@ on_tree_motion_notify_event(GtkWidget * widget,
   if (!gtk_clist_get_selection_info(GTK_CLIST(widget),
 				    (int) event->x, (int) event->y,
 				    &row, &column)) {
-    filetips_show(NULL, -1, -1, NULL);
-    return FALSE;
-  }
-
-  if (column != 1) {
-    filetips_show(NULL, -1, -1, NULL);
+    filetips_show(NULL, event->x_root, event->y_root, NULL);
     return FALSE;
   }
 
@@ -1176,16 +1218,16 @@ on_tree_motion_notify_event(GtkWidget * widget,
   node = gtk_clist_get_row_data(GTK_CLIST(widget), row);
   if (!node || !node->file) return FALSE;
 
-  filetips_show(widget, row, column, node->file);
+  filetips_show(widget, event->x_root, event->y_root, node->file);
   return FALSE;
 }
 
 
 gboolean
 on_tree_leave_notify_event(GtkWidget * widget ATTR_UNUSED,
-			   GdkEventCrossing * event ATTR_UNUSED, gpointer user_data ATTR_UNUSED)
+			   GdkEventCrossing* event, gpointer user_data ATTR_UNUSED)
 {
-  filetips_show(NULL, -1, -1, NULL);
+  filetips_show(NULL, event->x_root, event->y_root, NULL);
   return FALSE;
 }
 
@@ -1225,6 +1267,7 @@ file_tree_release_page(file_tree_t* tree) {
     gtk_signal_disconnect(GTK_OBJECT(tree->button[1]),
 			  tree->signal[6]);
   tree->signal[6] = -1;
+  
 }
 
 void popup_position(GtkMenu * menu, gint * x, gint * y, 
@@ -1245,7 +1288,7 @@ void
 on_file_tree_files1_clicked(GtkButton * button ATTR_UNUSED, 
 			    gpointer user_data) {
   GtkWidget *popup;
-  GtkWidget *item[MIME_SIZE+1];
+  GtkWidget *item[MEDIA_SIZE+1];
   file_tree_t* tree = user_data;
   int i1, i2;
   char str[1024];
@@ -1324,7 +1367,7 @@ void
 on_file_tree_files2_clicked(GtkButton * button ATTR_UNUSED, 
 			    gpointer user_data) {
   GtkWidget *popup;
-  GtkWidget *item[MIME_SIZE+1];
+  GtkWidget *item[MEDIA_SIZE+1];
   file_tree_t* tree = user_data;
   int i1, i2;
   char str[1024];
@@ -1350,7 +1393,7 @@ on_file_tree_files2_clicked(GtkButton * button ATTR_UNUSED,
   radio_group = NULL;
   cnt = 0;
   i2 = tree->show_time;
-  for (i1 = 0; i1 <= MIME_SIZE; i1++) {
+  for (i1 = 0; i1 <= MEDIA_SIZE; i1++) {
     if (tree->fci[i1][i2] == 0) {
       item[i1] = NULL;
       continue;
@@ -1359,11 +1402,11 @@ on_file_tree_files2_clicked(GtkButton * button ATTR_UNUSED,
     sprintf(str, "%s: %d (%.1f%%) %s (%.1f%%)",
 	    MNames(i1),  tree->fci[i1][i2],
 	    (double)(tree->fci[i1][i2])/
-	    (double)(tree->fci[MIME_SIZE][i2])*100.,
+	    (double)(tree->fci[MEDIA_SIZE][i2])*100.,
 	    str2,
-	    tree->fsi[i1][i2]/tree->fsi[MIME_SIZE][i2]*100.);
+	    tree->fsi[i1][i2]/tree->fsi[MEDIA_SIZE][i2]*100.);
 
-    if (i1 == MIME_SIZE) {
+    if (i1 == MEDIA_SIZE) {
       item[i1] = gtk_menu_item_new();
       gtk_widget_show(item[i1]);
       gtk_container_add(GTK_CONTAINER(popup), item[i1]);
@@ -1379,7 +1422,7 @@ on_file_tree_files2_clicked(GtkButton * button ATTR_UNUSED,
     gtk_container_add(GTK_CONTAINER(popup), item[i1]);
     cnt++;
   }
-  for (i1 = 0; i1 <= MIME_SIZE; i1++) {
+  for (i1 = 0; i1 <= MEDIA_SIZE; i1++) {
     if (item[i1])
       gtk_signal_connect(GTK_OBJECT(item[i1]), "activate",
 			 GTK_SIGNAL_FUNC(on_mime_activate),
