@@ -1,3 +1,7 @@
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
+   This is free software distributed under the terms of the
+   GNU Public License.  See the file COPYING for details. */
+
 #ifndef _FILES_H_
 #define _FILES_H_
 
@@ -11,5 +15,6 @@ file_t* file_create_from_browse_response(net_t* net, char *data);
 file_t* file_create_from_search_response(net_t* net, char *data);
 int file_compare(file_t* file1, file_t* file2);
 file_t* file_duplicate(file_t * file);
+void play_file(char *filename, char* app);
 
 #endif
