@@ -1,4 +1,4 @@
-/* Copyright (C) 2000/2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -41,8 +41,8 @@ static int hot_description_func(clist_rename_t* cr) {
   if (row < 0) return 0;
   
   hot = cr->data;
-  if (hot->description) l_free(hot->description);
-  hot->description = l_strdup(cr->new_text);
+  if (hot->description) g_free(hot->description);
+  hot->description = g_strdup(cr->new_text);
   gtk_clist_set_text(cr->clist, row, cr->col, hot->description);  
   
   return 1;
@@ -140,9 +140,9 @@ GtkWidget *create_hotlist_popup(hot_t* hot)
 hot_t *hotlist_new(char *user, char *des, int visible) {
   hot_t *result;
 
-  result = (hot_t *) l_malloc(sizeof(hot_t));
-  result->user = l_strdup(user);
-  if (des) result->description = l_strdup(des);
+  result = (hot_t *) g_malloc(sizeof(hot_t));
+  result->user = g_strdup(user);
+  if (des) result->description = g_strdup(des);
   else result->description = NULL;
   result->visible = visible;
   result->nets = NULL;
@@ -158,7 +158,7 @@ hot_t *hotlist_search_user(char *user) {
 
   for (dlist = global.hotlist; dlist; dlist = dlist->next) {
     hot = dlist->data;
-    if (!l_strcasecmp(hot->user, user)) return hot;
+    if (!g_strcasecmp(hot->user, user)) return hot;
   }
 
   return NULL;
@@ -172,10 +172,10 @@ hot_t* hotlist_add(char *nick, char *des, int visible) {
 
   hot = hotlist_search_user(nick);
   if (hot && (!visible || hot->visible)) {
-    text = l_strdup_printf("User <%s> is already in hotlist",
+    text = g_strdup_printf("User <%s> is already in hotlist",
 			   nick);
     info_dialog(text);
-    l_free(text);
+    g_free(text);
     return NULL;
   }
 
@@ -205,8 +205,8 @@ hot_t* hotlist_add(char *nick, char *des, int visible) {
     strcpy(tstr[0], nick);
     if (hot->nets) strcpy(tstr[1], LineSpeed(hot->speed));
     else strcpy(tstr[1], "");
-    if (hot->description) l_free(hot->description);
-    if (des) hot->description = l_strdup(des);
+    if (hot->description) g_free(hot->description);
+    if (des) hot->description = g_strdup(des);
     else hot->description = NULL;
     if (hot->description)
       strcpy(tstr[2], hot->description);
@@ -249,19 +249,19 @@ void hotlist_remove(char *nick, int visible) {
       // remove from list, if user has no private page.
       command_send(NULL, CMD_REMOVE_HOTLIST, hot->user);
       global.hotlist = g_list_remove(global.hotlist, hot);
-      if (hot->user) l_free(hot->user);
+      if (hot->user) g_free(hot->user);
       if (hot->nets) g_list_free(hot->nets);
-      if (hot->description) l_free(hot->description);
-      l_free(hot);
+      if (hot->description) g_free(hot->description);
+      g_free(hot);
     }
   } else if (!hot->visible) {
     // only remove, if this is no visible hotlist entry
     command_send(NULL, CMD_REMOVE_HOTLIST, hot->user);
     global.hotlist = g_list_remove(global.hotlist, hot);
-    if (hot->user) l_free(hot->user);
+    if (hot->user) g_free(hot->user);
     if (hot->nets) g_list_free(hot->nets);
-    if (hot->description) l_free(hot->description);
-    l_free(hot);
+    if (hot->description) g_free(hot->description);
+    g_free(hot);
   }
 }
 
@@ -309,7 +309,8 @@ void hotlist_user_online(net_t* net, hot_t * hot) {
       chat_print_text(page, M_PUBLIC, "user", "<");
       chat_print_nick(page, M_PUBLIC, hot->user, net);
       chat_print_text(page, M_PUBLIC, "user", "> ");
-      chat_print_text(page, M_PUBLIC, "message", "is on");
+      chat_print_text(page, M_PUBLIC, "message", "is on ");
+      chat_print_text(page, M_PUBLIC, "text", net->name);
       if (hot->description && hot->description[0]) {
 	chat_print_text(page, M_PUBLIC, "message", " (");
 	chat_print_text(page, M_PUBLIC, "message", hot->description);
