@@ -458,9 +458,9 @@ static int consume_command_irc(net_t* net) {
   net->c_type = irc_get_command_code(command);
   
   if (from)
-    result = l_strdup_printf("%s %s", from, args);
+    result = g_strdup_printf("%s %s", from, args);
   else
-    result = l_strdup(args);
+    result = g_strdup(args);
 
   net->c_length = strlen(result);
   net->c_message = result;
@@ -480,16 +480,16 @@ char *irc_irc2nap(net_t* net, char *data) {
   if (*data != ':') {
     if (!strncmp (data, "NOTICE ", 7)) {
       net->c_type = CIRC_PRIVATE_NOTICE;
-      result = l_strdup(data+7);
+      result = g_strdup(data+7);
     } else if (!strncmp (data, "PING ", 5)) {
       net->c_type = CIRC_PING;
-      result = l_strdup(data+5);
+      result = g_strdup(data+5);
     } else if (!strncmp (data, "ERROR ", 6)) {
       net->c_type = CIRC_ERROR;
-      result = l_strdup(data+6);
+      result = g_strdup(data+6);
     } else {
       net->c_type = 20000;
-      result = l_strdup(data);
+      result = g_strdup(data);
     }
     return result;
   }
@@ -516,7 +516,7 @@ char *irc_rpl_n(net_t* net, irc_t *irc, char *last)
 
   if (irc->command_num == 1) {
     net->c_type = NAP_EMAIL;
-    data = l_strdup_printf("admin@lopster.irc");
+    data = g_strdup_printf("admin@lopster.irc");
     irc_handle_command(net, data);
   }
 
@@ -525,7 +525,7 @@ char *irc_rpl_n(net_t* net, irc_t *irc, char *last)
     // <nick> <text>
     strtok_r(NULL," ", &last);
     if (last[0] == ':') last++;
-    return l_strdup_printf("SRV %s", last);
+    return g_strdup_printf("SRV %s", last);
   }
 
   switch (irc->command_num) {
@@ -534,7 +534,7 @@ char *irc_rpl_n(net_t* net, irc_t *irc, char *last)
     strtok_r(NULL," ", &last);
     strtok_r(NULL," ", &last);
     strtok_r(NULL," ", &last);
-    return l_strdup_printf("%s 0 0", strtok_r(NULL," ", &last));
+    return g_strdup_printf("%s 0 0", strtok_r(NULL," ", &last));
   case RPL_LUSEROP: // 252
   case RPL_LUSERCLIENT: // 251
   case RPL_LUSERUNKNOWN: // 253
@@ -542,18 +542,18 @@ char *irc_rpl_n(net_t* net, irc_t *irc, char *last)
     net->c_type = NAP_ANNOUNCE;
     strtok_r(NULL," ", &last);
     if (last[0] == ':') last++;
-    return l_strdup_printf("%s %s", IRC_SRV_MSG, last);
+    return g_strdup_printf("%s %s", IRC_SRV_MSG, last);
 
   case RPL_ISON: // 303
     if (last[0] == ':') last++;
     irc->params = last;
     if (irc->params[0]) {
       net->c_type = MODE_ERROR;
-      data = l_strdup_printf("Nickname is already in use: %s", last);
+      data = g_strdup_printf("Nickname is already in use: %s", last);
     } else {
       net->c_type = CMD_SERVER_ANNOUNCE;
       // [SrvMsg] Nickname can be used.
-      data = l_strdup_printf("%s Nickname can be used.", IRC_SRV_MSG);
+      data = g_strdup_printf("%s Nickname can be used.", IRC_SRV_MSG);
     }
     return data;
 
@@ -561,14 +561,14 @@ char *irc_rpl_n(net_t* net, irc_t *irc, char *last)
     net->c_type = NAP_MOTD;
     strtok_r(NULL," ", &last);
     if (last[0] == ':') last++;
-    return l_strdup_printf("%s", last);
+    return g_strdup_printf("%s", last);
 
   case RPL_CHANNELMODEIS: // 324
     // "<channel> <mode> <mode params>"
     net->c_type = NAP_PUBLIC; // 403
     irc->msgtarget = strtok_r(NULL," ", &last);
     irc->channel = strtok_r(NULL," ", &last);
-    return l_strdup_printf("%s --- %s sets channel mode to %s", irc->channel, irc->msgtarget, last);
+    return g_strdup_printf("%s --- %s sets channel mode to %s", irc->channel, irc->msgtarget, last);
 
   case ERR_NICKNAMEINUSE: // 433
     // "<nick> :Nickname is already in use"
@@ -584,7 +584,7 @@ char *irc_rpl_n(net_t* net, irc_t *irc, char *last)
     net->c_type = NAP_PART; // 401
     strtok_r(NULL," ", &last);
     irc->channel = strtok_r(NULL," ", &last);
-    return l_strdup(irc->channel);
+    return g_strdup(irc->channel);
 
   // WHOIS
   case RPL_WHOISUSER: // 311
@@ -595,7 +595,7 @@ char *irc_rpl_n(net_t* net, irc_t *irc, char *last)
     net->c_type = NAP_ANNOUNCE;
     strtok_r(NULL," ", &last);
     irc->nickname = strtok_r(NULL," :", &last);
-    return l_strdup_printf("WhoIs [%s] %s", irc->nickname, last);
+    return g_strdup_printf("WhoIs [%s] %s", irc->nickname, last);
 
   // WHOWAS
   case RPL_WHOWASUSER: // 314
@@ -603,23 +603,23 @@ char *irc_rpl_n(net_t* net, irc_t *irc, char *last)
     net->c_type = NAP_ANNOUNCE;
     strtok_r(NULL," ", &last);
     irc->nickname = strtok_r(NULL," :", &last);
-    return l_strdup_printf("WhoWas [%s] %s", irc->nickname, last);
+    return g_strdup_printf("WhoWas [%s] %s", irc->nickname, last);
 
   // WHO
   case RPL_WHOREPLY: // 352
     net->c_type = NAP_ANNOUNCE;
-    return l_strdup_printf("Who %s", last);
+    return g_strdup_printf("Who %s", last);
   case RPL_ENDOFWHO: // 315
     net->c_type = NAP_ANNOUNCE;
     strtok_r(NULL," ", &last);
     irc->nickname = strtok_r(NULL," :", &last);
-    return l_strdup_printf("Who [%s] %s", irc->nickname, last);
+    return g_strdup_printf("Who [%s] %s", irc->nickname, last);
 
   case RPL_INVITING: // 341
     net->c_type = NAP_ANNOUNCE;
     strtok_r(NULL," ", &last);
     irc->nickname = strtok_r(NULL," :", &last);
-    return l_strdup_printf("--- You're inviting %s to %s", irc->nickname, last);
+    return g_strdup_printf("--- You're inviting %s to %s", irc->nickname, last);
 
   case ERR_CHANOPRIVSNEEDED: // 482
     page = chat_page_search(net, irc->channel, P_PUBLIC, 1);
@@ -630,7 +630,7 @@ char *irc_rpl_n(net_t* net, irc_t *irc, char *last)
     net->c_type = NAP_ANNOUNCE;
     strtok_r(NULL," ", &last);
     irc->channel = strtok_r(NULL," :", &last);
-    return l_strdup_printf("--- %s :You're not channel operator", irc->channel);
+    return g_strdup_printf("--- %s :You're not channel operator", irc->channel);
 
   default:
     irc->msgtarget = strtok_r(NULL," ", &last);
@@ -643,19 +643,19 @@ char *irc_rpl_n(net_t* net, irc_t *irc, char *last)
   if (irc->command_num == RPL_ISON) { // 303
     if (irc->params[0]) {
       net->c_type = NAP_ERROR;
-      return l_strdup_printf("Nickname is already in use: %s", last);
+      return g_strdup_printf("Nickname is already in use: %s", last);
     } else
       net->c_type = NAP_ANNOUNCE;
-      return l_strdup_printf("Nickname is OK.");
+      return g_strdup_printf("Nickname is OK.");
   }
 
   if (irc->command_num >= 400 && irc->command_num < 600) { // 400 - 599
     net->c_type = NAP_ANNOUNCE;
-    return l_strdup_printf("SRV %s", last);
+    return g_strdup_printf("SRV %s", last);
   }
 
   net->c_type = NAP_ANNOUNCE;
-  return l_strdup_printf("SRV %s", last);
+  return g_strdup_printf("SRV %s", last);
   return NULL;
 }
   */
@@ -685,10 +685,10 @@ char *data;
 
     if (net->c_type == NAP_PUBLIC) {
       // <channel> <nick> <text>
-      return l_strdup_printf("%s %s %s", irc->msgtarget, irc->nickname, last);
+      return g_strdup_printf("%s %s %s", irc->msgtarget, irc->nickname, last);
     } else {
       // <nick> <message>
-      return l_strdup_printf("%s %s", irc->nickname, last);
+      return g_strdup_printf("%s %s", irc->nickname, last);
     }
   }
 
@@ -700,7 +700,7 @@ char *data;
 
   // FIXME
   net->c_type = NAP_ANNOUNCE;
-  return l_strdup(last);
+  return g_strdup(last);
   return NULL;
 }
   */
@@ -810,16 +810,16 @@ char *irc_ctcp_send_command(net_t* net, irc_t* irc, char *ctcp_msg)
 
   if (!strcasecmp(ctcp_cmd, IRC_CTCP_VERSION)) {
     net->c_type = CMD_NOTICE;
-    data = l_strdup_printf("%s %c%s Lopster %s%c", irc->nickname, ASCII_CTCP, IRC_CTCP_VERSION, VERSION, ASCII_CTCP);
+    data = g_strdup_printf("%s %c%s Lopster %s%c", irc->nickname, ASCII_CTCP, IRC_CTCP_VERSION, VERSION, ASCII_CTCP);
 #warning FIXME!
     //    send_command(net->c_type, data, net);
     net->c_type = CMD_ANNOUNCE;
-    return l_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_VERSION);
+    return g_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_VERSION);
   }
 
   if (!strcasecmp(ctcp_cmd, IRC_CTCP_USERINFO)) {
     net->c_type = CMD_NOTICE;
-    data = l_strdup_printf("%s %c%s :%s%c",
+    data = g_strdup_printf("%s %c%s :%s%c",
                            irc->nickname,
                            ASCII_CTCP,
                            IRC_CTCP_USERINFO,
@@ -827,14 +827,14 @@ char *irc_ctcp_send_command(net_t* net, irc_t* irc, char *ctcp_msg)
                            ASCII_CTCP);
     //    send_command(net->c_type, data, net);
     net->c_type = CMD_ANNOUNCE;
-    return l_strdup_printf("%s CTCP %s",
+    return g_strdup_printf("%s CTCP %s",
                            irc->nickname,
                            IRC_CTCP_USERINFO);
   }
 
   if (!strcasecmp(ctcp_cmd, IRC_CTCP_CLIENTINFO)) {
     net->c_type = CMD_NOTICE;
-    data = l_strdup_printf("%s %c%s :%s %s %s %s %s %s %s%c",
+    data = g_strdup_printf("%s %c%s :%s %s %s %s %s %s %s%c",
                            irc->nickname,
                            ASCII_CTCP,
                            IRC_CTCP_CLIENTINFO,
@@ -850,21 +850,21 @@ char *irc_ctcp_send_command(net_t* net, irc_t* irc, char *ctcp_msg)
                            );
     //    send_command(net->c_type, data, net);
     net->c_type = CMD_ANNOUNCE;
-    return l_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_CLIENTINFO);
+    return g_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_CLIENTINFO);
   }
 
   if (!strcasecmp(ctcp_cmd, IRC_CTCP_PING)) {
     net->c_type = CMD_NOTICE;
-    data = l_strdup_printf("%s %c%s %s%c", irc->nickname, ASCII_CTCP, ctcp_cmd, last, ASCII_CTCP);
+    data = g_strdup_printf("%s %c%s %s%c", irc->nickname, ASCII_CTCP, ctcp_cmd, last, ASCII_CTCP);
     //    send_command(net->c_type, data, net);
     net->c_type = CMD_ANNOUNCE;
-    return l_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_PING);
+    return g_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_PING);
   }
 
   if (!strcasecmp(ctcp_cmd, IRC_CTCP_TIME)) {
     net->c_type = CMD_NOTICE;
     wk = ctcp_time();
-    data = l_strdup_printf("%s %c%s :%s%c",
+    data = g_strdup_printf("%s %c%s :%s%c",
                            irc->nickname,
                            ASCII_CTCP,
                            IRC_CTCP_TIME,
@@ -872,15 +872,15 @@ char *irc_ctcp_send_command(net_t* net, irc_t* irc, char *ctcp_msg)
                            ASCII_CTCP
                            );
     //    send_command(net->c_type, data, net);
-    l_free(wk);
+    g_free(wk);
     net->c_type = CMD_ANNOUNCE;
-    return l_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_TIME);
+    return g_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_TIME);
   }
 
   if (!strcasecmp(ctcp_cmd, IRC_CTCP_SOURCE)) {
     net->c_type = CMD_NOTICE;
     wk = ctcp_time();
-    data = l_strdup_printf("%s %c%s :The latest version of Lopster can be obtained from %s%c",
+    data = g_strdup_printf("%s %c%s :The latest version of Lopster can be obtained from %s%c",
                            irc->nickname,
                            ASCII_CTCP,
                            IRC_CTCP_SOURCE,
@@ -888,44 +888,44 @@ char *irc_ctcp_send_command(net_t* net, irc_t* irc, char *ctcp_msg)
                            ASCII_CTCP
                            );
     //    send_command(net->c_type, data, net);
-    l_free(wk);
+    g_free(wk);
     net->c_type = CMD_ANNOUNCE;
-    return l_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_TIME);
+    return g_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_TIME);
   }
 
   if (!strcasecmp(ctcp_cmd, IRC_CTCP_FINGER)) {
     net->c_type = CMD_ANNOUNCE;
-    return l_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_FINGER);
+    return g_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_FINGER);
   }
 
   if (!strcasecmp(ctcp_cmd, IRC_CTCP_XDCC_LIST)) {
     net->c_type = CMD_ANNOUNCE;
-    return l_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_XDCC_LIST);
+    return g_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_XDCC_LIST);
   }
 
   if (!strcasecmp(ctcp_cmd, IRC_CTCP_CDCC_LIST)) {
     net->c_type = CMD_ANNOUNCE;
-    return l_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_CDCC_LIST);
+    return g_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_CDCC_LIST);
   }
 
   // Lopster.... Only
   if (!strcasecmp(ctcp_cmd, IRC_CTCP_X_NAP_BROWSE_DIRECT)) {
     net->c_type = CMD_BROWSE_DIRECT_OK; // 641
-    data = l_strdup_printf("%s", last); // <nick> <ip> <port>
+    data = g_strdup_printf("%s", last); // <nick> <ip> <port>
     irc_handle_command(net, data);
     net->c_type = CMD_ANNOUNCE;
-    return l_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_X_NAP_BROWSE_DIRECT);
+    return g_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_X_NAP_BROWSE_DIRECT);
   }
   if (!strcasecmp(ctcp_cmd, IRC_CTCP_X_NAP_BROWSE)) {
     // net->c_type = CMD_SERVER_BROWSE_RESPONSE; // 212
     net->c_type = CMD_BROWSE_DIRECT_OK; // 641 FIXME:::::
-    data = l_strdup_printf("%s", last); // <nick>
+    data = g_strdup_printf("%s", last); // <nick>
     irc_handle_command(net, data);
     net->c_type = CMD_ANNOUNCE;
-    return l_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_X_NAP_BROWSE);
+    return g_strdup_printf("%s CTCP %s", irc->nickname, IRC_CTCP_X_NAP_BROWSE);
   }
 
-  return l_strdup_printf("%s", irc->nickname);
+  return g_strdup_printf("%s", irc->nickname);
 
 }
 */
@@ -938,7 +938,7 @@ void irc_handle_command(net_t* net, char* data)
 {
   net->c_length = strlen(data);
   handle_command(net, data);
-  l_free(data);
+  g_free(data);
 }
 */
 //
@@ -954,7 +954,7 @@ char *irc_nap2irc(net_command_t * command, net_t* net)
 #ifdef HAVE_LIBLCONV_H
   if (!global.options.use_iconv) {
 #endif
-    lconv_buf = l_strdup(command->data);
+    lconv_buf = g_strdup(command->data);
 #ifdef HAVE_LIBLCONV_H
   } else {
     lconv_buf = lconv_strdup_conv(LCONV_SPEC, net->codeset, local_codeset, command->data);
@@ -967,7 +967,7 @@ char *irc_nap2irc(net_command_t * command, net_t* net)
     // Format: <user>
     net->queue = g_list_remove(net->queue, command);
     //    net->c_type = NAP_HOTLIST_ACK;
-    //    data = l_strdup(command->data);
+    //    data = g_strdup(command->data);
     //    irc_handle_command(net, data);
     return NULL;
   case CMD_DOWNLOAD_START: // 218
@@ -976,7 +976,7 @@ char *irc_nap2irc(net_command_t * command, net_t* net)
     net->queue = g_list_remove(net->queue, command);
     return NULL;
   case CMD_LOGIN: // server.c server_login()
-    msg = l_strdup_printf(
+    msg = g_strdup_printf(
             "%s %s\r\n%s %s\r\n%s %s %s %s :%s\r\n",
             IRC_PASS,
             net->user.password,  // PASS <password>
@@ -986,23 +986,23 @@ char *irc_nap2irc(net_command_t * command, net_t* net)
             net->user.username, "8", "*", net->user.username); // USER <user> <mode> <unused> <realname>
     break;
   case CMD_JOIN:
-    msg = l_strdup_printf("%s %s\r\n", IRC_JOIN, lconv_buf);
+    msg = g_strdup_printf("%s %s\r\n", IRC_JOIN, lconv_buf);
     break;
 
   // PRIVMSG
   case CMD_ANNOUNCE:       // 628 FIXME <text>
     // 4.7 Operwall message
-    msg = l_strdup_printf("%s %s\r\n", IRC_WALLOPS, lconv_buf);
+    msg = g_strdup_printf("%s %s\r\n", IRC_WALLOPS, lconv_buf);
     break;
   case CMD_PRIVMSG:        // 205   <nick> <message>
     last = lconv_buf;
     data = strtok_r(lconv_buf," ", &last);
-    msg = l_strdup_printf("%s %s :%s\r\n", IRC_PRIVMSG, data, last);
+    msg = g_strdup_printf("%s %s :%s\r\n", IRC_PRIVMSG, data, last);
     break;
   case CMD_NOTICE:
     last = lconv_buf;
     data = strtok_r(lconv_buf," ", &last);
-    msg = l_strdup_printf("%s %s :%s\r\n", IRC_NOTICE, data, last);
+    msg = g_strdup_printf("%s %s :%s\r\n", IRC_NOTICE, data, last);
     break;
   case CMD_EMOTE:          // 824   FIXME <channel> "<text>"
 
@@ -1010,10 +1010,10 @@ char *irc_nap2irc(net_command_t * command, net_t* net)
     channel = strtok_r(lconv_buf," ", &last);
     last++;
     last[strlen(last)-1] = 0;
-    msg = l_strdup_printf("%s %s :\001ACTION %s\001\r\n", IRC_PRIVMSG, channel, last);
+    msg = g_strdup_printf("%s %s :\001ACTION %s\001\r\n", IRC_PRIVMSG, channel, last);
 
     net->c_type = CIRC_CTCP;
-    data = l_strdup_printf("%s %s ACTION :%s", net->user.username,
+    data = g_strdup_printf("%s %s ACTION :%s", net->user.username,
 			   channel, last);
     irc_handle_command(net, data);
     break;
@@ -1023,28 +1023,28 @@ char *irc_nap2irc(net_command_t * command, net_t* net)
 
     last = lconv_buf;
     channel = strtok_r(lconv_buf," ", &last);
-    msg = l_strdup_printf("%s %s :%s\r\n", IRC_PRIVMSG, channel, last);
+    msg = g_strdup_printf("%s %s :%s\r\n", IRC_PRIVMSG, channel, last);
 
     net->c_type = CIRC_PUBLIC; // 403 c n t
     last = command->data;
     channel = strtok_r(command->data," ", &last);
-    data = l_strdup_printf("%s %s %s", channel, net->user.username, last);
+    data = g_strdup_printf("%s %s %s", channel, net->user.username, last);
     irc_handle_command(net, data);
     break;
 
   case CMD_SERVER_DISCONNECT:
-    msg = l_strdup_printf("%s Bye\r\n", IRC_QUIT);
+    msg = g_strdup_printf("%s Bye\r\n", IRC_QUIT);
   case CMD_FULL_CHANNEL_LIST:
-    msg = l_strdup_printf("%s\r\n", IRC_LIST);
+    msg = g_strdup_printf("%s\r\n", IRC_LIST);
     break;
   case CMD_NAMES_LIST:
-    msg = l_strdup_printf("%s %s\r\n", IRC_NAMES, lconv_buf);
+    msg = g_strdup_printf("%s %s\r\n", IRC_NAMES, lconv_buf);
     break;
 
   case CMD_SET_CHAN_MODE:  // 10209
   case CMD_SET_CHAN_LIMIT: // 826
     if ((msg = irc_mode_nap(command->type, lconv_buf)) == NULL) {
-      l_free(lconv_buf);
+      g_free(lconv_buf);
       return NULL;
     }
     break;
@@ -1053,22 +1053,22 @@ char *irc_nap2irc(net_command_t * command, net_t* net)
     return 0;
   case CMD_PART: // 401
     // PART
-    data = l_strdup(lconv_buf);
-    msg = l_strdup_printf("%s %s :See you.\r\n", IRC_PART, lconv_buf);
+    data = g_strdup(lconv_buf);
+    msg = g_strdup_printf("%s %s :See you.\r\n", IRC_PART, lconv_buf);
     break;
   case CMD_PONG:
-    msg = l_strdup_printf("%s %s\r\n", IRC_PONG, lconv_buf);
+    msg = g_strdup_printf("%s %s\r\n", IRC_PONG, lconv_buf);
     break;
   case CMD_CHAN_INVITE: // 10210
     last = lconv_buf;
     data = strtok_r(lconv_buf," ", &last);
-    msg = l_strdup_printf("%s %s %s\r\n", IRC_INVITE, last, data);
+    msg = g_strdup_printf("%s %s %s\r\n", IRC_INVITE, last, data);
     break;
   case CMD_KICK: // 829
-    msg = l_strdup_printf("%s %s\r\n", IRC_KICK, lconv_buf);
+    msg = g_strdup_printf("%s %s\r\n", IRC_KICK, lconv_buf);
     break;
   case CMD_KILL: // 610
-    msg = l_strdup_printf("%s %s\r\n", IRC_KILL, lconv_buf);
+    msg = g_strdup_printf("%s %s\r\n", IRC_KILL, lconv_buf);
     break;
   case CMD_SET_TOPIC: // 410
     // 410 <channel> <topic>
@@ -1076,43 +1076,43 @@ char *irc_nap2irc(net_command_t * command, net_t* net)
     last = lconv_buf;
     data = strtok_r(lconv_buf," ", &last);
     if (*last)
-      msg = l_strdup_printf("%s %s :%s\r\n", IRC_TOPIC, data, last);
+      msg = g_strdup_printf("%s %s :%s\r\n", IRC_TOPIC, data, last);
     else
-      msg = l_strdup_printf("%s %s\r\n", IRC_TOPIC, data);
+      msg = g_strdup_printf("%s %s\r\n", IRC_TOPIC, data);
     break;
   case CMD_WHOIS: // 603
-    msg = l_strdup_printf("%s %s\r\n", IRC_WHOIS, lconv_buf);
+    msg = g_strdup_printf("%s %s\r\n", IRC_WHOIS, lconv_buf);
     break;
   case CMD_WHICH_SERVER: // 10119
-    msg = l_strdup_printf("%s %s\r\n", IRC_WHO, lconv_buf);
+    msg = g_strdup_printf("%s %s\r\n", IRC_WHO, lconv_buf);
     break;
 
   // BAN
   case CMD_CHAN_BAN: // 422
     last = lconv_buf;
     channel = strtok_r(lconv_buf," ", &last);
-    msg = l_strdup_printf("MODE %s +b %s\r\n", channel, last);
+    msg = g_strdup_printf("MODE %s +b %s\r\n", channel, last);
     break;
   case CMD_CHAN_UNBAN: // 423
     last = lconv_buf;
     channel = strtok_r(lconv_buf," ", &last);
-    msg = l_strdup_printf("MODE %s -b %s\r\n", channel, last);
+    msg = g_strdup_printf("MODE %s -b %s\r\n", channel, last);
     break;
   // OP
   case CMD_OP: // 10204
     last = lconv_buf;
     channel = strtok_r(lconv_buf," ", &last);
-    msg = l_strdup_printf("MODE %s +o %s\r\n", channel, last);
+    msg = g_strdup_printf("MODE %s +o %s\r\n", channel, last);
     break;
   case CMD_DEOP: // 10205
     last = lconv_buf;
     channel = strtok_r(lconv_buf," ", &last);
-    msg = l_strdup_printf("MODE %s -o %s\r\n", channel, last);
+    msg = g_strdup_printf("MODE %s -o %s\r\n", channel, last);
     break;
 
   // Lopster Only. CTCP X-NAP-BROWSE
   case CMD_BROWSE: // 211
-    msg = l_strdup_printf("%s %s :%c%s %s%c",
+    msg = g_strdup_printf("%s %s :%c%s %s%c",
 			  IRC_PRIVMSG,
 			  lconv_buf, // <nick>
 			  ASCII_CTCP,
@@ -1123,7 +1123,7 @@ char *irc_nap2irc(net_command_t * command, net_t* net)
   // Lopster Only, CTCP X-NAP-BROWSE-DIRECT
   case CMD_BROWSE_DIRECT: // 640
     // Server: <nick> <ip> <port>
-    msg = l_strdup_printf("%s %s :%c%s %s %u %hu%c",
+    msg = g_strdup_printf("%s %s :%c%s %s %u %hu%c",
 			  IRC_PRIVMSG,
 			  lconv_buf, // <Dest nick>
 			  ASCII_CTCP,
@@ -1136,11 +1136,11 @@ char *irc_nap2irc(net_command_t * command, net_t* net)
 
   default:
     net->queue = g_list_remove(net->queue, command);
-    l_free(lconv_buf);
+    g_free(lconv_buf);
     return NULL;
   }
 
-  l_free(lconv_buf);
+  g_free(lconv_buf);
   return msg;
 }
 */
