@@ -205,7 +205,7 @@ HANDLER(irc_error) {
 
   if (!data) return;
 
-  server_message(net, data);
+  server_message(net, "%s", data);
   
   l_log(net, "Messages", LOG_MESSAGE, "%s\n", data);
   
@@ -220,7 +220,7 @@ HANDLER(irc_notice) {
 
   if (!sender_nick || strchr(sender_nick, '.')) {
     // we asumme its the server here.
-    server_message(net, iarg[1]);
+    server_message(net, "%s", iarg[1]);
   } else if (is_channel(iarg[0])) {
     // notice to channel
     public_notice(net, iarg[0], sender_nick, iarg[1]);
@@ -236,7 +236,7 @@ HANDLER(irc_login_ack) {
   if (get_iargs(data) != 2) return;
 
   network_success(net);
-  server_message(net, iarg[1]);
+  server_message(net, "%s", iarg[1]);
 }
 
 HANDLER(irc_motd) {
@@ -354,7 +354,7 @@ HANDLER(irc_message) {
   STRIP_TARGET(data);
   STRIP_COLON(data);
 
-  server_message(net, data);
+  server_message(net, "%s", data);
   l_log(net, "Messages", LOG_MESSAGE, "%s\n", data);
 }
 
@@ -418,14 +418,14 @@ HANDLER(irc_nick_changed) {
       if (!page->online) continue;
       if ((cu = search_user_in_list(GTK_CLIST(page->online),
 				    sender_nick, &row)) == NULL) continue;
-      l_free(cu->user);
-      cu->user = l_strdup(data);
+      g_free(cu->user);
+      cu->user = g_strdup(data);
       update_channel_user(page, cu);
     } else if (page->type == P_PRIVATE && !strcmp(page->name, sender_nick)) {
-      l_free(page->name);
-      l_free(page->vname);
-      page->name = l_strdup(data);
-      page->vname = l_strdup(data);
+      g_free(page->name);
+      g_free(page->vname);
+      page->name = g_strdup(data);
+      page->vname = g_strdup(data);
       chat_page_update_network(page, net);
     } else {
       continue;
@@ -574,7 +574,7 @@ static char* chmode2int_irc(chat_page_t* page, int change) {
 	break;
       case 'k':
 	arg_pos++;
-#warning FIXME: support channel keys
+	// FIXME: support channel keys
 	break;
       default:
 	for (i2 = 0; i2 < CMODE_NUMBER; i2++) {
@@ -670,8 +670,8 @@ HANDLER(irc_pong) {
     client_message("Message", "Pong from server <%s> [%d ms]",
 		   target, timestamp_difference(stamp));
     global.userstamp = g_list_remove(global.userstamp, stamp);
-    l_free(stamp->user);
-    l_free(stamp);
+    g_free(stamp->user);
+    g_free(stamp);
   }
 }
 
@@ -905,21 +905,21 @@ HANDLER(irc_ctcp) {
 
   if (!strcasecmp(command, "VERSION")) {
     if (INTERN_MICRO_VERSION)
-      buffer = l_strdup_printf
+      buffer = g_strdup_printf
 	("%s \001VERSION Lopster "VERSION".%d\001",
 	 user, INTERN_MICRO_VERSION);
     else
-      buffer = l_strdup_printf
+      buffer = g_strdup_printf
 	("%s \001VERSION Lopster "VERSION"\001", user);
     //    send_command(CMD_NOTICE, buffer, net);
-    l_free(buffer);
+    g_free(buffer);
   } else if (!strcasecmp(command, "ACTION")) {
     message = arg(NULL, 1);
     if (!message) message = "";
     if (*message == ':') message++;
-    buffer = l_strdup_printf("%s %s \"%s\"", channel, user, message);
+    buffer = g_strdup_printf("%s %s \"%s\"", channel, user, message);
     std_emote(net, buffer);
-    l_free(buffer);
+    g_free(buffer);
   } else {
     printf("*** irc_ctcp: not implemented %s\n", command);
   }
