@@ -1,4 +1,4 @@
-/* Copyright (C) 2000/2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -60,7 +60,7 @@ log_t *search_log(net_t* net, char *des, int type) {
 
   for (dlist = net?net->logs:other_logs; dlist; dlist = dlist->next) {
     log = dlist->data;
-    if (!l_strcasecmp(des, log->description) && (type == log->type)) {
+    if (!g_strcasecmp(des, log->description) && (type == log->type)) {
       if (!log->fd) log->fd = fopen(log->filename, "a+");
       if (global.options.log_expire &&
 	  (global.current_time - log->time > global.options.log_expire * 60 * 60)) {
@@ -74,7 +74,7 @@ log_t *search_log(net_t* net, char *des, int type) {
     }
   }
 
-  log = l_malloc(sizeof(log_t));
+  log = g_malloc(sizeof(log_t));
   sprintf(str, "%s/log/", global.options.config_dir);
   if (net) {
     strcat(str, net->name);
@@ -97,8 +97,8 @@ log_t *search_log(net_t* net, char *des, int type) {
     strcat(str, ".log");
   }
   log->fd = fopen(str, "a+");
-  log->filename = l_strdup(str);
-  log->description = l_strdup(des);
+  log->filename = g_strdup(str);
+  log->description = g_strdup(des);
   log->type = type;
   log->time = 0;
   if (net) net->logs = g_list_append(net->logs, log);
@@ -143,9 +143,9 @@ void close_logs(net_t* net) {
   for (dlist = net?net->logs:other_logs; dlist; dlist = dlist->next) {
     log = dlist->data;
     if (log->fd) fclose(log->fd);
-    l_free(log->filename);
-    l_free(log->description);
-    l_free(log);
+    g_free(log->filename);
+    g_free(log->description);
+    g_free(log);
   }
   g_list_free(net?net->logs:other_logs);
   net?net->logs:other_logs = NULL;
@@ -155,16 +155,16 @@ void log_file_destroy(log_file_t* lf) {
   GList* dlist2;
   log_session_t* ls;
 
-  l_free(lf->filename);
+  g_free(lf->filename);
   for (dlist2 = lf->sessions; dlist2; dlist2 = dlist2->next) {
     ls = dlist2->data;
-    l_free(ls->sname);
-    l_free(ls);
+    g_free(ls->sname);
+    g_free(ls);
   }
   g_list_free(lf->sessions);
   if (lf->fd) fclose(lf->fd);
   if (lf->status > 0) gtk_idle_remove(lf->status);
-  l_free(lf);
+  g_free(lf);
 }
 
 static void log_net_destroy(gpointer data) {
@@ -172,13 +172,13 @@ static void log_net_destroy(gpointer data) {
   log_net_t* ln = data;
   log_file_t* lf;
 
-  l_free(ln->dirname);
+  g_free(ln->dirname);
   for (dlist = ln->files; dlist; dlist = dlist->next) {
     lf = dlist->data;
     log_file_destroy(lf);
   }
   g_list_free(ln->files);
-  l_free(ln);
+  g_free(ln);
 }
 
 void log_show_all() {
@@ -202,11 +202,12 @@ void log_show_all() {
   }
 
   while ((entry = readdir(dir)) != NULL) {
-    if (entry->d_name[0] == '.') continue;
-    dname = l_strdup_printf("%s%c%s", name, DIR_SEP, entry->d_name);
+    if (!strcmp(entry->d_name, ".") ||
+	!strcmp(entry->d_name, "..")) continue;
+    dname = g_strdup_printf("%s%c%s", name, DIR_SEP, entry->d_name);
     stat(dname, &buf);
     if (buf.st_mode & S_IFDIR) {
-      ln = l_malloc(sizeof(*ln));
+      ln = g_malloc(sizeof(*ln));
       ln->dirname = dname;
       ln->files = NULL;
       
@@ -214,7 +215,7 @@ void log_show_all() {
       row = gtk_clist_append(clist, list);
       gtk_clist_set_row_data_full(clist, row, ln, log_net_destroy);
     } else {
-      l_free(dname);
+      g_free(dname);
     }
   }
 
@@ -240,7 +241,7 @@ text_compare(GtkCList * clist, gconstpointer ptr1, gconstpointer ptr2)
   if (!text1)
     return -1;
 
-  return l_strcasecmp(text1, text2);
+  return g_strcasecmp(text1, text2);
 }
 
 gint log_compare(GtkCList * clist, gconstpointer ptr1, gconstpointer ptr2)
@@ -257,7 +258,7 @@ gint log_compare(GtkCList * clist, gconstpointer ptr1, gconstpointer ptr2)
   if (!lf2) return 0;
 
   if (clist->sort_column == 0) {
-    return l_strcasecmp(lf1->filename, lf2->filename);
+    return g_strcasecmp(lf1->filename, lf2->filename);
   } else if (clist->sort_column == 1) {
     if (lf1->size < lf2->size) return -1;
     if (lf1->size > lf2->size) return 1;
@@ -323,13 +324,14 @@ void log_load_network(log_net_t* ln) {
 
   if ((dir = opendir(ln->dirname)) == NULL) return;
   while ((entry = readdir(dir)) != NULL) {
-    if (entry->d_name[0] == '.') continue;
-    dname = l_strdup_printf("%s%c%s", ln->dirname, 
+    if (!strcmp(entry->d_name, ".") ||
+	!strcmp(entry->d_name, "..")) continue;
+    dname = g_strdup_printf("%s%c%s", ln->dirname, 
 			    DIR_SEP, entry->d_name);
     stat(dname, &buf);
     
     if (buf.st_mode & S_IFREG) {
-      lf = l_malloc(sizeof(*lf));
+      lf = g_malloc(sizeof(*lf));
       lf->filename = dname;
       lf->size = buf.st_size;
       lf->mtime = buf.st_mtime;
@@ -339,7 +341,7 @@ void log_load_network(log_net_t* ln) {
       ln->files = g_list_append(ln->files, lf);
       lf->ln = ln;
     } else {
-      l_free(dname);
+      g_free(dname);
     }
   }
   closedir(dir);
@@ -396,9 +398,9 @@ gint log_updater(gpointer data) {
       in_session = 1;
       number = 0;
 
-      ls = l_malloc(sizeof(*ls));
+      ls = g_malloc(sizeof(*ls));
       ls->offset = off;
-      ls->sname = l_strdup(line+18);
+      ls->sname = g_strdup(line+18);
       ls->file = lf;
       lf->sessions = g_list_append(lf->sessions, ls);
       strcpy(tstr[0], line + 18);
@@ -587,7 +589,7 @@ static void log_file_close(log_file_t* lf) {
   if (found && flist) {
     *flist = g_list_remove(*flist, found);
     if (found->fd) fclose(found->fd);
-    l_free(found);
+    g_free(found);
   }
 }
 
@@ -616,11 +618,11 @@ on_log_delete(GtkMenuItem * menuitem ATTR_UNUSED,
       current_lf = NULL;
     }
     ln = lf->ln;
-    fname = l_strdup(lf->filename);
+    fname = g_strdup(lf->filename);
     ln->files = g_list_remove(ln->files, lf);
     log_file_destroy(lf);
     unlink(fname);
-    l_free(fname);
+    g_free(fname);
   }
 
   if (ln && !ln->files) {
@@ -662,11 +664,11 @@ log_delete_all(char* filename) {
 	  current_lf = NULL;
 	}
       }
-      fname = l_strdup(lf->filename);
+      fname = g_strdup(lf->filename);
       ln->files = g_list_remove(ln->files, lf);
       log_file_destroy(lf);
       unlink(fname);
-      l_free(fname);
+      g_free(fname);
       break;
     }
   }
