@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2003 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2004 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -16,6 +16,7 @@
 #include <stdio.h>
 
 #include "buffer.h"
+#include "mtypes.h"
 
 #ifdef HAVE_ZLIB
 #include <zlib.h>
@@ -94,11 +95,12 @@
 #define SERVER_TIMEOUT      500
 
 // server flags
-#define SERVER_REMOVE      0x01
-#define SERVER_PERMANENT   0x02
-#define SERVER_IGNORE      0x04
-#define SERVER_ONETIME     0x08
-#define SERVER_REDIRECT    0x10
+#define SERVER_MARKED      0x01
+#define SERVER_PERMANENT   0x02  // save
+#define SERVER_IGNORE      0x04  // save
+#define SERVER_NO_NAPI     0x08  // save
+#define SERVER_REDIRECT    0x10  // save
+#define SERVER_SAVE_MASK   (SERVER_PERMANENT|SERVER_IGNORE|SERVER_REDIRECT|SERVER_NO_NAPI)
 
 // server status
 #define SERVER_OFFLINE        0
@@ -108,15 +110,17 @@
 #define SERVER_ONLINE         4
 
 // network flags
-#define NETWORK_NO_WINMX   0x02
-#define NETWORK_MESSAGES   0x04
-#define NETWORK_IGN_GLOBAL 0x08
+#define NETWORK_NO_WINMX   0x02  // save
+#define NETWORK_MESSAGES   0x04  // save
+#define NETWORK_IGN_GLOBAL 0x08  // save
 #define NETWORK_CONNECTING 0x10
-#define NETWORK_DONT_SHARE 0x20
+#define NETWORK_DONT_SHARE 0x20  // save
+#define NETWORK_SAVE_MASK  (NETWORK_NO_WINMX|NETWORK_MESSAGES|NETWORK_IGN_GLOBAL|NETWORK_DONT_SHARE)
 
 // net group flags
 #define NG_STARTUP_CONNECT    0x2
 #define NG_NEVER_CONNECT      0x4
+#define NG_SAVE_MASK          (NG_STARTUP_CONNECT|NG_NEVER_CONNECT)
 
 // network types
 #define NETWORK_NAPSTER       1
@@ -155,18 +159,8 @@
 #define LIST_BANREASON      4
 #define LIST_NODOWNLOAD     5
 #define LIST_AFKREASON      6
-#define STRING_LIST_NO      7
-
-#define BROWSER_NONE        0
-#define BROWSER_NETSCAPE    1
-#define BROWSER_MOZILLA     2
-#define BROWSER_LYNX        3
-#define BROWSER_W3M         4
-#define BROWSER_OPERA       5
-#define BROWSER_DILLO       6
-#define BROWSER_GALEON      7
-#define BROWSER_KONQUEROR   8
-#define BROWSER_NO          9
+#define LIST_QSEARCH        7
+#define STRING_LIST_NO      8
 
 #define PIPE_AWAY_LOG    0x01
 #define PIPE_CHWALLOP    0x02
@@ -256,21 +250,10 @@
 #define STATUS_MUZZLED     5
 #define STATUS_UNKNOWN     6
 
-// major media types
-#define MIME_NONE          0
-#define MIME_MP3           1
-#define MIME_AUDIO         2
-#define MIME_VIDEO         3
-#define MIME_APPLICATION   4
-#define MIME_IMAGE         5
-#define MIME_TEXT          6
-#define MIME_SIZE          7
-
 // debugging output control
 //#define SHARE_DEBUG
 //#define RESUME_DEBUG
 //#define BROWSE_DEBUG
-//#define GLOBAL_DEBUG
 //#define SEARCH_DEBUG
 //#define TRANSFER_DEBUG
 //#define CONNECTION_DEBUG
@@ -282,7 +265,7 @@
 #define N_OPENNAP          1
 #define N_OPENNAP_NG       2
 #define N_SLAVANAP         3
-#define N_UNUSED           4
+#define N_CLEANNAP         4
 #define N_IRC              5
 #define N_NUMBER           6
 
@@ -298,6 +281,7 @@
 #ifdef ENABLE_WHITE_BOARD
  #define S_WHITEBOARD       8
 #endif
+//#define S_REMOTE           9
 
 // exit mode
 #define E_NONE             0
@@ -381,9 +365,11 @@
 typedef enum {
   L_LEECH,
   L_USER,
+  L_CHAT,
   L_MOD,
   L_ADMIN,
-  L_ELITE
+  L_ELITE,
+  L_MAX
 } user_level_t;
 
 typedef struct _geometry_t geometry_t;
@@ -391,10 +377,7 @@ typedef struct _word_info_t word_info_t;
 typedef struct _text_info_t text_info_t;
 typedef struct _chat_page_t chat_page_t;
 typedef struct _afk_t afk_t;
-typedef struct _clone_t clone_t;
 typedef struct _client_t client_t;
-typedef struct _suffix_t suffix_t;
-typedef struct _mime_t mime_t;
 typedef struct _user_t user_t;
 typedef struct _link_t link_t;
 typedef struct _network_t network_t;
@@ -404,7 +387,7 @@ typedef struct _status_t status_t;
 typedef struct _file_t file_t;
 typedef struct _pix_t pix_t;
 typedef struct _bandwidth_t bandwidth_t;
-typedef struct _napigator_t napigator_t;
+typedef struct _naplist_t naplist_t;
 typedef struct _global_t global_t;
 typedef struct _download_t download_t;
 typedef struct _upload_t upload_t;
@@ -425,7 +408,6 @@ typedef struct _band_t band_t;
 typedef struct _hot_t hot_t;
 typedef struct _whois_t whois_t;
 typedef struct _speed_t speed_t;
-typedef struct _file_tips_t file_tips_t;
 typedef struct _access_t access_t;
 typedef struct _browse_t browse_t;
 typedef struct _file_tree_t file_tree_t;
@@ -435,7 +417,7 @@ typedef struct _library_t library_t;
 typedef struct _shared_folder_t shared_folder_t;
 typedef struct _channel_entry_t channel_entry_t;
 typedef struct _server_net_t server_net_t;
-typedef struct _transfer_data_t transfer_data_t;
+typedef struct _transfer_t transfer_t;
 typedef struct _exec_t exec_t;
 typedef struct _file_segment_t file_segment_t;
 typedef struct _search_info_t search_info_t;
@@ -467,8 +449,8 @@ struct _file_tree_t {
   char *name;
   char *description;
   time_t last_refreshed;
-  double fsi[MIME_SIZE+1][3+1];
-  int fci[MIME_SIZE+1][3+1];
+  double fsi[MEDIA_SIZE+1][3+1];
+  int fci[MEDIA_SIZE+1][3+1];
   GtkWidget *main_link;
   GtkCTree *ctree;
   GtkCList *clist;
@@ -620,12 +602,6 @@ struct _afk_t {
   int warn_afk;
 };
 
-struct _clone_t
-{
-  GList *nicks;
-  char *ip;
-};
-
 struct _client_t
 {
   char *info;
@@ -633,19 +609,6 @@ struct _client_t
   int cnt;
 };
 
-struct _suffix_t
-{
-  char *suffix;
-  char *application;
-  int as_mp3;
-};
-
-struct _mime_t
-{
-  char *download;
-  GList *suffixes;
-};
-
 struct _user_t {
   char username[100];
   char password[100];
@@ -694,14 +657,17 @@ struct _option_t {
   unsigned char parse_colors;
   unsigned char strip_colors;
   unsigned char timestamps;
+  unsigned char switch_im;
+  unsigned char switch_browse;
   int dummy_md5;
+  int share_as_mp3;
 
+  char filetips;
   char colored_nicks;
   char colored_nicks2;
   char channel_rejoin;
   int allow_dcc;
   int check_lib_for_download;
-  int auto_search;
   char confirm_delete;
   int auto_resume;
   int ping_search;
@@ -713,22 +679,23 @@ struct _option_t {
   int auto_retry;
   int dl_autoremove;
   int ul_autoremove;
-  int resume_abort[4];
+  int resume_abort[3];
   unsigned int popup_create;
   unsigned int server_motd;
   int tab_style;
   int access_timeout;
+  char access_cleanup;
   int access_save;
   int check_version;
   int piping;
   int no_piping;
-  int resume_timeout;
+  int auto_search, resume_timeout;
   int access_transferring;
   int rejoin_when_kicked;
   int time_display;
   char *config_dir;
   char *config_file;
-  int browser;
+  char* browser;
   int retry_timeout[6];
   int upload_priority[4];
   //Weasel75
@@ -775,15 +742,6 @@ struct _file_t {
   unsigned char media_type;
 };
 
-struct _file_tips_t {
-  GtkWidget *widget;
-  GtkWidget *tips;
-  file_t *file;
-  int row;
-  int col;
-  int timeout;
-};
-
 struct _statistic_t {
   double incomplete[2];
   double total[2];
@@ -828,8 +786,14 @@ struct _pix_t {
   GdkBitmap *share_allb;
   GdkPixmap *server_con;
   GdkBitmap *server_conb;
-  GdkPixmap *server_ign;
-  GdkBitmap *server_ignb;
+  GdkPixmap *server_napi;
+  GdkBitmap *server_napib;
+  GdkPixmap *server_nnapi;
+  GdkBitmap *server_nnapib;
+  GdkPixmap *server_perm;
+  GdkBitmap *server_permb;
+  GdkPixmap *server_deact;
+  GdkBitmap *server_deactb;
 };
 
 struct _bandwidth_t {
@@ -843,11 +807,17 @@ struct _bandwidth_t {
   long bytes[2];
 };
 
-struct _napigator_t
-{
+#define NAPLIST_HOST "www.naplist.com"
+#define NAPLIST_DOC  "servers.txt"
+struct _naplist_t {
   int use_proxy;
-  char *proxy_url;
+  char *proxy_host;
   int proxy_port;
+  int auto_update;
+  int timer;
+  int auto_remove;
+  char* host;
+  char* document;
 };
 
 typedef struct {
@@ -886,7 +856,6 @@ struct _global_t {
 
   time_t current_time;
   socket_t *upload_socket;	// listen socket file transfer
-  mime_t mimetype[MIME_SIZE];
   status_t status;
   pix_t pix;			// pix for intern use
 
@@ -897,7 +866,6 @@ struct _global_t {
   GList *hotlist;		// users in hot list
   GList *incomplete;		// list of incomplete downloads
   GList *client_list;
-  GList *clones;
   GList *search_pattern;
   GList *searches;
   GList *chat_pages;
@@ -912,7 +880,6 @@ struct _global_t {
   int timer;
   afk_t afk;
   net_user_t user_reply;
-  chat_page_t* reply_page;
   char *session_start;
   time_t start_time;
   GList *server_links;
@@ -939,7 +906,7 @@ struct _global_t {
   limit_t limit;		// limit options
   option_t options;		// other options
   statistic_t statistic;
-  napigator_t napigator;	// napigator options
+  naplist_t naplist;	// naplist options
 
   char *rainbow_colors;
   GList *allowed_ports;
@@ -951,7 +918,6 @@ struct _global_t {
   GList *userstamp;
   GList *whois_requests;
   GList *speed_requests;
-  GList *userinfo;
   int reconnect_timer;
   int resume_timer;
   unsigned long my_ip;
@@ -992,7 +958,9 @@ struct _user_info_t {
   unsigned char client;
 };
 
-struct _transfer_data_t {
+#define TRANS(t) ((transfer_t*)(t))
+
+struct _transfer_t {
   user_info_t *user_info;
   double rate;
   int timeleft;
@@ -1009,20 +977,21 @@ struct _transfer_data_t {
 };
 
 struct _download_t {
-  transfer_data_t* data;
+  transfer_t data;
   resume_t *resume;
   int queue;
   GList* csegment;
   int fd;
   GList* nets;
   file_t* file;
+  GtkCTreeNode* node;
 #ifdef ADVANCED_CONNECTION
   char advanced;
 #endif
 };
 
 struct _share_t {
-  transfer_data_t* data;
+  transfer_t data;
   net_user_t nu;
   buffer_t* buffer;
 };
@@ -1037,7 +1006,7 @@ struct _browse_t {
 };
 
 struct _upload_t {
-  transfer_data_t* data;
+  transfer_t data;
   FILE* fd;
   file_segment_t* segment;
   net_user_t nu;
@@ -1133,6 +1102,7 @@ struct _server_t {
   int timeout;
   time_t last_resolved;
   net_t* net;
+  GtkCTreeNode* node;
 };
 
 struct _net_t {
@@ -1148,6 +1118,7 @@ struct _net_t {
   char* codeset;
   net_group_t* group;
 
+  buffer_t* temp_buffer;
   buffer_t* out_buffer;
   buffer_t* in_buffer;
   socket_t* socket;
@@ -1159,6 +1130,7 @@ struct _net_t {
   int cur_searches;   // currently active searches
   int max_searches;   // max concurrent searches allowed
   int skip_search_end;// true if we skip next search_end tag
+  int max_share;
 
   gint16 c_type;
   gint16 c_length;
@@ -1167,6 +1139,8 @@ struct _net_t {
   GList* temp_list;
   GList *logs;			// list of open logfiles
 
+  GtkCTreeNode* node;
+  
 #ifdef HAVE_ZLIB
   int compress;
 #endif
@@ -1310,6 +1284,8 @@ struct _net_group_t {
   int flags;
   int how_many;
   GList* nets;
+  GList* nets_connecting;
+  GtkCTreeNode* node;
 };
 
 #define FILE_NODE(obj) ((file_node_t*)obj)
@@ -1323,8 +1299,8 @@ struct _file_node_t {
   file_node_t* next;
   file_node_t* prev;
   file_t* file;
-  double fsi[MIME_SIZE+1][3+1];
-  int fci[MIME_SIZE+1][3+1];
+  double fsi[MEDIA_SIZE+1][3+1];
+  int fci[MEDIA_SIZE+1][3+1];
   int flags;
 };
 
