@@ -1,3 +1,7 @@
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
+   This is free software distributed under the terms of the
+   GNU Public License.  See the file COPYING for details. */
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -260,15 +264,13 @@ static void command_pack_nap(net_t* net, gint16 type, va_list ap) {
 
   buffer = net->out_buffer;
 
-  // FIXME: we assume that we dont need more space than 1024 bytes
-  if (buffer->datamax - buffer->datasize < 1024)
-    buffer_resize(buffer, 2048);
-  pos = buffer->data + buffer->datasize;
+  // FIXME: we assume that we dont need more space than 2048 bytes
+  pos = buffer_need_space(buffer, 2048);
 
   switch (type) {
   case CMD_RAW:
     str1  = va_arg(ap, char*);
-    str2 = l_strdup(str1);
+    str2 = g_strdup(str1);
     str3 = arg(str2, 0);
     if (!str3) return;
     str4 = arg(NULL, 1);
@@ -446,9 +448,9 @@ static void command_pack_nap(net_t* net, gint16 type, va_list ap) {
 			 " SIZE \"AT BEST\" \"%ld\"", long2);
       }
     }
-    if (int2 != MIME_MP3) {
+    if (int2 != MEDIA_MP3) {
       added += sprintf(pos+added,
-		       " TYPE %s", MimeNames(int2, 1));
+		       " TYPE %s", int2media(int2, 1));
     }
     // remove header from counter again
     added -= 4;
@@ -590,6 +592,19 @@ static void command_pack_nap(net_t* net, gint16 type, va_list ap) {
   
   pos[added+4] = 0;
 
+#ifdef HAVE_LIBLCONV_H
+  if (global.options.use_iconv) {
+    char* lconv_buf =
+      lconv_strdup_conv(LCONV_SPEC, global.options.dest_codeset, local_codeset, 
+			pos+4);
+    added = strlen(lconv_buf);
+    strncpy(pos+4, lconv_buf, added);
+    free(lconv_buf);
+  }
+#endif
+
+  pos[added+4] = 0;
+
 #ifdef PROTOCOL_DEBUG
   l_log(net, "protocol",
 	LOG_PROTOCOL, "S: %5d L: %4d [%s]\n", command,
@@ -607,91 +622,6 @@ static void command_pack_nap(net_t* net, gint16 type, va_list ap) {
   memcpy(pos+2, &command, 2);
 }
 
-
-/*
-gint send_command_real_nap(net_command_t* command, net_t* net) {
-  unsigned char *msg;
-  gint16 length;
-  gint16 length2;
-  gint16 type;
-  gint16 type2;
-  char* pos;
-  socket_t* socket;
-  int real_sent = 0;
-#ifdef HAVE_LIBLCONV_H
-  char lconv_buf[4097];
-#endif
-
-  if (!net) return -1;
-  if (!NET_NAP(net)) return -1;
-
-#ifdef HAVE_LIBLCONV_H
-  if (!global.options.use_iconv) {
-#endif
-    length = strlen(command->data);
-#ifdef HAVE_LIBLCONV_H
-  } else {
-    lconv_conv(LCONV_SPEC, global.options.dest_codeset, local_codeset, lconv_buf, command->data);
-    length = strlen(lconv_buf);
-    strncpy(command->data, lconv_buf, length);
-  }
-#endif
-  msg = l_malloc((length + 5) * sizeof(unsigned char));
-
-  type = napster_command_id[command->type];
-  length2 = BSWAP16(length);
-  type2 = BSWAP16(type);
-
-  memcpy(msg, &length2, 2);
-  memcpy(&msg[2], &type2, 2);
-  memcpy(&msg[4], command->data, length);
-  msg[length + 4] = 0;
-
-  socket = net->socket;
-
-#ifdef HAVE_ZLIB
-  if (net->compress) {
-    if (buffer_compress(socket->zcom, socket->zout, msg, length+4) == 0) {
-      if (send_safe(socket->fd, socket->zcom->data, socket->zcom->datasize,
-		    socket->zcom->datasize) < 0) {
-	printf("[to server] could not send command\n");
-	l_free(msg);
-	return -1;
-      }
-      real_sent = socket->zcom->datasize;
-      socket->zcom->datasize = 0;
-    } else {
-      l_free(msg);
-      printf("*** [ZLIB] [%s] compression error\n", net->name);
-      server_disconnect(net->active_server, "Compression error", 1);
-      return -1;
-    }
-  } else {
-#endif
-    if (send_safe(socket->fd, msg, length+4, length+4) < 0) {
-      printf("[to server] could not send command\n");
-      l_free(msg);
-      return -1;
-    }
-    real_sent = length+4;
-#ifdef HAVE_ZLIB
-  }
-#endif
-
-#ifdef PROTOCOL_DEBUG
-  l_log(net, "protocol",
-	LOG_PROTOCOL, "S: %5d L: %4d [%s]\n", type,
-	strlen(command->data), command->data);
-#endif
-
-  net->sent_compressed += real_sent;
-  net->sent_uncompressed += length+4;
-
-  l_free(msg);
-  return 0;
-}
-*/
-
 static int delete_0chars(char* message, int length) {
   int pos1, pos2;
 
@@ -728,7 +658,7 @@ static int consume_command_nap(net_t* net) {
     return -1;
   }
 
-  result = l_malloc(sizeof(char)*net->c_length+1);
+  result = g_malloc(sizeof(char)*net->c_length+1);
   memcpy(result, buffer->data+buffer->consumed+4, net->c_length);
   result[net->c_length] = 0;
 
