@@ -1,7 +1,11 @@
-/* Copyright (C) 2000-2003 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -82,7 +86,7 @@ static HANDLER_ENTRY Protocol_Napster[] = {
   {NAP_ALTER_PORT,         nap_set_port},   	   // 613
   {NAP_BANLIST,            NULL}, 	           // 615
   {NAP_IP_BANLIST,         nap_banlist_entry},	   // 616
-  {NAP_CHANNEL_LIST_END,   NULL},	           // 617
+  {NAP_CHANNEL_LIST_END,   nap_channel_list_end},  // 617
   {NAP_CHANNEL_LIST,       nap_channel_list},	   // 618
   {NAP_LIMIT,              nap_remote_queued},	   // 620
   {NAP_MOTD,               nap_motd},       	   // 621
@@ -161,7 +165,6 @@ HANDLER(nap_login_ack) {
   } else {
     net->compress = 0;
   }
-
 #endif
 
   if (!net->active_server) return;
@@ -224,7 +227,8 @@ HANDLER(nap_download_ack) {
   char *md5;
   int linespeed;
   net_user_t* nu;
-
+  transfer_t* trans;
+  
   username = arg(data, 0);
   ip_long = strtoul(arg(NULL, 0), NULL, 10);
   port = atoi(arg(NULL, 0));
@@ -242,38 +246,33 @@ HANDLER(nap_download_ack) {
     return;
   }
 
+  if (!download_real_allowed()) {
+    socket_end(socket, S_QUEUED);
+    return;
+  }
+
   if (socket->timer >= 0) {
     gtk_timeout_remove(socket->timer);
     socket->timer = -1;
   }
 
   download = socket->data;
+  trans = TRANS(download);
   nu = download->nets->data;
 
   // dont allow peer to start DCCs
-  if (download->data->is_dcc) return;
+  if (trans->is_dcc) return;
   
   if (!download_grab_resume(socket, download->resume)) {
     socket_end(socket, S_QUEUED);
     return;
   }
 
-  /* On Slavanap "download_ack" everythings ok, cause the peer
-   * has actually accepted the download.
-   * Opennap instead automatically returns the "download_ack"
-   * if the peer is firewalled. In this case the uploader never
-   * has seen our request. 
-   * So only return, if we are on Slavanap here */
-  if (net->subtype == N_SLAVANAP && !download_real_allowed()) {
-    socket_end(socket, S_QUEUED);
-    return;
-  }
-
   // set/update IP/port
-  socket->port    = ntohs(port);      // network->host
-  socket->ip_long = BSWAP32(ip_long); // little ->host
+  socket->port    = htons(port);      // host  ->net
+  socket->ip_long = BSWAP32(ip_long); // little->net
   // update linespeed
-  download->data->user_info->linespeed = linespeed;
+  trans->user_info->linespeed = linespeed;
 
   if (socket->port == 0) {
     if (!global.upload_socket) {
@@ -385,16 +384,16 @@ HANDLER(nap_browse_end) {
   browse = browse_search_user_net(nick, net);
   if (!browse) return;
 
-  if (ip) 
+  if (ip) {
     ip_long = BSWAP32(strtoul(ip, NULL, 10));
-  else
+    /*
+    if (!g_strncasecmp(nick, net->user.username))
+      global.my_ip = ip_long;
+    */
+  } else {
     ip_long = 0;
+  }
   browse_finish(browse, ip_long);
-  
-  /*
-  if (!l_strncasecmp(nick, net->user.username))
-    global.my_ip = ip_long;
-  */
 }
 
 HANDLER(nap_server_stats) {
@@ -507,34 +506,34 @@ int check_not_known1(net_t* net, char *data)
   if (!strncmp("User is not currently online", data, 28))
     return 1;
 
-  data2 = l_strdup(data);
+  data2 = g_strdup(data);
   text = arg(data2, 0);
 
   if (!text) {
-    l_free(data2);
+    g_free(data2);
     return 0;
   }
-  if (l_strcasecmp("user", text)) {
-    l_free(data2);
+  if (g_strcasecmp("user", text)) {
+    g_free(data2);
     return 0;
   }
   user = arg(NULL, 0);
   if (!user) {
-    l_free(data2);
+    g_free(data2);
     return 0;
   }
   text = arg(NULL, 1);
   if (!text) {
-    l_free(data2);
+    g_free(data2);
     return 0;
   }
   if (strncmp("is not a known user", text, 19)) {
-    l_free(data2);
+    g_free(data2);
     return 0;
   }
 
   whois_eval_not_online(net, user);
-  l_free(data2);
+  g_free(data2);
   return 1;
 }
 
@@ -543,44 +542,44 @@ int check_not_known2(net_t* net, char *data) {
   char *user;
   char *data2;
 
-  data2 = l_strdup(data);
+  data2 = g_strdup(data);
   text = arg(data2, 0);
   if (!text) {
-    l_free(data2);
+    g_free(data2);
     return 0;
   }
-  if (l_strcasecmp("information", text)) {
-    l_free(data2);
+  if (g_strcasecmp("information", text)) {
+    g_free(data2);
     return 0;
   }
 
   text = arg(NULL, 0);
   if (!text) {
-    l_free(data2);
+    g_free(data2);
     return 0;
   }
-  if (l_strcasecmp("on", text)) {
-    l_free(data2);
+  if (g_strcasecmp("on", text)) {
+    g_free(data2);
     return 0;
   }
 
   user = arg(NULL, 0);
   if (!user) {
-    l_free(data2);
+    g_free(data2);
     return 0;
   }
   text = arg(NULL, 1);
   if (!text) {
-    l_free(data2);
+    g_free(data2);
     return 0;
   }
   if (strncmp("is not available", text, 16)) {
-    l_free(data2);
+    g_free(data2);
     return 0;
   }
 
   whois_eval_not_online(net, user);
-  l_free(data2);
+  g_free(data2);
   return 1;
 }
 
@@ -605,38 +604,38 @@ void check_ping(net_t* net ATTR_UNUSED, char *data) {
   char *data2;
   user_timestamp_t *stamp;
 
-  data2 = l_strdup(data);
+  data2 = g_strdup(data);
   text = arg(data2, 0);
   if (!text) {
-    l_free(data2);
+    g_free(data2);
     return;
   }
-  if (l_strcasecmp("ping", text)) {
-    l_free(data2);
+  if (g_strcasecmp("ping", text)) {
+    g_free(data2);
     return;
   }
 
   text = arg(NULL, 0);
   if (!text) {
-    l_free(data2);
+    g_free(data2);
     return;
   }
-  if (l_strcasecmp("failed,", text)) {
-    l_free(data2);
+  if (g_strcasecmp("failed,", text)) {
+    g_free(data2);
     return;
   }
 
   user = arg(NULL, 0);
   if (!user) {
-    l_free(data2);
+    g_free(data2);
     return;
   }
   
   stamp = timestamp_search(global.userstamp, user);
   if (!stamp) return;
   global.userstamp = g_list_remove(global.userstamp, stamp);
-  l_free(stamp->user);
-  l_free(stamp);
+  g_free(stamp->user);
+  g_free(stamp);
   return;
 }
 
@@ -675,16 +674,17 @@ static int check_channel_kick(net_t* net, char* data) {
   char* user;
   chat_page_t* page;
 
+  if (!data) return 0;
   if (!strncmp("You were kicked from channel ", data, 29)) {
-    temp = l_strdup(data+29);
+    temp = g_strdup(data+29);
     channel = arg(temp, 0);
     who = arg(NULL, 0);   // by
     who = arg(NULL, 0);
     if (!who) {
-      l_free(temp);
+      g_free(temp);
       return 0;
     }
-    if (!l_strcasecmp(who, "server")) {
+    if (!g_strcasecmp(who, "server")) {
       who = arg(NULL, 0);
       who = NULL;
     }
@@ -696,49 +696,49 @@ static int check_channel_kick(net_t* net, char* data) {
     if (page)
       user_parted(net, page, net->user.username, 
 		  reason, -1, -1, 2, who);
-    l_free(temp);
+    g_free(temp);
     return 1;
   }
   if (strstr(data, "kicked") && strstr(data, " out of channel ")) {
-    temp = l_strdup(data);
+    temp = g_strdup(data);
     who = arg(temp, 0);
-    if (!l_strcasecmp(who, "server")) {
+    if (who && !g_strcasecmp(who, "server")) {
       who = arg(temp, 0);
       who = NULL;
     }
     user = arg(NULL, 0);   // kicked
     user = arg(NULL, 0);
     reason = arg(NULL, 1);
-    if (strncmp("out of channel ", reason, 15)) {
-      l_free(temp);
+    if (!reason || strncmp("out of channel ", reason, 15)) {
+      g_free(temp);
       return 0;
     }
     channel = arg(reason+15, 0);
     reason = arg(NULL, 1);
     if (!channel) {
-      l_free(temp);
+      g_free(temp);
       return 0;
     }
     channel[strlen(channel)-1] = 0;
     if (!strcmp(user, net->user.username)) {
-      l_free(temp);
+      g_free(temp);
       return 1;
     }
     page = chat_page_search(net, channel, P_PUBLIC, 1);
     if (page)
       user_parted(net, page, user, reason, -1, -1, 2, who);
-    l_free(temp);
+    g_free(temp);
     return 1;
   }
   if (strstr(data, " cleared channel ")) {
-    temp = l_strdup(data);
+    temp = g_strdup(data);
     who = arg(temp, 0);
     channel = arg(NULL, 0);   // cleared
     channel = arg(NULL, 0);   // channel
     channel = arg(NULL, 0);
     reason = arg(NULL, 1);
     if (!channel) {
-      l_free(temp);
+      g_free(temp);
       return 0;
     }
     channel[strlen(channel)-1] = 0;
@@ -760,7 +760,7 @@ static int check_channel_kick(net_t* net, char* data) {
       }
       chat_print_text(page, M_PUBLIC, "text", "\n");
     }
-    l_free(temp);
+    g_free(temp);
     return 1;
   }
   return 0;
@@ -775,10 +775,15 @@ static int check_level_set(net_t* net, char* data) {
   pos = strstr(data, " changed your user level to ");
   if (pos) {
     who = arg(data, 0);
-    pos = strchr(pos+28, '(');
-    if (!pos) return 1;
-
-    level = atoi(pos+1);
+    level = level2int(pos+28);
+    if (level < 0) {
+      pos = strchr(pos+28, '(');
+      if (!pos) return 1;
+      level = atoi(pos+1);
+      // need to add one level on slavanap (and maybe other non-opennaps)
+      // because there is no chatter
+      if (net->subtype != N_OPENNAP && level >= L_CHAT) level++;
+    }
     if (net->user.level != (unsigned)level)
       set_user_level(net, level);
     
@@ -796,10 +801,15 @@ static int check_level_set(net_t* net, char* data) {
   }
   pos = strstr(data, " set your user level to ");
   if (pos) {
-    pos = strchr(pos+24, '(');
-    if (!pos) return 1;
-
-    level = atoi(pos+1);
+    level = level2int(pos+24);
+    if (level < 0) {
+      pos = strchr(pos+24, '(');
+      if (!pos) return 1;
+      level = atoi(pos+1);
+      // need to add one level on slavanap (and maybe other non-opennaps)
+      // because there is no chatter
+      if (net->subtype != N_OPENNAP && level >= L_CHAT) level++;
+    }
     if (net->user.level != (unsigned)level)
       set_user_level(net, level);
     
@@ -821,15 +831,15 @@ static int chmode2int_nap(char* mode) {
   if (*mode == '+') mode++;
   else if (*mode == '-') mode++;
 
-  if (!l_strcasecmp(mode, "REGISTERED"))
+  if (!g_strcasecmp(mode, "REGISTERED"))
     return CMODE_REGISTERED;
-  else if (!l_strcasecmp(mode, "TOPIC"))
+  else if (!g_strcasecmp(mode, "TOPIC"))
     return CMODE_TOPIC;
-  else if (!l_strcasecmp(mode, "PRIVATE"))
+  else if (!g_strcasecmp(mode, "PRIVATE"))
     return CMODE_PRIVATE;
-  else if (!l_strcasecmp(mode, "INVITE"))
+  else if (!g_strcasecmp(mode, "INVITE"))
     return CMODE_INVITE;
-  else if (!l_strcasecmp(mode, "MODERATED"))
+  else if (!g_strcasecmp(mode, "MODERATED"))
     return CMODE_MODERATED;
   else return 0;
 }
@@ -843,17 +853,17 @@ static int check_channel_mode(net_t* net, char* data) {
 
   if (strncmp("mode for channel ", data, 17)) return 0;
 
-  temp = l_strdup(data+17);
+  temp = g_strdup(data+17);
   channel = arg(temp, 0);
   data = arg(NULL, 1);
   if (!channel) {
-    l_free(temp);
+    g_free(temp);
     return 0;
   }
 
   page = chat_page_search(net, channel, P_PUBLIC, 1);
   if (!page) {
-    l_free(temp);
+    g_free(temp);
     return 0;
   }
 
@@ -866,7 +876,7 @@ static int check_channel_mode(net_t* net, char* data) {
 
   channel_set_mode(page, NULL, chmode & CMODE_MASK_NAP,
 		   (~chmode) & CMODE_MASK_NAP, 0);
-  l_free(temp);
+  g_free(temp);
   return 1;
 }
 
@@ -881,7 +891,7 @@ static int check_channel_topic(net_t* net, char* data) {
   pos = strstr(data, " set topic on ");
   if (!pos) return 0;
 
-  temp = l_strdup(data);
+  temp = g_strdup(data);
   pos = strstr(temp, " set topic on ");
   *pos = 0;
 
@@ -890,7 +900,7 @@ static int check_channel_topic(net_t* net, char* data) {
 
   channel = arg(pos, 0);
   if (!channel) {
-    l_free(temp);
+    g_free(temp);
     return 0;
   }
   channel[strlen(channel)-1] = 0;
@@ -901,7 +911,7 @@ static int check_channel_topic(net_t* net, char* data) {
 
   channel_set_topic(page, who, topic, 0);
 
-  l_free(temp);
+  g_free(temp);
   return 1;
 }
 
@@ -918,7 +928,7 @@ static int check_channel_mode2(net_t* net, char* data) {
   pos = strstr(data, " changed mode on channel ");
   if (!pos) return 0;
 
-  temp = l_strdup(data);
+  temp = g_strdup(data);
   pos = strstr(temp, " changed mode on channel ");
   *pos = 0;
 
@@ -926,7 +936,7 @@ static int check_channel_mode2(net_t* net, char* data) {
   pos += 25;
   channel = arg(pos, 0);
   if (!channel) {
-    l_free(temp);
+    g_free(temp);
     return 0;
   }
   channel[strlen(channel)-1] = 0;
@@ -954,7 +964,7 @@ static int check_channel_mode2(net_t* net, char* data) {
 
   channel_set_mode(page, who, chmode1, chmode2, 0);
 
-  l_free(temp);
+  g_free(temp);
   return 1;
 }
 
@@ -991,32 +1001,32 @@ static int check_channel_limit_level(net_t* net, char* data) {
   if (net->subtype == N_OPENNAP ||
       net->subtype == N_OPENNAP_NG) {
     if (strncmp("Channel ", data, 8)) return 0;
-    temp = l_strdup(data+8);
+    temp = g_strdup(data+8);
     channel = arg(temp, 0);
     if (!channel) {
-      l_free(temp);
+      g_free(temp);
       return 0;
     }
     rest = arg(NULL, 1);
     if (!rest) {
-      l_free(temp);
+      g_free(temp);
       return 0;
     }
     if (!strncmp("has limit ", rest, 10)) {
       page = chat_page_search(net, channel, P_PUBLIC, 1);
       if (page)
 	channel_set_limit(page, NULL, atoi(rest+10), 0);
-      l_free(temp);
+      g_free(temp);
       return 1;
     }
     if (!strncmp("is level ", rest, 9)) {
       page = chat_page_search(net, channel, P_PUBLIC, 1);
       if (page)
 	channel_set_level(page, NULL, level2int(rest+9), 0);
-      l_free(temp);
+      g_free(temp);
       return 1;
     }
-    l_free(temp);
+    g_free(temp);
     return 1;
   }
   /*
@@ -1037,7 +1047,7 @@ int check_channel_limit(net_t* net, char* data) {
   pos1 = strstr(data, " set limit on channel ");
   if (!pos1) return 0;
 
-  temp = l_strdup(data);
+  temp = g_strdup(data);
   pos1 = strstr(temp, " set limit on channel ");
   *pos1 = 0;
 
@@ -1047,14 +1057,14 @@ int check_channel_limit(net_t* net, char* data) {
   pos1 = arg(NULL, 0);
   limit = arg(NULL, 0);
   if (!limit) {
-    l_free(temp);
+    g_free(temp);
     return 0;
   }
 
   page = chat_page_search(net, channel, P_PUBLIC, 1);
   if (!page) return 0;
   channel_set_limit(page, who, atoi(limit), 0);
-  l_free(temp);
+  g_free(temp);
   return 1;
 }
 
@@ -1071,7 +1081,7 @@ int check_channel_level(net_t* net, char* data) {
   pos2 = strstr(data, " to level ");
   if (!pos1 || !pos2) return 0;
 
-  temp = l_strdup(data);
+  temp = g_strdup(data);
   pos1 = strstr(temp, " set channel ");
   pos2 = strstr(temp, " to level ");
   *pos1 = 0;
@@ -1082,14 +1092,14 @@ int check_channel_level(net_t* net, char* data) {
   channel = arg(pos1, 0);
   level = arg(pos2, 0);
   if (!channel || !level) {
-    l_free(temp);
+    g_free(temp);
     return 0;
   }
   page = chat_page_search(net, channel, P_PUBLIC, 1);
   if (!page) return 0;
 
   channel_set_level(page, who, level2int(level), 0);
-  l_free(temp);
+  g_free(temp);
   return 1;
 }
 
@@ -1100,7 +1110,7 @@ int check_search_valid_words(net_t* net, char* data) {
   if (strcmp(data, "search failed: request contained no valid words")) 
     return 0;
 
-  server_message(net, data);
+  server_message(net, "%s", data);
 
   if (!net->active_searches) {
     server_message(net, "[Search]: No active search found!");
@@ -1128,6 +1138,17 @@ void check_pending_searches(net_t* net, char *data) {
 	       net->subtype == N_OPENNAP_NG) {
       // we do too many concurrent searches
       if (net->max_searches > 1) net->max_searches--;
+    } else if (net->max_searches > 1) { /* UNKNOWN server type below*/
+      // we have assumed opennap style (default on unknown servers)
+      net->max_searches--;
+    } else if (net->max_searches_pm > 1) {
+      // we had switched to slavanap style, now adjust the values
+      net->max_searches_pm--;
+      net->skip_search_end++;
+    } else if (net->max_searches_pm == 0) {
+      // we have opennap style, but it seems the wrong one,
+      // switch to slavanap style now.
+      net->max_searches_pm = 3;
     }
 
 #ifdef SEARCH_DEBUG
@@ -1146,8 +1167,17 @@ void check_pending_searches(net_t* net, char *data) {
   }
 }
 
+static void check_share_limit(net_t* net, char *data) {
+  if (!strncmp(data, "You may only share ", 19)) {
+    net->max_share = atoi(data+19);
+#ifdef SHARE_DEBUG
+    printf("[SHARE] max share limited to %d\n", net->max_share);
+#endif
+  }
+}
+
 int check_locate(net_t* net, char *data) {
-  char* data2 = l_strdup(data);
+  char* data2 = g_strdup(data);
   char* user;
   char* rest;
   chat_page_t* page;
@@ -1156,11 +1186,11 @@ int check_locate(net_t* net, char *data) {
   rest = arg(NULL, 1);
 
   if (!user || !rest) {
-    l_free(data2);
+    g_free(data2);
     return 0;
   }
   if (strncmp(rest, "is on ", 6)) {
-    l_free(data2);
+    g_free(data2);
     return 0;
   }
   
@@ -1176,7 +1206,7 @@ int check_locate(net_t* net, char *data) {
   chat_print_network(page, M_PUBLIC, net, 0, 1);
   chat_print_text(page, M_PUBLIC, "user", "]\n");
 
-  l_free(data2);
+  g_free(data2);
   return 1;
 }
 
@@ -1189,23 +1219,23 @@ static int check_server_pong(net_t* net, char *data) {
   if (strncmp("Pong from server", data, 16))
     return 0;
 
-  temp = l_strdup(data + 17);
+  temp = g_strdup(data + 17);
   serv = arg(temp, 0);
   ping = arg(NULL, 0);
   if (ping) ping++;
   else {
-    l_free(temp);
+    g_free(temp);
     return 1;
   }
 
   if ((link = link_search_rec(net->links, serv)) == NULL) {
-    l_free(temp);
+    g_free(temp);
     return 1;
   }
 
-  if (link->ping) l_free(link->ping);
-  link->ping = l_strdup(ping);
-  l_free(temp);
+  if (link->ping) g_free(link->ping);
+  link->ping = g_strdup(ping);
+  g_free(temp);
   return 1;
 }
 
@@ -1219,17 +1249,17 @@ static int check_op_voice_ban(net_t* net, char *data) {
   int set = 0;
 
   if (!data) return 0;
-  data2 = l_strdup(data);
+  data2 = g_strdup(data);
   who = arg(data2, 0);
   if (!who) {
-    l_free(data2);
+    g_free(data2);
     return 0;
   }
   if (!strcmp("Server", who)) who = arg(NULL, 0);
 
   pos1 = arg(NULL, 1);
   if (!pos1) {
-    l_free(data2);
+    g_free(data2);
     return 0;
   }
 
@@ -1266,7 +1296,7 @@ static int check_op_voice_ban(net_t* net, char *data) {
   if (flag != -1) {
     pos1 = arg(NULL, 1);
     if (!pos1) {
-      l_free(data2);
+      g_free(data2);
       return 0;
     }
 
@@ -1276,7 +1306,7 @@ static int check_op_voice_ban(net_t* net, char *data) {
       char* reason;
       channel = arg(pos1+11, 0);
       if (!channel) {
-	l_free(data2);
+	g_free(data2);
 	return 0;
       }
       if (channel[strlen(channel)-1] == ':') {
@@ -1288,10 +1318,10 @@ static int check_op_voice_ban(net_t* net, char *data) {
       page = chat_page_search(net, channel, P_PUBLIC, 3);
       if (page) 
 	user_flagged(page, who, nick, set, flag, reason);
-      l_free(data2);
+      g_free(data2);
       return 1;
     }
-    l_free(data2);
+    g_free(data2);
     return 0;
   }
 
@@ -1310,7 +1340,7 @@ static int check_op_voice_ban(net_t* net, char *data) {
   if (flag != -1) {
     pos1 = arg(NULL, 1);
     if (!pos1) {
-      l_free(data2);
+      g_free(data2);
       return 0;
     }
     // nick cant be NULL here, unless pos1 is NULL
@@ -1319,7 +1349,7 @@ static int check_op_voice_ban(net_t* net, char *data) {
       char* reason;
       channel = arg(pos1+5, 0);
       if (!channel) {
-	l_free(data2);
+	g_free(data2);
 	return 0;
       }
       channel[strlen(channel)-1] = 0;
@@ -1327,10 +1357,10 @@ static int check_op_voice_ban(net_t* net, char *data) {
       page = chat_page_search(net, channel, P_PUBLIC, 3);
       if (page) 
 	user_flagged(page, who, nick, set, flag, reason);
-      l_free(data2);
+      g_free(data2);
       return 1;
     }
-    l_free(data2);
+    g_free(data2);
     return 0;
   }
 
@@ -1342,19 +1372,22 @@ static int check_op_voice_ban(net_t* net, char *data) {
     page = chat_page_search(net, who+24, P_PUBLIC, 3);
     if (page)
       user_flagged(page, ".", net->user.username, 1, CU_OP, NULL);
-    l_free(data2);
+    g_free(data2);
     return 1;
   }
-  l_free(data2);
+  g_free(data2);
   return 0;
 }
 
 HANDLER(nap_error_message) {
+  if (!data) return;
+
   if (check_channel_wallop(net, data)) return;
   if (check_locate(net, data)) return;
   if (check_not_known1(net, data)) return;
   if (check_not_known2(net, data)) return;
   if (check_search_valid_words(net, data)) return;
+  check_share_limit(net, data);
   check_whois(net, data);
   check_server_join_quit(net, data);
   check_server_pong(net, data);
@@ -1372,7 +1405,7 @@ HANDLER(nap_error_message) {
   
   check_chat_page(net, data);
 
-  server_message(net, data);
+  server_message(net, "%s", data);
   
   l_log(net, "Messages", LOG_MESSAGE, "%s\n", data);
   
@@ -1517,11 +1550,11 @@ HANDLER(nap_channel_alt_topic)
 {
   (void)net;
 
-  client_message("Additional Topic", data);
+  client_message("Additional Topic", "%s", data);
 }
 
 HANDLER(nap_alternate_ack) {
-  upload_t *upload;
+  transfer_t *trans;
   socket_t *socket;
 
   char *username;
@@ -1530,7 +1563,7 @@ HANDLER(nap_alternate_ack) {
   char *winname;
   char *md5;
   char *linespeed;
-
+  
   username = arg(data, 0);
   ip = arg(NULL, 0);
   port = arg(NULL, 0);
@@ -1542,12 +1575,12 @@ HANDLER(nap_alternate_ack) {
   
   socket = upload_search_mapable(net, username, winname);
   if (!socket || !socket->data) return;
-
-  upload = socket->data;
-
-  if (upload->data->status != S_WAITING) return;
   
-  upload->data->user_info->linespeed = atoi(linespeed);
+  trans = TRANS(socket->data);
+
+  if (trans->status != S_WAITING) return;
+  
+  trans->user_info->linespeed = atoi(linespeed);
   socket->port = htons(atoi(port));
   socket->ip_long = BSWAP32(strtoul(ip, NULL, 10));
 
@@ -1588,7 +1621,6 @@ HANDLER(nap_eval_whowas)
 void upload_queue(upload_t* upload);
 
 HANDLER(nap_upload_request) {
-  upload_t *new_trans;
   socket_t *socket;
   char *username;
   char *winname;
@@ -1597,7 +1629,9 @@ HANDLER(nap_upload_request) {
   char* found;
   file_t* file;
   file_node_t* node;
-  
+  upload_t* upload;
+  transfer_t* trans = NULL;
+
   username = arg(data, 0);
   winname = arg(NULL, 0);
   speed = arg(NULL, 0);
@@ -1614,15 +1648,16 @@ HANDLER(nap_upload_request) {
 				    NULL, winname);
     if (!node || ((node->flags & LIB_SHARED) == 0)) return;
 
-    new_trans = upload_new(net, node->file->longname, username,
-			   node->flags&LIB_VIRTUAL);
-    file = new_trans->file;
+    upload = upload_new(net, node->file->longname, username,
+			node->flags&LIB_VIRTUAL);
+    trans = TRANS(upload);
+    file = upload->file;
     socket = socket_new(S_UPLOAD);
-    socket->data = new_trans;
+    socket->data = upload;
 
-    if (speed) new_trans->data->user_info->linespeed = atoi(speed);
+    if (speed) trans->user_info->linespeed = atoi(speed);
     
-    //    new_trans->size = file->size;
+    //    upload->size = file->size;
     upload_show(socket);
 
     sub = subscription_lookup_file(username, file->longname, &found);
@@ -1638,29 +1673,30 @@ HANDLER(nap_upload_request) {
       chat_print_file(page, file->longname, file->filename);
       chat_print_text(page, M_PUBLIC, "user", "\n");
 
-      found = l_strdup_printf("Automatic Notification: File [%s] was not subscribed by you, this request will be queued",
+      found = g_strdup_printf("Automatic Notification: File [%s] was not subscribed by you, this request will be queued",
 			      file->shortname);
       send_notice(net, username, found, 0);
-      l_free(found);
+      g_free(found);
       send_notice(net, username, 
 		  "To view your subscriptions, send me a private message: '!Subscription'", 0);
     }
     ext_handle(EVENT_UPLOAD_REQUEST, username, winname);
   } else {
-    new_trans = socket->data;
+    upload = socket->data;
+    trans = TRANS(upload);
     // updating network
-    new_trans->nu.net = net;
-    user_info_get(new_trans->data->user_info, net);
+    upload->nu.net = net;
+    user_info_get(trans->user_info, net);
   }
 
   /* only count now if access_transferring is not enabled */
   if (!global.options.access_transferring)
-    access_new_request(new_trans);
+    access_new_request(upload);
 
-  if (!upload_in_progress(new_trans)) {
-    if (new_trans->data->status == S_QUEUED) {
+  if (!transfer_in_progress(trans)) {
+    if (trans->status == S_QUEUED) {
       // immediately send queue
-      upload_queue(new_trans);
+      upload_queue(upload);
     } else {
       upload_status_set(socket, S_REQUESTED);
       check_uploads_again();
@@ -1681,9 +1717,9 @@ HANDLER(nap_set_port) {
   chat_print_text(page, M_PUBLIC, "user", " [");
   chat_print_network(page, M_PUBLIC, net, 0, 1);
   chat_print_text(page, M_PUBLIC, "user", "] ");
-  text = l_strdup_printf("has set your data port to %d\n", port);
+  text = g_strdup_printf("has set your data port to %d\n", port);
   chat_print_text(page, M_PUBLIC, "message", text);
-  l_free(text);
+  g_free(text);
 
   create_upload_port(port, FALSE);
 }
@@ -1695,7 +1731,7 @@ HANDLER(nap_channel_list) {
 
   name = arg(data, 0);
   online = arg(NULL, 0);
-  topic = arg(NULL, 0);
+  topic = arg(NULL, 1);
   
   if (!online) return;
   channel_entry_new(net, name, online, topic, NULL, NULL);
@@ -1725,7 +1761,7 @@ HANDLER(nap_remote_queued) {
 
   if (queue) download->queue = atoi(queue);
 
-  if (download->data->status == S_WAITING)
+  if (TRANS(download)->status == S_WAITING)
     socket_end(socket, S_REMOTE);
 
   return;
@@ -1774,14 +1810,17 @@ HANDLER(nap_motd) {
     
     if (net->subtype == N_OPENNAP ||
 	net->subtype == N_OPENNAP_NG) {
+      // fast search (opennap's the better one)
       net->max_searches_pm = 0;
       net->max_searches = 3;
     } else if (net->subtype == N_SLAVANAP) {
+      // slow search
       net->max_searches_pm = 3;
       net->max_searches = 1;
     } else {
-      net->max_searches = 1;
+      // first try the fast version
       net->max_searches_pm = 0;
+      net->max_searches = 1;
     }
     network_go(net);
   }
@@ -1923,13 +1962,13 @@ HANDLER(nap_browse_direct_err)
   socket = browse_search_socket(nick);
   if (!socket) return;
 
-  client_message("Browse", message);
+  client_message("Browse", "%s", message);
   socket_destroy(socket, 0);
 }
 
 HANDLER(nap_ghost) {
   (void)data;
-  server_message(net, "Someone else tries to login with your Nick");
+  server_message(net, "Someone else is trying to login with your Nick");
 }
 
 HANDLER(nap_sping) {
@@ -1948,8 +1987,8 @@ HANDLER(nap_sping) {
     client_message("Message", "Pong from server <%s> [%d ms]",
 		   serv, timestamp_difference(stamp));
     global.userstamp = g_list_remove(global.userstamp, stamp);
-    l_free(stamp->user);
-    l_free(stamp);
+    g_free(stamp->user);
+    g_free(stamp);
   }
 }
 
@@ -2023,8 +2062,8 @@ HANDLER(nap_pong) {
   chat_print_text(page, M_PUBLIC, "message", t);
 
   global.userstamp = g_list_remove(global.userstamp, stamp);
-  l_free(stamp->user);
-  l_free(stamp);
+  g_free(stamp->user);
+  g_free(stamp);
 }
 
 HANDLER(nap_emote) {
@@ -2083,22 +2122,11 @@ HANDLER(nap_global_user)
   char *nick;
   char *ip;
   GtkCList *clist;
-/*  clone_t *clone;*/
 
   nick = arg(data, 0);
   ip = arg(NULL, 0);
   (void)net;
-  /*
-  if ((clone = is_ip_in_clones(global.clones, ip)) != NULL)
-  {
-    clone->nicks = g_list_prepend(clone->nicks, l_strdup(nick));
-  } else {
-    clone = (clone_t *) l_malloc(sizeof(clone_t));
-    clone->ip = l_strdup(ip);
-    clone->nicks = g_list_prepend(NULL, l_strdup(nick));
-    global.clones = g_list_prepend(global.clones, clone);
-  }
-  */
+
   if (!global.user_win) return;
 
   clist = GTK_CLIST(lookup_widget(global.user_win, "clist12"));
@@ -2328,9 +2356,9 @@ HANDLER(nap_version_stats)
   if (*data) {
     info = arg(data, 0);
     number = arg(NULL, 0);
-    client = (client_t *) l_malloc(sizeof(client_t));
+    client = (client_t *) g_malloc(sizeof(client_t));
     client->cnt = 0;
-    client->info = l_strdup(info);
+    client->info = g_strdup(info);
     if (number)
       client->logins = atoi(number);
     else
@@ -2343,9 +2371,9 @@ HANDLER(nap_version_stats)
       short_info = info;
     client2 = is_client_in_list(global.client_list, short_info);
     if (!client2) {
-      client2 = (client_t *) l_malloc(sizeof(client_t));
+      client2 = (client_t *) g_malloc(sizeof(client_t));
       client2->cnt = 1;
-      client2->info = l_strdup(short_info);
+      client2->info = g_strdup(short_info);
       client2->logins = client->logins;
       global.client_list = g_list_append(global.client_list, client2);
     } else {
@@ -2398,10 +2426,9 @@ HANDLER(nap_version_stats)
   gtk_clist_thaw(clist);
 }
 
-HANDLER(nap_user_mode)
-{
+HANDLER(nap_user_mode) {
   (void)net;
-  client_message("Usermode", data);
+  client_message("Usermode", "%s", data);
 }
 
 // deprecated
@@ -2414,16 +2441,13 @@ HANDLER(nap_browse_new) {
 HANDLER(nap_redirect) {
   char* address;
   char* port;
-  char str[1024];
   server_t* server;
 
   if (net->active_server->status != SERVER_CONNECTING) {
-    sprintf(str, "[REDIRECT] %s (ignoring)", data);
-    server_message(net, str);
+    server_message(net, "[REDIRECT] %s (ignoring)", data);
     return;
   } else {
-    sprintf(str, "[REDIRECT] %s", data);
-    server_message(net, str);
+    server_message(net, "[REDIRECT] %s", data);
   }
 
   address = arg(data, 0);
@@ -2445,15 +2469,12 @@ HANDLER(nap_redirect) {
   }
 
   server_disconnect(net->active_server, "Redirect", 1);
-  server_connect(server);
+  if (!(server->flags & SERVER_IGNORE))
+    server_connect(server);
 }
 
-HANDLER(nap_cycle)
-{
-  char str[1024];
-  
-  sprintf(str, "Cycle [%s] (ignoring)", data);
-  server_message(net, str);
+HANDLER(nap_cycle) {
+  server_message(net, "Cycle [%s] (ignoring)", data);
 }
 
 HANDLER(nap_histogram) {
@@ -2479,10 +2500,10 @@ HANDLER(nap_histogram) {
   bytes = strtod(arg1, NULL);
 
   if (count > 0) {
-    arg1 = l_strdup_printf("%5d", id);
+    arg1 = g_strdup_printf("%5d", id);
     print_size(str, bytes);
     client_message(arg1, "%10lu %10s", count, str);
-    l_free(arg1);
+    g_free(arg1);
   }
 }
 
@@ -2510,9 +2531,9 @@ HANDLER(nap_histogram_end)
   bytes = strtod(arg1, NULL);
 
   print_size(str, bytes);
-  arg1 = l_strdup_printf("%5d", id);
+  arg1 = g_strdup_printf("%5d", id);
   client_message(arg1, "%10lu %10s", count, str);
-  l_free(arg1);
+  g_free(arg1);
 
   client_message("-ID--", "----calls- ----bytes-");
 
@@ -2554,10 +2575,10 @@ HANDLER(nap_shistogram) {
     return;
 
   if (count > 0) {
-    arg1 = l_strdup_printf("%5d", id);
+    arg1 = g_strdup_printf("%5d", id);
     print_size(str, bytes);
     client_message(arg1, "%10lu %10s %s", count, str, name);
-    l_free(arg1);
+    g_free(arg1);
   }
 }
 
