@@ -1,4 +1,4 @@
-/* Copyright (C) 2000/2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2004 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -37,6 +37,37 @@
 #include "log.h"
 #include "utils.h"
 #include "bannet.h"
+#include "mtypes.h"
+
+typedef struct {
+  GtkEntry* app_entry;
+  GtkEntry* label_entry;
+  GtkWidget* widget;   // the container
+  void* parent;
+} gapp_t;
+
+typedef struct {
+  GList* app;
+  GtkEntry* ext_entry;
+  GtkWidget* widget;   // the container
+  GtkWidget* widget2;  // the subcontainer
+  GtkWidget* vbox;     // container for apps
+  void* parent;
+} gsuffix_t;
+
+typedef struct {
+  GtkEntry* download_entry;
+  GList* app;
+  GList* suffix;
+  GtkWidget* vbox1;    // container for apps
+  GtkWidget* vbox2;    // container for suffixes
+  GtkWidget* widget2;  // the subcontainer
+} gtype_t;
+
+static gtype_t* Types[MEDIA_SIZE] = {
+  NULL, NULL, NULL, NULL, 
+  NULL, NULL, NULL, NULL
+};
 
 void check_uploads_again();
 
@@ -68,22 +99,425 @@ char* DownloadPage(int id) {
   }
 }
 
+static void on_toggle_button(GtkButton* button, GtkWidget* frame) {
+  GtkWidget* pix_open;
+  GtkWidget* pix_close;
+
+  pix_open = gtk_object_get_data(GTK_OBJECT(button), "pix_open");
+  pix_close = gtk_object_get_data(GTK_OBJECT(button), "pix_close");
+
+  if (GTK_WIDGET_VISIBLE(frame)) {
+    gtk_widget_show(pix_close);
+    gtk_widget_hide(pix_open);
+    gtk_widget_hide(frame);
+  } else {
+    gtk_widget_hide(pix_close);
+    gtk_widget_show(pix_open);
+    gtk_widget_show(frame);
+  }
+}
+
+static void on_remove_main_app(GtkButton* button, gapp_t* app) {
+  gtype_t* type = app->parent;
+
+  (void)button;
+  type->app = g_list_remove(type->app, app);
+  gtk_widget_destroy(app->widget);
+  g_free(app);
+}
+
+static void on_remove_sub_app(GtkButton* button, gapp_t* app) {
+  gsuffix_t* suf = app->parent;
+
+  (void)button;
+  suf->app = g_list_remove(suf->app, app);
+  gtk_widget_destroy(app->widget);
+  g_free(app);
+}
+
+static void on_remove_suffix(GtkButton* button, gsuffix_t* suf) {
+  gtype_t* type = suf->parent;
+  GList* dlist;
+
+  (void)button;
+  type->suffix = g_list_remove(type->suffix, suf);
+  for (dlist = suf->app; dlist; dlist = dlist->next) {
+    gapp_t* app = dlist->data;
+    g_free(app);
+  }
+  g_list_free(suf->app);
+
+  gtk_widget_destroy(suf->widget);
+  gtk_widget_destroy(suf->widget2);
+  g_free(suf);
+}
+
+static gapp_t* g_add_app(GtkWidget* vbox, app_t* app, int sub) {
+  GtkWidget* hbox;
+  GtkWidget* label;
+  GtkWidget* entry;
+  GtkWidget* button;
+  GtkWidget* pixmap;
+
+  gapp_t* gapp = g_malloc0(sizeof(*gapp));
+
+  hbox = gtk_hbox_new(FALSE, 5);
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+  gapp->widget = hbox;
+
+  label = gtk_label_new("command");
+  gtk_widget_show(label);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+  entry = gtk_entry_new();
+  gapp->app_entry = GTK_ENTRY(entry);
+  gtk_widget_set_usize (entry, 1, -2);
+  if (app && app->app) gtk_entry_set_text(GTK_ENTRY(entry), app->app);
+  gtk_widget_show(entry);
+  gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+	
+  label = gtk_label_new("label");
+  gtk_widget_show(label);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+  entry = gtk_entry_new();
+  gapp->label_entry = GTK_ENTRY(entry);
+  gtk_widget_set_usize (entry, 1, -2);
+  if (app && app->label) gtk_entry_set_text(GTK_ENTRY(entry), app->label);
+  gtk_widget_show(entry);
+  gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+  
+  button = gtk_button_new();
+  GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
+  pixmap = create_pixmap(vbox, "close.xpm");
+  gtk_widget_show(pixmap);
+  gtk_container_add(GTK_CONTAINER(button), pixmap);
+  gtk_widget_show(button);
+  gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+  
+  if (sub)
+    gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		       (GtkSignalFunc)on_remove_sub_app, gapp);
+  else
+    gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		       (GtkSignalFunc)on_remove_main_app, gapp);
+
+  return gapp;
+
+}
+
+static void on_add_sub_app(GtkButton* button, gsuffix_t* suf) {
+  GList* dlist;
+  gapp_t* app;
+  char* text;
+
+  (void)button;
+  for (dlist = suf->app; dlist; dlist = dlist->next) {
+    app = dlist->data;
+    text = gtk_entry_get_text(app->app_entry);
+    if (!text || *text == 0) return;
+  }
+  
+  gtk_widget_show(suf->widget2);
+
+  app = g_add_app(suf->vbox, NULL, 1);
+  suf->app = g_list_append(suf->app, app);
+  app->parent = suf;
+}
+
+static void on_add_main_app(GtkButton* button, gtype_t* type) {
+  GList* dlist;
+  gapp_t* app;
+  char* text;
+
+  (void)button;
+  if (!type->vbox1) return;
+
+  for (dlist = type->app; dlist; dlist = dlist->next) {
+    app = dlist->data;
+    text = gtk_entry_get_text(app->app_entry);
+    if (!text || *text == 0) return;
+  }
+  
+  gtk_widget_show(type->widget2);
+  app = g_add_app(type->vbox1, NULL, 0);
+  type->app = g_list_append(type->app, app);
+  app->parent = type;
+}
+
+static gsuffix_t* g_add_suffix(GtkWidget* vbox, suffix_t* suf) {
+  GtkWidget* hbox;
+  GtkWidget* button2;
+  GtkWidget* hbox2;
+  GtkWidget* frame;
+  GtkWidget* pixmap;
+  GtkWidget* label;
+  GtkWidget* entry;
+  GtkWidget* button;
+  GtkWidget* vbox3;
+
+  gsuffix_t* gsuffix = g_malloc0(sizeof(*gsuffix));
+
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+  gsuffix->widget = hbox;
+
+  button2 = gtk_button_new();
+  GTK_WIDGET_UNSET_FLAGS(button2, GTK_CAN_FOCUS);
+  gtk_widget_show(button2);
+  gtk_box_pack_start(GTK_BOX(hbox), button2, TRUE, TRUE, 0);
+  hbox2 = gtk_hbox_new(FALSE, 0);
+  gtk_widget_show(hbox2);
+  gtk_container_add(GTK_CONTAINER(button2), hbox2);
+
+  pixmap = create_pixmap(vbox, "arrowr.xpm");
+  gtk_widget_show(pixmap);
+  gtk_box_pack_start(GTK_BOX(hbox2), pixmap, FALSE, FALSE, 0);
+  gtk_object_set_data(GTK_OBJECT(button2), "pix_close", pixmap);
+      
+  pixmap = create_pixmap(vbox, "arrowd.xpm");
+  gtk_box_pack_start(GTK_BOX(hbox2), pixmap, FALSE, FALSE, 0);
+  gtk_object_set_data(GTK_OBJECT(button2), "pix_open", pixmap);
+  
+  label = gtk_label_new("SUFFIX");
+  gtk_widget_show(label);
+  gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
+  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+  gtk_misc_set_padding(GTK_MISC(label), 5, 0);
+  
+  entry = gtk_entry_new();
+  gtk_widget_set_usize (entry, 1, -2);
+  gtk_widget_show(entry);
+  gtk_box_pack_start(GTK_BOX(hbox2), entry, TRUE, TRUE, 0);
+  if (suf)
+    gtk_entry_set_text(GTK_ENTRY(entry), 
+		       make_string_from_list(suf->ext, " "));
+  gsuffix->ext_entry = GTK_ENTRY(entry);
+  
+  button = gtk_button_new_with_label("Add application");
+  GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
+  gtk_widget_show(button);
+  gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		     (GtkSignalFunc)on_add_sub_app, gsuffix);
+  
+  button = gtk_button_new();
+  GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
+  pixmap = create_pixmap(vbox, "close.xpm");
+  gtk_widget_show(pixmap);
+  gtk_container_add(GTK_CONTAINER(button), pixmap);
+  gtk_widget_show(button);
+  gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		     (GtkSignalFunc)on_remove_suffix, gsuffix);
+  
+  frame = gtk_frame_new(NULL);
+  gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
+  gtk_container_set_border_width(GTK_CONTAINER(frame), 0);
+  gtk_signal_connect(GTK_OBJECT(button2), "clicked",
+		     (GtkSignalFunc)on_toggle_button, frame);
+  gsuffix->widget2 = frame;
+  
+  vbox3 = gtk_vbox_new(FALSE, 0);
+  gtk_widget_show(vbox3);
+  gtk_container_add(GTK_CONTAINER(frame), vbox3);
+  gtk_container_set_border_width(GTK_CONTAINER(vbox3), 5);
+  gsuffix->vbox = vbox3;
+  
+  return gsuffix;
+}
+
+static void on_add_suffix(GtkButton* button, gtype_t* type) {
+  GList* dlist;
+  gsuffix_t* suf;
+  char* text;
+
+  (void)button;
+  for (dlist = type->suffix; dlist; dlist = dlist->next) {
+    suf = dlist->data;
+    text = gtk_entry_get_text(suf->ext_entry);
+    if (!text || *text == 0) return;
+  }
+
+  gtk_widget_show(type->widget2);
+
+  suf = g_add_suffix(type->vbox2, NULL);
+  type->suffix = g_list_append(type->suffix, suf);
+  suf->parent = type;
+}
+
+static void g_type_destroy(gtype_t* type) {
+  GList* dlist;
+
+  g_list_foreach(type->app, (GFunc)g_free, NULL);
+  g_list_free(type->app);
+  for (dlist = type->suffix; dlist; dlist = dlist->next) {
+    gsuffix_t* suf = dlist->data;
+    g_list_foreach(suf->app, (GFunc)g_free, NULL);
+    g_list_free(suf->app);
+    g_free(suf);
+  }
+  g_list_free(type->suffix);
+  g_free(type);
+}
+
+static void setup_media_types() {
+  GtkWidget* win;
+  GtkWidget* vbox;
+  GtkWidget* vbox2;
+  GtkWidget* vbox3;
+  GtkWidget* hbox;
+  GtkWidget* hbox2;
+  GtkWidget* button;
+  GtkWidget* button2;
+  GtkWidget* label;
+  GtkWidget* entry;
+  GtkWidget* frame;
+  GtkWidget* pixmap;
+  int i1;
+  char* str;
+  GList* dlist;
+  GList* dlist2;
+
+  if (!global.options_win) return;
+  else win = global.options_win;
+
+  vbox = lookup_widget(win, "vbox259");
+  for (i1 = 0; i1 < MEDIA_SIZE; i1++) {
+    mtype_t* mtype = mtype_find(i1);
+    if (Types[i1]) g_type_destroy(Types[i1]);
+    Types[i1] = g_malloc0(sizeof(gtype_t));
+
+    hbox = gtk_hbox_new(FALSE, 0);
+    gtk_widget_show(hbox);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+
+    button2 = gtk_button_new();
+    GTK_WIDGET_UNSET_FLAGS (button2, GTK_CAN_FOCUS);
+    gtk_widget_show(button2);
+    gtk_box_pack_start(GTK_BOX(hbox), button2, TRUE, TRUE, 0);
+    hbox2 = gtk_hbox_new(FALSE, 0);
+    gtk_widget_show(hbox2);
+    gtk_container_add(GTK_CONTAINER(button2), hbox2);
+
+    pixmap = create_pixmap(win, "arrowr.xpm");
+    gtk_widget_show(pixmap);
+    gtk_box_pack_start(GTK_BOX(hbox2), pixmap, FALSE, FALSE, 0);
+    gtk_object_set_data(GTK_OBJECT(button2), "pix_close", pixmap);
+    
+    pixmap = create_pixmap(win, "arrowd.xpm");
+    gtk_box_pack_start(GTK_BOX(hbox2), pixmap, FALSE, FALSE, 0);
+    gtk_object_set_data(GTK_OBJECT(button2), "pix_open", pixmap);
+
+    str = g_strdup_printf("MEDIA %s", int2media(i1, 2));
+    label = gtk_label_new(str);
+    g_free(str);
+    gtk_widget_show(label);
+    gtk_box_pack_start(GTK_BOX(hbox2), label, TRUE, TRUE, 0);
+    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+    gtk_misc_set_padding(GTK_MISC(label), 5, 0);
+
+    button = gtk_button_new_with_label("Add suffix");
+    GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
+    gtk_widget_show(button);
+    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		       (GtkSignalFunc)on_add_suffix, Types[i1]);
+
+    if (i1 == MEDIA_NONE || i1 == MEDIA_FOLDER)
+      gtk_widget_set_sensitive(button, FALSE);
+    button = gtk_button_new_with_label("Add application");
+    GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
+    gtk_widget_show(button);
+    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		       (GtkSignalFunc)on_add_main_app, Types[i1]);
+    
+    frame = gtk_frame_new(NULL);
+    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(frame), 0);
+    gtk_signal_connect(GTK_OBJECT(button2), "clicked",
+		       (GtkSignalFunc)on_toggle_button, frame);
+    Types[i1]->widget2 = frame;
+
+    vbox3 = gtk_vbox_new(FALSE, 0);
+    gtk_widget_show(vbox3);
+    gtk_container_add(GTK_CONTAINER(frame), vbox3);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox3), 5);
+
+    vbox2 = gtk_vbox_new(FALSE, 0);
+    gtk_widget_show(vbox2);
+    gtk_box_pack_start(GTK_BOX(vbox3), vbox2, FALSE, FALSE, 0);
+    Types[i1]->vbox1 = vbox2;
+      
+    Types[i1]->download_entry = NULL;
+    if (i1 != MEDIA_FOLDER) {
+      hbox = gtk_hbox_new(FALSE, 5);
+      gtk_widget_show(hbox);
+      gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);
+      label = gtk_label_new("Download folder");
+      gtk_widget_show(label);
+      gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+      entry = gtk_entry_new();
+      Types[i1]->download_entry = GTK_ENTRY(entry);
+    
+      if (mtype->download) 
+	gtk_entry_set_text(GTK_ENTRY(entry), mtype->download);
+      gtk_widget_show(entry);
+      gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+      button = gtk_button_new_with_label("Browse...");
+      GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
+      //    gtk_widget_show(button);
+      gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    }
+    
+    for (dlist = mtype->app; dlist; dlist = dlist->next) {
+      app_t* app = dlist->data;
+      gapp_t* gapp = g_add_app(vbox2, app, 0);
+
+      Types[i1]->app = g_list_append(Types[i1]->app, gapp);
+      gapp->parent = Types[i1];
+    }
+    
+    vbox2 = gtk_vbox_new(FALSE, 0);
+    gtk_widget_show(vbox2);
+    gtk_box_pack_start(GTK_BOX(vbox3), vbox2, FALSE, FALSE, 0);
+    Types[i1]->vbox2 = vbox2;
+    
+    for (dlist = mtype->suffix; dlist; dlist = dlist->next) {
+      suffix_t* suffix = dlist->data;
+      gsuffix_t* gsuffix =  g_add_suffix(vbox2, suffix);
+
+      Types[i1]->suffix = g_list_append(Types[i1]->suffix, gsuffix);
+      gsuffix->parent = Types[i1];
+      
+      for (dlist2 = suffix->app; dlist2; dlist2 = dlist2->next) {
+	app_t* app = dlist2->data;
+	gapp_t* gapp = g_add_app(gsuffix->vbox, app, 1);
+
+	gsuffix->app = g_list_append(gsuffix->app, gapp);
+	gapp->parent = gsuffix;
+      }
+    }
+  }
+}
+
 void setup_preferences(unsigned long flags) {
   GtkWidget *win;
   GtkWidget *temp;
   GList *dlist;
   char *pos;
 
-  if (!global.options_win)
-    return;
-  else
-    win = global.options_win;
+  if (!global.options_win) return;
+  else win = global.options_win;
 
   if (flags & P_TREE) {
     GtkCTreeNode *node;
     GtkCTreeNode *node2;
     GtkCTree *ctree;
 
+    setup_media_types();
+
     ctree = GTK_CTREE(lookup_widget(win, "ctree2"));
     gtk_ctree_set_line_style(ctree, GTK_CTREE_LINES_NONE);
     gtk_ctree_set_expander_style(ctree, GTK_CTREE_EXPANDER_TRIANGLE);
@@ -99,46 +533,38 @@ void setup_preferences(unsigned long flags) {
     node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 15,
 				  NULL, NULL, NULL, NULL, TRUE, FALSE);
     gtk_ctree_node_set_row_data(ctree, node2, (void *) 3);
-    strcpy(tstr[0], "Mimetypes");
+    strcpy(tstr[0], "Mediatypes");
     node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 15,
 				  NULL, NULL, NULL, NULL, TRUE, FALSE);
     gtk_ctree_node_set_row_data(ctree, node2, (void *) 4);
-    strcpy(tstr[0], "Paths");
-    node = gtk_ctree_insert_node(ctree, NULL, NULL, list, 15,
-				 NULL, NULL, NULL, NULL, FALSE, TRUE);
-    gtk_ctree_node_set_row_data(ctree, node, (void *) 5);
-    strcpy(tstr[0], "Download");
+    strcpy(tstr[0], "Share/Paths");
     node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 15,
 				  NULL, NULL, NULL, NULL, TRUE, FALSE);
-    gtk_ctree_node_set_row_data(ctree, node2, (void *) 6);
-    strcpy(tstr[0], "Shares");
-    node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 15,
-				  NULL, NULL, NULL, NULL, TRUE, FALSE);
-    gtk_ctree_node_set_row_data(ctree, node2, (void *) 7);
+    gtk_ctree_node_set_row_data(ctree, node2, (void *) 5);
     strcpy(tstr[0], "Appearance");
     node = gtk_ctree_insert_node(ctree, NULL, NULL, list, 15,
 				 NULL, NULL, NULL, NULL, FALSE, TRUE);
-    gtk_ctree_node_set_row_data(ctree, node, (void *) 8);
+    gtk_ctree_node_set_row_data(ctree, node, (void *) 6);
     strcpy(tstr[0], "Chat");
     node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 15,
 				  NULL, NULL, NULL, NULL, TRUE, FALSE);
-    gtk_ctree_node_set_row_data(ctree, node2, (void *) 9);
+    gtk_ctree_node_set_row_data(ctree, node2, (void *) 7);
     strcpy(tstr[0], "Transfers");
     node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 15,
 				  NULL, NULL, NULL, NULL, TRUE, FALSE);
-    gtk_ctree_node_set_row_data(ctree, node2, (void *) 10);
+    gtk_ctree_node_set_row_data(ctree, node2, (void *) 8);
     strcpy(tstr[0], "Schemes");
     node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 15,
 				  NULL, NULL, NULL, NULL, TRUE, FALSE);
-    gtk_ctree_node_set_row_data(ctree, node2, (void *) 11);
+    gtk_ctree_node_set_row_data(ctree, node2, (void *) 9);
     strcpy(tstr[0], "Advanced");
     node = gtk_ctree_insert_node(ctree, NULL, NULL, list, 15,
 				 NULL, NULL, NULL, NULL, FALSE, TRUE);
-    gtk_ctree_node_set_row_data(ctree, node, (void *) 12);
+    gtk_ctree_node_set_row_data(ctree, node, (void *) 10);
     strcpy(tstr[0], "Netbans/Events");
     node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 15,
 				  NULL, NULL, NULL, NULL, TRUE, FALSE);
-    gtk_ctree_node_set_row_data(ctree, node2, (void *) 13);
+    gtk_ctree_node_set_row_data(ctree, node2, (void *) 11);
 
     temp = lookup_widget(win, "frame267");
     gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
@@ -148,10 +574,16 @@ void setup_preferences(unsigned long flags) {
     gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
     temp = lookup_widget(win, "frame37");
     gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
+    temp = lookup_widget(win, "frame1421");
+    gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
     temp = lookup_widget(win, "frame39");
     gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
     temp = lookup_widget(win, "frame398");
     gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
+    temp = lookup_widget(win, "frame1418");
+    gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
+    temp = lookup_widget(win, "frame1424");
+    gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
     temp = lookup_widget(win, "frame11");
     gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
     temp = lookup_widget(win, "frame155");
@@ -166,16 +598,8 @@ void setup_preferences(unsigned long flags) {
     gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
     temp = lookup_widget(win, "frame184");
     gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
-    temp = lookup_widget(win, "label506");
-    gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
     temp = lookup_widget(win, "label1659");
     gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
-    temp = lookup_widget(win, "frame187");
-    gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
-    temp = lookup_widget(win, "frame188");
-    gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
-    temp = lookup_widget(win, "label513");
-    gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
     temp = lookup_widget(win, "frame240");
     gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
     temp = lookup_widget(win, "label780");
@@ -203,6 +627,7 @@ void setup_preferences(unsigned long flags) {
     temp = lookup_widget(win, "frame397");
     gtk_widget_set_style(temp, global.styles[STYLE_LABEL]);
   }
+
   if (flags & P_GENERAL) {
     temp = lookup_widget(win, "entry4");
     gtk_entry_set_text(GTK_ENTRY(temp), global.username);
@@ -227,10 +652,10 @@ void setup_preferences(unsigned long flags) {
     temp = lookup_widget(win, "entry83");
     pos = create_string_from_ports(global.allowed_ports);
     if (pos) gtk_entry_set_text(GTK_ENTRY(temp), pos);
-    l_free(pos);
+    g_free(pos);
 
     // Weasel75
-    pos = l_malloc(100);
+    pos = g_malloc(100);
     if (pos) {
       temp = lookup_widget(win, "entry_speedred");
       print_bytes(pos, global.options.down_speed[0]);
@@ -256,8 +681,54 @@ void setup_preferences(unsigned long flags) {
       print_bytes(pos, global.options.up_speed[2]);
       gtk_entry_set_text(GTK_ENTRY(temp), pos);
       
-      l_free(pos);
+      g_free(pos);
     }
+
+    
+    temp = lookup_widget(win, "entry163");
+    gtk_entry_set_text(GTK_ENTRY(temp), 
+		       global.naplist.host?global.naplist.host:"");
+    temp = lookup_widget(win, "entry164");
+    gtk_entry_set_text(GTK_ENTRY(temp), 
+		       global.naplist.document?
+		       global.naplist.document:"");
+    temp = lookup_widget(win, "checkbutton96");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp),
+				 global.options.filetips);
+    temp = lookup_widget(win, "checkbutton92");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), 
+				 (global.naplist.auto_update > 0));
+    temp = lookup_widget(win, "spinbutton81");
+    if (global.naplist.auto_update >= 0) {
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp),
+				global.naplist.auto_update);
+    } else {
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp),
+				-global.naplist.auto_update);
+    }
+
+    temp = lookup_widget(win, "checkbutton97");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp),
+				 global.options.switch_im);
+    temp = lookup_widget(win, "checkbutton98");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp),
+				 global.options.switch_browse);
+
+    temp = lookup_widget(win, "checkbutton93");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), 
+				 global.naplist.use_proxy);
+
+    temp = lookup_widget(win, "entry159");
+    gtk_entry_set_text(GTK_ENTRY(temp), 
+		       global.naplist.proxy_host?
+		       global.naplist.proxy_host:"");
+    temp = lookup_widget(win, "spinbutton82");
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp),
+			      global.naplist.proxy_port);
+
+    temp = lookup_widget(win, "checkbutton95");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), 
+				 global.naplist.auto_remove);
   }
 
   if (flags & P_RSEARCH) {
@@ -327,27 +798,10 @@ void setup_preferences(unsigned long flags) {
     temp = lookup_widget(win, "spinbutton72");
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp),
 			      (gfloat) (global.limit.large_size));
-  }
 
-  if (flags & P_MIMETYPES) {
-    int i1;
-    suffix_t *suffix;
-
-    temp = lookup_widget(win, "clist17");
-    for (i1 = 1; i1 < MIME_SIZE; i1++) {
-      for (dlist = global.mimetype[i1].suffixes; dlist;
-	   dlist = dlist->next) {
-	suffix = dlist->data;
-	strcpy(tstr[0], suffix->suffix);
-	strcpy(tstr[1], MimeNames(i1, 0));
-	strcpy(tstr[2], suffix->application);
-	if (suffix->as_mp3)
-	  strcpy(tstr[3], "Yes");
-	else
-	  strcpy(tstr[3], "No");
-	gtk_clist_append(GTK_CLIST(temp), list);
-      }
-    }
+    // now the media types
+    //    temp = lookup_widget(win, "ctree13");
+    //    gui_mtype_fill_tree(GTK_CTREE(temp));
   }
 
   if (flags & P_PATHS) {
@@ -358,19 +812,6 @@ void setup_preferences(unsigned long flags) {
       gtk_entry_set_text(GTK_ENTRY(temp), "");
   }
 
-  if (flags & P_DOWNLOAD) {
-    int i1;
-    temp = lookup_widget(win, "clist18");
-    for (i1 = 0; i1 < MIME_SIZE; i1++) {
-      strcpy(tstr[0], MimeNames(i1, 0));
-      if (global.mimetype[i1].download)
-	strcpy(tstr[1], global.mimetype[i1].download);
-      else
-	strcpy(tstr[1], "");
-      gtk_clist_append(GTK_CLIST(temp), list);
-    }
-  }
-
   if (flags & P_SHARES) {
     shared_folder_t* folder;
     int row;
@@ -380,7 +821,7 @@ void setup_preferences(unsigned long flags) {
     if (global.lib) {
       for (dlist = global.lib->shared_folder; dlist; dlist = dlist->next) {
 	folder = dlist->data;
-	strcpy(tstr[0], MimeNames(folder->type, 1));
+	strcpy(tstr[0], int2media(folder->type, 1));
 	strcpy(tstr[1], folder->name);
 	row = gtk_clist_append(GTK_CLIST(temp), list);
       }
@@ -531,9 +972,28 @@ void setup_preferences(unsigned long flags) {
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp),
 			      global.options.log_expire);
 
-    temp = lookup_widget(win, "combo_entry27");
-    gtk_entry_set_text(GTK_ENTRY(temp), WebBrowser(global.options.browser));
+    temp = lookup_widget(win, "entry162");
+    gtk_entry_set_text(GTK_ENTRY(temp), 
+		       global.options.browser?global.options.browser:"");
 
+#ifdef HAVE_LIBLCONV_H
+    temp = lookup_widget(win, "frame1418");
+    gtk_widget_set_sensitive(temp, TRUE);
+    temp = lookup_widget(win, "checkbutton91");
+    if (global.options.use_iconv)
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), TRUE);
+    else
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), FALSE);
+    temp = lookup_widget(win, "entry158");
+    gtk_entry_set_text(GTK_ENTRY(temp), global.options.dest_codeset);
+#else
+    temp = lookup_widget(win, "frame1418");
+    gtk_widget_set_sensitive(temp, FALSE);
+    temp = lookup_widget(win, "checkbutton91");
+    gtk_widget_set_sensitive(temp, FALSE);
+    temp = lookup_widget(win, "entry158");
+    gtk_widget_set_sensitive(temp, FALSE);
+#endif
     temp = lookup_widget(win, "spinbutton73");
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp),
 			      (gfloat) global.network.transfer_delay);
@@ -601,7 +1061,7 @@ void setup_preferences(unsigned long flags) {
 
     temp = lookup_widget(win, "clist41");
     for (dlist = global.bannet; dlist; dlist = dlist->next) {
-      bn = l_malloc(sizeof(*bn));
+      bn = g_malloc(sizeof(*bn));
       memcpy(bn, dlist->data, sizeof(*bn));
       sprintf(tstr[0], "%d.%d.%d.%d", 
 	      (int)((bn->ip_from>>24)&0xff),
@@ -615,7 +1075,7 @@ void setup_preferences(unsigned long flags) {
 	      (int)((bn->ip_to    )&0xff));
       row = gtk_clist_append(GTK_CLIST(temp), list);
       gtk_clist_set_row_data_full(GTK_CLIST(temp), row, bn, 
-				  l_free);
+				  g_free);
     }
 
     if (global.external_handler) {
@@ -633,26 +1093,6 @@ void setup_preferences(unsigned long flags) {
   }
 }
 
-void clear_suffixes()
-{
-  int i1;
-  GList *dlist;
-  suffix_t *suffix;
-
-  for (i1 = 0; i1 < MIME_SIZE; i1++) {
-    if (!global.mimetype[i1].suffixes)
-      continue;
-    for (dlist = global.mimetype[i1].suffixes; dlist; dlist = dlist->next) {
-      suffix = (suffix_t *) (dlist->data);
-      l_free(suffix->suffix);
-      l_free(suffix->application);
-      l_free(suffix);
-    }
-    g_list_free(global.mimetype[i1].suffixes);
-    global.mimetype[i1].suffixes = NULL;
-  }
-}
-
 int page2int(char* class) {
   if (!strcmp(class, "Active Page"))
     return 0;
@@ -677,6 +1117,68 @@ int list2int(char* class) {
   return 3;
 }
 
+static void apply_media_types() {
+  GtkWidget* win;
+  int i1;
+  char* text;
+  char* text2;
+  app_t* app;
+  GList* dlist;
+  GList* dlist2;
+  suffix_t* suf;
+  
+  if (!global.options_win) return;
+  else win = global.options_win;
+  
+  mtype_clear_all();
+  
+  for (i1 = 0; i1 < MEDIA_SIZE; i1++) {
+    gtype_t* type = Types[i1];
+    mtype_t* mtype = mtype_find(i1);
+
+    if (type->download_entry) {
+      text = gtk_entry_get_text(type->download_entry);
+      if (text && *text)
+	mtype->download = g_strdup(text);
+    }
+    
+    for (dlist = type->app; dlist; dlist = dlist->next) {
+      gapp_t* gapp = dlist->data;
+      text = gtk_entry_get_text(gapp->app_entry);
+      text2 = gtk_entry_get_text(gapp->label_entry);
+      if (!text || *text == 0) continue;
+      
+      app = g_malloc0(sizeof(*app));
+      app->app = g_strdup(text);
+      if (text2 && *text2) app->label = g_strdup(text2);
+      mtype->app = g_list_append(mtype->app, app);
+    }
+    for (dlist = type->suffix; dlist; dlist = dlist->next) {
+      gsuffix_t* gsuf = dlist->data;
+      text = gtk_entry_get_text(gsuf->ext_entry);
+      if (!text || *text == 0) continue;
+      
+      suf = g_malloc0(sizeof(*suf));
+      make_list_from_string(&(suf->ext), text, " ");
+      mtype->suffix = g_list_append(mtype->suffix, suf);
+
+      for (dlist2 = gsuf->app; dlist2; dlist2 = dlist2->next) {
+	gapp_t* gapp = dlist2->data;
+	text = gtk_entry_get_text(gapp->app_entry);
+	text2 = gtk_entry_get_text(gapp->label_entry);
+	if (!text || *text == 0) continue;
+
+	app = g_malloc0(sizeof(*app));
+	app->app = g_strdup(text);
+	if (text2 && *text2)
+	  app->label = g_strdup(text2);
+	suf->app = g_list_append(suf->app, app);
+      }
+    }
+  }
+  mtype_save();
+}
+
 void apply_preferences() {
   GtkWidget *win;
   GtkWidget *temp;
@@ -696,15 +1198,15 @@ void apply_preferences() {
 
   // general
   temp = lookup_widget(win, "entry4");
-  if (global.username) l_free(global.username);
-  global.username = l_strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
+  if (global.username) g_free(global.username);
+  global.username = g_strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
   temp = lookup_widget(win, "entry5");
-  if (global.password) l_free(global.password);
-  global.password = l_strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
+  if (global.password) g_free(global.password);
+  global.password = g_strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
   temp = lookup_widget(win, "entry7");
   pos = gtk_entry_get_text(GTK_ENTRY(temp));
   if (strcmp(pos, global.email)) {
-    global.email = l_strdup(pos);
+    global.email = g_strdup(pos);
     command_send(NULL, CMD_CHANGE_EMAIL, global.email);
   }
   temp = GTK_COMBO(lookup_widget(win, "combo3"))->entry;
@@ -717,13 +1219,13 @@ void apply_preferences() {
   }
 
   temp = lookup_widget(win, "entry83");
-  pos = l_strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
+  pos = g_strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
   destroy_string_list(global.allowed_ports);
   global.allowed_ports = create_allowed_ports(pos);
-  l_free(pos);
+  g_free(pos);
   pos = create_string_from_ports(global.allowed_ports);
   if (pos) gtk_entry_set_text(GTK_ENTRY(temp), pos);
-  l_free(pos);
+  g_free(pos);
 
   temp = lookup_widget(win, "radiobutton6");
   i2 = 0;
@@ -741,6 +1243,63 @@ void apply_preferences() {
     create_upload_port(i1, TRUE);
   }
 
+  i1 = global.naplist.auto_update;
+
+  temp = lookup_widget(win, "spinbutton81");
+  global.naplist.auto_update =
+    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp));
+  temp = lookup_widget(win, "checkbutton92");
+  if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
+    global.naplist.auto_update = -global.naplist.auto_update;
+  }
+  if (i1 != global.naplist.auto_update) {
+    napigator_restart_update(NULL);
+  }
+  temp = lookup_widget(win, "checkbutton96");
+  global.options.filetips =
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));
+
+  temp = lookup_widget(win, "checkbutton97");
+  global.options.switch_im =
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));
+
+  temp = lookup_widget(win, "checkbutton98");
+  global.options.switch_browse =
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));
+
+  temp = lookup_widget(win, "entry163");
+  pos = gtk_entry_get_text(GTK_ENTRY(temp));
+  g_free(global.naplist.host);
+  if (!pos || *pos == 0)
+    global.naplist.host = g_strdup(NAPLIST_HOST);
+  else
+    global.naplist.host = g_strdup(pos);
+
+  temp = lookup_widget(win, "entry164");
+  pos = gtk_entry_get_text(GTK_ENTRY(temp));
+  g_free(global.naplist.document);
+  if (!pos || *pos == 0)
+    global.naplist.document = g_strdup(NAPLIST_DOC);
+  else
+    global.naplist.document = g_strdup(pos);
+
+  temp = lookup_widget(win, "entry159");
+  pos = gtk_entry_get_text(GTK_ENTRY(temp));
+  g_free(global.naplist.proxy_host);
+  global.naplist.proxy_host = g_strdup(pos);
+
+  temp = lookup_widget(win, "spinbutton82");
+  global.naplist.proxy_port =
+    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp));
+
+  temp = lookup_widget(win, "checkbutton93");
+  global.naplist.use_proxy =
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));
+
+  temp = lookup_widget(win, "checkbutton95");
+  global.naplist.auto_remove =
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));
+
   // Weasel75
   temp = lookup_widget(win, "entry_speedred");
   i1 = extract_bytes(gtk_entry_get_text(GTK_ENTRY(temp)));
@@ -785,8 +1344,8 @@ void apply_preferences() {
 
   temp = lookup_widget(win, "entry85");
   pos = gtk_entry_get_text(GTK_ENTRY(temp));
-  l_free(global.ping_command);
-  global.ping_command = l_strdup(pos);
+  g_free(global.ping_command);
+  global.ping_command = g_strdup(pos);
 
   temp = lookup_widget(win, "checkbutton49");
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
@@ -849,9 +1408,15 @@ void apply_preferences() {
     socket_t* socket = gtk_clist_get_row_data(GTK_CLIST(temp), i1);
     upload_t* upload = socket->data;
     if (!upload) continue;
-    if (upload_in_progress(upload) && upload->file)
-      if (upload->file->size >= global.limit.large_size *1024 * 1024)
+    if (transfer_in_progress(TRANS(upload)) && upload->file) {
+      int size;
+      if (upload->segment)
+	size = upload->segment->stop - upload->segment->start;
+      else
+	size = upload->file->size;
+      if (size >= global.limit.large_size *1024 * 1024)
 	global.limit.cur_large++;
+    }
   }
   
   temp = lookup_widget(win, "checkbutton34");
@@ -876,53 +1441,11 @@ void apply_preferences() {
   global.limit.download_abort_number =
     gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp));
 
-  // mimetypes
-  clear_suffixes();
-  temp = lookup_widget(win, "clist17");
-  for (i1 = 0; i1 < GTK_CLIST(temp)->rows; i1++) {
-    int mime;
-    char *suf;
-    char *app;
-    char *as_mp3;
-    suffix_t *suffix;
-
-    gtk_clist_get_text(GTK_CLIST(temp), i1, 1, &pos);
-    mime = mime2int(pos);
-    gtk_clist_get_text(GTK_CLIST(temp), i1, 0, &suf);
-    gtk_clist_get_text(GTK_CLIST(temp), i1, 2, &app);
-    gtk_clist_get_text(GTK_CLIST(temp), i1, 3, &as_mp3);
-    suffix = l_malloc(sizeof(suffix_t));
-    suffix->suffix = l_strdup(suf);
-    suffix->application = l_strdup(app);
-    if (!l_strcasecmp(as_mp3, "Yes"))
-      suffix->as_mp3 = 1;
-    else
-      suffix->as_mp3 = 0;
-    if (mime == MIME_MP3)
-      suffix->as_mp3 = 1;
-    global.mimetype[mime].suffixes =
-	g_list_append(global.mimetype[mime].suffixes, suffix);
-  }
-
-
   // paths
   temp = lookup_widget(win, "entry75");
   if (global.incomplete_path)
-    l_free(global.incomplete_path);
-  global.incomplete_path = l_strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
-
-  // download
-  temp = lookup_widget(win, "clist18");
-  for (i1 = 0; i1 < GTK_CLIST(temp)->rows; i1++) {
-    int mime;
-
-    gtk_clist_get_text(GTK_CLIST(temp), i1, 0, &pos);
-    mime = mime2int(pos);
-    gtk_clist_get_text(GTK_CLIST(temp), i1, 1, &pos);
-    if (global.mimetype[mime].download)
-      l_free(global.mimetype[mime].download);
-    global.mimetype[mime].download = l_strdup(pos);
-  }
+    g_free(global.incomplete_path);
+  global.incomplete_path = g_strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
 
   // shares
   temp_lib = lib_new("__temp_lib__");
@@ -930,7 +1453,7 @@ void apply_preferences() {
   for (i1 = 0; i1 < GTK_CLIST(temp)->rows; i1++) {
     int mime;
     gtk_clist_get_text(GTK_CLIST(temp), i1, 0, &pos);
-    mime = mime2int(pos);
+    mime = media2int(pos);
     gtk_clist_get_text(GTK_CLIST(temp), i1, 1, &pos);
     lib_add_folder(temp_lib, pos, mime);
   }
@@ -972,16 +1495,16 @@ void apply_preferences() {
 
   temp = lookup_widget(win, "combo_entry24");
   pos = gtk_entry_get_text(GTK_ENTRY(temp));
-  if (!l_strcasecmp(pos, "None"))
+  if (!g_strcasecmp(pos, "None"))
     global.options.show_joins = 0;
   else
     global.options.show_joins = 1;
 
   temp = lookup_widget(win, "combo_entry25");
   pos = gtk_entry_get_text(GTK_ENTRY(temp));
-  if (!l_strcasecmp(pos, "None"))
+  if (!g_strcasecmp(pos, "None"))
     global.options.show_parts = 0;
-  else if (!l_strcasecmp(pos, "Recent Users"))
+  else if (!g_strcasecmp(pos, "Recent Users"))
     global.options.show_parts = 1;
   else
     global.options.show_parts = 2;
@@ -1018,9 +1541,9 @@ void apply_preferences() {
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
     temp = lookup_widget(win, "combo_entry32");
     pos = gtk_entry_get_text(GTK_ENTRY(temp));
-    if (!l_strcasecmp(pos, "Dark Colors"))
+    if (!g_strcasecmp(pos, "Dark Colors"))
       global.options.colored_nicks = 3;
-    else if (!l_strcasecmp(pos, "Medium Colors"))
+    else if (!g_strcasecmp(pos, "Medium Colors"))
       global.options.colored_nicks = 2;
     else
       global.options.colored_nicks = 1;
@@ -1031,9 +1554,9 @@ void apply_preferences() {
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
     temp = lookup_widget(win, "entry156");
     pos = gtk_entry_get_text(GTK_ENTRY(temp));
-    if (!l_strcasecmp(pos, "Dark Colors"))
+    if (!g_strcasecmp(pos, "Dark Colors"))
       global.options.colored_nicks2 = 3;
-    else if (!l_strcasecmp(pos, "Medium Colors"))
+    else if (!g_strcasecmp(pos, "Medium Colors"))
       global.options.colored_nicks2 = 2;
     else
       global.options.colored_nicks2 = 1;
@@ -1115,9 +1638,22 @@ void apply_preferences() {
   global.limit.max_seek_add =
     gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp));
 
-  temp = lookup_widget(win, "combo_entry27");
+  temp = lookup_widget(win, "entry162");
   pos = gtk_entry_get_text(GTK_ENTRY(temp));
-  global.options.browser = browser2int(pos);
+  g_free(global.options.browser);
+  while (*pos == ' ') pos++;
+  global.options.browser = (pos&&*pos)?g_strdup(pos):NULL;
+
+#ifdef HAVE_LIBLCONV_H
+  temp = lookup_widget(win, "checkbutton91");
+  global.options.use_iconv =
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));
+  if (global.options.dest_codeset)
+    g_free(global.options.dest_codeset);
+  temp = lookup_widget(win, "entry158");
+  global.options.dest_codeset = 
+    g_strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
+#endif
 
   temp = lookup_widget(win, "radiobutton5");
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp)))
@@ -1153,10 +1689,10 @@ void apply_preferences() {
   temp = lookup_widget(win, "entry142");
   pos = gtk_entry_get_text(GTK_ENTRY(temp));
   if (global.external_handler)
-    l_free(global.external_handler);
+    g_free(global.external_handler);
   global.external_handler = 0;
   if (pos && *pos)
-    global.external_handler = l_strdup(pos);
+    global.external_handler = g_strdup(pos);
 
   for (i1 = 0; i1 < EVENT_NUMBER; i1++) {
     char text[100];
@@ -1179,11 +1715,12 @@ void apply_preferences() {
   }
   lib_destroy(temp_lib);
       
+  apply_media_types();
+
   write_rc();
 }
 
-void preferences_goto(int section)
-{
+void preferences_goto(int section) {
   GtkCList *clist;
 
   if (!global.options_win) {
