@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2003 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -42,23 +42,26 @@ char *search_arg(char *data);
 void info_dialog(char *text);
 
 typedef struct {
-  int fd;
   char* filename;
+  GtkBox* box;
+  GtkToggleButton* check;
+  GtkProgressBar* progress;
+  GtkLabel* action;
+  long size;
+  int last_progress;
+
+  int fd;
   GList* segments;
   long current_pos;
   long start_found;
-} scan_t;
+} dead_t;
 
-static int scanning_for_dead = 0;
-GtkWidget* scan_win = NULL;
-GList* dead_files = NULL;
-scan_t* scan = NULL;
-int scan_idle = -1;
-int scan_timeout = -1;
-double total_to_go = 0;
-double file_to_go = 0;
-double total_progress = 0;
 
+static GList* DeadFiles = NULL;
+static GtkWidget* ScanWin = NULL;
+static int ScanIdle = -1;
+
+//#define RESUME_DEBUG
 
 char *get_search_string(char *str) {
   static char result[1024];
@@ -88,7 +91,7 @@ resume_t *resume_new(long access) {
   GList* dlist;
   int cnt;
 
-  resume = l_malloc(sizeof(resume_t));
+  resume = g_malloc(sizeof(resume_t));
   resume->filename = NULL;
   resume->dirname = NULL;
   resume->comp_size = 0;
@@ -121,16 +124,16 @@ void resume_destroy(resume_t * resume) {
     return;
 
   if (resume->filename)
-    l_free(resume->filename);
+    g_free(resume->filename);
   if (resume->dirname)
-    l_free(resume->dirname);
+    g_free(resume->dirname);
   if (resume->search)
     resume_remove_search(resume);
-  if (resume->search_string) l_free(resume->search_string);
+  if (resume->search_string) g_free(resume->search_string);
   g_list_free(resume->downloads);
   global.incomplete = g_list_remove(global.incomplete, resume);
   resume_save(NULL);
-  l_free(resume);
+  g_free(resume);
 }
 
 void resume_load(char *fname, int import) {
@@ -148,8 +151,8 @@ void resume_load(char *fname, int import) {
   char *short_file;
   char* status;
   int have_incomplete;
-  long start;
-  long length;
+  unsigned long start;
+  unsigned long length;
   char* temp1;
   char* temp2;
   char* temp3;
@@ -160,7 +163,8 @@ void resume_load(char *fname, int import) {
   int cnt = 0;
 
 #ifdef RESUME_DEBUG
-  printf("reading [%s][%s]\n", fname, global.incomplete_path);
+  printf("reading [%s][%s]\n", fname, 
+	 global.incomplete_path?global.incomplete_path:"(null)");
 #endif
   if (!global.incomplete_path || 
       !create_dir(global.incomplete_path)) {
@@ -182,9 +186,8 @@ void resume_load(char *fname, int import) {
       temp3 = arg(NULL, 0);
       temp4 = arg(NULL, 0);
       if (!temp2 || !resume) continue;
-      start = strtol(temp1, NULL, 10);
-      length = strtol(temp2, NULL, 10);
-      //      if (length <= 0) continue;
+      start = strtoul(temp1, NULL, 10);
+      length = strtoul(temp2, NULL, 10);
       cnt++;    // increase the segment counter
       if (length < 1000) continue;
       segment = file_segment_new(start, start+length);
@@ -239,13 +242,14 @@ void resume_load(char *fname, int import) {
       cnt = 0;   // reset the segment counter;
       if (have_incomplete && import)
 	resume->filename =
-	  l_strdup_printf("%s/%s", global.incomplete_path, short_file);
+	  g_strdup_printf("%s%c%s", global.incomplete_path,
+			  DIR_SEP, short_file);
       else
-	resume->filename = l_strdup(filename);
+	resume->filename = g_strdup(filename);
       
       resume->comp_size = strtoul(size, NULL, 10);
-      resume->dirname = l_strdup(dirname);
-      if (*search) resume->search_string = l_strdup(search);
+      resume->dirname = g_strdup(dirname);
+      if (*search) resume->search_string = g_strdup(search);
       else resume->search_string = NULL;
       
       resume->flags = atoi(status);
@@ -281,14 +285,16 @@ int resume_save(gpointer data ATTR_UNUSED) {
   long size;
   file_segment_t* segment;
 
-  fname = l_strdup_printf("%s/incomplete.list", global.options.config_dir);
+  fname = g_strdup_printf("%s%cincomplete.list", 
+			  global.options.config_dir, DIR_SEP);
   fname_new =
-    l_strdup_printf("%s/incomplete.list.new", global.options.config_dir);
+    g_strdup_printf("%s%cincomplete.list.new", 
+		    global.options.config_dir, DIR_SEP);
 
   if ((fd = fopen(fname_new, "w")) == NULL) {
     g_warning("Could not write [%s]\n", fname);
-    l_free(fname);
-    l_free(fname_new);
+    g_free(fname);
+    g_free(fname_new);
     return 1;
   }
 
@@ -326,23 +332,23 @@ int resume_save(gpointer data ATTR_UNUSED) {
     g_warning("Could not write [%s]\n", fname);
   }
   
-  l_free(fname);
-  l_free(fname_new);
+  g_free(fname);
+  g_free(fname_new);
   return 1;
 }
 
 int resume_insert_file(resume_t * resume, file_t * file) {
   socket_t *socket;
-  download_t *download;
+  transfer_t *trans;
   int mtype;
 
   /* media types have to match, if they are MIME_NONE then
    * then suffixes have to match.
    * Necessary if downloading multi volume archives with same
    * size, same basename, but different suffixes */
-  mtype = get_media_type(resume->filename);
+  mtype = mtype_get(resume->filename);
   if (mtype != file->media_type) return 0;
-  if (mtype == MIME_NONE) {
+  if (mtype == MEDIA_NONE) {
     char* suf1 = strrchr(file->shortname, '.');
     char* suf2 = strrchr(resume->filename, '.');
     if (suf1 && !suf2) return 0;
@@ -351,21 +357,19 @@ int resume_insert_file(resume_t * resume, file_t * file) {
   }
 
   socket = download_create(file, resume, NULL, NULL, 0);
-  if (!socket) return 0;
-  
-  download = socket->data;
-  
-  if (!download_in_progress(download) && (socket->timer == -1) &&
-      (download->data->status != S_REMOTE) && 
-      (download->data->status != S_QUEUED) && 
-      (download->data->status != S_FINISHED)) {
+  if (!socket || !socket->data) return 0;
+  trans = TRANS(socket->data);
+  if (!transfer_in_progress(trans) && (socket->timer == -1) &&
+      (trans->status != S_REMOTE) && 
+      (trans->status != S_QUEUED) && 
+      (trans->status != S_CANCELED) &&  // dont start canceled
+      (trans->status != S_FINISHED)) {
     download_status_set(socket, S_QUEUED);
   }
   return 1;
 }
 
-resume_t *resume_search_short_file(char *filename)
-{
+resume_t* resume_search_short_file(char* filename) {
   GList *dlist;
   resume_t *resume;
   char *fname;
@@ -406,8 +410,7 @@ resume_t *resume_search_file(file_t * file)
   return NULL;
 }
 
-GList *resume_search_size(long size)
-{
+GList *resume_search_size(long size) {
   GList *dlist;
   resume_t *resume;
   GList* result = NULL;
@@ -420,25 +423,52 @@ GList *resume_search_size(long size)
   return result;
 }
 
+GList *resume_search_size_real(long size) {
+  GList *dlist;
+  resume_t *resume;
+  GList* result = NULL;
+
+  for (dlist = global.incomplete; dlist; dlist = dlist->next) {
+    resume = dlist->data;
+    if (size == resume->comp_size)
+      result = g_list_append(result, resume);
+  }
+  return result;
+}
+
+void resume_update_search(resume_t* resume, char* text) {
+  char* pos;
+
+  if (!resume || !resume->search) return;
+  
+  if (resume->search->pattern->include)
+    g_free(resume->search->pattern->include);
+
+  if (text && *text) {
+    resume->search->pattern->include = g_strdup(text);
+  } else {
+    pos = g_strdup(get_file_name(resume->filename));
+    resume->search->pattern->include = g_strdup(get_search_string(pos));
+    g_free(pos);
+  }
+}
+
 void resume_create_search(resume_t * resume) {
   search_t *search;
   search_pattern_t *pattern;
-  char *pos;
 
   search = search_new();
   search->resume = 1;
   search->link = resume;
 
-  pos = l_strdup(get_file_name(resume->filename));
-
   pattern = search_pattern_new();
-  if (resume->search_string && *(resume->search_string))
-    pattern->include = l_strdup(resume->search_string);
-  else
-    pattern->include = l_strdup(get_search_string(pos));
-  pattern->name = l_strdup(pattern->include);
-  l_free(pos);
-  pattern->exclude = l_strdup("");
+  search->pattern = pattern;
+
+  resume->search = search;
+
+  resume_update_search(resume, resume->search_string);
+  pattern->name = g_strdup(pattern->include);
+  pattern->exclude = g_strdup("");
   pattern->max_results = 0;
   pattern->destination = DEST_NAPSTER;
 
@@ -447,10 +477,7 @@ void resume_create_search(resume_t * resume) {
   else
     pattern->size_lo = resume->inc_size;
 
-  pattern->media_type = MIME_NONE;
-
-  search->pattern = pattern;
-  resume->search = search;
+  pattern->media_type = MEDIA_NONE;
 }
 
 void resume_search(resume_t * resume, net_t* net) {
@@ -488,7 +515,7 @@ void resume_remove_search(resume_t * resume) {
   if (resume->search) search_remove(resume->search);
   resume->search = NULL;
   
-  resume_queued_update(resume);
+  resume->needs_update = 1;
 }
 
 static int resume_belongs_to(resume_t * resume) {
@@ -509,11 +536,6 @@ static int resume_belongs_to(resume_t * resume) {
   return class;
 }
 
-void resume_queued_update(resume_t* resume) {
-  if (!resume) return;
-  resume->needs_update = 1;
-}
-
 void resume_show(resume_t * resume) {
   GtkCTree *ctree;
   int tree_no;
@@ -533,6 +555,7 @@ void resume_show(resume_t * resume) {
     if (resume->tree_no != tree_no) resume_hide(resume);
     else return;
   }
+
   pos = get_file_name(resume->filename);
   
   strcpy(tstr[0], pos);
@@ -569,7 +592,7 @@ static int resume_update_resume(resume_t * resume) {
   char text[1024];
   GdkPixmap *pixmap = NULL;
   GdkBitmap *bitmap = NULL;
-  download_t* download;
+  transfer_t* trans;
   GList* dlist;
   int timeleft;
   double rate;
@@ -636,12 +659,12 @@ static int resume_update_resume(resume_t * resume) {
   for (dlist = resume->downloads; dlist; dlist = dlist->next) {
     socket = dlist->data;
     if (!socket) continue;
-    download = socket->data;
-    if (download->data->status != S_DOWNLOADING) continue;
-    rate += download->data->rate;
+    trans = TRANS(socket->data);
+    if (trans->status != S_DOWNLOADING) continue;
+    rate += trans->rate;
     cnt++;
-//    if (download->data->timeleft < timeleft)
-//      timeleft = download->data->timeleft;
+//    if (trans->timeleft < timeleft)
+//      timeleft = trans->timeleft;
   }
 
   if (cnt > 0) {
@@ -794,7 +817,7 @@ void resume_update(resume_t * resume) {
 void resume_cancel(resume_t * resume) {
   GList *dlist;
   socket_t *socket;
-  download_t *download;
+  transfer_t *trans;
 
   if (!resume) return;
 
@@ -805,23 +828,19 @@ void resume_cancel(resume_t * resume) {
   for (dlist = resume->downloads; dlist; dlist = dlist->next) {
     socket = dlist->data;
     if (!socket) return;
-    download = socket->data;
-    if (!download) return;
-    if (download->data->status == S_QUEUED ||
-	download->data->status == S_REMOTE ||
-	download_in_progress(download) ||
+    trans = TRANS(socket->data);
+    if (!trans) return;
+    if (trans->status == S_QUEUED ||
+	trans->status == S_REMOTE ||
+	transfer_in_progress(trans) ||
 	socket->timer != -1) {
-#ifdef RESUME_DEBUG
-      printf("cancel t:%p\n", download);
-#endif
       socket_end(socket, S_CANCELED);
     }
   }
   resume_update(resume);
 }
 
-void resume_remove(resume_t * resume)
-{
+void resume_remove(resume_t * resume) {
   socket_t *socket;
 
   if (!resume) return;
@@ -843,16 +862,14 @@ void resume_remove(resume_t * resume)
   resume_destroy(resume);
 }
 
-void resume_freeze(resume_t * resume)
-{
+void resume_freeze(resume_t * resume) {
   if (resume->flags & (RESUME_INACTIVE)) return;
   resume_cancel(resume);
   resume->flags |= RESUME_DEACTIVATED;
   resume_update(resume);
 }
 
-void resume_thaw(resume_t * resume)
-{
+void resume_thaw(resume_t * resume) {
   if ((resume->flags & RESUME_DEACTIVATED) == 0) return;
   resume->flags &= ~RESUME_DEACTIVATED;
   resume_update(resume);
@@ -874,6 +891,7 @@ int resume_timer(gpointer data ATTR_UNUSED) {
 int resume_has_potentials(resume_t* resume) {
   socket_t *socket;
   download_t *download;
+  transfer_t *trans;
   GList *dlist;
   int res = 0;
 
@@ -883,12 +901,12 @@ int resume_has_potentials(resume_t* resume) {
     socket = dlist->data;
     if (socket->timer != -1) res |= 1;
     download = socket->data;
-    if (download_in_progress(download))
+    trans = TRANS(download);
+    if (transfer_in_progress(trans))
       res |= 1;
-    if ((download->data->status == S_QUEUED) &&
-	global.options.auto_resume)
+    if (trans->status == S_QUEUED && global.options.auto_resume)
       res |= 4;
-    if (download->data->status == S_PART)
+    if (trans->status == S_PART)
       res |= 4;
   }
   
@@ -904,19 +922,13 @@ time_t resume_time_of_death(resume_t* resume) {
   lsize = resume->inc_size;
   lsize2 = resume->comp_size;
 
-  if ((double)lsize*100. < 
-      (double)(global.options.resume_abort[0])*(double)lsize2) {
-    return 1;  // now
-  }
-  if (!global.options.resume_abort[1]) {
-    return 0;
-  }
+  if (!global.options.resume_abort[0]) return 0;
 
   if (lsize2) per = (double)lsize/(double)lsize2;
   else per = 1.0;
 
-  value = (time_t)global.options.resume_abort[2]*24*60*60 +
-    (time_t)((double)global.options.resume_abort[3]*24*60*60*per);
+  value = (time_t)global.options.resume_abort[1]*24*60*60 +
+    (time_t)((double)global.options.resume_abort[2]*24*60*60*per);
   if (resume->last_access + value <= global.current_time) {
     return 1;
   } else {
@@ -950,8 +962,7 @@ void resume_remove_outdated() {
   g_list_free(result);
 }
 
-void resume_hide(resume_t * resume)
-{
+void resume_hide(resume_t * resume) {
   if (resume->tree_no < 0 || !resume->node) {
 #ifdef RESUME_DEBUG
     printf("resume_hide(): resume not shown\n");
@@ -970,9 +981,6 @@ GtkWidget* create_abort_win (void)
   GtkWidget *frame324;
   GtkWidget *vbox192;
   GtkWidget *vbox193;
-  GtkWidget *hbox687;
-  GtkWidget *label1012;
-  GtkWidget *hscale1;
   GtkWidget *hbox686;
   GtkWidget *checkbutton69;
   GtkObject *spinbutton63_adj;
@@ -988,7 +996,6 @@ GtkWidget* create_abort_win (void)
   GtkWidget *hbox685;
   GtkWidget *button318;
   GtkWidget *button319;
-  GtkObject *adj;
 
   abort_win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_object_set_data (GTK_OBJECT (abort_win), "abort_win", abort_win);
@@ -1009,36 +1016,19 @@ GtkWidget* create_abort_win (void)
   gtk_widget_show (vbox193);
   gtk_box_pack_start (GTK_BOX (vbox192), vbox193, TRUE, TRUE, 0);
 
-  hbox687 = gtk_hbox_new (FALSE, 0);
-  gtk_widget_show (hbox687);
-  gtk_box_pack_start (GTK_BOX (vbox193), hbox687, FALSE, FALSE, 0);
-
-  label1012 = gtk_label_new ("Keep incomplete files only if progress was at least");
-  gtk_widget_show (label1012);
-  gtk_box_pack_start (GTK_BOX (hbox687), label1012, FALSE, FALSE, 0);
-
-  adj = gtk_adjustment_new ((double)global.options.resume_abort[0]/(double)100,
-			    0, 1, 0, 0, 0);
-  gtk_object_set_data (GTK_OBJECT (abort_win), "adj1", adj);
-  hscale1 = gtk_hscale_new (GTK_ADJUSTMENT (adj));
-  gtk_widget_show (hscale1);
-  gtk_box_pack_start (GTK_BOX (vbox193), hscale1, FALSE, FALSE, 0);
-  gtk_scale_set_value_pos (GTK_SCALE (hscale1), GTK_POS_RIGHT);
-  gtk_scale_set_digits (GTK_SCALE (hscale1), 2);
-
   hbox686 = gtk_hbox_new (FALSE, 5);
   gtk_widget_show (hbox686);
   gtk_box_pack_start (GTK_BOX (vbox193), hbox686, FALSE, FALSE, 0);
 
   checkbutton69 = gtk_check_button_new_with_label ("Keep incomplete files at most");
-  if (global.options.resume_abort[1]) 
+  if (global.options.resume_abort[0]) 
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton69), TRUE);
   gtk_widget_show (checkbutton69);
   gtk_object_set_data (GTK_OBJECT (abort_win), "check", checkbutton69);
   gtk_box_pack_start (GTK_BOX (hbox686), checkbutton69, FALSE, FALSE, 0);
 
-  spinbutton63_adj = gtk_adjustment_new (global.options.resume_abort[2],
-					 0, 365*20, 1, 10, 10);
+  spinbutton63_adj = gtk_adjustment_new
+    (global.options.resume_abort[1], 0, 365*20, 1, 10, 10);
   gtk_object_set_data (GTK_OBJECT (abort_win), "adj2", spinbutton63_adj);
   spinbutton63 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton63_adj), 1, 0);
   gtk_widget_show (spinbutton63);
@@ -1059,8 +1049,8 @@ GtkWidget* create_abort_win (void)
   gtk_widget_show (label1018);
   gtk_box_pack_start (GTK_BOX (hbox686), label1018, FALSE, FALSE, 0);
 
-  spinbutton64_adj = gtk_adjustment_new (global.options.resume_abort[3],
-					 0, 1000, 1, 10, 10);
+  spinbutton64_adj = gtk_adjustment_new
+    (global.options.resume_abort[2], 0, 1000, 1, 10, 10);
   gtk_object_set_data (GTK_OBJECT (abort_win), "adj3", spinbutton64_adj);
   spinbutton64 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton64_adj), 1, 0);
   gtk_widget_show (spinbutton64);
@@ -1080,27 +1070,27 @@ GtkWidget* create_abort_win (void)
   gtk_widget_set_usize (frame325, -2, 41);
   gtk_frame_set_shadow_type (GTK_FRAME (frame325), GTK_SHADOW_IN);
 
-  hbox685 = gtk_hbox_new (TRUE, 5);
+  hbox685 = gtk_hbox_new(TRUE, 5);
   gtk_widget_show (hbox685);
-  gtk_container_add (GTK_CONTAINER (frame325), hbox685);
+  gtk_container_add(GTK_CONTAINER(frame325), hbox685);
   gtk_container_set_border_width (GTK_CONTAINER (hbox685), 5);
 
-  button318 = gtk_button_new_with_label ("Ok");
+  button318 = gtk_button_new_with_label("Ok");
   gtk_widget_show (button318);
   gtk_box_pack_start (GTK_BOX (hbox685), button318, TRUE, TRUE, 0);
 
-  button319 = gtk_button_new_with_label ("Cancel");
+  button319 = gtk_button_new_with_label("Cancel");
   gtk_widget_show (button319);
   gtk_box_pack_start (GTK_BOX (hbox685), button319, TRUE, TRUE, 0);
 
-  gtk_signal_connect (GTK_OBJECT (abort_win), "destroy",
-                      GTK_SIGNAL_FUNC (gtk_widget_destroy),
-                      NULL);
-  gtk_signal_connect (GTK_OBJECT (button318), "clicked",
-                      GTK_SIGNAL_FUNC (on_button318_clicked),
-                      (gpointer)abort_win);
-  gtk_signal_connect (GTK_OBJECT (button319), "clicked",
-                      GTK_SIGNAL_FUNC (on_button319_clicked),
+  gtk_signal_connect(GTK_OBJECT(abort_win), "destroy",
+		     GTK_SIGNAL_FUNC(gtk_widget_destroy),
+		     NULL);
+  gtk_signal_connect(GTK_OBJECT(button318), "clicked",
+		     GTK_SIGNAL_FUNC(on_button318_clicked),
+		     (gpointer)abort_win);
+  gtk_signal_connect(GTK_OBJECT(button319), "clicked",
+		     GTK_SIGNAL_FUNC(on_button319_clicked),
         (gpointer)abort_win);
 
   return abort_win;
@@ -1112,7 +1102,7 @@ void resume_saver_start() {
   if (global.resume_saver >= 0) return;
 
   global.resume_saver = 
-    gtk_timeout_add(10000, resume_save, NULL);
+    gtk_timeout_add(20000, resume_save, NULL);
 }
 
 void resume_saver_stop() {
@@ -1233,7 +1223,7 @@ void seg_list_destroy(gpointer data) {
   file_segment_t** segment = data;
 
   file_segment_destroy(segment[1]);
-  l_free(segment);
+  g_free(segment);
 }
 
 void resume_options_setup(GtkWidget* win, resume_t* resume) {
@@ -1252,7 +1242,7 @@ void resume_options_setup(GtkWidget* win, resume_t* resume) {
     if (segment->size < 1000) continue;
     sprintf(tstr[0], "%d - %d", segment->start, segment->start+segment->size);
     row = gtk_clist_append(clist, list);
-    seg_list = l_malloc(2*sizeof(*seg_list));
+    seg_list = g_malloc(2*sizeof(*seg_list));
     seg_list[0] = segment;
     seg_list[1] = file_segment_new(segment->start, segment->stop);
     seg_list[1]->size = segment->size;
@@ -1432,93 +1422,76 @@ int resume_finished(resume_t* resume) {
   else return 0;
 }
 
-int scan_start() {
-  if (scanning_for_dead) {
-    info_dialog("Already scanning incomplete folder\nPlease stand by.");
-    return 0;
-  }
-  scanning_for_dead = 1;
-  dead_files = NULL;
-  scan_win = create_scan_progress_win();
-  gtk_widget_show(scan_win);
-  total_to_go = 0;
-  file_to_go = 0;
-  total_progress = 0;
-  scan_idle = -1;
-  scan_timeout = -1;
 
-  return 1;
+static void dead_destroy(dead_t* dead) {
+  if (dead->fd >= 0) close(dead->fd);
+  g_free(dead->filename);
+  g_list_foreach(dead->segments, (GFunc)g_free, NULL);
+  g_list_free(dead->segments);
+  g_free(dead);
 }
 
-gint scan_update(gpointer data ATTR_UNUSED) {
-  GtkProgressBar* bar;
-  double per;
+static void scan_end(gboolean close_win) {
 
-  bar = GTK_PROGRESS_BAR(lookup_widget(scan_win, "progressbar2"));
-  if (total_to_go > 0 && scan)
-    per = (total_progress+scan->current_pos)/total_to_go;
-  else per = 0;
-  gtk_progress_bar_update(bar, per);
+  if (!ScanWin) return;
 
-  bar = GTK_PROGRESS_BAR(lookup_widget(scan_win, "progressbar3"));
-  if (file_to_go > 0 && scan)
-    per = scan->current_pos/file_to_go;
-  else per = 0;
-  gtk_progress_bar_update(bar, per);
+  if (ScanIdle >= 0) {
+    gtk_idle_remove(ScanIdle);
+    ScanIdle = -1;
+  }
+
+  if (close_win) {
+    GList* dlist;
+
+    gtk_widget_destroy(ScanWin);
+    ScanWin = NULL;
+
+    for (dlist = DeadFiles; dlist; dlist = dlist->next) {
+      dead_t* dead = dlist->data;
+      dead_destroy(dead);
+    }
+    g_list_free(DeadFiles);
+    DeadFiles = NULL;
+  }
 
-  return 1;
 }
 
-void scan_end(int close_win) {
-  GList* dlist;
+static void on_scan_destroy() {
+  scan_end(TRUE);
+}
 
-  if (!scanning_for_dead) return;
+static gboolean on_scan_delete() {
+  scan_end(TRUE);
+  return TRUE;
+}
 
-  if (scan_idle >= 0) {
-    gtk_idle_remove(scan_idle);
-    scan_idle = -1;
-  }
-  if (scan_timeout >= 0) {
-    gtk_timeout_remove(scan_timeout);
-    scan_timeout = -1;
-  }
+static gint scan_files();
+static void on_scan_start(GtkWidget* button) {
+  gtk_widget_set_sensitive(button, FALSE);
+  ScanIdle = gtk_idle_add(scan_files, NULL);
+}
 
-  if (scan) {
-    l_free(scan->filename);
-    close(scan->fd);
-    for (dlist = scan->segments; dlist; dlist = dlist->next)
-      l_free(dlist->data);
-    g_list_free(scan->segments);
-    l_free(scan);
-    scan = NULL;
-  }
-  if (scan_win && close_win) {
-    gtk_widget_destroy(scan_win);
-    scan_win = NULL;
-    scanning_for_dead = 0;
-  } else {
-    scan_update(NULL);
-  }
+static void on_scan_abort() {
+  scan_end(TRUE);
 }
 
 #define MIN_ZERO_SIZE 1024*4
 #define PRE_LOAD  2
-gint scan_files(gpointer data);
 
-long check_data(scan_t* scan, long pos) {
+static long check_data(dead_t* dead, long pos) {
   long i1;
   int res;
   static char buffer[MIN_ZERO_SIZE];
   
-  if (lseek(scan->fd, pos, SEEK_SET) == (off_t)-1) return -1;
-  res = read(scan->fd, buffer, PRE_LOAD);
+  if (lseek(dead->fd, pos, SEEK_SET) == (off_t)-1) return -1;
+  res = read(dead->fd, buffer, PRE_LOAD);
   if (res < 0) return -2;
   if (res == 0) return -3;
   for (i1 = 0; i1 < PRE_LOAD; i1++) {
     if (buffer[i1] != 0) return pos+i1;
   }
 
-  res = read(scan->fd, buffer+PRE_LOAD, MIN_ZERO_SIZE-PRE_LOAD);
+  res = read(dead->fd, buffer+PRE_LOAD, MIN_ZERO_SIZE-PRE_LOAD);
   if (res < 0) return -2;
   if (res == 0) return -3;
   for (i1 = PRE_LOAD; i1 < res+PRE_LOAD; i1++)
@@ -1526,200 +1499,314 @@ long check_data(scan_t* scan, long pos) {
   return -4;
 }
 
-void scan_set_status(char* file, char* status) {
-  GtkCList* clist;
-  int row;
-
-  clist = GTK_CLIST(lookup_widget(scan_win, "clist42"));
-  
-  row = gtk_clist_find_row_from_data(clist, file);
-  if (row < 0) return;
-  gtk_clist_set_text(clist, row, 0, status);
-}
-
-gint file_scan_idle(gpointer data ATTR_UNUSED) {
+static gint file_scan_idle(dead_t* dead) {
   GList* dlist;
   int cnt = 0;
   long temp;
   resume_t* newfile;
+  double per;
 
   for (cnt = 0; cnt < 10; cnt++) {
-    temp = check_data(scan, scan->current_pos);
+    temp = check_data(dead, dead->current_pos);
     switch (temp) {
     case -1: // lseek error, should be EOF
     case -3: // read 0 bytes, should be EOF
-      if (scan->start_found >= 0) {
+      if (dead->start_found >= 0) {
 	file_segment_t* segment;
-	segment = file_segment_new(scan->start_found, 
-				   scan->current_pos - MIN_ZERO_SIZE);
+	segment = file_segment_new(dead->start_found, 
+				   dead->current_pos - MIN_ZERO_SIZE);
 	segment->size = segment->stop - segment->start;
-	scan->start_found = -1;
+	dead->start_found = -1;
 #ifdef RESUME_DEBUG
-	printf("** new segment from %d to %d\n",
+	printf(" - new segment from %d to %d\n",
 	       segment->start, segment->stop);
 #endif
-	scan->segments = g_list_append(scan->segments, segment);
+	dead->segments = g_list_append(dead->segments, segment);
       }
-      scan_set_status(scan->filename, "Done");
+
+      gtk_label_set_text(dead->action, "Finished");
+      gtk_toggle_button_set_active(dead->check, FALSE);
+      gtk_widget_set_sensitive(GTK_WIDGET(dead->box), FALSE);
+
+      gtk_progress_bar_update(dead->progress, 1.0);
 
       // now create resume and attach segments
       newfile = resume_new(global.current_time);
-      newfile->filename = scan->filename;
-      for(dlist = scan->segments; dlist; dlist = dlist->next) {
+      newfile->filename = dead->filename;
+      dead->filename = NULL;
+      for (dlist = dead->segments; dlist; dlist = dlist->next) {
         file_segment_t* t = dlist->data;
         newfile->inc_size += t->stop - t->start;
       }
-      newfile->parts = scan->segments;
+      newfile->parts = dead->segments;
+      dead->segments = NULL;
       resume_show(newfile);
       resume_save(NULL);
-      close(scan->fd);
-      l_free(scan);
-      scan = NULL;
-      total_progress += file_to_go;
-      gtk_timeout_remove(scan_timeout);
-      scan_timeout = -1;
 
-      gtk_idle_remove(scan_idle);
-      scan_idle = gtk_idle_add(scan_files, NULL);
+      DeadFiles = g_list_remove(DeadFiles, dead);
+      dead_destroy(dead);
+
+      gtk_idle_remove(ScanIdle);
+      ScanIdle = gtk_idle_add(scan_files, NULL);
       return 0;
     case -2: // read error
-      scan_set_status(scan->filename, "Read error");
-      
-      l_free(scan->filename);
-      close(scan->fd);
-      for (dlist = scan->segments; dlist; dlist = dlist->next)
-	l_free(dlist->data);
-      g_list_free(scan->segments);
-      l_free(scan);
-      scan = 0;
-      total_progress += file_to_go;
+      gtk_label_set_text(dead->action, "Read error");
+      gtk_toggle_button_set_active(dead->check, FALSE);
+      gtk_widget_set_sensitive(GTK_WIDGET(dead->box), FALSE);
 
-      gtk_timeout_remove(scan_timeout);
-      scan_timeout = -1;
-      
-      gtk_idle_remove(scan_idle);
-      scan_idle = gtk_idle_add(scan_files, NULL);
+      DeadFiles = g_list_remove(DeadFiles, dead);
+      dead_destroy(dead);
+
+      gtk_progress_bar_update(dead->progress, 0.0);
+
+      gtk_idle_remove(ScanIdle);
+      ScanIdle = gtk_idle_add(scan_files, NULL);
       return 0;
     case -4: // hole found, so finish segment if there is one
-      if (scan->start_found >= 0) {
+      if (dead->start_found >= 0 && 
+	  dead->start_found+MIN_ZERO_SIZE < dead->current_pos) {
 	file_segment_t* segment;
-	segment = file_segment_new(scan->start_found,
-				   scan->current_pos - MIN_ZERO_SIZE);
+	segment = file_segment_new(dead->start_found,
+				   dead->current_pos - MIN_ZERO_SIZE);
 	segment->size = segment->stop - segment->start;
-	scan->start_found = -1;
 #ifdef RESUME_DEBUG
-	printf("** new segment from %d to %d\n",
+	printf(" - new segment from %d to %d\n",
 	       segment->start, segment->stop);
 #endif
-	scan->segments = g_list_append(scan->segments, segment);
+	dead->segments = g_list_append(dead->segments, segment);
       }
+      dead->start_found = -1;
       break;
     default: // >= 0, data found at position
-      if (scan->start_found < 0) {
-	scan->start_found = temp;
-      }
+      if (dead->start_found < 0) dead->start_found = temp;
       break;
     }
-    scan->current_pos += MIN_ZERO_SIZE;
+    dead->current_pos += MIN_ZERO_SIZE;
+    if (dead->size >= dead->current_pos) {
+      per = (double)dead->current_pos/dead->size;
+      gtk_progress_bar_update(dead->progress, per);
+    }
   }
 
   return 1;
 }
 
-gint scan_files(gpointer data ATTR_UNUSED) {
-  char* filename;
+static gint scan_files() {
   int fd;
-  struct stat st;
+  dead_t* dead;
 
-  if (!dead_files) {
-    scan_end(0);
+  if (!DeadFiles) {
+    scan_end(FALSE);
     return 0;
   }
 #ifdef RESUME_DEBUG
-  printf("still %d files to go\n", g_list_length(dead_files));
+  printf("still %d files to go\n", g_list_length(DeadFiles));
 #endif
-  filename = dead_files->data;
-  dead_files = g_list_remove(dead_files, filename);
-  
-  if ((fd  = open(filename, O_RDONLY)) < 0) {
-    g_warning("Unable to open() file '%s' (%s) !\n", filename,
-	      g_strerror(errno));
-    l_free(filename);
+  dead = DeadFiles->data;
+
+  if (!gtk_toggle_button_get_active(dead->check)) {
+    gtk_label_set_text(dead->action, "Skipped");
+    gtk_toggle_button_set_active(dead->check, FALSE);
+    gtk_widget_set_sensitive(GTK_WIDGET(dead->box), FALSE);
+
+    DeadFiles = g_list_remove(DeadFiles, dead);
+    dead_destroy(dead);
     return 1;
   }
 
-  scan_set_status(filename, "In progress");
+  if ((fd  = open(dead->filename, O_RDONLY)) < 0) {
+    g_warning("Unable to open() file '%s' (%s) !\n", 
+	      dead->filename, g_strerror(errno));
+    DeadFiles = g_list_remove(DeadFiles, dead);
+    dead_destroy(dead);
+    return 1;
+  }
+
+  gtk_label_set_text(dead->action, "Scanning...");
 
 #ifdef RESUME_DEBUG
-  printf("now scanning [%s]\n", filename);
+  printf("now scanning [%s]\n", dead->filename);
 #endif
-  scan = l_malloc(sizeof(*scan));
-  scan->filename = filename;
-  scan->fd = fd;
-  scan->segments = NULL;
-  scan->start_found = -1;
-  scan->current_pos = 0;
-  stat(scan->filename, &st);
-  file_to_go = st.st_size;
+  dead->fd = fd;
+  dead->segments = NULL;
+  dead->start_found = -1;
+  dead->current_pos = 0;
 
-  scan_idle = gtk_idle_add(file_scan_idle, NULL);
-  scan_timeout = gtk_timeout_add(500, scan_update, NULL);
+  ScanIdle = gtk_idle_add((GtkFunction)file_scan_idle, dead);
 
   return 0;
 }
 
-void scan_start_real() {  
-  GtkWidget* button;
+static void on_scan_unlink() {
+  GList* dlist;
 
-  button = lookup_widget(scan_win, "button355");
-  if (button) gtk_widget_set_sensitive(button, FALSE);
-  else return;
+  dlist = DeadFiles;
+  while (dlist) {
+    dead_t* dead = dlist->data;
+    dlist = dlist->next;
+    
+    if (!gtk_toggle_button_get_active(dead->check)) continue;
 
-  scan_idle = gtk_idle_add(scan_files, NULL);
+    gtk_widget_destroy(GTK_WIDGET(dead->box));
+    unlink(dead->filename);
+    dead_destroy(dead);
+    DeadFiles = g_list_remove(DeadFiles, dead);
+  }
 }
 
+static GtkWidget* create_scan_progress_win() {
+  GtkWidget* win;
+  GtkWidget* vbox;
+  GtkWidget* vbox2;
+  GtkWidget* sw;
+  GtkWidget* hbox;
+  GtkWidget* frame;
+  GtkWidget* button;
+  GtkWidget* sep;
+  GtkWidget* label;
+
+  if (ScanWin) return ScanWin;
+
+  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title(GTK_WINDOW(win), "Scan progress");
+  gtk_window_set_default_size(GTK_WINDOW(win), 500, 300);
+  ScanWin = win;
+  
+  gtk_signal_connect(GTK_OBJECT(win), "destroy",
+		     GTK_SIGNAL_FUNC(on_scan_destroy), NULL);
+  gtk_signal_connect(GTK_OBJECT(win), "delete_event",
+		     GTK_SIGNAL_FUNC(on_scan_delete), NULL);
+
+
+  vbox = gtk_vbox_new(FALSE, 5);
+  gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
+  gtk_container_add(GTK_CONTAINER(win), vbox);
+
+  label = gtk_label_new("In the folder for your incomplete files normally should only be the files lopster currently has in its download list. If there are some more files in it, they will be listed below. You either can delete them, or reconstruct to add them to your download list.");
+  gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+
+  sw = gtk_scrolled_window_new(NULL, NULL);
+  gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
+  gtk_scrolled_window_set_policy
+    (GTK_SCROLLED_WINDOW(sw), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  
+  vbox2 = gtk_vbox_new(FALSE, 5);
+  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sw), vbox2);
+  gtk_viewport_set_shadow_type
+    (GTK_VIEWPORT(GTK_BIN(sw)->child), GTK_SHADOW_NONE);
+  
+  {
+    struct dirent* dent;
+    DIR* dir;
+    char* name;
+    struct stat st;
+    GtkWidget* vbox3;
+    GtkWidget* check;
+    GtkWidget* bar;
+    long size;
+    dead_t* dead;
+
+    dir = opendir(global.incomplete_path);
+    while (dir && (dent = readdir(dir)) != NULL) {
+      if (dent->d_name[0] == '.') continue;
+#ifdef RESUME_DEBUG
+      printf("found file [%s]\n", dent->d_name);
+#endif
+      if (resume_search_short_file(dent->d_name)) {
+#ifdef RESUME_DEBUG
+	printf(" - is in incomplete.list\n");
+#endif
+	continue;
+      }
+      name = g_strdup_printf("%s%c%s", global.incomplete_path,
+			     DIR_SEP, dent->d_name);
+      if (stat(name, &st) < 0) size = 0;
+      else size = st.st_size;
+
+      dead = g_malloc0(sizeof(*dead));
+      dead->filename = name;
+      dead->size = size;
+      dead->fd = -1;
+      dead->start_found = -1;
+
+      vbox3 = gtk_vbox_new(FALSE, 0);
+      gtk_box_pack_start(GTK_BOX(vbox2), vbox3, FALSE, FALSE, 0);
+      dead->box = GTK_BOX(vbox3);
+      
+      check = gtk_check_button_new_with_label(dent->d_name);
+      gtk_box_pack_start(dead->box, check, FALSE, FALSE, 0);
+      dead->check = GTK_TOGGLE_BUTTON(check);
+      //      if (dead->size >= 1024)
+      //	gtk_toggle_button_set_active(dead->check, TRUE);
+
+      hbox = gtk_hbox_new(FALSE, 5);
+      gtk_box_pack_start(dead->box, hbox, FALSE, FALSE, 0);
+      
+      label = gtk_label_new("Waiting");
+      gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+      dead->action = GTK_LABEL(label);
+
+      bar = gtk_progress_bar_new();
+      gtk_box_pack_end(GTK_BOX(hbox), bar, FALSE, FALSE, 0);
+      dead->progress = GTK_PROGRESS_BAR(bar);
+
+      name = g_strdup_printf("Size: %ld (%s)", dead->size,
+			     print_size(tstr[0], dead->size));
+      label = gtk_label_new(name);
+      g_free(name);
+
+      gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+      
+      DeadFiles = g_list_append(DeadFiles, dead);
+    }
+    closedir(dir);
+  }
+
+  if (!DeadFiles) {
+    label = gtk_label_new("No dead files found");
+    gtk_box_pack_start(GTK_BOX(vbox2), label, FALSE, FALSE, 0);
+  }
+
+  sep = gtk_hseparator_new();
+  gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 0);
+
+  frame = gtk_frame_new(NULL);
+  gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
+  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
+
+  hbox = gtk_hbox_new(TRUE, 5);
+  gtk_container_add(GTK_CONTAINER(frame), hbox);
+  gtk_container_set_border_width(GTK_CONTAINER(hbox), 3);
+
+  button = gtk_button_new_with_label("Reconstruct selected");
+  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		     GTK_SIGNAL_FUNC(on_scan_start), NULL);
+
+  button = gtk_button_new_with_label("Delete selected");
+  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		     GTK_SIGNAL_FUNC(on_scan_unlink), NULL);
+  
+  button = gtk_button_new_with_label("Abort");
+  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		     GTK_SIGNAL_FUNC(on_scan_abort), NULL);
+
+  gtk_widget_show_all(win);
+  return win;
+}
+
+
 void resume_scan_dead() {
-  struct dirent* dent;
-  DIR* dir;
-  char* name;
-  struct stat st;
-  GtkCList* clist;
-  int row;
-
-  if (!scan_start()) return;
-  
-  clist = GTK_CLIST(lookup_widget(scan_win, "clist42"));
-
-  dir = opendir(global.incomplete_path);
-  if (!dir) return;
-  while ((dent = readdir(dir)) != NULL) {
-    if (dent->d_name[0] == '.') continue;
-    name = l_strdup_printf("%s/%s", global.incomplete_path,
-			   dent->d_name);
-#ifdef RESUME_DEBUG
-    printf("found file [%s]\n", name);
-#endif
-    if (!resume_search_long_file(name)) {
-#ifdef RESUME_DEBUG
-      printf(" **is not in resume.list\n");
-#endif
-      if (stat(name, &st) >= 0 && st.st_size > 1000) {
-	dead_files = g_list_append(dead_files, name);
-	strcpy(tstr[0], "Waiting");
-	strcpy(tstr[1], name);
-	row = gtk_clist_append(clist, list);
-	gtk_clist_set_row_data(clist, row, name);
-	total_to_go += st.st_size;
-      } else {
-	strcpy(tstr[0], "Skip: too small");
-	strcpy(tstr[1], name);
-	row = gtk_clist_append(clist, list);
-      }
-    }
-  }
-  closedir(dir);
+  create_scan_progress_win();
+  //  ScanIdle = gtk_idle_add(scan_files, NULL);
 }
 
+
+
 void resume_set_size(resume_t* resume, long size) {
   GList* dlist;
   socket_t* socket;
@@ -1756,31 +1843,23 @@ void segment_size_show(GtkWidget* area, int new_size) {
   char str[1024];
   char str2[1024];
   int width, height;
-  int dx, dy;
   GtkStyle* style;
 
   if (!area->window) return;
 
   width = area->allocation.width;
   height = area->allocation.height;
+  style = area->style;
   
-  if (!GTK_WIDGET_REALIZED(global.progressbar))
-    gtk_widget_realize(global.progressbar);
-  style = global.progressbar->style;
-  dx = style->klass->xthickness;
-  dy = style->klass->ythickness;
+  gdk_draw_rectangle(area->window,
+		     style->bg_gc[0], TRUE,
+		     0, 0, width, height);
 
-  gtk_paint_box (style, area->window,
-		 GTK_STATE_NORMAL, GTK_SHADOW_IN, 
-		 NULL, area, "trough",
-		 0, 0, width, height);
-
-  posx = (width - 2*dx) * (new_size-1) / 19;
-  if (posx > width-2*dx) posx = width-2*dx;
+  posx = (width - 3) * (new_size-1) / 19;
+  if (posx > width-3) posx = width-3;
   if (posx > 0)
-    gtk_paint_box(style, area->window,
-		  GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-		  NULL, area, "bar", dy, dy, posx, height-2*dy);
+    gdk_draw_rectangle(area->window, style->bg_gc[3], TRUE,
+		       1, 1, posx, height-3);
 
   global.limit.min_segment_size = (1<<new_size);
   print_bytes(str, global.limit.min_segment_size<<10);
@@ -1788,8 +1867,8 @@ void segment_size_show(GtkWidget* area, int new_size) {
 
   //  posx = (width - gdk_string_width(area->style->font, str2))/2;
   posx = gdk_string_width(style->font, "A");
-  posy = (height + style->font->ascent)/2 - dy +1;
+  posy = (height + style->font->ascent)/2 - 1;
   gdk_draw_text (area->window, style->font,
-		 style->fg_gc[GTK_STATE_NORMAL],
+		 style->fg_gc[3],
 		 posx, posy, str2, strlen (str2));
 }
