@@ -1,4 +1,4 @@
-/* Copyright (C) 2000/2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -15,9 +15,11 @@ resume_t *resume_search_short_file(char *filename);
 resume_t *resume_search_long_file(char *filename);
 resume_t *resume_search_file(file_t * file);
 GList *resume_search_size(long size);
+GList *resume_search_size_real(long size);
 void resume_search(resume_t * resume, net_t* net);
 void resume_search_all(net_t* net, int force);
 void resume_remove_search(resume_t * resume);
+void resume_update_search(resume_t* resume, char* text);
 
 void resume_show(resume_t * resume);
 void resume_queued_update(resume_t* resume);
@@ -52,5 +54,3 @@ void resume_scan_dead();
 void resume_set_size(resume_t* resume, long size);
 void segment_size_show(GtkWidget* area, int new_size);
 
-void scan_end(int close_win);
-void scan_start_real();
