@@ -1,4 +1,4 @@
-/* Copyright (C) 2000/2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -27,14 +27,14 @@
 scheme_t *scheme_new(void) {
   scheme_t *scheme;
 
-  scheme = l_malloc(sizeof(scheme_t));
+  scheme = g_malloc(sizeof(scheme_t));
   scheme->name = NULL;
   scheme->colors = NULL;
   scheme->server_prefix = NULL;
   scheme->client_prefix = NULL;
   scheme->join_prefix = NULL;
   scheme->part_prefix = NULL;
-  scheme->style = gtk_style_new();
+  scheme->style = gtk_style_copy(gtk_widget_get_style(global.win));
   return scheme;
 }
 
@@ -64,7 +64,7 @@ static scheme_t *verify_scheme(scheme_t* scheme) {
 
   if (!scheme) {
     scheme = scheme_new();
-    scheme->name = l_strdup("default");
+    scheme->name = g_strdup("default");
     mods = 1;
   }
   if (!scheme) return NULL;
@@ -188,19 +188,19 @@ static scheme_t *verify_scheme(scheme_t* scheme) {
   }
   
   if (!scheme->client_prefix) {
-    scheme->client_prefix = l_strdup("%K=%c=%C= ");
+    scheme->client_prefix = g_strdup("%K=%c=%C= ");
     mods = 1;
   }
   if (!scheme->server_prefix) {
-    scheme->server_prefix = l_strdup("%K;%r;%R; ");
+    scheme->server_prefix = g_strdup("%K;%r;%R; ");
     mods = 1;
   }
   if (!scheme->join_prefix) {
-    scheme->join_prefix = l_strdup("%K-%g=%G> ");
+    scheme->join_prefix = g_strdup("%K-%g=%G> ");
     mods = 1;
   }    
   if (!scheme->part_prefix) {
-    scheme->part_prefix = l_strdup("%G<%g=%K- ");
+    scheme->part_prefix = g_strdup("%G<%g=%K- ");
     mods = 1;
   }
 
@@ -215,7 +215,7 @@ void scheme_destroy(scheme_t * scheme) {
   style_t *style;
 
   if (!scheme) return;
-  if (scheme->name) l_free(scheme->name);
+  if (scheme->name) g_free(scheme->name);
 
   for (dlist = scheme->colors; dlist; dlist = dlist->next) {
     style = dlist->data;
@@ -223,13 +223,13 @@ void scheme_destroy(scheme_t * scheme) {
   }
 
   g_list_free(scheme->colors);
-  l_free(scheme->server_prefix);
-  l_free(scheme->client_prefix);
-  l_free(scheme->join_prefix);
-  l_free(scheme->part_prefix);
+  g_free(scheme->server_prefix);
+  g_free(scheme->client_prefix);
+  g_free(scheme->join_prefix);
+  g_free(scheme->part_prefix);
   gtk_style_unref(scheme->style);
 
-  l_free(scheme);
+  g_free(scheme);
 }
 
 static scheme_t* scheme_load_2(char* filename, char* name) {
@@ -250,26 +250,26 @@ static scheme_t* scheme_load_2(char* filename, char* name) {
   }
 
   scheme = scheme_new();
-  scheme->name = l_strdup(name);
+  scheme->name = g_strdup(name);
 
   fgets(temp, sizeof(temp), file);
   while (mfgets(temp, sizeof(temp), file)) {
     if (temp[0] == '#') continue;
     if (!strncasecmp(temp, "ServerPrefix", 12)) {
       pos = strchr(temp, '=') + 1;
-      scheme->server_prefix = l_strdup(pos);
+      scheme->server_prefix = g_strdup(pos);
       continue;
     } else if (!strncasecmp(temp, "ClientPrefix", 12)) {
       pos = strchr(temp, '=') + 1;
-      scheme->client_prefix = l_strdup(pos);
+      scheme->client_prefix = g_strdup(pos);
       continue;
     } else if (!strncasecmp(temp, "JoinPrefix", 10)) {
       pos = strchr(temp, '=') + 1;
-      scheme->join_prefix = l_strdup(pos);
+      scheme->join_prefix = g_strdup(pos);
       continue;
     } else if (!strncasecmp(temp, "PartPrefix", 10)) {
       pos = strchr(temp, '=') + 1;
-      scheme->part_prefix = l_strdup(pos);
+      scheme->part_prefix = g_strdup(pos);
       continue;
     }
     ///
@@ -413,7 +413,7 @@ void scheme_update_list(char *current) {
 
   // dont know why i have to duplicate the string here
   // original would be messed up till end of function
-  if (current) cur = l_strdup(current);
+  if (current) cur = g_strdup(current);
 
   sprintf(dirname, "%s/schemes", global.options.config_dir);
 
@@ -428,8 +428,8 @@ void scheme_update_list(char *current) {
       stat(filename, &buf);
       if (S_ISREG(buf.st_mode)) {
 	if (scheme_is_valid(entry->d_name)) {
-	  list2 = g_list_insert_sorted(list2, l_strdup(entry->d_name),
-				       (GCompareFunc) l_strcasecmp);
+	  list2 = g_list_insert_sorted(list2, g_strdup(entry->d_name),
+				       (GCompareFunc) g_strcasecmp);
 	}
       }
     }
@@ -440,14 +440,14 @@ void scheme_update_list(char *current) {
   if (list2) {
     gtk_combo_set_popdown_strings(GTK_COMBO(temp), list2);
     for (dlist = list2; dlist; dlist = dlist->next)
-      l_free(dlist->data);
+      g_free(dlist->data);
     g_list_free(list2);
   }
 
   temp = lookup_widget(global.options_win, "combo_entry15");
   if (cur) {
     gtk_entry_set_text(GTK_ENTRY(temp), cur);
-    l_free(cur);
+    g_free(cur);
   }
 }
 
@@ -595,11 +595,11 @@ style_t *style_new(char *name, char *fore, char *back, char *font)
   style_t *style;
   GdkColor color;
 
-  style = l_malloc(sizeof(style_t));
-  style->name = l_strdup(name);
+  style = g_malloc(sizeof(style_t));
+  style->name = g_strdup(name);
 
   if (fore) {
-    style->n_fore = l_strdup(fore);
+    style->n_fore = g_strdup(fore);
     if (strcmp(fore, "NULL")) {
       gdk_color_parse(fore, &color);
       style->fore = gdk_color_copy(&color);
@@ -607,12 +607,12 @@ style_t *style_new(char *name, char *fore, char *back, char *font)
       style->fore = NULL;
     }
   } else {
-    style->n_fore = l_strdup("NULL");
+    style->n_fore = g_strdup("NULL");
     style->fore = NULL;
   }
 
   if (back) {
-    style->n_back = l_strdup(back);
+    style->n_back = g_strdup(back);
     if (strcmp(back, "NULL")) {
       gdk_color_parse(back, &color);
       style->back = gdk_color_copy(&color);
@@ -620,20 +620,20 @@ style_t *style_new(char *name, char *fore, char *back, char *font)
       style->back = NULL;
     }
   } else {
-    style->n_back = l_strdup("NULL");
+    style->n_back = g_strdup("NULL");
     style->back = NULL;
   }
 
 
   if (font) {
-    style->n_font = l_strdup(font);
+    style->n_font = g_strdup(font);
     if (strcmp(font, "NULL")) {
       style->font = gdk_fontset_load(font);
     } else {
       style->font = NULL;
     }
   } else {
-    style->n_font = l_strdup("NULL");
+    style->n_font = g_strdup("NULL");
     style->font = NULL;
   }
 
@@ -641,12 +641,12 @@ style_t *style_new(char *name, char *fore, char *back, char *font)
 }
 
 void style_destroy(style_t * style) {
-  if (style->name) l_free(style->name);
-  if (style->n_fore) l_free(style->n_fore);
-  if (style->n_back) l_free(style->n_back);
-  if (style->n_font) l_free(style->n_font);
+  if (style->name) g_free(style->name);
+  if (style->n_fore) g_free(style->n_fore);
+  if (style->n_back) g_free(style->n_back);
+  if (style->n_font) g_free(style->n_font);
 
-  l_free(style);
+  g_free(style);
 }
 
 void style_update(scheme_t * scheme,
@@ -663,8 +663,8 @@ void style_update(scheme_t * scheme,
   }
 
   if (fore) {
-    if (style->n_fore) l_free(style->n_fore);
-    style->n_fore = l_strdup(fore);
+    if (style->n_fore) g_free(style->n_fore);
+    style->n_fore = g_strdup(fore);
     if (strcmp(fore, "NULL")) {
       gdk_color_parse(fore, &color);
       style->fore = gdk_color_copy(&color);
@@ -673,8 +673,8 @@ void style_update(scheme_t * scheme,
     }
   }
   if (back) {
-    if (style->n_back) l_free(style->n_back);
-    style->n_back = l_strdup(back);
+    if (style->n_back) g_free(style->n_back);
+    style->n_back = g_strdup(back);
     if (strcmp(back, "NULL")) {
       gdk_color_parse(back, &color);
       style->back = gdk_color_copy(&color);
@@ -683,8 +683,8 @@ void style_update(scheme_t * scheme,
     }
   }
   if (font) {
-    if (style->n_font) l_free(style->n_font);
-    style->n_font = l_strdup(font);
+    if (style->n_font) g_free(style->n_font);
+    style->n_font = g_strdup(font);
     if (strcmp(font, "NULL")) {
       style->font = gdk_fontset_load(font);
     } else {
@@ -701,7 +701,7 @@ style_t *style_search(scheme_t * scheme, char *name) {
 
   for (dlist = scheme->colors; dlist; dlist = dlist->next) {
     style = dlist->data;
-    if (!l_strcasecmp(name, style->name)) return style;
+    if (!g_strcasecmp(name, style->name)) return style;
   }
   return NULL;
 }
