@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2003 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -37,9 +37,14 @@
 #include "utils.h"
 #include "files.h"
 #include "filetips.h"
+#include "file_tree.h"
 #include "dialog.h"
 #include "sarray.h"
 
+GdkColor library_color = {0, 45874, 24247, 5242 };
+GdkColor resume_color = {0, 0, 0, 65535};
+GdkColor normal_color = {0, 0, 0, 0};
+
 typedef int (*IntFunc) (char *str);
 
 #define WORD_CHAR(c) \
@@ -63,27 +68,27 @@ static char *Destination(int id) {
 }
 
 static int get_real_speed(char *trans) {
-  if (!l_strcasecmp(trans, "not specified"))
+  if (!g_strcasecmp(trans, "not specified"))
     return 0;
   else
     return speed2int(trans);
 }
 
 static int get_real_int(char *trans) {
-  if (!l_strcasecmp(trans, "not specified"))
+  if (!g_strcasecmp(trans, "not specified"))
     return 0;
   else
     return atoi(trans);
 }
 
 int life2int(char *dest) {
-  if (!l_strcasecmp(dest, "Ignore"))
+  if (!g_strcasecmp(dest, "Ignore"))
     return PATTERN_IGNORE;
-  else if (!l_strcasecmp(dest, "Save"))
+  else if (!g_strcasecmp(dest, "Save"))
     return PATTERN_SAVE;
-  else if (!l_strcasecmp(dest, "Template"))
+  else if (!g_strcasecmp(dest, "Template"))
     return PATTERN_TEMPLATE;
-  else if (!l_strcasecmp(dest, "Autosearch"))
+  else if (!g_strcasecmp(dest, "Autosearch"))
     return PATTERN_SEARCH;
   else
     return PATTERN_TEMPORARY;
@@ -156,9 +161,9 @@ on_search_resize_column               (GtkCList        *clist,
 
 static gboolean
 on_list_leave_notify_event(GtkWidget * widget ATTR_UNUSED,
-			   GdkEventCrossing * event ATTR_UNUSED, gpointer user_data ATTR_UNUSED)
+			   GdkEventCrossing* event, gpointer user_data ATTR_UNUSED)
 {
-  filetips_show(NULL, -1, -1, NULL);
+  filetips_show(NULL, event->x_root, event->y_root, NULL);
   return FALSE;
 }
 
@@ -172,12 +177,7 @@ on_search_motion_notify_event(GtkWidget * widget,
   if (!gtk_clist_get_selection_info(GTK_CLIST(widget),
 				    (int) event->x, (int) event->y,
 				    &row, &column)) {
-    filetips_show(NULL, -1, -1, NULL);
-    return FALSE;
-  }
-
-  if (column != 2) {
-    filetips_show(NULL, -1, -1, NULL);
+    filetips_show(NULL, event->x_root, event->y_root, NULL);
     return FALSE;
   }
 
@@ -186,7 +186,7 @@ on_search_motion_notify_event(GtkWidget * widget,
   file = gtk_clist_get_row_data(GTK_CLIST(widget), row);
   if (!file) return FALSE;
 
-  filetips_show(widget, row, column, file);
+  filetips_show(widget, event->x_root, event->y_root, file);
   return FALSE;
 }
 
@@ -220,6 +220,8 @@ void search_create_page(search_t * search)
   gtk_widget_show(search_list);
   gtk_widget_set_events(search_list,
 			GDK_POINTER_MOTION_MASK | GDK_LEAVE_NOTIFY_MASK);
+  gtk_ctree_set_line_style(GTK_CTREE(search_list),
+			   GTK_CTREE_LINES_DOTTED);
 
   gtk_clist_set_sort_column(GTK_CLIST(search_list), 1);
   //  gtk_clist_set_auto_sort(GTK_CLIST(search_list), TRUE);
@@ -316,9 +318,9 @@ void search_create_page(search_t * search)
                       GTK_SIGNAL_FUNC (on_search_resize_column),
                       NULL);
 
-  str = l_strdup_printf("%s: %s\n%s",
+  str = g_strdup_printf("%s: %s\n%s",
 			Destination(pattern->destination),
-			MimeNames(pattern->media_type, 1),
+			int2media(pattern->media_type, 1),
 			pattern->include);
 
   tab_label = gtk_hbox_new (FALSE, 2);
@@ -328,7 +330,7 @@ void search_create_page(search_t * search)
   label = gtk_label_new(str);
   gtk_widget_show(label);
   gtk_box_pack_start (GTK_BOX (tab_label), label, TRUE, TRUE, 0);
-  l_free(str);
+  g_free(str);
 
   button = gtk_button_new();
   gtk_widget_show(button);
@@ -400,12 +402,8 @@ GtkWidget *create_search_list_popup(file_t * file)
 
   if (file) {
     if (!file->user || file->user[0] == 0) {
-      download = gtk_menu_item_new_with_label("Open file");
-      gtk_widget_show(download);
-      gtk_container_add(GTK_CONTAINER(popup), download);
-      gtk_signal_connect(GTK_OBJECT(download), "activate",
-			 GTK_SIGNAL_FUNC(on_play_file3_activate), 
-			 file);
+      item = create_open_menu(file->media_type, file->longname, 1);
+      gtk_container_add(GTK_CONTAINER(popup), item);
     } else if (item_num < 2) {
       download = gtk_menu_item_new_with_label("Download");
       gtk_widget_show(download);
@@ -576,7 +574,7 @@ gint search_compare(GtkCList * clist,
     else if (atoi(text1) > atoi(text2)) return 1;
     else return 0;
   } else if (clist->sort_column == 1) {
-    return l_strcasecmp(file1->shortname, file2->shortname);
+    return g_strcasecmp(file1->shortname, file2->shortname);
   } else if (clist->sort_column == 2) {
     if (file1->size < file2->size)
       return -1;
@@ -596,7 +594,7 @@ gint search_compare(GtkCList * clist,
       return 1;
     return 0;
   } else if (clist->sort_column == 5) {
-    return l_strcasecmp(file1->user, file2->user);
+    return g_strcasecmp(file1->user, file2->user);
   } else if (clist->sort_column == 6) {
     unsigned long ip1 = ntohl(file1->ip);
     unsigned long ip2 = ntohl(file2->ip);
@@ -631,7 +629,7 @@ gint check_search_timeout(gpointer data) {
   if (si->last_result + global.options.search_timeout 
       < global.current_time) {
     search_finish(search, si->net, 1);
-    l_free(data);
+    g_free(data);
     return 0;
   }
   return 1;
@@ -688,7 +686,7 @@ gint search_update(gpointer data) {
 search_info_t* search_info_new(net_t* net) {
   search_info_t* st;
 
-  st = l_malloc(sizeof(*st));
+  st = g_malloc(sizeof(*st));
   st->net = net;
   st->timer = -1;
   st->timeout = 0;
@@ -709,7 +707,7 @@ void search_info_destroy(search_info_t* si) {
   if (si->timer != -1) {
     gtk_timeout_remove(si->timer);
   }
-  l_free(si);
+  g_free(si);
 }
 
 void search_info_remove(search_t* search, net_t* net) {
@@ -768,7 +766,7 @@ void napster_search(search_t * search, net_t* net) {
     si->active = 1;
     si->last_result = global.current_time;
     if (si->timeout > 0 && si->timer == -1 &&
-	(p=(void **)l_malloc(sizeof(void *)*2))){
+	(p=(void **)g_malloc(sizeof(void *)*2))){
       p[0] = search;
       p[1] = si;
       si->timer = gtk_timeout_add(SEARCH_TO_IVAL,
@@ -872,7 +870,7 @@ void search_pattern_insert(search_pattern_t* pattern, int save) {
   strcpy(tstr[0], pattern->name);
   sprintf(tstr[1], "%s: %s",
 	  Destination(pattern->destination),
-	  MimeNames(pattern->media_type, 1));
+	  int2media(pattern->media_type, 1));
   strcpy(tstr[2], LifeTime(pattern->level));
 
   row = gtk_clist_append(clist, list);
@@ -884,8 +882,8 @@ void search_pattern_insert(search_pattern_t* pattern, int save) {
 }
 
 int search_pattern_cmp(search_pattern_t* p1, search_pattern_t* p2) {
-  if (l_strcasecmp(p1->include, p2->include)) return 1;
-  if (l_strcasecmp(p1->exclude, p2->exclude)) return 1;
+  if (g_strcasecmp(p1->include, p2->include)) return 1;
+  if (g_strcasecmp(p1->exclude, p2->exclude)) return 1;
   if (p1->max_results != p2->max_results) return 1;
   if (p1->media_type != p2->media_type) return 1;
   if (p1->destination != p2->destination) return 1;
@@ -957,13 +955,13 @@ static int find_free_pattern_id() {
 }
 
 static void search_pattern_reset(search_pattern_t* search) {
-  if (search->include)    l_free(search->include);
-  if (search->exclude)    l_free(search->exclude);
+  if (search->include)    g_free(search->include);
+  if (search->exclude)    g_free(search->exclude);
 
   search->include = NULL;
   search->exclude = NULL;
 
-  search->media_type = MIME_MP3;
+  search->media_type = MEDIA_MP3;
   search->destination = DEST_NAPSTER;
   search->bitrate_lo = 0;
   search->bitrate_hi = 0;
@@ -981,7 +979,7 @@ static void search_pattern_reset(search_pattern_t* search) {
 search_pattern_t *search_pattern_new() {
   search_pattern_t *pattern;
 
-  pattern = l_malloc(sizeof(*pattern));
+  pattern = g_malloc(sizeof(*pattern));
 
   pattern->name = NULL;
   pattern->include = NULL;
@@ -1038,7 +1036,7 @@ static void search_pattern_get(search_pattern_t* search) {
 
   search->destination = 0;
 
-  text = l_strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
+  text = g_strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
   if (text[0] == ':') {
     pos = arg(text, 0)+1;
     search->destination = extract_destination(pos);
@@ -1046,14 +1044,14 @@ static void search_pattern_get(search_pattern_t* search) {
   } else {
     pos = text;
   }
-  if (pos) search->include = l_strdup(pos);
-  else search->include = l_strdup("");
-  search->name = l_strdup(search->include);
+  if (pos) search->include = g_strdup(pos);
+  else search->include = g_strdup("");
+  search->name = g_strdup(search->include);
 
-  l_free(text);
+  g_free(text);
 
   temp = lookup_widget(global.win, "entry69");
-  search->exclude = l_strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
+  search->exclude = g_strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
   
   temp = lookup_widget(global.win, "search_results");
   search->max_results =
@@ -1061,7 +1059,7 @@ static void search_pattern_get(search_pattern_t* search) {
 
   temp = lookup_widget(global.win, "combo_entry17");
   if (GTK_WIDGET_IS_SENSITIVE(temp))
-    search->media_type = mime2int(gtk_entry_get_text(GTK_ENTRY(temp)));
+    search->media_type = media2int(gtk_entry_get_text(GTK_ENTRY(temp)));
 
   /* if not specified with the search string, get destination
    * from buttons */
@@ -1136,10 +1134,10 @@ search_pattern_include(search_pattern_t* pattern, char* token) {
 
   if (!token || *token == 0) return;
   if (pattern->include) {
-    new_inc = l_strdup_printf("%s %s", pattern->include, token);
-    l_free(pattern->include);
+    new_inc = g_strdup_printf("%s %s", pattern->include, token);
+    g_free(pattern->include);
   } else {
-    new_inc = l_strdup(token);
+    new_inc = g_strdup(token);
   }
   pattern->include = new_inc;
 }
@@ -1150,10 +1148,10 @@ search_pattern_exclude(search_pattern_t* pattern, char* token) {
 
   if (!token || *token == 0) return;
   if (pattern->exclude) {
-    new_exc = l_strdup_printf("%s %s", pattern->exclude, token);
-    l_free(pattern->exclude);
+    new_exc = g_strdup_printf("%s %s", pattern->exclude, token);
+    g_free(pattern->exclude);
   } else {
-    new_exc = l_strdup(token);
+    new_exc = g_strdup(token);
   }
   pattern->exclude = new_exc;
 }
@@ -1177,7 +1175,7 @@ void send_search_request_from_string(char* data) {
       search_pattern_exclude(pattern, token);
     } else if (*token == '<') {
       token++;
-      pattern->media_type = mime2int(token);
+      pattern->media_type = media2int(token);
     } else {
       if (*token == '+') token++;
       search_pattern_include(pattern, token);
@@ -1188,7 +1186,7 @@ void send_search_request_from_string(char* data) {
     search_pattern_destroy(pattern);
     return;
   }
-  pattern->name = l_strdup(pattern->include);
+  pattern->name = g_strdup(pattern->include);
 
   pattern2 = search_pattern_find_same(pattern);
   if (pattern2) {
@@ -1356,6 +1354,7 @@ void search_insert_file_real(GtkCTree* ctree, file_t * file) {
   GdkPixmap *pixmap = NULL;
   GdkBitmap *mask = NULL;
   GtkCTreeNode* node;
+  GList * tmp_list;
 
   node = search_find_node(ctree, file);
   file->visible = 1;
@@ -1416,16 +1415,24 @@ void search_insert_file_real(GtkCTree* ctree, file_t * file) {
 			       pixmap, mask, pixmap, mask,
 			       FALSE, FALSE);
   gtk_ctree_node_set_row_data(ctree, node, file);
+  
+  /* mark library file and resume with color */
+  if((tmp_list = resume_search_size_real(file->size)) != NULL) {
+    gtk_ctree_node_set_foreground(ctree, node, &resume_color);
+    g_list_free(tmp_list);
+  } else if ((tmp_list = file_tree_search_filesize(FILE_TREE(global.lib), file)) != NULL)  {
+    gtk_ctree_node_set_foreground(ctree, node, &library_color);
+    g_list_free(tmp_list);
+  }
 }
 
 
-int file_cmp(file_t * file1, file_t * file2)
-{
+int file_cmp(file_t * file1, file_t * file2) {
   if (file1->size != file2->size)
     return 1;
   if (strcmp(file1->winname, file2->winname))
     return 2;
-  if (l_strcasecmp(file1->user, file2->user))
+  if (g_strcasecmp(file1->user, file2->user))
     return 3;
   //  if (file1->bitrate != file2->bitrate) return 4;
   //  if (file1->frequency != file2->frequency) return 5;
@@ -1484,14 +1491,14 @@ void search_pattern_load() {
   char *pos;
   char *pos2;
 
-  filename = l_strdup_printf("%s%csearches.save", 
+  filename = g_strdup_printf("%s%csearches.save", 
 			     global.options.config_dir, DIR_SEP);
   if ((file = fopen(filename, "r")) == NULL) {
     global.status.search_read = 1;
-    l_free(filename);
+    g_free(filename);
     return;
   }
-  l_free(filename);
+  g_free(filename);
 
   search = NULL;
   while (mfgets(line, sizeof(line), file)) {
@@ -1505,24 +1512,24 @@ void search_pattern_load() {
 	pos2[0] = 0;
 	if (search) search_pattern_insert(search, 0);
 	search = search_pattern_new();
-	search->name = l_strdup(pos);
+	search->name = g_strdup(pos);
       }
     } else if (search) {
       if (!strncasecmp(line, "contains", 8)) {
 	pos = strchr(line, '=') + 1;
-	search->include = l_strdup(pos);
+	search->include = g_strdup(pos);
       } else if (!strncasecmp(line, "excludes", 8)) {
 	pos = strchr(line, '=') + 1;
-	search->exclude = l_strdup(pos);
+	search->exclude = g_strdup(pos);
       } else if (!strncasecmp(line, "MaxResults", 10)) {
 	pos = strchr(line, '=') + 1;
 	search->max_results = atoi(pos);
       } else if (!strncasecmp(line, "MediaType", 9)) {
 	pos = strchr(line, '=') + 1;
 	if (*pos == 0)	// backward compatible
-	  search->media_type = MIME_MP3;
+	  search->media_type = MEDIA_MP3;
 	else
-	  search->media_type = mime2int(pos);
+	  search->media_type = media2int(pos);
       } else if (!strncasecmp(line, "Destination", 11)) {
 	pos = strchr(line, '=') + 1;
 	search->destination = atoi(pos);
@@ -1599,29 +1606,29 @@ void search_pattern_save() {
   if (!clist) return;
 
   filename =
-    l_strdup_printf("%s%csearches.save",
+    g_strdup_printf("%s%csearches.save",
 		    global.options.config_dir, DIR_SEP);
   if ((file = fopen(filename, "w")) == NULL) {
     g_warning("Could not write searches");
-    l_free(filename);
+    g_free(filename);
     return;
   }
-  l_free(filename);
+  g_free(filename);
 
   for (row = 0; row < clist->rows; row++) {
     search = gtk_clist_get_row_data(clist, row);
     if (!search || search->level < PATTERN_SAVE) continue;
 
-    pos2 = l_strdup(search->name);
+    pos2 = g_strdup(search->name);
     for (pos = pos2; *pos; pos++)
       if (*pos == '\n')
 	*pos = ' ';
     fprintf(file, "PATTERN [%s]\n", pos2);
-    l_free(pos2);
+    g_free(pos2);
     fprintf(file, "Contains=%s\n", search->include?search->include:"");
     fprintf(file, "Excludes=%s\n", search->exclude?search->exclude:"");
     fprintf(file, "MaxResults=%d\n", search->max_results);
-    fprintf(file, "MediaType=%s\n", MimeNames(search->media_type, 1));
+    fprintf(file, "MediaType=%s\n", int2media(search->media_type, 1));
     fprintf(file, "Destination=%d\n", search->destination);
     fprintf(file, "Bitrate=%d:%d\n", 
 	    search->bitrate_lo, search->bitrate_hi);
@@ -1641,10 +1648,10 @@ void search_pattern_save() {
 
 void search_pattern_destroy(search_pattern_t * search) {
   if (!search) return;
-  if (search->name)       l_free(search->name);
-  if (search->include)    l_free(search->include);
-  if (search->exclude)    l_free(search->exclude);
-  l_free(search);
+  if (search->name)       g_free(search->name);
+  if (search->include)    g_free(search->include);
+  if (search->exclude)    g_free(search->exclude);
+  g_free(search);
 }
 
 int search_pattern_fits_file(search_pattern_t * search, file_t * file,
@@ -1668,27 +1675,27 @@ int search_pattern_fits_file(search_pattern_t * search, file_t * file,
       (search->duration_hi && file->duration > search->duration_hi))
     return 0;
   
-  text = l_strdup(search->include);
+  text = g_strdup(search->include);
   pos = search_arg(text);
   while (pos) {
     if (!strcasestr(file->longname, pos)) {
-      l_free(text);
+      g_free(text);
       return 0;
     }
     pos = search_arg(NULL);
   }
-  l_free(text);
+  g_free(text);
 
-  text = l_strdup(search->exclude);
+  text = g_strdup(search->exclude);
   pos = arg(text, 0);
   while (pos) {
     if (strcasestr(file->longname, pos))
       return 0;
     pos = arg(NULL, 0);
   }
-  l_free(text);
+  g_free(text);
 
-  if (with_type && search->media_type != MIME_NONE &&
+  if (with_type && search->media_type != MEDIA_NONE &&
       search->media_type != file->media_type)
     return 0;
 
@@ -1736,7 +1743,7 @@ void search_pattern_show(search_pattern_t * pattern, int mode) {
     temp = lookup_widget(global.win, "search_results");
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), pattern->max_results);
     temp = lookup_widget(global.win, "combo_entry17");
-    gtk_entry_set_text(GTK_ENTRY(temp), MimeNames(pattern->media_type, 1));
+    gtk_entry_set_text(GTK_ENTRY(temp), int2media(pattern->media_type, 1));
     
     search_set_destination(pattern->destination);
 
@@ -1858,11 +1865,11 @@ search_pattern_t *search_pattern_copy(search_pattern_t * search) {
 
   pattern = search_pattern_new();
   if (search->name)
-    pattern->name = l_strdup(search->name);
+    pattern->name = g_strdup(search->name);
   if (search->include)
-    pattern->include = l_strdup(search->include);
+    pattern->include = g_strdup(search->include);
   if (search->exclude)
-    pattern->exclude = l_strdup(search->exclude);
+    pattern->exclude = g_strdup(search->exclude);
   pattern->media_type = search->media_type;
   pattern->destination = search->destination;
   pattern->bitrate_lo = search->bitrate_lo;
@@ -1907,7 +1914,7 @@ static void search_delete(void* item) {
 search_t *search_new() {
   search_t *search;
 
-  search = l_malloc(sizeof(search_t));
+  search = g_malloc(sizeof(search_t));
   search->results = sarray_new(search_compare_item, search_delete);
   search->resume = 0;
   search->local = 0;
@@ -2034,7 +2041,7 @@ void search_destroy(search_t * search) {
       net->queued_searches =
 	g_list_remove(net->queued_searches, search);
     }
-    l_free(si);
+    g_free(si);
   }
 
   if (search->updater != -1) {
@@ -2045,7 +2052,7 @@ void search_destroy(search_t * search) {
   search_clear(search);
   search_pattern_destroy(search->pattern);
   g_list_free(search->net_info);
-  l_free(search);
+  g_free(search);
 }
 
 void search_remove(search_t * search) {
@@ -2064,8 +2071,7 @@ void search_remove(search_t * search) {
   }
 }
 
-void search_update_stats(search_t * search, int check)
-{
+void search_update_stats(search_t * search, int check) {
   char t[1024];
   GtkWidget *label;
   GtkNotebook* notebook;
@@ -2337,7 +2343,7 @@ void search_queue(search_t * search, net_t* net, int top) {
 
     // clearing old results of finished searches
     if (!search->resume) search_clear_old(search, net);
-    else resume_queued_update((resume_t*)(search->link));
+    else ((resume_t*)(search->link))->needs_update = 1;
   }
 }
 
@@ -2377,7 +2383,7 @@ int search_finish(search_t* search, net_t* net, int next) {
 #endif
       search_info_remove(search, net);
       if (search->resume) 
-	resume_queued_update((resume_t*)(search->link));
+	((resume_t*)(search->link))->needs_update = 1;
       else if (search->updater == -1) {
 	search->updater = gtk_idle_add(search_update, search);
       }
@@ -2492,8 +2498,8 @@ int search_pattern_rename_func(clist_rename_t* cr) {
   if (row < 0) return 0;
   
   pattern = cr->data;
-  if (pattern->name) l_free(pattern->name);
-  pattern->name = l_strdup(cr->new_text);
+  if (pattern->name) g_free(pattern->name);
+  pattern->name = g_strdup(cr->new_text);
 
   //  gtk_clist_set_text(cr->clist, row, cr->col, pattern->name);  
   search_pattern_visual_update(pattern);
@@ -2808,3 +2814,81 @@ void search_remove_network(net_t* net) {
     }
   }
 }
+
+GtkWidget*
+create_search_legend_menu (void) {
+  GtkWidget *menu1;
+  GtkAccelGroup *menu1_accels;
+  GtkWidget *library1;
+  GtkWidget *resume1;
+  GtkWidget *normal1;
+  GtkStyle * style;
+  GdkColor white = {0, 65535, 65535, 65535};
+
+  menu1 = gtk_menu_new ();
+  gtk_object_set_data (GTK_OBJECT (menu1), "menu1", menu1);
+  menu1_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (menu1));
+  
+  style = gtk_style_copy(gtk_widget_get_style(menu1));
+  style->bg[GTK_STATE_NORMAL] = white;
+  style->bg[GTK_STATE_ACTIVE] = white;
+  style->bg[GTK_STATE_PRELIGHT] = white;
+  style->bg[GTK_STATE_SELECTED] = white;
+  gtk_widget_set_style(menu1, style);
+ 
+  normal1 = gtk_menu_item_new_with_label ("- Normal");
+  gtk_widget_show (normal1);
+  gtk_container_add (GTK_CONTAINER (menu1), normal1);
+  gtk_menu_item_right_justify (GTK_MENU_ITEM (normal1));
+  GTK_WIDGET_UNSET_FLAGS (normal1, GTK_SENSITIVE);
+  style = gtk_style_copy(gtk_widget_get_style(GTK_BIN(normal1)->child));
+  style->fg[GTK_STATE_NORMAL] = normal_color;
+  style->fg[GTK_STATE_ACTIVE] = normal_color;
+  style->fg[GTK_STATE_PRELIGHT] = normal_color;
+  style->fg[GTK_STATE_SELECTED] = normal_color;
+  gtk_widget_set_style(GTK_BIN(normal1)->child, style);
+  style = gtk_style_copy(gtk_widget_get_style(normal1));
+  style->bg[GTK_STATE_NORMAL] = white;
+  style->bg[GTK_STATE_ACTIVE] = white;
+  style->bg[GTK_STATE_PRELIGHT] = white;
+  style->bg[GTK_STATE_SELECTED] = white;
+  gtk_widget_set_style(normal1, style);
+  
+  library1 = gtk_menu_item_new_with_label ("- Already in library");
+  gtk_widget_show (library1);
+  gtk_container_add (GTK_CONTAINER (menu1), library1);
+  gtk_menu_item_right_justify (GTK_MENU_ITEM (library1));
+  GTK_WIDGET_UNSET_FLAGS (library1, GTK_SENSITIVE);
+  style = gtk_style_copy(gtk_widget_get_style(GTK_BIN(library1)->child));
+  style->fg[GTK_STATE_NORMAL] = library_color;
+  style->fg[GTK_STATE_ACTIVE] = library_color;
+  style->fg[GTK_STATE_PRELIGHT] = library_color;
+  style->fg[GTK_STATE_SELECTED] = library_color;
+  gtk_widget_set_style(GTK_BIN(library1)->child, style);
+  style = gtk_style_copy(gtk_widget_get_style(library1));
+  style->bg[GTK_STATE_NORMAL] = white;
+  style->bg[GTK_STATE_ACTIVE] = white;
+  style->bg[GTK_STATE_PRELIGHT] = white;
+  style->bg[GTK_STATE_SELECTED] = white;
+  gtk_widget_set_style(library1, style);
+ 
+  resume1 = gtk_menu_item_new_with_label ("- Can be used for resume");
+  gtk_widget_show (resume1);
+  gtk_container_add (GTK_CONTAINER (menu1), resume1);
+  gtk_menu_item_right_justify (GTK_MENU_ITEM (resume1));
+  GTK_WIDGET_UNSET_FLAGS (resume1, GTK_SENSITIVE);
+  style = gtk_style_copy(gtk_widget_get_style(GTK_BIN(resume1)->child));
+  style->fg[GTK_STATE_NORMAL] = resume_color;
+  style->fg[GTK_STATE_ACTIVE] = resume_color;
+  style->fg[GTK_STATE_PRELIGHT] = resume_color;
+  style->fg[GTK_STATE_SELECTED] = resume_color;
+  gtk_widget_set_style(GTK_BIN(resume1)->child, style); 
+  style = gtk_style_copy(gtk_widget_get_style(resume1));
+  style->bg[GTK_STATE_NORMAL] = white;
+  style->bg[GTK_STATE_ACTIVE] = white;
+  style->bg[GTK_STATE_PRELIGHT] = white;
+  style->bg[GTK_STATE_SELECTED] = white;
+  gtk_widget_set_style(resume1, style);
+ 
+  return menu1;
+}
