@@ -1,4 +1,4 @@
-/* Copyright (C) 2000/2002 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -27,6 +27,7 @@
 #include "share2.h"
 #include "mp3_ext.h"
 #include "ogg_ext.h"
+#include "flac_ext.h"
 #include "md5.h"
 #include "scheme.h"
 #include "callbacks.h"
@@ -39,6 +40,7 @@
 #include "file_tree.h"
 #include "utils.h"
 #include "files.h"
+#include "mtypes.h"
 
 static int lib_version = 0;
 static int stype = 0;
@@ -60,18 +62,15 @@ static void file_parse_header(file_t * file) {
     file->bitrate = 32;
     file->frequency = 32000;
     file->duration = 60;
-  } else if (!l_strcasecmp(suf, "MP3")) {
+  } else if (!g_strcasecmp(suf, "MP3")) {
     file_parse_mp3_header(file);
 #ifdef HAVE_OGG    
-  } else if (!l_strcasecmp(suf, "OGG")) {
-    // for now set dummy values
-    file->bitrate = 32;
-    file->frequency = 32000;
-    file->duration = 60;
-
-    // First try to parse vorbis info from the file ...
-    // 
+  } else if (!g_strcasecmp(suf, "OGG")) {
     file_parse_ogg_header(file);
+#endif
+#ifdef HAVE_FLAC    
+  } else if (!g_strcasecmp(suf, "FLAC")) {
+     file_parse_flac_header(file);
 #endif
   } else {
     file->bitrate = 24;
@@ -85,13 +84,13 @@ static void file_calc_md5(file_t * file) {
   char str[33];
   char hex[16] = "0123456789abcdef";
 
-  if (file->md5) l_free(file->md5);
+  if (file->md5) g_free(file->md5);
   file->md5 = NULL;
   if (global.options.dummy_md5) {
     for (i1 = 0; i1 < 32; i1++)
       str[i1] = hex[random() % 16];
     str[32] = 0;
-    file->md5 = l_strdup_printf("%s-%ld", str, file->size);
+    file->md5 = g_strdup_printf("%s-%ld", str, file->size);
   } else {
     file->md5 = MD5File(file->longname, file->md5);
   }
@@ -131,23 +130,6 @@ static time_t file_update_size(file_t* file) {
   return stats.st_mtime;
 }
 
-suffix_t *check_suffix(char *name, int mime) {
-  GList *dlist;
-  char *suffix;
-  suffix_t *suf;
-
-  suffix = strrchr(name, '.');
-  if (!suffix) return NULL;
-  else suffix++;
-
-  for (dlist = global.mimetype[mime].suffixes; dlist; dlist = dlist->next) {
-    suf = dlist->data;
-    if (!l_strcasecmp(suffix, suf->suffix))
-      return suf;
-  }
-  return NULL;
-}
-
 ///////////////////////////////////////////////////////
 
 static dir_position_t* dir_position_new(library_t* lib, 
@@ -158,25 +140,25 @@ static dir_position_t* dir_position_new(library_t* lib,
   char* base_folder;
 
   if (!base) return NULL;
-  pos = l_malloc(sizeof(dir_position_t));
+  pos = g_malloc(sizeof(dir_position_t));
   if (!pos) return NULL;
   
   pos->lib = lib;
   if (base[strlen(base)-1] == DIR_SEP)
-    base_folder = l_strdup(base);
+    base_folder = g_strdup(base);
   else
-    base_folder = l_strdup_printf("%s%c", base, DIR_SEP);
+    base_folder = g_strdup_printf("%s%c", base, DIR_SEP);
 
   dir = opendir(base_folder);
   if (!dir) {
-    l_free(base_folder);
-    l_free(pos);
+    g_free(base_folder);
+    g_free(pos);
     return NULL;
   }
   pos->type = type;
-  pos->base = l_strdup(base_folder);
+  pos->base = g_strdup(base_folder);
 
-  entry = l_malloc(sizeof(dir_pos_entry_t));
+  entry = g_malloc(sizeof(dir_pos_entry_t));
   entry->dir = dir;
   entry->folder = base_folder;
   entry->dent = NULL;
@@ -189,8 +171,8 @@ static void dir_position_destroy(dir_position_t* pos) {
   if (!pos) return;
   // FIXME: close left open DIRs, free entries
   if (pos->stack) g_list_free(pos->stack);
-  if (pos->base) l_free(pos->base);
-  l_free(pos);
+  if (pos->base) g_free(pos->base);
+  g_free(pos);
 }
 
 static int dir_position_next(dir_position_t* pos) {
@@ -208,28 +190,28 @@ static int dir_position_next(dir_position_t* pos) {
     dent = readdir(entry->dir);
     if (!dent) {
       if (entry->dir) closedir(entry->dir);
-      l_free(entry->folder);
-      l_free(entry);
+      g_free(entry->folder);
+      g_free(entry);
       pos->stack = g_list_remove_link(pos->stack, pos->stack);
     } else {
       if (dent->d_name[0] == '.') continue;
-      name = l_strdup_printf("%s%s", entry->folder, dent->d_name);
+      name = g_strdup_printf("%s%s", entry->folder, dent->d_name);
       stat(name, &buf);
       if (buf.st_mode & S_IFDIR) {
 	dir = opendir(name);
 	if (dir) {
-	  n_entry = l_malloc(sizeof(dir_pos_entry_t));
-	  n_entry->folder = l_strdup_printf("%s/", name);
-	  l_free(name);
+	  n_entry = g_malloc(sizeof(dir_pos_entry_t));
+	  n_entry->folder = g_strdup_printf("%s/", name);
+	  g_free(name);
 	  n_entry->dir = dir;
 	  n_entry->dent = NULL;
 	  pos->stack = g_list_prepend(pos->stack, n_entry);
 	  break;
-	} else l_free(name);
+	} else g_free(name);
       } else {
 	//	printf("is no dir\n");
 	entry->dent = dent;
-	l_free(name);
+	g_free(name);
 	break;
       }
     }      
@@ -255,19 +237,17 @@ dir_position_get_current_name(dir_position_t* pos, char** file) {
 }
 ////////////////////////////////////////////////////
 
-static void
-file_unshare(file_node_t* node, net_t* net) {
+static void file_unshare(file_node_t* node, net_t* net) {
   // dont unshare offline files on server (they are not shared)
   if (node->flags & LIB_VIRTUAL) return;
 
-  command_send(net, CMD_REMOVE_FILE, node->file->winname);
+  command_send_unimportant
+    (net, CMD_REMOVE_FILE, node->file->winname);
   LIB_NODE(node)->nets = 
     g_list_remove(LIB_NODE(node)->nets, net);
 }
 
-static void 
-file_share(file_node_t* node, net_t* net) {
-  suffix_t *suffix;
+static void file_share(file_node_t* node, net_t* net) {
   file_t* file = node->file;
 
   // dont share offline files on server (but they are shown on direct browse)
@@ -278,17 +258,18 @@ file_share(file_node_t* node, net_t* net) {
     return;
   }
 
-  suffix = get_suffix(file->winname);
-  if (suffix && suffix->as_mp3) {
-    command_send(net, CMD_ADD_FILE, 
-		 file->winname, file->md5, file->size, 
-		 find_valid_bitrate(file->bitrate),
-		 file->frequency, file->duration);
+  if (file->media_type == MEDIA_MP3 || global.options.share_as_mp3) {
+    command_send_unimportant
+      (net, CMD_ADD_FILE, 
+       file->winname, file->md5, file->size, 
+       find_valid_bitrate(file->bitrate),
+       file->frequency, file->duration);
   } else {
-    command_send(net, CMD_SHARE_FILE,
-		 file->winname, file->size, file->md5,
-		 MimeNames((file->media_type == MIME_NONE) ?
-			   (MIME_APPLICATION):(file->media_type), 2));
+    command_send_unimportant
+      (net, CMD_SHARE_FILE,
+       file->winname, file->size, file->md5,
+       int2media((file->media_type == MEDIA_NONE) ?
+		 (MEDIA_APPLICATION):(file->media_type), 2));
   }
   LIB_NODE(node)->nets = 
     g_list_append(LIB_NODE(node)->nets, net);
@@ -307,22 +288,22 @@ lib_node_unshared(file_node_t* node) {
     parent->fci[node->file->media_type][1]--;
     parent->fsi[node->file->media_type][0] += node->file->size;
     parent->fsi[node->file->media_type][1] -= node->file->size;
-    parent->fci[MIME_SIZE][0]++;
-    parent->fci[MIME_SIZE][1]--;
-    if (parent->fci[MIME_SIZE][1] == 0)
+    parent->fci[MEDIA_SIZE][0]++;
+    parent->fci[MEDIA_SIZE][1]--;
+    if (parent->fci[MEDIA_SIZE][1] == 0)
       parent->flags &= ~LIB_SHARED;
-    parent->fsi[MIME_SIZE][0] += node->file->size;
-    parent->fsi[MIME_SIZE][1] -= node->file->size;
+    parent->fsi[MEDIA_SIZE][0] += node->file->size;
+    parent->fsi[MEDIA_SIZE][1] -= node->file->size;
     parent = parent->parent;
   }
   tree->fci[node->file->media_type][0]++;
   tree->fci[node->file->media_type][1]--;
   tree->fsi[node->file->media_type][0] += node->file->size;
   tree->fsi[node->file->media_type][1] -= node->file->size;
-  tree->fci[MIME_SIZE][0]++;
-  tree->fci[MIME_SIZE][1]--;
-  tree->fsi[MIME_SIZE][0] += node->file->size;
-  tree->fsi[MIME_SIZE][1] -= node->file->size;
+  tree->fci[MEDIA_SIZE][0]++;
+  tree->fci[MEDIA_SIZE][1]--;
+  tree->fsi[MEDIA_SIZE][0] += node->file->size;
+  tree->fsi[MEDIA_SIZE][1] -= node->file->size;
 }
 
 static void
@@ -366,20 +347,20 @@ lib_node_removed(file_node_t* node) {
     parent->fci[node->file->media_type][3]--;
     parent->fsi[node->file->media_type][0] -= node->file->size;
     parent->fsi[node->file->media_type][3] -= node->file->size;
-    parent->fci[MIME_SIZE][0]--;
-    parent->fci[MIME_SIZE][3]--;
-    parent->fsi[MIME_SIZE][0] -= node->file->size;
-    parent->fsi[MIME_SIZE][3] -= node->file->size;
+    parent->fci[MEDIA_SIZE][0]--;
+    parent->fci[MEDIA_SIZE][3]--;
+    parent->fsi[MEDIA_SIZE][0] -= node->file->size;
+    parent->fsi[MEDIA_SIZE][3] -= node->file->size;
     parent = parent->parent;
   }
   tree->fci[node->file->media_type][0]--;
   tree->fci[node->file->media_type][3]--;
   tree->fsi[node->file->media_type][0] -= node->file->size;
   tree->fsi[node->file->media_type][3] -= node->file->size;
-  tree->fci[MIME_SIZE][0]--;
-  tree->fci[MIME_SIZE][3]--;
-  tree->fsi[MIME_SIZE][0] -= node->file->size;
-  tree->fsi[MIME_SIZE][3] -= node->file->size;
+  tree->fci[MEDIA_SIZE][0]--;
+  tree->fci[MEDIA_SIZE][3]--;
+  tree->fsi[MEDIA_SIZE][0] -= node->file->size;
+  tree->fsi[MEDIA_SIZE][3] -= node->file->size;
 }
 
 static void
@@ -395,25 +376,24 @@ lib_node_shared(file_node_t* node) {
     parent->fci[node->file->media_type][1]++;
     parent->fsi[node->file->media_type][0] -= node->file->size;
     parent->fsi[node->file->media_type][1] += node->file->size;
-    parent->fci[MIME_SIZE][0]--;
-    parent->fci[MIME_SIZE][1]++;
+    parent->fci[MEDIA_SIZE][0]--;
+    parent->fci[MEDIA_SIZE][1]++;
     parent->flags |= LIB_SHARED;
-    parent->fsi[MIME_SIZE][0] -= node->file->size;
-    parent->fsi[MIME_SIZE][1] += node->file->size;
+    parent->fsi[MEDIA_SIZE][0] -= node->file->size;
+    parent->fsi[MEDIA_SIZE][1] += node->file->size;
     parent = parent->parent;
   }
   tree->fci[node->file->media_type][0]--;
   tree->fci[node->file->media_type][1]++;
   tree->fsi[node->file->media_type][0] -= node->file->size;
   tree->fsi[node->file->media_type][1] += node->file->size;
-  tree->fci[MIME_SIZE][0]--;
-  tree->fci[MIME_SIZE][1]++;
-  tree->fsi[MIME_SIZE][0] -= node->file->size;
-  tree->fsi[MIME_SIZE][1] += node->file->size;
+  tree->fci[MEDIA_SIZE][0]--;
+  tree->fci[MEDIA_SIZE][1]++;
+  tree->fsi[MEDIA_SIZE][0] -= node->file->size;
+  tree->fsi[MEDIA_SIZE][1] += node->file->size;
 }
 
-static void
-lib_node_share(file_node_t* node) {
+static void lib_node_share(file_node_t* node) {
   GList* dlist;
   net_t* net;
   file_node_t* child;
@@ -483,21 +463,28 @@ static void lib_update(library_t* lib) {
   lib_update_rec(tree, tree->files);
 }
 
-static file_node_t*
-share_cb(file_node_t* node, void* data) {
+static file_node_t* share_cb(file_node_t* node, void* data) {
   net_t* net = data;
 
-  if (node->file && (node->flags & LIB_SHARED))
+  if (node->file && (node->flags & LIB_SHARED)) {
     file_share(node, net);
-
+  }
+  
   return NULL;
 }
 
 void lib_share(library_t* lib, net_t* net) {
-  if (!net) return;
-  if (net->flags & NETWORK_DONT_SHARE) return;
-
-  file_node_action(FILE_TREE(lib), NULL, share_cb, net);
+  if (!net) {
+    GList* dlist;
+    for (dlist = global.net_active; dlist; dlist = dlist->next) {
+      net = dlist->data;
+      if (net->flags & NETWORK_DONT_SHARE) continue;
+      file_node_action(FILE_TREE(lib), NULL, share_cb, net);
+    }
+  } else {
+    if (net->flags & NETWORK_DONT_SHARE) return;
+    file_node_action(FILE_TREE(lib), NULL, share_cb, net);
+  }
 }
 
 static void lib_finish(library_t* lib) {
@@ -542,8 +529,9 @@ static gint dir_for_all_idle(gpointer data) {
   folder = dir_position_get_current_name(pos, &filename);
 
   if (filename && 
-      ((pos->type == MIME_NONE) || check_suffix(filename, pos->type))) {
-    name = l_strdup_printf("%s%s", folder, filename);
+      (pos->type == MEDIA_NONE ||
+       mtype_contains_suffix(pos->type, filename))) {
+    name = g_strdup_printf("%s%s", folder, filename);
 
     node = file_tree_search_filename(tree, name);
     if (!node) {
@@ -553,7 +541,7 @@ static gint dir_for_all_idle(gpointer data) {
       lib_update_user(tree);
       node->flags &= ~LIB_REMOVE;
     }
-    l_free(name);
+    g_free(name);
     lib_node_update_size(LIB_NODE(node));
   }
 
@@ -581,27 +569,28 @@ static gint dir_for_all_idle2(gpointer data) {
   folder = dir_position_get_current_name(pos, &filename);
 
   if (filename && 
-      ((pos->type == MIME_NONE) || check_suffix(filename, pos->type))) {
+      (pos->type == MEDIA_NONE ||
+       mtype_contains_suffix(pos->type, filename))) {
 
-    name = l_strdup_printf("%s%s", folder, filename);
+    name = g_strdup_printf("%s%s", folder, filename);
     file = file_create_from_local(name);
-    l_free(name);
+    g_free(name);
 
     mod = file_update_size(file);
     
     // changing filename
-    l_free(file->longname);
-    l_free(file->winname);
+    g_free(file->longname);
+    g_free(file->winname);
     len = strlen("[OFFLINE] ") + strlen(filename);
     if (snode->name[strlen(snode->name)-1] != DIR_SEP) {
-      char* str = l_strdup_printf("%s%c", snode->name, DIR_SEP);
-      l_free(snode->name);
+      char* str = g_strdup_printf("%s%c", snode->name, DIR_SEP);
+      g_free(snode->name);
       snode->name = str;
     }
     file->longname = file_node_get_folder(snode, &len);
     sprintf(file->longname+len, "[OFFLINE] %s", filename);
 
-    file->winname = l_strdup(file->longname);
+    file->winname = g_strdup(file->longname);
     file->shortname = get_short_name(file->longname);
     file->filename = get_file_name(file->shortname);
     local_to_napster_hide(file->winname);
@@ -627,7 +616,7 @@ static file_node_t* lib_node_new(char* name) {
 
   if (!name) return NULL;
 
-  node = l_malloc(sizeof(lib_node_t));
+  node = g_malloc(sizeof(lib_node_t));
   if (!node) return NULL;
   file_node_init(FILE_NODE(node), name);
 
@@ -666,15 +655,15 @@ static int lib_node_rename_func(clist_rename_t* cr) {
   pos = strrchr(node->file->longname, DIR_SEP);
   if (pos) {
     *pos = 0;
-    new_name = l_strdup_printf("%s%c%s", node->file->longname,
+    new_name = g_strdup_printf("%s%c%s", node->file->longname,
 			       DIR_SEP, cr->new_text);
     *pos = DIR_SEP;
   } else {
-    new_name = l_strdup(cr->new_text);
+    new_name = g_strdup(cr->new_text);
   }
 
   if (stat(new_name, &stats) == 0) {
-    l_free(new_name);
+    g_free(new_name);
     info_dialog("Cannot rename file, filename already in use!");
     return 0;
   }
@@ -682,11 +671,11 @@ static int lib_node_rename_func(clist_rename_t* cr) {
   if ((res = rename(node->file->longname, new_name)) != 0) {
     char* str;
     str = 
-      l_strdup_printf("Renaming from\n  [%s]\nto\n  [%s]\nfailed (%s)\n",
+      g_strdup_printf("Renaming from\n  [%s]\nto\n  [%s]\nfailed (%s)\n",
 		      node->file->longname, new_name, strerror(errno));
     info_dialog(str);
-    l_free(new_name);
-    l_free(str);
+    g_free(new_name);
+    g_free(str);
     return 0;
   }
 
@@ -698,12 +687,12 @@ static int lib_node_rename_func(clist_rename_t* cr) {
     lib_node_unshare(node);
   }
 
-  l_free(node->file->longname);
-  l_free(node->file->winname);
+  g_free(node->file->longname);
+  g_free(node->file->winname);
   node->file->longname = new_name;
   node->file->shortname = get_short_name(new_name);
   node->file->filename = get_file_name(new_name);
-  node->file->winname = l_strdup(new_name);
+  node->file->winname = g_strdup(new_name);
   local_to_napster_hide(node->file->winname);
   
   if (old_shared) {
@@ -711,8 +700,8 @@ static int lib_node_rename_func(clist_rename_t* cr) {
     lib_node_share(node);
   }
 
-  l_free(node->name);
-  node->name = l_strdup(cr->new_text);
+  g_free(node->name);
+  node->name = g_strdup(cr->new_text);
 
   file_node_update(FILE_TREE(global.lib), node, 0, 0);
   
@@ -799,7 +788,7 @@ lib_node_init(file_node_t* node) {
 library_t* lib_new(char* name) {
   library_t *lib;
 
-  lib = l_malloc(sizeof(library_t));
+  lib = g_malloc(sizeof(library_t));
   if (!lib) return NULL;
 
   file_tree_init(FILE_TREE(lib), name, NULL);
@@ -877,8 +866,8 @@ void shared_folder_destroy(GList* slist) {
   if (!slist) return;
   for (dlist = slist; dlist; dlist = dlist->next) {
     folder = dlist->data;
-    l_free(folder->name);
-    l_free(folder);
+    g_free(folder->name);
+    g_free(folder);
   }
   g_list_free(slist);
 }
@@ -973,8 +962,8 @@ void lib_save(library_t* lib, char* name) {
 
   if (!lib) return;
   if (name && name != tree->name) {
-    l_free(tree->name);
-    tree->name = l_strdup(name);
+    g_free(tree->name);
+    tree->name = g_strdup(name);
   }
   sprintf(filename, "%s%cshare%c%s", global.options.config_dir,
 	  DIR_SEP, DIR_SEP, tree->name);
@@ -1045,9 +1034,9 @@ static int lib_load_folders(library_t* lib) {
     s1 = arg(line, 2);
     s2 = arg(NULL, 2);
     if (!s1 || !s2) return 0;
-    folder = l_malloc(sizeof(shared_folder_t));
+    folder = g_malloc(sizeof(shared_folder_t));
     if (!folder) return 0;
-    folder->name = l_strdup(s1);
+    folder->name = g_strdup(s1);
     if (!folder->name) return 0;
     folder->type = atoi(s2);
     lib->shared_folder = g_list_append(lib->shared_folder, folder);
@@ -1092,9 +1081,9 @@ static int lib_load_folder_1(library_t* lib, lib_node_t* parent) {
 	strcpy(file->longname+pos, filename);
 	file->shortname = get_short_name(file->longname);
 	file->filename = get_file_name(file->shortname);
-	file->winname = l_strdup(file->longname);
+	file->winname = g_strdup(file->longname);
 	local_to_napster_hide(file->winname);
-	file->md5 = l_strdup(md5);
+	file->md5 = g_strdup(md5);
 	file->size = strtoul(size, NULL, 10);
 	if (!strncmp(bitrate, "vbr", 3)) {
 	  file->bitrate = atoi(bitrate + 3);
@@ -1105,11 +1094,11 @@ static int lib_load_folder_1(library_t* lib, lib_node_t* parent) {
 	}
 	file->frequency = atoi(freq);
 	file->duration = atoi(duration);
-	file->media_type = get_media_type(file->filename);
-	file->user = l_strdup("");
+	file->media_type = mtype_get(file->filename);
+	file->user = g_strdup("");
 	
 	child = LIB_NODE
-	  (file_node_insert_file(tree, FILE_NODE(parent), file));
+	  (file_node_insert_file(tree, FILE_NODE(parent), file, FALSE));
 	if (atoi(share)) FILE_NODE(child)->flags |= LIB_SHARED;
 	else FILE_NODE(child)->flags &= ~LIB_SHARED;
 	child->mod_time = strtoul(modif, NULL, 10);
@@ -1168,7 +1157,7 @@ static int lib_load_folder_0(library_t* lib) {
   
   file = file_new();
   file_set_longname(file, filename);
-  file->md5 = l_strdup(md5);
+  file->md5 = g_strdup(md5);
   file->size = strtoul(size, NULL, 10);
   if (!strncmp(bitrate, "vbr", 3)) {
     file->bitrate = atoi(bitrate + 3);
@@ -1179,7 +1168,7 @@ static int lib_load_folder_0(library_t* lib) {
   }
   file->frequency = atoi(freq);
   file->duration = atoi(duration);
-  file->user = l_strdup("");
+  file->user = g_strdup("");
 
   child = file_tree_insert_file(FILE_TREE(lib), file);
   if (atoi(share)) child->flags |= LIB_SHARED;
@@ -1224,7 +1213,7 @@ lib_set_share_flag(library_t* lib) {
   node = tree->files;
   while (node) {
     if (!node->file) {
-      if (node->fci[MIME_SIZE][1] > 0)
+      if (node->fci[MEDIA_SIZE][1] > 0)
 	node->flags |= LIB_SHARED;
       else
 	node->flags &= ~LIB_SHARED;
@@ -1276,7 +1265,7 @@ library_t* lib_load_saved(char *fname) {
 
   file_tree_calc_matrix(tree);
   lib_set_share_flag(lib);
-  file_tree_show(tree, MIME_SIZE, 3);
+  file_tree_show(tree, MEDIA_SIZE, 3);
   file_tree_mark(tree, 0);
 
   temp = lookup_widget(global.win, "entry84");
@@ -1316,7 +1305,7 @@ void lib_unshare_all(library_t* lib) {
   
   file_node_action(FILE_TREE(lib), NULL, set_unshared_cb, NULL);
   
-  command_send(NULL, CMD_UNSHARE_ALL);
+  command_send_unimportant(NULL, CMD_UNSHARE_ALL);
 }
 
 static file_node_t*
@@ -1334,7 +1323,7 @@ void lib_unshare_all_network(library_t* lib, net_t* net) {
 
   file_node_action(FILE_TREE(lib), NULL, set_unshared_server_cb, net);
 
-  command_send(net, CMD_UNSHARE_ALL);
+  command_send_unimportant(net, CMD_UNSHARE_ALL);
 }
 
 static GList* get_selected_nodes(GtkCList* clist, int* count, int* count2) {
@@ -1522,14 +1511,10 @@ GtkWidget *create_lib1_popup(file_node_t* node) {
     selected = get_selected_nodes(clist, &shared, &virtual);
     item_num = g_list_length(selected);
 
-    item = gtk_menu_item_new_with_label("Open file");
-    gtk_widget_show(item);
+    item = create_open_menu(file->media_type, file->longname,
+			    !(node->flags & LIB_VIRTUAL));
     gtk_container_add(GTK_CONTAINER(popup), item);
-    gtk_signal_connect(GTK_OBJECT(item), "activate",
-		       GTK_SIGNAL_FUNC(on_play_file3_activate), file);
-    if (node->flags & LIB_VIRTUAL) 
-      gtk_widget_set_sensitive(item, FALSE);
-    
+
     item = gtk_menu_item_new();
     gtk_widget_show(item);
     gtk_container_add(GTK_CONTAINER(popup), item);
@@ -1537,12 +1522,12 @@ GtkWidget *create_lib1_popup(file_node_t* node) {
     
     if (item_num - shared > 0) {
       if ((item_num > 1) || (item_num - shared > 1))
-	text = l_strdup_printf("Share Selected (%d)",
+	text = g_strdup_printf("Share Selected (%d)",
 			       item_num-shared);
       else
-	text = l_strdup("Share File");
+	text = g_strdup("Share File");
       item = gtk_menu_item_new_with_label(text);
-      l_free(text);
+      g_free(text);
       gtk_widget_show(item);
       gtk_container_add(GTK_CONTAINER(popup), item);
       gtk_signal_connect(GTK_OBJECT(item), "activate",
@@ -1550,11 +1535,11 @@ GtkWidget *create_lib1_popup(file_node_t* node) {
     }
     if (shared > 0) {
       if ((item_num > 1) || (shared > 1))
-	text = l_strdup_printf("Unshare Selected (%d)", shared);
+	text = g_strdup_printf("Unshare Selected (%d)", shared);
       else
-	text = l_strdup("Unshare File");
+	text = g_strdup("Unshare File");
       item = gtk_menu_item_new_with_label(text);
-      l_free(text);
+      g_free(text);
       gtk_widget_show(item);
       gtk_container_add(GTK_CONTAINER(popup), item);
       gtk_signal_connect(GTK_OBJECT(item), "activate",
@@ -1581,11 +1566,11 @@ GtkWidget *create_lib1_popup(file_node_t* node) {
 
     if (virtual) {
       if (virtual > 1)
-	text = l_strdup_printf("Remove Files (%d)", virtual);
+	text = g_strdup_printf("Remove Files (%d)", virtual);
       else
-	text = l_strdup("Remove File");
+	text = g_strdup("Remove File");
       item = gtk_menu_item_new_with_label(text);
-      l_free(text);
+      g_free(text);
       gtk_widget_show(item);
       gtk_container_add(GTK_CONTAINER(popup), item);
       gtk_signal_connect(GTK_OBJECT(item), "activate",
@@ -1594,12 +1579,12 @@ GtkWidget *create_lib1_popup(file_node_t* node) {
 
     if (item_num - virtual > 0) {
       if (item_num - virtual > 1) 
-	text = l_strdup_printf("Delete %d files (from disc)",
+	text = g_strdup_printf("Delete %d files (from disc)",
 			       item_num);
       else
-	text = l_strdup("Delete File (from disc)");
+	text = g_strdup("Delete File (from disc)");
       item = gtk_menu_item_new_with_label(text);
-      l_free(text);
+      g_free(text);
       gtk_widget_show(item);
       gtk_container_add(GTK_CONTAINER(popup), item);
       gtk_signal_connect(GTK_OBJECT(item), "activate",
@@ -1673,12 +1658,14 @@ on_unshare_subtree_activate(GtkMenuItem * menuitem ATTR_UNUSED,
   lib_save(global.lib, NULL);
 }
 
+/*
 static void
 on_refresh_subtree_activate(GtkMenuItem * menuitem ATTR_UNUSED,
 			    gpointer user_data ATTR_UNUSED) {
   printf("on_refresh_subtree not defined\n");
   //  lib_node_refresh((lib_node_t*)user_data);
 }
+*/
 
 static int lib_rename_folder_func(clist_rename_t* cr) {
   GtkCTreeNode* cnode;
@@ -1692,8 +1679,8 @@ static int lib_rename_folder_func(clist_rename_t* cr) {
   cnode = gtk_ctree_find_by_row_data(ctree, NULL, node);
   if (!cnode) return 0;
   
-  if (node->name) l_free(node->name);
-  node->name = l_strdup(cr->new_text);
+  if (node->name) g_free(node->name);
+  node->name = g_strdup(cr->new_text);
 
   file_node_update(FILE_TREE(global.lib), node, 0, 0);
   
@@ -1776,19 +1763,29 @@ GtkWidget *create_lib2_popup(lib_node_t* node) {
   GtkWidget *item2;
   file_node_t* n2 = FILE_NODE(node);
   file_tree_t* tree = FILE_TREE(global.lib);
+  char* folder;
+  int temp = 0;
 
   if (!node || !tree) return NULL;
   popup = gtk_menu_new();
   
-  
-  if (tree->fci[MIME_SIZE][0] > 0) {
+  folder = file_node_get_folder(FILE_NODE(node), &temp);
+  item = create_open_menu(MEDIA_FOLDER, folder, 1);
+  gtk_container_add(GTK_CONTAINER(popup), item);
+			  
+  item = gtk_menu_item_new();
+  gtk_widget_show(item);
+  gtk_container_add(GTK_CONTAINER(popup), item);
+  gtk_widget_set_sensitive(item, FALSE);
+
+  if (tree->fci[MEDIA_SIZE][0] > 0) {
     item = gtk_menu_item_new_with_label("Share Subtree");
     gtk_widget_show(item);
     gtk_container_add(GTK_CONTAINER(popup), item);
     gtk_signal_connect(GTK_OBJECT(item), "activate",
 		       GTK_SIGNAL_FUNC(on_share_subtree_activate), node);
   }
-  if (tree->fci[MIME_SIZE][1] > 0) {
+  if (tree->fci[MEDIA_SIZE][1] > 0) {
     item = gtk_menu_item_new_with_label("Unshare Subtree");
     gtk_widget_show(item);
     gtk_container_add(GTK_CONTAINER(popup), item);
@@ -1827,49 +1824,49 @@ GtkWidget *create_lib2_popup(lib_node_t* node) {
     gtk_container_add(GTK_CONTAINER(popup2), item2);
     gtk_signal_connect(GTK_OBJECT(item2), "activate",
 		       GTK_SIGNAL_FUNC(on_offline_add), 
-		       (gpointer) MIME_MP3);
+		       (gpointer) MEDIA_MP3);
 
     item2 = gtk_menu_item_new_with_label("Video");
     gtk_widget_show(item2);
     gtk_container_add(GTK_CONTAINER(popup2), item2);
     gtk_signal_connect(GTK_OBJECT(item2), "activate",
 		       GTK_SIGNAL_FUNC(on_offline_add),
-		       (gpointer) MIME_VIDEO);
+		       (gpointer) MEDIA_VIDEO);
     
     item2 = gtk_menu_item_new_with_label("Audio");
     gtk_widget_show(item2);
     gtk_container_add(GTK_CONTAINER(popup2), item2);
     gtk_signal_connect(GTK_OBJECT(item2), "activate",
 		       GTK_SIGNAL_FUNC(on_offline_add),
-		       (gpointer) MIME_AUDIO);
+		       (gpointer) MEDIA_AUDIO);
     
     item2 = gtk_menu_item_new_with_label("Application");
     gtk_widget_show(item2);
     gtk_container_add(GTK_CONTAINER(popup2), item2);
     gtk_signal_connect(GTK_OBJECT(item2), "activate",
 		       GTK_SIGNAL_FUNC(on_offline_add),
-		       (gpointer) MIME_APPLICATION);
+		       (gpointer) MEDIA_APPLICATION);
     
     item2 = gtk_menu_item_new_with_label("Image");
     gtk_widget_show(item2);
     gtk_container_add(GTK_CONTAINER(popup2), item2);
     gtk_signal_connect(GTK_OBJECT(item2), "activate",
 		       GTK_SIGNAL_FUNC(on_offline_add),
-		       (gpointer) MIME_IMAGE);
+		       (gpointer) MEDIA_IMAGE);
     
     item2 = gtk_menu_item_new_with_label("Text");
     gtk_widget_show(item2);
     gtk_container_add(GTK_CONTAINER(popup2), item2);
     gtk_signal_connect(GTK_OBJECT(item2), "activate",
 		       GTK_SIGNAL_FUNC(on_offline_add),
-		       (gpointer) MIME_TEXT);
+		       (gpointer) MEDIA_TEXT);
     
     item2 = gtk_menu_item_new_with_label("All Files");
     gtk_widget_show(item2);
     gtk_container_add(GTK_CONTAINER(popup2), item2);
     gtk_signal_connect(GTK_OBJECT(item2), "activate",
 		       GTK_SIGNAL_FUNC(on_offline_add),
-		       (gpointer) MIME_NONE);
+		       (gpointer) MEDIA_NONE);
     
   } else {
     item = gtk_menu_item_new_with_label("Create Virtual Folder");
@@ -1918,8 +1915,7 @@ on_load_list_activate(GtkMenuItem * menuitem, gpointer user_data ATTR_UNUSED)
   label = GTK_LABEL(GTK_BIN(menuitem)->child);
   s1 = label->label;
 
-  lib_load_saved(s1);
-
+  lib_share(lib_load_saved(s1), NULL);
   setup_preferences(P_SHARES);
 }
 
@@ -2138,8 +2134,8 @@ shared_folder_t* lib_add_folder(library_t* lib, char* folder, int type) {
   
   sf = lib_search_folder(lib, folder, type);
   if (!sf) {
-    sf = l_malloc(sizeof(*sf));
-    sf->name = l_strdup(folder);
+    sf = g_malloc(sizeof(*sf));
+    sf->name = g_strdup(folder);
     sf->type = type;
     lib->shared_folder = g_list_append(lib->shared_folder, sf);
     return sf;
