@@ -1,4 +1,4 @@
-/* Copyright (C) 2000/2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -69,7 +69,9 @@ struct _band_t {
   double* temp1;
   double* temp2;
   int* tno;
+  int sssss;
   int step;
+  int real_step;
   int smooth;
   int mode;
   int show;
@@ -120,7 +122,7 @@ static band_t* band_new(int up) {
   band_t* result;
   int i1;
 
-  result = l_malloc(sizeof(*result));
+  result = g_malloc(sizeof(*result));
   if (!up)
     result->area = lookup_widget(global.win, "drawingarea3");
   else
@@ -130,17 +132,17 @@ static band_t* band_new(int up) {
     gtk_widget_realize(result->area);
   result->pixmap = NULL;
 
-  result->val1  = l_malloc(sizeof(double*) * BAND_SIZE);
-  result->val2  = l_malloc(sizeof(double*) * BAND_SIZE);
-  result->size  = l_malloc(sizeof(int) * BAND_SIZE);
-  result->pos   = l_malloc(sizeof(int) * BAND_SIZE);
-  result->temp1 = l_malloc(sizeof(double) * BAND_SIZE);
-  result->temp2 = l_malloc(sizeof(double) * BAND_SIZE);
-  result->tno   = l_malloc(sizeof(double) * BAND_SIZE);
+  result->val1  = g_malloc(sizeof(double*) * BAND_SIZE);
+  result->val2  = g_malloc(sizeof(double*) * BAND_SIZE);
+  result->size  = g_malloc(sizeof(int) * BAND_SIZE);
+  result->pos   = g_malloc(sizeof(int) * BAND_SIZE);
+  result->temp1 = g_malloc(sizeof(double) * BAND_SIZE);
+  result->temp2 = g_malloc(sizeof(double) * BAND_SIZE);
+  result->tno   = g_malloc(sizeof(double) * BAND_SIZE);
   for (i1 = 0; i1 < BAND_SIZE; i1++) {
-    result->val1[i1]  = l_malloc(sizeof(double)*BAND_WIDTH);
+    result->val1[i1]  = g_malloc(sizeof(double)*BAND_WIDTH);
     memset(result->val1[i1], 0, sizeof(double) * BAND_WIDTH);
-    result->val2[i1]  = l_malloc(sizeof(double)*BAND_WIDTH);
+    result->val2[i1]  = g_malloc(sizeof(double)*BAND_WIDTH);
     memset(result->val2[i1], 0, sizeof(double) * BAND_WIDTH);
     result->pos[i1]  = 0;
     result->size[i1] = BAND_WIDTH;
@@ -148,7 +150,10 @@ static band_t* band_new(int up) {
     result->temp2[i1] = 0;
     result->tno[i1] = 0;
   }
-  result->step   = global.options.graph_interval[up];
+  result->step = global.options.graph_interval[up];
+  if (result->step < 0) result->real_step = 0;
+  else result->real_step = result->step;
+
   result->smooth = global.options.graph_smooth[up];
   result->mode   = global.options.graph_mode[up];
   result->show   = global.options.graph_show[up];
@@ -171,7 +176,7 @@ static void band_new_vals(band_t* band, long val1, long val2) {
       band->tno[i1] = 0;
       band->pos[i1]++;
       if (band->pos[i1] >= band->size[i1]) band->pos[i1] = 0;
-      if (i1 == band->step) band->need_update = 1;
+      if (i1 == band->real_step) band->need_update = 1;
     }
   }    
 }
@@ -188,7 +193,7 @@ static void band_draw_graph(band_t* band, int up) {
   double sum1, sum2;
   int unit;
   int sindex;
-  int i1, i2, i2x, i3, i4, ix;
+  int i1, i2, i2x=0/*avoid compiler warning*/, i3, i4, ix;
   char str[1024];
   long bytes1, bytes2;
   int add;
@@ -200,7 +205,7 @@ static void band_draw_graph(band_t* band, int up) {
   char* text;
   time_t tim;
   int dwidth, dheight;
-  int BX1 = 0;
+  int BX1 = 40;   // just a guess
   char str1[128];
   char str2[128];
 
@@ -214,7 +219,13 @@ static void band_draw_graph(band_t* band, int up) {
   dwidth  = width-BX1-BX2;
   dheight = height-BY1-BY2;
 
-  step = band->step;
+  if (band->step < 0 && band->real_step < BAND_SIZE-1 &&
+      steps[band->real_step]*dwidth < global.current_time - global.start_time)
+    band->real_step++;
+  if (band->step < 0 && band->real_step > 0 &&
+      steps[band->real_step-1]*dwidth > global.current_time - global.start_time)
+    band->real_step--;
+  step = band->real_step;
   si = band->smooth;
 
   if (band->pixmap) {
@@ -241,17 +252,17 @@ static void band_draw_graph(band_t* band, int up) {
   sum2 = global.statistic.incomplete[1];
   if (sum2 > 0 && !up) {
     print_size(str1, sum2);
-    text = l_strdup_printf("%.1f%% of %s completed",
+    text = g_strdup_printf("%.1f%% of %s completed",
 			   sum1 / sum2 * 100, str1);
     pos = gdk_string_width(font1, text);
     gdk_draw_string(band->pixmap, font1, gc4, 
 		    width-BX2-pos, BY1 - 5, text);
-    l_free(text);
+    g_free(text);
   }
 
-  old_val1 = l_malloc(si*sizeof(double));
+  old_val1 = g_malloc(si*sizeof(double));
   memset(old_val1, 0, sizeof(double) * si);
-  old_val2 = l_malloc(si*sizeof(double));
+  old_val2 = g_malloc(si*sizeof(double));
   memset(old_val2, 0, sizeof(double) * si);
   sum1 = sum2 = 0;
   max = 0;
@@ -318,16 +329,16 @@ static void band_draw_graph(band_t* band, int up) {
   if (!up) {
     print_size(str1, global.statistic.total[0]);
     print_speed(str2, global.statistic.total[0] / (tim - global.start_time), 1);
-    text = l_strdup_printf("Total Down: %s (%s)", str1, str2);
+    text = g_strdup_printf("Total Down: %s (%s)", str1, str2);
   } else {
     print_size(str1, global.statistic.total[1]);
     print_speed(str2, global.statistic.total[1] / (tim - global.start_time), 1);
-    text = l_strdup_printf("Total Up: %s (%s)", str1, str2);
+    text = g_strdup_printf("Total Up: %s (%s)", str1, str2);
   }
   gdk_draw_string(band->pixmap, font1, gc4, 
 		  BX1, BY1 - 5, text);
 
-  l_free(text);
+  g_free(text);
 
   pos = band->pos[step];
   sum1 = sum2 = 0;
@@ -458,8 +469,8 @@ static void band_draw_graph(band_t* band, int up) {
 		  BX1-1, dheight+BY1-1 - i3);
   }
 
-  l_free(old_val1);
-  l_free(old_val2);
+  g_free(old_val1);
+  g_free(old_val2);
   band->need_update = 0;
 
  just_paint:
@@ -573,10 +584,10 @@ void statistic_update(statistic_t * stat) {
 
 void statistic_draw_graphs(statistic_t* stat, int expose) {
   if (stat->band[0] &&
-      (expose || stat->band[0]->tno[stat->band[0]->step] == 0))
+      (expose || stat->band[0]->tno[stat->band[0]->real_step] == 0))
     band_draw_graph(stat->band[0], 0);
   if (stat->band[1] &&
-      (expose || stat->band[1]->tno[stat->band[1]->step] == 0))
+      (expose || stat->band[1]->tno[stat->band[1]->real_step] == 0))
     band_draw_graph(stat->band[1], 1);
 }
 
@@ -610,8 +621,8 @@ int statistic_get_no_steps() {
 static access_t *access_new(char *name) {
   access_t *result;
 
-  result = l_malloc(sizeof(access_t));
-  result->name = l_strdup(name);
+  result = g_malloc(sizeof(access_t));
+  result->name = g_strdup(name);
   result->last = 0;
   result->accesses = 0;
   result->access_list = NULL;
@@ -647,7 +658,7 @@ access_add(access_t * access, char *name, int depth) {
     sub = access_new(name);
     access_ctree_insert_access(access, sub, depth);
     if (!access->access_list) {
-      access->access_list = l_malloc(ACCESS_HASH_LENGTH * sizeof(access_t*));
+      access->access_list = g_malloc(ACCESS_HASH_LENGTH * sizeof(access_t*));
       for (key = 0; key < ACCESS_HASH_LENGTH; key++)
 	access->access_list[key] = NULL;
     }
@@ -681,10 +692,10 @@ void access_destroy(access_t * access, int depth) {
       }
       g_list_free(access->access_list[i1]);
     }
-    l_free(access->access_list);
+    g_free(access->access_list);
   }
-  if (access->name) l_free(access->name);
-  l_free(access);
+  if (access->name) g_free(access->name);
+  g_free(access);
 }
 
 static void access_remove(access_t* child) {
@@ -717,19 +728,19 @@ static GtkCTreeNode* access_find_node(access_t* access) {
 void access_new_request(upload_t *upload) {
   access_t *access1;
   access_t *access2;
-  transfer_data_t* data;
-
-  data = upload->data;
-  if (data->is_dcc) return;
+  transfer_t* trans;
   
-  if (global.options.access_timeout == 0) return;
+  trans = TRANS(upload);
+  if (trans->is_dcc) return;
+  
+  if (global.options.access_timeout <= 0) return;
   
   if (!upload->access) {
     if (global.statistic.access_format == 0) {
       access1 = access_add(global.statistic.file_access, upload->file->longname, 1);
-      access2 = access_add(access1, data->user_info->nick, 2);
+      access2 = access_add(access1, trans->user_info->nick, 2);
     } else {
-      access1 = access_add(global.statistic.file_access, data->user_info->nick, 1);
+      access1 = access_add(global.statistic.file_access, trans->user_info->nick, 1);
       access2 = access_add(access1, upload->file->longname, 2);
     }
     upload->access = access2;
@@ -779,9 +790,9 @@ access_t* access_get_access(GtkCTreeNode* node) {
   return access;
 }
 
-GtkCTreeNode* access_ctree_insert_real(GtkCTree* ctree, GtkCTreeNode* node,
-				       access_t * access, int depth)
-{
+static GtkCTreeNode* 
+access_ctree_insert_real(GtkCTree* ctree, GtkCTreeNode* node,
+			 access_t * access, int depth) {
   GtkCTreeNode *node2;
   char str[100];
 
@@ -793,16 +804,14 @@ GtkCTreeNode* access_ctree_insert_real(GtkCTree* ctree, GtkCTreeNode* node,
     else strcpy(tstr[0], access->name);
   }
 
-  if (access->done > 0)
-    sprintf(tstr[1], "%d (%s)", access->accesses, 
-	    print_size(str, access->done));
-  else sprintf(tstr[1], "%d", access->accesses);
+  sprintf(tstr[1], "%d", access->accesses);
+  sprintf(tstr[2], "%s", print_size(str, access->done));
 
   if (access->last) {
-    strcpy(tstr[2], ctime(&access->last));
-    tstr[2][strlen(tstr[2]) - 1] = 0;
+    strcpy(tstr[3], ctime(&access->last));
+    tstr[3][strlen(tstr[3]) - 1] = 0;
   } else
-    strcpy(tstr[2], "None yet");
+    strcpy(tstr[3], "None yet");
 
   if (depth == 1) {
     node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 5,
@@ -845,11 +854,10 @@ void access_ctree_update(GtkCTreeNode* node, access_t * access) {
     node = gtk_ctree_find_by_row_data(ctree, NULL, access);
   if (!node) return;
 
-  if (access->done+access->temp > 0)
-    sprintf(tstr[0], "%d (%s)", access->accesses, 
-	    print_size(str, access->done+access->temp));
-  else sprintf(tstr[0], "%d", access->accesses);
+  sprintf(tstr[0], "%d", access->accesses);
   gtk_ctree_node_set_text(ctree, node, 1, tstr[0]);
+  sprintf(tstr[0], "%s", print_size(str, access->done+access->temp));
+  gtk_ctree_node_set_text(ctree, node, 2, tstr[0]);
   access->temp = 0;
   
   if (access->last) {
@@ -857,7 +865,7 @@ void access_ctree_update(GtkCTreeNode* node, access_t * access) {
     tstr[0][strlen(tstr[0]) - 1] = 0;
   } else
     strcpy(tstr[0], "None yet");
-  gtk_ctree_node_set_text(ctree, node, 2, tstr[0]);
+  gtk_ctree_node_set_text(ctree, node, 3, tstr[0]);
 }
 
 void upload_remove_access(access_t* access);
@@ -874,7 +882,7 @@ static gint access_ctree_remove_old(gpointer data ATTR_UNUSED) {
   long new_timeout = 0;  // when next item has to be removed
   int temp;
 
-  if (global.options.access_timeout <= 0) {
+  if (!global.options.access_cleanup) {
 #ifdef STAT_DEBUG
     printf("[ACCESS] cleanup canceled\n");
 #endif
@@ -897,7 +905,7 @@ static gint access_ctree_remove_old(gpointer data ATTR_UNUSED) {
       for (i2 = 0; i2 < ACCESS_HASH_LENGTH; i2++) {
 	for (dlist2 = access1->access_list[i2]; dlist2; dlist2 = dlist2->next) {
 	  access2 = dlist2->data;
-	  temp = (global.options.access_timeout) *60 * 60
+	  temp = global.options.access_timeout *60 * 60
 	    - global.current_time + access2->last;
 
 	  if (temp <= 0)
@@ -979,7 +987,7 @@ access_t *access_add2(access_t * access, char *name, int search)
   if (!sub) {
     sub = access_new(name);
     if (!access->access_list) {
-      access->access_list = l_malloc(ACCESS_HASH_LENGTH * sizeof(access_t*));
+      access->access_list = g_malloc(ACCESS_HASH_LENGTH * sizeof(access_t*));
       for (key = 0; key < ACCESS_HASH_LENGTH; key++)
 	access->access_list[key] = NULL;
     }
@@ -1010,13 +1018,13 @@ gint access_load_idle(gpointer data)
       if (last) {
 	global.statistic.access_format = atoi(last);
 	if (global.statistic.file_access->name)
-	  l_free(global.statistic.file_access->name);
+	  g_free(global.statistic.file_access->name);
 	if (global.statistic.access_format)
 	  global.statistic.file_access->name = 
-	    l_strdup("User Request Statistic");
+	    g_strdup("User Request Statistic");
 	else
 	  global.statistic.file_access->name = 
-	    l_strdup("File Request Statistic");
+	    g_strdup("File Request Statistic");
       }
       continue;
     } else if (*name == '-') {
@@ -1072,14 +1080,14 @@ void access_load() {
   global.statistic.access_format = 0;
   global.statistic.file_access = access_new("File Request Statistic");
 
-  fname = l_strdup_printf("%s/access.list", global.options.config_dir);
+  fname = g_strdup_printf("%s/access.list", global.options.config_dir);
   if ((fd = fopen(fname, "r")) == NULL) {
     global.status.access_read = 1;
-    l_free(fname);
+    g_free(fname);
     access_show(global.statistic.file_access);
     return;
   }
-  l_free(fname);
+  g_free(fname);
 
   gtk_idle_add(access_load_idle, (gpointer) fd);
 }
@@ -1095,19 +1103,19 @@ void access_save()
   access_t *access2;
   int i1, i2;
 
-  fname = l_strdup_printf("%s/access.list", global.options.config_dir);
+  fname = g_strdup_printf("%s/access.list", global.options.config_dir);
 
   if (!global.statistic.file_access->access_list) {
     unlink(fname);
-    l_free(fname);
+    g_free(fname);
     return;
   }
-  fname_new = l_strdup_printf("%s/access.list.new", global.options.config_dir);
+  fname_new = g_strdup_printf("%s/access.list.new", global.options.config_dir);
 
   if ((fd = fopen(fname_new, "w")) == NULL) {
     g_warning("Could not write [%s]\n", fname);
-    l_free(fname);
-    l_free(fname_new);
+    g_free(fname);
+    g_free(fname_new);
     return;
   }
 
@@ -1137,8 +1145,8 @@ void access_save()
   }
 
   fclose(fd);
-  l_free(fname);
-  l_free(fname_new);
+  g_free(fname);
+  g_free(fname_new);
 }
 
 static upload_t* list_remove_upload(GList** uploads, access_t* access) {
@@ -1243,11 +1251,11 @@ static void access_convert() {
 	    
 	    access1 = access_add2(file_access, acc2->name, 1);
 	    
-	    l_free(acc2->name);
-	    acc2->name = l_strdup(acc1->name);
+	    g_free(acc2->name);
+	    acc2->name = g_strdup(acc1->name);
 
 	    if (!access1->access_list) {
-	      access1->access_list = l_malloc(ACCESS_HASH_LENGTH * sizeof(access_t*));
+	      access1->access_list = g_malloc(ACCESS_HASH_LENGTH * sizeof(access_t*));
 	      for (key = 0; key < ACCESS_HASH_LENGTH; key++)
 		access1->access_list[key] = NULL;
 	    }
@@ -1292,13 +1300,6 @@ void on_access_expand(GtkMenuItem * menuitem ATTR_UNUSED, gpointer user_data ATT
   gtk_ctree_expand_recursive(ctree, NULL);
 }
 
-void on_access_play(GtkMenuItem * menuitem ATTR_UNUSED, gpointer user_data)
-{
-  access_t *access = user_data;
-  
-  if (access) play_file(access->name);
-}
-
 void on_access_mode(GtkMenuItem * menuitem ATTR_UNUSED, gpointer user_data ATTR_UNUSED)
 {
   access_convert();
@@ -1339,12 +1340,9 @@ GtkWidget *create_access_popup(void)
   }
   if (mode != 0) {
     if (mode == 1) {
-      item = gtk_menu_item_new_with_label("Open File");
-      gtk_widget_show(item);
+      item = create_open_menu(mtype_get(access->name),
+			      access->name, 1);
       gtk_container_add(GTK_CONTAINER(popup), item);
-      gtk_signal_connect(GTK_OBJECT(item), "activate",
-			 GTK_SIGNAL_FUNC(on_access_play), access);
-    
     } else if (mode == 2) {
       item = gtk_menu_item_new_with_label("User Menu");
       gtk_widget_show(item);
@@ -1408,6 +1406,9 @@ static void on_graph_interval(GtkMenuItem * menuitem ATTR_UNUSED,
 
   if (popup_band->step == val) return;
   popup_band->step = val;
+  if (val < 0) popup_band->real_step = 0;
+  else popup_band->real_step = val;
+
   popup_band->need_update = 1;
 
   if (popup_band == global.statistic.band[0])
@@ -1503,19 +1504,28 @@ GtkWidget* create_band_popup(band_t* band) {
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(item[0]), popup2);
 
   group = 0;
+  item[0] = gtk_radio_menu_item_new_with_label (group, "Auto adjust");
+  group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (item[0]));
+  gtk_widget_show (item[0]);
+  gtk_container_add (GTK_CONTAINER (popup2), item[0]);
+  item[1] = gtk_menu_item_new();
+  gtk_widget_show(item[1]);
+  gtk_container_add(GTK_CONTAINER(popup2), item[1]);
+  if (band->step == -1)
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item[0]), TRUE);
   for (i1 = 0; i1 < BAND_SIZE; i1++) {
     print_time_unit(str, steps[i1]);
-    item[i1] = gtk_radio_menu_item_new_with_label (group, str);
-    group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (item[i1]));
-    gtk_widget_show (item[i1]);
-    gtk_container_add (GTK_CONTAINER (popup2), item[i1]);
+    item[i1+1] = gtk_radio_menu_item_new_with_label (group, str);
+    group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (item[i1+1]));
+    gtk_widget_show (item[i1+1]);
+    gtk_container_add (GTK_CONTAINER (popup2), item[i1+1]);
     if (band->step == i1)
-      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item[i1]), TRUE);
+      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item[i1+1]), TRUE);
   }
-  for (i1 = 0; i1 < BAND_SIZE; i1++) {
+  for (i1 = 0; i1 <= BAND_SIZE; i1++) {
     gtk_signal_connect(GTK_OBJECT(item[i1]), "activate",
 		       GTK_SIGNAL_FUNC(on_graph_interval), 
-		       (void*)(i1));
+		       (void*)(i1-1));
   }
 
   // Smooth submenu
