@@ -1,4 +1,4 @@
-/* Copyright (C) 2000/2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -37,7 +37,7 @@ void string_list_clear(int lists) {
     if ((lists & (1 << i1)) == 0)
       continue;
     for (dlist = global.string_list[i1]; dlist; dlist = dlist->next) {
-      l_free(dlist->data);
+      g_free(dlist->data);
     }
     if (global.string_list[i1]) {
       g_list_free(global.string_list[i1]);
@@ -58,7 +58,7 @@ char *string_list_search(int slist, char *item)
   if (!item) return NULL;
   for (dlist = global.string_list[slist]; dlist; dlist = dlist->next) {
     string = (char *) (dlist->data);
-    if (!l_strcasecmp(string, item))
+    if (!g_strcasecmp(string, item))
       return string;
   }
   return NULL;
@@ -90,6 +90,9 @@ GtkCList *string_list_get_clist(int slist)
   case LIST_NODOWNLOAD:
     clist = GTK_CLIST(lookup_widget(global.string_win, "clist33"));
     break;
+  case LIST_QSEARCH:
+    clist = GTK_CLIST(lookup_widget(global.string_win, "clist44"));
+    break;
   default:
     break;
   }
@@ -122,6 +125,9 @@ GtkEntry *string_list_get_entry(int slist)
   case LIST_NODOWNLOAD:
     entry = GTK_ENTRY(lookup_widget(global.string_win, "entry126"));
     break;
+  case LIST_QSEARCH:
+    entry = GTK_ENTRY(lookup_widget(global.string_win, "entry161"));
+    break;
   default:
     break;
   }
@@ -137,7 +143,7 @@ void string_list_add(int slist, char *item)
   if (string_list_search(slist, item))
     return;
 
-  new_item = l_strdup(item);
+  new_item = g_strdup(item);
   global.string_list[slist] =
     g_list_append(global.string_list[slist], new_item);
 
@@ -152,6 +158,7 @@ void string_list_add(int slist, char *item)
   case LIST_BANREASON:
   case LIST_AFKREASON:
   case LIST_NODOWNLOAD:
+  case LIST_QSEARCH:
   default:
     break;
   }
@@ -189,13 +196,14 @@ void string_list_remove(int slist, char *item) {
   case LIST_HIGHLIGHT:
   case LIST_BANREASON:
   case LIST_NODOWNLOAD:
+  case LIST_QSEARCH:
   default:
     break;
   }
 
   // probably update list window
   if (!global.string_win) {
-    l_free(string);
+    g_free(string);
     return;
   }
 
@@ -205,7 +213,7 @@ void string_list_remove(int slist, char *item) {
 
   row = gtk_clist_find_row_from_data(clist, string);
   if (row >= 0) gtk_clist_remove(clist, row);
-  l_free(string);
+  g_free(string);
 }
 
 void highlight_add(char *item)
@@ -216,6 +224,14 @@ void highlight_remove(char *item)
 {
   string_list_remove(LIST_HIGHLIGHT, item);
 }
+void qsearch_add(char *item)
+{
+  string_list_add(LIST_QSEARCH, item);
+}
+void qsearch_remove(char *item)
+{
+  string_list_remove(LIST_QSEARCH, item);
+}
 void friend_add(char *item)
 {
   string_list_add(LIST_FRIEND, item);
