@@ -1,4 +1,4 @@
-/* Copyright (C) 2000/2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -23,5 +23,7 @@ void nodownload_add(char *item);
 void nodownload_remove(char *item);
 void afkreason_add(char *item);
 void afkreason_remove(char *item);
+void qsearch_add(char *item);
+void qsearch_remove(char *item);
 
 GtkWidget *create_string_list_popup(int slist);
