@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -35,12 +35,12 @@ void subscription_load() {
 
   subscription_t* sub;
 
-  filename = l_strdup_printf("%s/subscriptions", global.options.config_dir);
+  filename = g_strdup_printf("%s/subscriptions", global.options.config_dir);
   if ((file = fopen(filename, "r")) == NULL) {
-    l_free(filename);
+    g_free(filename);
     return;
   }
-  l_free(filename);
+  g_free(filename);
 
   sub = NULL;
   while (mfgets(line, sizeof(line), file)) {
@@ -50,7 +50,7 @@ void subscription_load() {
     if (line[0] == '"') {
       filename = arg(line, 2);
       if (sub && filename) {
-	filename = l_strdup(filename);
+	filename = g_strdup(filename);
 	sub->files = g_list_append(sub->files, filename);
 	subscription_file_add_ctree(sub, filename);
       }
@@ -75,14 +75,13 @@ void subscription_save() {
   char* filename;
   subscription_t* sub;
 
-  printf("saveing sub\n");
-  filename = l_strdup_printf("%s/subscriptions", global.options.config_dir);
+  filename = g_strdup_printf("%s/subscriptions", global.options.config_dir);
   if ((file = fopen(filename, "w")) == NULL) {
     g_warning("Could not write subscriptions");
-    l_free(filename);
+    g_free(filename);
     return;
   }
-  l_free(filename);
+  g_free(filename);
 
   for (dlist = global.subscriptions; dlist; dlist = dlist->next) {
     sub = dlist->data;
@@ -99,8 +98,8 @@ void subscription_save() {
 subscription_t* subscription_new(char* user) {
   subscription_t* sub;
 
-  sub = l_malloc(sizeof(subscription_t));
-  sub->user = l_strdup(user);
+  sub = g_malloc(sizeof(subscription_t));
+  sub->user = g_strdup(user);
   sub->friend_while_subs = 1;
   sub->files = NULL;
   global.subscriptions = g_list_append(global.subscriptions, sub);
@@ -115,11 +114,11 @@ void subscription_destroy(subscription_t* sub) {
 
   for (dlist = sub->files; dlist; dlist = dlist->next) {
     filename = dlist->data;
-    l_free(filename);
+    g_free(filename);
   }
   g_list_free(sub->files);
-  l_free(sub->user);
-  l_free(sub);
+  g_free(sub->user);
+  g_free(sub);
   global.subscriptions = g_list_remove(global.subscriptions, sub);
   check_uploads_again();
 
@@ -278,7 +277,7 @@ void subscription_file_add(subscription_t* sub, char* name) {
   if (!sub) return;
   if (subscription_file_search(sub, name)) return;
 
-  tfile = l_strdup(name);
+  tfile = g_strdup(name);
   sub->files = g_list_append(sub->files, tfile);
   subscription_save();
   subscription_file_add_ctree(sub, tfile);
@@ -327,7 +326,7 @@ void subscription_file_remove(subscription_t* sub, char* file) {
   check_uploads_again();
 
   subscription_file_remove_ctree(sub, tfile);
-  l_free(tfile);
+  g_free(tfile);
   subscription_save();
 }
 
@@ -380,14 +379,6 @@ void on_friend_while_activate(GtkMenuItem * menuitem ATTR_UNUSED, gpointer user_
 }
 
 
-void on_play_file5_activate(GtkMenuItem * menuitem ATTR_UNUSED, gpointer user_data)
-{
-  char* file = user_data;
-  
-  if (!file) return;
-  play_file(file);
-}
-
 GtkWidget *create_subscription_popup()
 {
   GtkWidget *popup = NULL;
@@ -421,12 +412,8 @@ GtkWidget *create_subscription_popup()
     gtk_signal_connect(GTK_OBJECT(item), "activate",
 		       GTK_SIGNAL_FUNC(on_sub_remove_activate), filename);
 
-    item = gtk_menu_item_new_with_label("Open File");
-    gtk_widget_show(item);
+    item = create_open_menu(mtype_get(filename), filename, 1);
     gtk_container_add(GTK_CONTAINER(popup), item);
-    gtk_signal_connect(GTK_OBJECT(item), "activate",
-		       GTK_SIGNAL_FUNC(on_play_file5_activate), filename);
-
   } else {
     popup = gtk_menu_new();
     item = gtk_menu_item_new_with_label("Remove user");
