@@ -1,11 +1,12 @@
-/* Copyright (C) 2000/2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
 extern const int StatusInfo[S_NUMBER];
 
-transfer_data_t* transfer_data_new(int status);
-void transfer_data_destroy(transfer_data_t* data);
+void transfer_init(transfer_t* trans, int status);
+void transfer_destroy(transfer_t* data);
+int transfer_in_progress(transfer_t *trans);
 long upload_calc_max(user_info_t* userinfo);
 int transfer_status_dist(int s1, int s2);
 GdkPixmap*
@@ -20,7 +21,6 @@ upload_t *upload_new(net_t* net, char* filename, char* user,
 		     int offline);
 void upload_destroy(upload_t *upload);
 void upload_hide(socket_t *socket);
-int upload_in_progress(upload_t *upload);
 void upload_status_set(socket_t * socket, int status);
 socket_t *upload_search_mapable(net_t* net, char *user, 
 				char *winname);
@@ -44,7 +44,6 @@ download_t *download_new(file_t* file);
 void download_destroy(download_t *download);
 void download_hide(socket_t * socket);
 void download_queued_update(socket_t* socket);
-int download_in_progress(download_t * download);
 void download_status_set(socket_t * socket, int status);
 socket_t *download_search_mapable(net_t* net, char *user, 
 				  char *winname);
@@ -81,7 +80,7 @@ void retry_timeout_check();
 GtkWidget *create_advanced_popup();
 GtkWidget *create_download_popup();
 void create_bandwidth_win(user_info_t* userinfo, int download);
-void create_bandman_win (int download);
+void create_bandman_win();
 GtkWidget* create_bandwidth_popup(user_info_t* userinfo);
 
 void dcc_send_file(char *nick, char *file, net_t* net);
