@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -26,82 +26,100 @@
 #include "whois.h"
 #include "utils.h"
 
-#define INFO_LIFE        15    // days
+#define INFO_LIFE        10    // days
 
 struct server_t;
 
+static GHashTable* UserInfo = NULL;
+
+/*
+static void user_info_destroy(user_info_t* userinfo) {
+  if (!userinfo) return;
+  if (userinfo->nick) g_free(userinfo->nick);
+  if (userinfo->last_server) g_free(userinfo->last_server);
+  g_free(userinfo);
+}
+*/
 user_info_t* user_info_new(char* user) {
-  user_info_t* user_info;
+  user_info_t* info;
 
-  user_info = l_malloc(sizeof(user_info_t));
-  user_info->last_server = NULL;
-  user_info->nick = l_strdup(user);
-  user_info->cur[0] = 0;
-  user_info->cur[1] = 0;
-  user_info->real_cur[0] = 0;
-  user_info->real_cur[1] = 0;
-  user_info->max[0] = -1;
-  user_info->max[1] = -1;
-  user_info->limit[0] = 0;
-  user_info->limit[1] = 0;
-  user_info->bytes[0] = 0;
-  user_info->bytes[1] = 0;
-  user_info->ignore_global[0] = 0;
-  user_info->ignore_global[1] = 0;
-  user_info->linespeed = -1;
-  user_info->timestamp = 1;
-  user_info->last_seen = global.current_time;
-  user_info->shares = -1;
-  user_info->remotes = 0;
-  user_info->client = C_UNKNOWN;
-  global.userinfo = g_list_prepend(global.userinfo, user_info);
+  info = g_malloc(sizeof(*info));
+  info->last_server = NULL;
+  info->nick = g_strdup(user);
+  info->cur[0] = 0;
+  info->cur[1] = 0;
+  info->real_cur[0] = 0;
+  info->real_cur[1] = 0;
+  info->max[0] = -1;
+  info->max[1] = -1;
+  info->limit[0] = 0;
+  info->limit[1] = 0;
+  info->bytes[0] = 0;
+  info->bytes[1] = 0;
+  info->ignore_global[0] = 0;
+  info->ignore_global[1] = 0;
+  info->linespeed = -1;
+  info->timestamp = 1;
+  info->last_seen = global.current_time;
+  info->shares = -1;
+  info->remotes = 0;
+  info->client = C_UNKNOWN;
 
-  return user_info;
+  if (!UserInfo) {
+    UserInfo =
+      g_hash_table_new((GHashFunc)g_str_hash, (GCompareFunc)g_str_equal);
+  }
+  g_hash_table_insert(UserInfo, info->nick, info);
+
+  return info;
 }
 
-void user_info_destroy(user_info_t* userinfo) {
-  if (!userinfo) return;
-  global.userinfo = g_list_remove(global.userinfo, userinfo);
-  if (userinfo->nick) l_free(userinfo->nick);
-  if (userinfo->last_server) l_free(userinfo->last_server);
-  l_free(userinfo);
+int user_info_is_default(user_info_t* info) {
+  if (info->max[0] == -1 && info->max[1] == -1 &&
+      info->limit[0] == 0 && info->limit[1] == 0 &&
+      info->ignore_global[0] == 0 && info->ignore_global[1] == 0)
+    return 1;
+  else
+    return 0;
+}
+
+static void
+user_info_save_entry(const char* nick, user_info_t* info,
+		     FILE* file) {
+  (void)nick;
+  // dont save nicks that are expired and have default settings
+  if (global.current_time - info->last_seen > 60*60*24*INFO_LIFE &&
+      user_info_is_default(info)) return;
+
+  qfprintf(file, "%s %d %d %ld %ld %d %d %lu %S\n",
+	   info->nick, info->max[0], info->max[1], info->limit[0], info->limit[1],
+	   info->ignore_global[0], info->ignore_global[1], info->last_seen,
+	   info->last_server?info->last_server:"");
 }
 
 void user_info_save() {
-  GList *dlist;
   char *fname;
   char *fname_new;
   FILE *fd;
-  user_info_t *info;
 
-  fname = l_strdup_printf("%s/userinfo.list", global.options.config_dir);
-  fname_new = l_strdup_printf("%s/access.list.new", global.options.config_dir);
+  fname = g_strdup_printf("%s/userinfo.list", global.options.config_dir);
+  fname_new = g_strdup_printf("%s/access.list.new", global.options.config_dir);
 
   if ((fd = fopen(fname_new, "w")) == NULL) {
     g_warning("Could not write [%s]\n", fname);
-    l_free(fname);
-    l_free(fname_new);
+    g_free(fname);
+    g_free(fname_new);
     return;
   }
 
-  for (dlist = global.userinfo; dlist; dlist = dlist->next) {
-    info = dlist->data;
+  g_hash_table_foreach(UserInfo, (GHFunc)user_info_save_entry, fd);
 
-    if (global.current_time - info->last_seen > 60*60*24*INFO_LIFE) continue;
-    qfprintf(fd, "%s %d %d %ld %ld %d %d %lu %S\n",
-	     info->nick, info->max[0], info->max[1], info->limit[0], info->limit[1],
-	     info->ignore_global[0], info->ignore_global[1], info->last_seen,
-             info->last_server?info->last_server:"");
-  }
-  
   if (!ferror(fd)) rename(fname_new, fname);
-  else {
-    g_warning("Could not write [%s]\n", fname);
-  }
+  else g_warning("Could not write [%s]\n", fname);
 
   fclose(fd);
-  l_free(fname);
-  l_free(fname_new);
+  g_free(fname);
+  g_free(fname_new);
 }
 
 void user_info_load() {
@@ -118,12 +136,12 @@ void user_info_load() {
   char line[2048];
 
   //  printf("user_info_load(): %ld\n", time(NULL));
-  fname = l_strdup_printf("%s/userinfo.list", global.options.config_dir);
+  fname = g_strdup_printf("%s/userinfo.list", global.options.config_dir);
   if ((fd = fopen(fname, "r")) == NULL) {
-    l_free(fname);
+    g_free(fname);
     return;
   }
-  l_free(fname);
+  g_free(fname);
 
   while (mfgets(line, sizeof(line), fd)) {
     user = arg(line, 2);
@@ -136,7 +154,7 @@ void user_info_load() {
     seen = arg(NULL, 2);
     server = arg(NULL, 2);
     
-    if (!user || !max1 || !max2 || !limit1 || !limit2 || !ignore1 || !ignore2) continue;
+    if (!ignore2) continue;
 
     if (seen) last_seen = strtol(seen, NULL, 10);
     else last_seen = global.current_time;
@@ -150,9 +168,9 @@ void user_info_load() {
     userinfo->limit[0] = strtol(limit1, NULL, 10);
     userinfo->limit[1] = strtol(limit2, NULL, 10);
     userinfo->ignore_global[0] = atoi(ignore1);
-    userinfo->ignore_global[0] = atoi(ignore2);
+    userinfo->ignore_global[1] = atoi(ignore2);
     userinfo->last_seen = last_seen;
-    if (server) userinfo->last_server = l_strdup(server);
+    userinfo->last_server = server?g_strdup(server):NULL;
   }
 
   fclose(fd);
@@ -160,20 +178,10 @@ void user_info_load() {
   return;
 }
 
-user_info_t *user_info_search(char *user) {
-  GList *source;
-  GList *dlist;
-  user_info_t *user_info;
-
-  source = global.userinfo;
-
-  for (dlist = source; dlist; dlist = dlist->next) {
-    user_info = dlist->data;
-    if (!l_strcasecmp(user_info->nick, user))
-      return user_info;
-  }
-
-  return NULL;
+user_info_t* user_info_search(char* user) {
+  user_info_t* info;
+  info = g_hash_table_lookup(UserInfo, user);
+  return info;
 }
 
 int user_info_priority(user_info_t* userinfo, int* class) {
@@ -220,7 +228,6 @@ void user_info_show(user_info_t* userinfo, int download, int active) {
   GtkWidget* spin;
   int posx;
   int posy;
-  int dx, dy;
   long limit;
   char* ss;
   int width, height;
@@ -228,49 +235,42 @@ void user_info_show(user_info_t* userinfo, int download, int active) {
   char str[1024];
   GtkStyle* style;
 
-  if (!GTK_WIDGET_REALIZED(global.progressbar))
-    gtk_widget_realize(global.progressbar);
-  style = global.progressbar->style;
-  dx = style->klass->xthickness;
-  dy = style->klass->ythickness;
-
   if (download) {
     limit = global.down_width.limit;
     area = lookup_widget(global.win, "drawingarea12");
     spin = lookup_widget(global.win, "spinbutton15");
+    style = area->style;
   } else {
     limit = global.up_width.limit;
     area = lookup_widget(global.win, "drawingarea11");
     spin = lookup_widget(global.win, "spinbutton49");
+    style = area->style;
   }
 
+  if (!area->window) return;
 
   if (limit <= 0) limit = 1024*80;
-
   gtk_object_set_data(GTK_OBJECT(area), "userinfo", userinfo);
   gtk_object_set_data(GTK_OBJECT(spin), "userinfo", userinfo);
 
-  if (!area->window) return;
   width = area->allocation.width;
   height = area->allocation.height;
   
-  gtk_paint_box (style, area->window,
-		 GTK_STATE_NORMAL, GTK_SHADOW_IN, 
-		 NULL, area, "trough",
-		 0, 0, width, height);
+  gdk_draw_rectangle(area->window,
+		     style->bg_gc[0], TRUE,
+		     0, 0, width, height);
 
   if (!userinfo) return;
 
   if (!active) 
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), (gfloat) (userinfo->max[upload]));
 
-  posx = (width - 2*dx) * userinfo->limit[upload] / limit;
-  if (posx > width) posx = width;
+  posx = (width - 4) * userinfo->limit[upload] / limit;
+  if (posx > width-4) posx = width-4;
   if (posx)
-    gtk_paint_box(style, area->window,
-		  GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-		  NULL, area, "bar", dx, dy, posx, height-2*dy);
-
+    gdk_draw_rectangle(area->window, style->bg_gc[3], TRUE,
+		       2, 2, posx, height-4);
+  
   if (!active) {
     ss = userinfo->nick;
   } else if (userinfo->limit[upload] == 0) {
@@ -279,17 +279,19 @@ void user_info_show(user_info_t* userinfo, int download, int active) {
   } else {
     ss = print_speed(str, userinfo->limit[upload], 1);
   }
-  posx = (width - 2*dx - gdk_string_width(style->font, ss))/2;
-  posy = (height + style->font->ascent - 2*dy)/2+1;
+  posx = (width - 4 - gdk_string_width(style->font, ss))/2;
+  posy = (height + style->font->ascent - 4)/2+1;
   gdk_draw_text (area->window, style->font,
-		 style->fg_gc[GTK_STATE_NORMAL],
+		 style->fg_gc[3],
 		 posx, posy, ss, strlen (ss));
 }
 
-// this function could be dangerous, as it does delete user infos that
-// might be still in use somewhere else
-// deactivated at the moment.
-
+/* this function could be dangerous, as it does delete user infos that
+ * might be still in use somewhere else (actually this might only
+ * happen if uptime > INFO_FILE (10 days)
+ */
+// deactivated at the moment, cleanup is done at exit
+/*
 int user_info_cleanup(gpointer data ATTR_UNUSED) {
   GList* dlist;
   user_info_t* info;
@@ -305,6 +307,7 @@ int user_info_cleanup(gpointer data ATTR_UNUSED) {
   // not finished yet.
   return 1;
 }
+*/
 
 user_info_t* user_info_detect(net_t* net, char* user) {
   user_info_t *user_info;
@@ -317,3 +320,19 @@ user_info_t* user_info_detect(net_t* net, char* user) {
   return user_info;
 }
 
+static void ui_fe(char* key, user_info_t* info, void** data) {
+  UserInfoFunc func = data[0];
+  void* user_data = data[1];
+
+  (void)key;
+  if (info) func(info, user_data);
+}
+
+void user_info_foreach(UserInfoFunc func, void* data) {
+  static void* array[2];
+  
+  if (!UserInfo) return;
+  array[0] = func;
+  array[1] = data;
+  g_hash_table_foreach(UserInfo, (GHFunc)ui_fe, array);
+}
