@@ -1,4 +1,4 @@
-/* Copyright (C) 2000/2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -26,6 +26,13 @@
 #include "whois.h"
 #include "utils.h"
 
+/*
+static void request_online_servers(whois_t* whois) {
+  if (!whois || !NET_CONNECTED(whois->net)) return;
+  send_notice(whois->net, whois->user, "!GetOnline", 0);
+}
+*/
+
 void whois_request(net_t* net, char *user, int flag) {
   whois_t *whois;
 
@@ -51,13 +58,13 @@ whois_t *whois_new(char *user, int flag)
 
   if (!user) return NULL;
 
-  result = (whois_t *) l_malloc(sizeof(whois_t));
+  result = (whois_t *) g_malloc(sizeof(whois_t));
   result->flag = flag;
   result->received = 0;
   result->online = 0;
   result->advanced = 0;
   result->yourself = 0;
-  result->user = l_strdup(user);
+  result->user = g_strdup(user);
   result->level = L_USER;
   result->online_time = 0;
   result->channels = NULL;
@@ -84,20 +91,20 @@ whois_t *whois_new(char *user, int flag)
 void whois_destroy(whois_t * whois)
 {
   if (whois->user)
-    l_free(whois->user);
+    g_free(whois->user);
   if (whois->channels)
-    l_free(whois->channels);
+    g_free(whois->channels);
   if (whois->status)
-    l_free(whois->status);
+    g_free(whois->status);
   if (whois->client)
-    l_free(whois->client);
+    g_free(whois->client);
   if (whois->ip)
-    l_free(whois->ip);
+    g_free(whois->ip);
   if (whois->email)
-    l_free(whois->email);
+    g_free(whois->email);
   if (whois->where)
-    l_free(whois->where);
-  l_free(whois);
+    g_free(whois->where);
+  g_free(whois);
 }
 
 static char* get_whois_tag(int id) {
@@ -177,33 +184,33 @@ static void whois_action_text(whois_t * whois) {
   if (whois->online) {
     print_whois_tag(page, 1);
     if (whois->online_time >= 60 * 60 * 24)
-      text = l_strdup_printf("%ld days %ld hours %ld minutes %ld seconds",
+      text = g_strdup_printf("%ld days %ld hours %ld minutes %ld seconds",
 			     whois->online_time / (24 * 60 * 60),
 			     (whois->online_time % (24 * 60 * 60)) / (60 * 60),
 			     (whois->online_time % (60 * 60)) / 60,
 			     whois->online_time % 60);
     else if (whois->online_time >= 60 * 60)
-      text = l_strdup_printf("%ld hours %ld minutes %ld seconds",
+      text = g_strdup_printf("%ld hours %ld minutes %ld seconds",
 			     whois->online_time / (60 * 60),
 			     (whois->online_time % (60 * 60)) / 60,
 			     whois->online_time % 60);
     else
-      text = l_strdup_printf("%ld minutes %ld seconds",
+      text = g_strdup_printf("%ld minutes %ld seconds",
 			     whois->online_time / 60,
 			     whois->online_time % 60);
     chat_print_text(page, M_PUBLIC, "text", text);
     chat_print_text(page, M_PUBLIC, "text", "\n");
-    l_free(text);
+    g_free(text);
   } else if (whois->online_time > 0) {
     print_whois_tag(page, 2);
-    text = l_strdup_printf("%s", ctime(&whois->online_time));
+    text = g_strdup_printf("%s", ctime(&whois->online_time));
     chat_print_text(page, M_PUBLIC, "text", text);
-    l_free(text);
+    g_free(text);
   } else {
     print_whois_tag(page, 3);
-    text = l_strdup("Unregistered\n");
+    text = g_strdup("Unregistered\n");
     chat_print_text(page, M_PUBLIC, "text", text);
-    l_free(text);
+    g_free(text);
     return;
   }
   
@@ -219,7 +226,7 @@ static void whois_action_text(whois_t * whois) {
   if (!whois->online) return;
 
   print_whois_tag(page, 5);
-  channels = l_strdup(whois->channels);
+  channels = g_strdup(whois->channels);
   channel = arg(channels, 0);
   while (channel) {
     chat_print_text(page, M_PUBLIC, "user", "[");
@@ -227,7 +234,7 @@ static void whois_action_text(whois_t * whois) {
     chat_print_text(page, M_PUBLIC, "user", "]");
     channel = arg(NULL, 0);
   }
-  l_free(channels);
+  g_free(channels);
   chat_print_text(page, M_PUBLIC, "text", "\n");
   
   print_whois_tag(page, 6);
@@ -299,10 +306,10 @@ static void whois_action_show(whois_t * whois)
   gtk_widget_show(global.whois_win);
   gtk_object_set_data(GTK_OBJECT(global.whois_win), "network", whois->net);
   
-  text = l_strdup_printf("%s [%s]", "Whois User", 
+  text = g_strdup_printf("%s [%s]", "Whois User", 
 			 whois->net->name);
   gtk_window_set_title(GTK_WINDOW(global.whois_win), text);
-  l_free(text);
+  g_free(text);
 
   temp = lookup_widget(global.whois_win, "label116");
   if (whois->online)
@@ -322,28 +329,28 @@ static void whois_action_show(whois_t * whois)
   if (whois->online) {
     if (whois->online_time >= 60 * 60 * 24)
       text =
-	  l_strdup_printf("%ld days %ld hours %ld minutes %ld seconds",
+	  g_strdup_printf("%ld days %ld hours %ld minutes %ld seconds",
 			  whois->online_time / (24 * 60 * 60),
 			  (whois->online_time % (24 * 60 * 60)) / (60 *
 								   60),
 			  (whois->online_time % (60 * 60)) / 60,
 			  whois->online_time % 60);
     else if (whois->online_time >= 60 * 60)
-      text = l_strdup_printf("%ld hours %ld minutes %ld seconds",
+      text = g_strdup_printf("%ld hours %ld minutes %ld seconds",
 			     whois->online_time / (60 * 60),
 			     (whois->online_time % (60 * 60)) / 60,
 			     whois->online_time % 60);
     else
-      text = l_strdup_printf("%ld minutes %ld seconds",
+      text = g_strdup_printf("%ld minutes %ld seconds",
 			     whois->online_time / 60,
 			     whois->online_time % 60);
   } else if (whois->online_time > 0) {
-    text = l_strdup_printf("%s", ctime(&whois->online_time));
+    text = g_strdup_printf("%s", ctime(&whois->online_time));
   } else {
-    text = l_strdup("Unregistered");
+    text = g_strdup("Unregistered");
   }
   gtk_entry_set_text(entry, text);
-  l_free(text);
+  g_free(text);
 
   entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry27"));
   if (whois->online) {
@@ -475,8 +482,8 @@ static void whois_action_userinfo(whois_t * whois) {
   userinfo->shares = whois->shared;
   userinfo->timestamp = global.current_time;
   userinfo->last_seen = global.current_time;
-  if (userinfo->last_server) l_free(userinfo->last_server);
-  userinfo->last_server = l_strdup(whois->net->name);
+  if (userinfo->last_server) g_free(userinfo->last_server);
+  userinfo->last_server = g_strdup(whois->net->name);
   if (!strncasecmp(whois->client, "WinMX v2.6", 10))
     userinfo->client = C_WINMX_26;
   else if (!strncasecmp(whois->client, "WinMX", 5))
@@ -489,12 +496,24 @@ static void whois_action_userinfo(whois_t * whois) {
     int v3 = 0;
     int v4 = 0;
     if (sscanf(whois->client+7, " %d.%d.%d-dev%d", 
-	       &v1, &v2, &v3, &v4) < 2) userinfo->client = C_OTHER;
-    else if ((v1 > 1) ||
-	     (v1 == 1 && v2 > 0) ||
-	     (v1 == 1 && v2 == 0 && v3 > 1) ||
-	     (v1 == 1 && v2 == 0 && v3 == 1 && v4 >= 6))
+	       &v1, &v2, &v3, &v4) < 2) {
+      userinfo->client = C_OTHER;
+      /*
+    } else if ((v1 > 1) ||
+	       (v1 == 1 && v2 > 2) ||
+	       (v1 == 1 && v2 == 2 && v3 > 0) ||
+	       (v1 == 1 && v2 == 2 && v3 == 0 && v4 >= 2)) {
       userinfo->client = C_LOPSTER_ADV;
+      request_online_servers(whois);
+      */
+    } else if ((v1 > 1) ||
+	       (v1 == 1 && v2 > 0) ||
+	       (v1 == 1 && v2 == 0 && v3 > 1) ||
+	       (v1 == 1 && v2 == 0 && v3 == 1 && v4 >= 6)) {
+      userinfo->client = C_LOPSTER_ADV;
+    } else {
+      userinfo->client = C_OTHER;
+    }
   } else
     userinfo->client = C_OTHER;
 }
@@ -523,7 +542,7 @@ whois_t *whois_search(net_t* net, char *user) {
   for (dlist = global.whois_requests; dlist; dlist = dlist->next) {
     whois = dlist->data;
     if (whois->net != net) continue;
-    if (!l_strcasecmp(whois->user, user))
+    if (!g_strcasecmp(whois->user, user))
       return whois;
   }
   return NULL;
@@ -557,7 +576,7 @@ whois_t *whois_search_unreceived(net_t* net, char *user)
   for (dlist = global.whois_requests; dlist; dlist = dlist->next) {
     whois = dlist->data;
     if (whois->net != net) continue;
-    if (!l_strcasecmp(whois->user, user) && !whois->received)
+    if (!g_strcasecmp(whois->user, user) && !whois->received)
       return whois;
   }
   return NULL;
@@ -574,10 +593,10 @@ void whois_eval_whois(net_t* net, char *data)
 
   if (!whois) return;
 
-  l_free(whois->user);
-  whois->user = l_strdup(pos1);
+  g_free(whois->user);
+  whois->user = g_strdup(pos1);
 
-  if (!l_strcasecmp(whois->user, net->user.username))
+  if (!g_strcasecmp(whois->user, net->user.username))
     whois->yourself = 1;
 
   whois->online = 1;
@@ -593,15 +612,17 @@ void whois_eval_whois(net_t* net, char *data)
 
   pos1 = arg(NULL, 0);
   if (!pos1) return;
-  if (whois->channels)
-    l_free(whois->channels);
-  whois->channels = l_strdup(pos1);
+  if (whois->channels) g_free(whois->channels);
+  if (net->subtype == N_CLEANNAP)
+    whois->channels = g_strdup(strcmp(pos1, "none")?pos1:"");
+  else
+    whois->channels = g_strdup(pos1);
 
   pos1 = arg(NULL, 0);
   if (!pos1) return;
   if (whois->status)
-    l_free(whois->status);
-  whois->status = l_strdup(pos1);
+    g_free(whois->status);
+  whois->status = g_strdup(pos1);
 
   pos1 = arg(NULL, 0);
   if (!pos1) return;
@@ -621,8 +642,8 @@ void whois_eval_whois(net_t* net, char *data)
 
   pos1 = arg(NULL, 0);
   if (!pos1) return;
-  if (whois->client) l_free(whois->client);
-  whois->client = l_strdup(pos1);
+  if (whois->client) g_free(whois->client);
+  whois->client = g_strdup(pos1);
 
   pos1 = arg(NULL, 0);
   if (!pos1) {
@@ -643,8 +664,8 @@ void whois_eval_whois(net_t* net, char *data)
       whois_action(whois);
       return;
     }
-    if (whois->ip) l_free(whois->ip);
-    whois->ip = l_strdup(pos1);
+    if (whois->ip) g_free(whois->ip);
+    whois->ip = g_strdup(pos1);
 
     pos1 = arg(NULL, 0);
     if (!pos1) {
@@ -662,14 +683,14 @@ void whois_eval_whois(net_t* net, char *data)
 
     pos1 = arg(NULL, 0);
     if (pos1) {
-      if (whois->email) l_free(whois->email);
-      whois->email = l_strdup(pos1);
+      if (whois->email) g_free(whois->email);
+      whois->email = g_strdup(pos1);
     }
 
     pos1 = arg(NULL, 0);
     if (pos1) {
-      if (whois->where) l_free(whois->where);
-      whois->where = l_strdup(pos1);
+      if (whois->where) g_free(whois->where);
+      whois->where = g_strdup(pos1);
     }
   }
 
@@ -686,8 +707,8 @@ void whois_eval_whowas(net_t* net, char *data)
   whois = whois_search_unreceived(net, pos1);
   if (!whois) return;
 
-  l_free(whois->user);
-  whois->user = l_strdup(pos1);
+  g_free(whois->user);
+  whois->user = g_strdup(pos1);
 
   whois->online = 0;
   whois->advanced = 0;
@@ -710,8 +731,8 @@ void whois_eval_not_online(net_t* net, char *data)
 
   if (!whois) return;
 
-  l_free(whois->user);
-  whois->user = l_strdup(data);
+  g_free(whois->user);
+  whois->user = g_strdup(data);
 
   whois->online = 0;
   whois->advanced = 0;
@@ -728,10 +749,10 @@ static speed_t *speed_new(char *user) {
   if (!user || !(*user))
     return NULL;
 
-  speed = (speed_t *) l_malloc(sizeof(speed_t));
+  speed = (speed_t *) g_malloc(sizeof(speed_t));
   speed->speed = -1;
   speed->sent = 0;
-  speed->user = l_strdup(user);
+  speed->user = g_strdup(user);
   speed->net = NULL;
   global.speed_requests = g_list_prepend(global.speed_requests, speed);
 
@@ -763,8 +784,8 @@ void speed_request(net_t* net, char *user) {
 
 void speed_destroy(speed_t * speed) {
   if (speed->user)
-    l_free(speed->user);
-  l_free(speed);
+    g_free(speed->user);
+  g_free(speed);
 }
 
 void speed_remove(speed_t * speed) {
@@ -804,12 +825,12 @@ static void speed_action_show(speed_t * speed) {
   chat_print_nick(page, M_PUBLIC, speed->user, speed->net);
   chat_print_text(page, M_PUBLIC, "user", "> ");
   if (speed->speed >= 0)
-    str = l_strdup_printf("has linespeed [%s]\n",
+    str = g_strdup_printf("has linespeed [%s]\n",
 			  LineSpeed(speed->speed));
   else
-    str = l_strdup_printf("seems to be offline\n");
+    str = g_strdup_printf("seems to be offline\n");
   chat_print_text(page, M_PUBLIC, "message", str);
-  l_free(str);
+  g_free(str);
 }
 
 void speed_action(speed_t * speed) {
@@ -826,7 +847,7 @@ speed_t *speed_search(net_t* net, char *user) {
 
   for (dlist = global.speed_requests; dlist; dlist = dlist->next) {
     speed = dlist->data;
-    if (!l_strcasecmp(speed->user, user) && (speed->net == net))
+    if (!g_strcasecmp(speed->user, user) && (speed->net == net))
       return speed;
   }
   return NULL;
@@ -858,7 +879,7 @@ GtkWidget* create_join_popup(net_t* net) {
     return popup;
   } else {
     entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry27"));
-    channels = l_strdup(gtk_entry_get_text(entry));
+    channels = g_strdup(gtk_entry_get_text(entry));
     cnt = 0;
     channel = arg(channels, 0);
     while (channel) {
@@ -871,7 +892,7 @@ GtkWidget* create_join_popup(net_t* net) {
       
       channel = arg(NULL, 0);
     }
-    l_free(channels);
+    g_free(channels);
     if (cnt == 0) {
       item = gtk_menu_item_new_with_label("No channels");
       gtk_widget_show(item);
