@@ -1,4 +1,4 @@
-/* Copyright (C) 2000/2001 sgop@users.sourceforge.net
+/* Copyright (C) 2000-2003 Markus Lausser (sgop@users.sf.net)
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
@@ -51,6 +51,7 @@
 #include "string_list.h"
 #include "utils.h"
 #include "dialog.h"
+#include "mtypes.h"
 
 #define PAGES   5
 
@@ -241,6 +242,8 @@ static void wizard_finish() {
   lib_refresh(global.lib);
 
   gtk_widget_destroy(ww.win);
+
+  global.status.rc_read = 1;
 }
 
 static void wizard_apply_current() {
@@ -249,17 +252,16 @@ static void wizard_apply_current() {
   GtkToggleButton* toggle;
   char *text;
   char *text2;
-  suffix_t *suffix;
   int mime, i1, i2;
 
   switch (ww.page) {
   case 1:
     entry = GTK_ENTRY(ww.page1_entry[0]);
     text = gtk_entry_get_text(entry);
-    global.username = l_strdup(text);
+    global.username = g_strdup(text);
     entry = GTK_ENTRY(ww.page1_entry[1]);
     text = gtk_entry_get_text(entry);
-    global.password = l_strdup(text);
+    global.password = g_strdup(text);
     break;
   case 2:
     entry = GTK_ENTRY(ww.page2_entry[0]);
@@ -267,10 +269,10 @@ static void wizard_apply_current() {
     global.linespeed = speed2int(text);
 
     entry = GTK_ENTRY(ww.page2_entry[1]);
-    text = strdup(gtk_entry_get_text(entry));
+    text = g_strdup(gtk_entry_get_text(entry));
     destroy_string_list(global.allowed_ports);
     global.allowed_ports = create_allowed_ports(text);
-    free(text);
+    g_free(text);
 
     toggle = GTK_TOGGLE_BUTTON(ww.page2_radio[0]);
     if (gtk_toggle_button_get_active(toggle)) i2 = 1;
@@ -283,143 +285,76 @@ static void wizard_apply_current() {
     
     global.network.port = i1;
     global.network.firewall = i2;
-    create_upload_port(i1, TRUE);
+    create_upload_port(i1, FALSE);
     break;
   case 3:
-    clear_suffixes();
+    mtype_clear_all();
     entry = GTK_ENTRY(ww.page3_entry[0]);
-    text = l_strdup(gtk_entry_get_text(entry));
-    text2 = arg(text, 0);
-    while (text2) {
-      suffix = l_malloc(sizeof(suffix_t));
-      suffix->suffix = l_strdup(text2);
-      suffix->application = l_strdup("");
-      suffix->as_mp3 = 1;
-      global.mimetype[MIME_MP3].suffixes =
-	g_list_append(global.mimetype[MIME_MP3].suffixes, suffix);
-      text2 = arg(NULL, 0);
-    }
-    l_free(text);
+    text = g_strdup(gtk_entry_get_text(entry));
+    mtype_create_media(MEDIA_MP3, text);
+    g_free(text);
     
     entry = GTK_ENTRY(ww.page3_entry[1]);
-    text = l_strdup(gtk_entry_get_text(entry));
-    text2 = arg(text, 0);
-    while (text2) {
-      suffix = l_malloc(sizeof(suffix_t));
-      suffix->suffix = l_strdup(text2);
-      suffix->application = l_strdup("");
-      suffix->as_mp3 = 0;
-      global.mimetype[MIME_VIDEO].suffixes =
-	  g_list_append(global.mimetype[MIME_VIDEO].suffixes, suffix);
-      text2 = arg(NULL, 0);
-    }
-    l_free(text);
+    text = g_strdup(gtk_entry_get_text(entry));
+    mtype_create_media(MEDIA_VIDEO, text);
+    g_free(text);
 
     entry = GTK_ENTRY(ww.page3_entry[2]);
-    text = l_strdup(gtk_entry_get_text(entry));
-    text2 = arg(text, 0);
-    while (text2) {
-      suffix = l_malloc(sizeof(suffix_t));
-      suffix->suffix = l_strdup(text2);
-      suffix->application = l_strdup("");
-      suffix->as_mp3 = 0;
-      global.mimetype[MIME_AUDIO].suffixes =
-	g_list_append(global.mimetype[MIME_AUDIO].suffixes, suffix);
-      text2 = arg(NULL, 0);
-    }
-    l_free(text);
+    text = g_strdup(gtk_entry_get_text(entry));
+    mtype_create_media(MEDIA_AUDIO, text);
+    g_free(text);
 
     entry = GTK_ENTRY(ww.page3_entry[3]);
-    text = l_strdup(gtk_entry_get_text(entry));
-    text2 = arg(text, 0);
-    while (text2) {
-      suffix = l_malloc(sizeof(suffix_t));
-      suffix->suffix = l_strdup(text2);
-      suffix->application = l_strdup("");
-      suffix->as_mp3 = 0;
-      global.mimetype[MIME_APPLICATION].suffixes =
-	  g_list_append(global.mimetype[MIME_APPLICATION].suffixes,
-			suffix);
-      text2 = arg(NULL, 0);
-    }
-    l_free(text);
+    text = g_strdup(gtk_entry_get_text(entry));
+    mtype_create_media(MEDIA_APPLICATION, text);
+    g_free(text);
 
     entry = GTK_ENTRY(ww.page3_entry[4]);
-    text = l_strdup(gtk_entry_get_text(entry));
-    text2 = arg(text, 0);
-    while (text2) {
-      suffix = l_malloc(sizeof(suffix_t));
-      suffix->suffix = l_strdup(text2);
-      suffix->application = l_strdup("");
-      suffix->as_mp3 = 0;
-      global.mimetype[MIME_IMAGE].suffixes =
-	  g_list_append(global.mimetype[MIME_IMAGE].suffixes, suffix);
-      text2 = arg(NULL, 0);
-    }
-    l_free(text);
+    text = g_strdup(gtk_entry_get_text(entry));
+    mtype_create_media(MEDIA_IMAGE, text);
+    g_free(text);
 
     entry = GTK_ENTRY(ww.page3_entry[5]);
-    text = l_strdup(gtk_entry_get_text(entry));
-    text2 = arg(text, 0);
-    while (text2) {
-      suffix = l_malloc(sizeof(suffix_t));
-      suffix->suffix = l_strdup(text2);
-      suffix->application = l_strdup("");
-      suffix->as_mp3 = 0;
-      global.mimetype[MIME_TEXT].suffixes =
-	  g_list_append(global.mimetype[MIME_TEXT].suffixes, suffix);
-      text2 = arg(NULL, 0);
-    }
-    l_free(text);
+    text = g_strdup(gtk_entry_get_text(entry));
+    mtype_create_media(MEDIA_TEXT, text);
+    g_free(text);
+
+    mtype_create_media(MEDIA_FOLDER, NULL);
     break;
   case 4:
     entry = GTK_ENTRY(ww.page4_entry[0]);
-    text = l_strdup(gtk_entry_get_text(entry));
+    text = g_strdup(gtk_entry_get_text(entry));
     if (global.incomplete_path)
-      l_free(global.incomplete_path);
+      g_free(global.incomplete_path);
     global.incomplete_path = text;
 
     entry = GTK_ENTRY(ww.page4_entry[1]);
-    text = l_strdup(gtk_entry_get_text(entry));
-    if (global.mimetype[MIME_NONE].download)
-      l_free(global.mimetype[MIME_NONE].download);
-    global.mimetype[MIME_NONE].download = text;
+    text = gtk_entry_get_text(entry);
+    mtype_set_download(MEDIA_NONE, text);
 
     entry = GTK_ENTRY(ww.page4_entry[2]);
-    text = l_strdup(gtk_entry_get_text(entry));
-    if (global.mimetype[MIME_MP3].download)
-      l_free(global.mimetype[MIME_MP3].download);
-    global.mimetype[MIME_MP3].download = text;
+    text = gtk_entry_get_text(entry);
+    mtype_set_download(MEDIA_MP3, text);
 
     entry = GTK_ENTRY(ww.page4_entry[3]);
-    text = l_strdup(gtk_entry_get_text(entry));
-    if (global.mimetype[MIME_VIDEO].download)
-      l_free(global.mimetype[MIME_VIDEO].download);
-    global.mimetype[MIME_VIDEO].download = text;
+    text = gtk_entry_get_text(entry);
+    mtype_set_download(MEDIA_VIDEO, text);
 
     entry = GTK_ENTRY(ww.page4_entry[4]);
-    text = l_strdup(gtk_entry_get_text(entry));
-    if (global.mimetype[MIME_AUDIO].download)
-      l_free(global.mimetype[MIME_AUDIO].download);
-    global.mimetype[MIME_AUDIO].download = text;
+    text = gtk_entry_get_text(entry);
+    mtype_set_download(MEDIA_AUDIO, text);
 
     entry = GTK_ENTRY(ww.page4_entry[5]);
-    text = l_strdup(gtk_entry_get_text(entry));
-    if (global.mimetype[MIME_APPLICATION].download)
-      l_free(global.mimetype[MIME_APPLICATION].download);
-    global.mimetype[MIME_APPLICATION].download = text;
+    text = gtk_entry_get_text(entry);
+    mtype_set_download(MEDIA_APPLICATION, text);
 
     entry = GTK_ENTRY(ww.page4_entry[6]);
-    text = l_strdup(gtk_entry_get_text(entry));
-    if (global.mimetype[MIME_IMAGE].download)
-      l_free(global.mimetype[MIME_IMAGE].download);
-    global.mimetype[MIME_IMAGE].download = text;
+    text = gtk_entry_get_text(entry);
+    mtype_set_download(MEDIA_IMAGE, text);
 
     entry = GTK_ENTRY(ww.page4_entry[7]);
-    text = l_strdup(gtk_entry_get_text(entry));
-    if (global.mimetype[MIME_TEXT].download)
-      l_free(global.mimetype[MIME_TEXT].download);
-    global.mimetype[MIME_TEXT].download = text;
+    text = gtk_entry_get_text(entry);
+    mtype_set_download(MEDIA_TEXT, text);
     break;
   case 5:
     if (global.lib) {
@@ -430,13 +365,13 @@ static void wizard_apply_current() {
     }
     for (i1 = 0; i1 < GTK_CLIST(ww.clist)->rows; i1++) {
       gtk_clist_get_text(GTK_CLIST(ww.clist), i1, 0, &text);
-      mime = mime2int(text);
+      mime = media2int(text);
       gtk_clist_get_text(GTK_CLIST(ww.clist), i1, 1, &text);
-      text2 = l_strdup(text);
+      text2 = g_strdup(text);
       if (text2[strlen(text2) - 1] == DIR_SEP)
 	text2[strlen(text2) - 1] = 0;
       lib_add_folder(global.lib, text2, mime);
-      l_free(text2);
+      g_free(text2);
     }
     break;
   }
@@ -472,7 +407,7 @@ static void wizard_help() {
 
 static void on_wizard_dir_select2(gchar * dir) {
 
-  strcpy(list[0], MimeNames(ww.browse_mime, 1));
+  strcpy(list[0], int2media(ww.browse_mime, 1));
   strcpy(list[1], dir);
   if (list[1][strlen(list[1]) - 1] == DIR_SEP)
     list[1][strlen(list[1]) - 1] = 0;
@@ -503,47 +438,49 @@ static GtkWidget *create_wizard_popup() {
   gtk_container_add(GTK_CONTAINER(popup), item);
   gtk_signal_connect(GTK_OBJECT(item), "activate",
 		     GTK_SIGNAL_FUNC(on_folder_add), 
-		     (gpointer) MIME_MP3);
+		     (gpointer) MEDIA_MP3);
   
   item = gtk_menu_item_new_with_label("Video");
   gtk_widget_show(item);
   gtk_container_add(GTK_CONTAINER(popup), item);
   gtk_signal_connect(GTK_OBJECT(item), "activate",
 		     GTK_SIGNAL_FUNC(on_folder_add),
-		     (gpointer) MIME_VIDEO);
+		     (gpointer) MEDIA_VIDEO);
 
   item = gtk_menu_item_new_with_label("Audio");
   gtk_widget_show(item);
   gtk_container_add(GTK_CONTAINER(popup), item);
   gtk_signal_connect(GTK_OBJECT(item), "activate",
 		     GTK_SIGNAL_FUNC(on_folder_add),
-		     (gpointer) MIME_AUDIO);
+		     (gpointer) MEDIA_AUDIO);
 
   item = gtk_menu_item_new_with_label("Application");
   gtk_widget_show(item);
   gtk_container_add(GTK_CONTAINER(popup), item);
   gtk_signal_connect(GTK_OBJECT(item), "activate",
 		     GTK_SIGNAL_FUNC(on_folder_add),
-		     (gpointer) MIME_APPLICATION);
+		     (gpointer) MEDIA_APPLICATION);
 
   item = gtk_menu_item_new_with_label("Image");
   gtk_widget_show(item);
   gtk_container_add(GTK_CONTAINER(popup), item);
   gtk_signal_connect(GTK_OBJECT(item), "activate",
 		     GTK_SIGNAL_FUNC(on_folder_add),
-		     (gpointer) MIME_IMAGE);
+		     (gpointer) MEDIA_IMAGE);
 
   item = gtk_menu_item_new_with_label("Text");
   gtk_widget_show(item);
   gtk_container_add(GTK_CONTAINER(popup), item);
   gtk_signal_connect(GTK_OBJECT(item), "activate",
-		     GTK_SIGNAL_FUNC(on_folder_add), (gpointer) MIME_TEXT);
+		     GTK_SIGNAL_FUNC(on_folder_add), 
+		     (gpointer) MEDIA_TEXT);
 
   item = gtk_menu_item_new_with_label("All Files");
   gtk_widget_show(item);
   gtk_container_add(GTK_CONTAINER(popup), item);
   gtk_signal_connect(GTK_OBJECT(item), "activate",
-		     GTK_SIGNAL_FUNC(on_folder_add), (gpointer) MIME_NONE);
+		     GTK_SIGNAL_FUNC(on_folder_add),
+		     (gpointer) MEDIA_NONE);
 
   return popup;
 }
@@ -570,11 +507,11 @@ on_wizard_destroy(GtkObject* object,
 static void on_wizard_dir_select(gchar * dir) {
   gchar *dir2;
 
-  dir2 = l_strdup(dir);
+  dir2 = g_strdup(dir);
   if (dir2[strlen(dir2) - 1] == DIR_SEP)
     dir2[strlen(dir2) - 1] = 0;
   gtk_entry_set_text(GTK_ENTRY(ww.browse_entry), dir2);
-  l_free(dir2);
+  g_free(dir2);
 }
 
 static void
@@ -1033,17 +970,25 @@ static GtkWidget* create_wizard (void) {
   gtk_widget_show (label);
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
-  entry = gtk_entry_new ();
-  gtk_widget_show (entry);
-  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 0, 1,
-                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-                    (GtkAttachOptions) (0), 0, 0);
+  entry = gtk_entry_new();
+  gtk_widget_show(entry);
+  gtk_table_attach(GTK_TABLE (table), entry, 1, 2, 0, 1,
+		   (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
+		   (GtkAttachOptions)(0), 0, 0);
   ww.page3_entry[0] = entry;
 #ifdef HAVE_OGG
+# ifdef HAVE_FLAC
+  gtk_entry_set_text (GTK_ENTRY (entry), "mp3 ogg flac");
+# else
   gtk_entry_set_text (GTK_ENTRY (entry), "mp3 ogg");
+# endif // HAVE_FLAC
 #else
+# ifdef HAVE_FLAC
+  gtk_entry_set_text (GTK_ENTRY (entry), "mp3 flac");
+# else
   gtk_entry_set_text (GTK_ENTRY (entry), "mp3");
-#endif
+# endif // HAVE_FLAC
+#endif // HAVE_OGG
 
   entry = gtk_entry_new ();
   gtk_widget_show (entry);
@@ -1051,7 +996,7 @@ static GtkWidget* create_wizard (void) {
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   ww.page3_entry[1] = entry;
-  gtk_entry_set_text(GTK_ENTRY(entry), "mpg mpeg avi asf mov wmf rm ram");
+  gtk_entry_set_text(GTK_ENTRY(entry), "\"mpg mpeg\" avi asf mov wmf \"rm ram\"");
 
   entry = gtk_entry_new ();
   gtk_widget_show (entry);
@@ -1060,9 +1005,17 @@ static GtkWidget* create_wizard (void) {
                     (GtkAttachOptions) (0), 0, 0);
   ww.page3_entry[2] = entry;
 #ifdef HAVE_OGG
+# ifdef HAVE_FLAC
   gtk_entry_set_text(GTK_ENTRY(entry), "wav au");
+# else
+  gtk_entry_set_text(GTK_ENTRY(entry), "wav au flac");
+# endif
 #else
+# ifdef HAVE_FLAC
   gtk_entry_set_text(GTK_ENTRY(entry), "wav au ogg");
+# else
+  gtk_entry_set_text(GTK_ENTRY(entry), "wav au ogg flac");
+# endif
 #endif
 
   entry = gtk_entry_new ();
@@ -1079,7 +1032,7 @@ static GtkWidget* create_wizard (void) {
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   ww.page3_entry[4] = entry;
-  gtk_entry_set_text(GTK_ENTRY(entry), "jpg jpeg bmp gif png");
+  gtk_entry_set_text(GTK_ENTRY(entry), "\"jpg jpeg\" bmp gif png");
 
   entry = gtk_entry_new ();
   gtk_widget_show (entry);
